 /* spacer */ (function () { var $ = jQuery; 

/*global ox : true, jQuery, Class, $, _, resizeEvents, getFullImgSrc, activefolder */

/**
 * @namespace
 */
ox = ox !== undefined ? ox : {};

/**
 * @namespace
 */
ox.gui = {
    themePath: getFullImgSrc("")
};

/**
 * Simple JavaScript Inheritance -- John Resig
 * adopted from http://ejohn.org/blog/simple-javascript-inheritance/
 */

(function () {
    
    var constructing = false, fnTest = /xyz/.test(function (xyz) {
        return xyz;
    }) ? /\b_super\b/ : /.*/;
    
    // The base Class implementation (does nothing)
    this.Class = function () {
    };

    // Create a new Class that inherits from this class
    this.Class.extend = function (prop) {
        
        var _super = this.prototype;

        // Instantiate a base class (but only create the instance,
        // don't run the "construct" constructor)
        constructing = true;
        var prototype = new this();
        constructing = false;
        
        // allow functions
        if (typeof prop === "function") {
            // create tmp object
            var obj = {};
            // execute function bound to tmp object
            $.proxy(prop, obj)();
            // return obj (which is now changed)
            prop = obj;
        }

        // factory
        var factory = function (name, fn) {
            return function () {
                var tmp = this._super;
                // Add a new ._super() method that is the same method
                // but on the super-class
                this._super = _super[name];
                // The method only need to be bound temporarily, so we
                // remove it when we're done executing
                var ret = fn.apply(this, arguments);
                this._super = tmp;
                return ret;
            };
        };
        
        // Copy the properties over onto the new prototype
        for (var name in prop) {
            // Check if we're overwriting an existing function
            prototype[name] = $.isFunction(prop[name]) && $.isFunction(_super[name]) && fnTest.test(prop[name]) ? factory(name, prop[name]) : prop[name];
        }
        
        // add clone
        prototype.getConstructor = function () {
            return Class;
        };

        // The dummy class constructor
        var Class = function () {
            // All construction is actually done in the "construct" method
            if (!constructing && this.construct) {
                this.construct.apply(this, arguments);
            }
        };

        // Populate our constructed prototype object
        Class.prototype = prototype;
        // Enforce the constructor to be what we expect
        Class.constructor = Class;
        // And make this class extendable
        Class.extend = arguments.callee;

        return Class;
    };
}());

// helper

var $handleContextMenu = function (e) {
    //var e = event || window.event;
    var tag = e.target.tagName;
    var type = e.target.type || "";
    if (tag === "INPUT" && type.search(/^(text|password)$/i) >= 0) {
        return true;
    } else {
        e.preventDefault();
        e.stopPropagation();
        return false;
    }
};

var $handleSelection = function (event) {
    var e = $.event.fix(event || window.event);
    var tag = e.target.tagName;
    var type = e.target.type || "";
    if (tag === "INPUT" && type.search(/^(text|password)$/i) >= 0) {
        return true;
    } else {
        e.preventDefault();
        e.stopPropagation();
        return false;
    }
};

/**
 * Callback wrapper
 */
ox.call = function () {
    var args = $.makeArray(arguments);
    var callback = args.shift();
    if (typeof callback === "function") {
        callback.apply(this, args);
    }
};

/**
 * jQuery plugin
 */
$.fn.intoViewport = function (node) {

    if (!node) {
        return this;
    }
    
    // IE sometimes crashes so...
    try {
        
        var pane = $(node);
        var height = pane.height();
        
        // get visible area
        var y1 = pane.scrollTop();
        var y2 = y1 + height;
        
        // get position
        var pos = this.position();
        var top = pos.top + y1;
        
        // out of visible area?
        if (top < y1) {
            top = top < 50 ? 0 : top;
            pane.scrollTop(top);
        } else if (top > y2) {
            pane.scrollTop(top);
        }
        
    } catch (e) {
        console.error(e);
    }
    
    return this;
};

/**
 * Mousewheel support
 */

(function($) {

    // handler
    var handler = function (event) {
        
        // get event
        var e = event || window.event;
        
        // fix event object
        e = $.event.fix(e);
        
        // set properties
        e.type = "mousewheel";
        e.delta = 0;
        e.deltaSign = 0;
        
        // get delta
        if (e.wheelDelta) {
            e.delta = e.wheelDelta / 120;
        }
        if (e.detail) {
            e.delta = -e.detail / 3;
        }
        
        // signed delta
        if (e.delta !== 0) {
            e.deltaSign = e.delta / Math.abs(e.delta);
        }
        
        // go!
        return $.event.handle.apply(this, [e]);
    };
    
    // event type
    var type = ox.browser.Gecko ? "DOMMouseScroll" : "mousewheel";
    
    // setup/teardown
    $.event.special.mousewheel = {
        setup: function () {
            // add listener
            if (this.addEventListener) {
                // Firefox, Safari, etc.
                this.addEventListener(type, handler, false);
            } else {
                // IE specific
                this.onmousewheel = handler;
            }
        },
        
        teardown: function () {
            // remove listener
            if (this.removeEventListener) {
                // Firefox, Safari, etc.
                this.removeEventListener(type, handler, false);
            } else {
                this.onmousewheel = null;
            }
        }
    };

}(jQuery));

/**
 * IFrame fix
 */

(function ($) {
    
    var tmpl = $("<div/>").css({
        position: "absolute",
        top: "0px",
        left: "0px",
        width: "0px",
        height: "0px",
        opacity: 0.01,
        backgroundColor: "white"
    });
    
    $.fn.overlay = function (flag) {
        
        if (flag === undefined || flag === true) {
            // loop
            this.each(function (i, n) {
                var node = $(n), p = node.parent();
                var w = node.outerWidth(), h = node.outerHeight();
                var pos = node.position();
                // create overlay
                var overlay = tmpl.clone(false).css({
                    top: pos.top + "px",
                    left: pos.left + "px",
                    width: w + "px",
                    height: h + "px"
                });
                // add
                p.append(overlay);
                node.data("overlay", overlay);
            });
        } else {
            // loop
            this.each(function (i, n) {
                var node = $(n), overlay = node.data("overlay");
                if (overlay) {
                    overlay.remove();
                    node.removeData("overlay");
                }
            });
        }
        
        return this;
    };
    
}(jQuery));

$.fixIFrames = (function () {
    
    var overlays = $();
    
    return function (flag) {
        // hide overlays (always)
        overlays.remove();
        overlays = $();
        // show?
        if (flag) {
            // get all visible iframes
            var iframes = $("iframe:visible"), i = 0, $i = iframes.length;
            var iframe, offset, width, height, fix;
            // loop
            for (; i < $i; i++) {
                // get
                iframe = iframes.eq(i);
                // get dimensions
                offset = iframe.offset();
                width = iframe.outerWidth();
                height = iframe.outerHeight();
                // add overlay
                overlays = overlays.add($("<div/>").css({
                    position: "absolute",
                    top: offset.top,
                    left: offset.left,
                    width: width,
                    height: height,
                    zIndex: 5000,
                    backgroundColor: "white",
                    opacity: 0.01
                }).text(" ").appendTo(document.body));
            }
        }
    };
    
}());

$.quickConfig = (function () {
    
    return function (containerSelector, configSelector, linkSelector, textClosed, textOpen) {
        
        var container = $(containerSelector);
        var config = $(configSelector);
        var link = $(linkSelector);
        
        textClosed = textClosed !== undefined ? textClosed : "Quick config"; /*i18n*/
        textOpen = textOpen !== undefined ? textOpen : "Close quick config"; /*i18n*/
        
        if (!config.hasClass("ox-quick-config")) {
            // adjust container
            container.css({
                position: "absolute",
                zIndex: 2
            });
            // remember top position
            container.data("top", parseInt(container.css("top") || 0, 10));
            // adjust config
            config.addClass("ox-quick-config").css({
                position: "absolute",
                zIndex: 1,
                top: (config.css("top") || 0) + "px",
                right: "0px",
                height: "auto",
                left: "0px",
                minHeight: "25px"
            });
            // adjust link
            link
                .css({
                    color: "#00f",
                    textDecoration: "underline",
                    whiteSpace: "nowrap"
                })
                .data("open", false)
                .append(addTranslated(textClosed));
        }
        
        // rebind events
        link.unbind("click")
            .bind("click", function (e) {
                // open
                var top = container.data("top");
                if (link.data("open") === false) {
                    link.data("open", true)
                        .empty().append(addTranslated(textOpen));
                    config.show();
                    var h = Math.max(config.outerHeight(), 25);
                    container.stop().animate({ top: (top + h) + "px" }, 500);
                } else {
                    link.data("open", false)
                        .empty().append(addTranslated(textClosed));
                    container.stop().animate({ top: top + "px" }, 500, function () {
                        config.hide();
                    });
                }
                return false;
            });
    };
}());

$.checkbox = function (id, value, checked, label, fn, data, enabled) {
    
    // text node or string/object?
    var text = label.nodeType === 3 ? label : document.createTextNode(label + ""),
        input = $("<input/>", {
                type: "checkbox",
                id: id,
                value: value,
                defaultChecked: !!checked,
                checked: !!checked
            })
            .css("border", "0px none"), // for IE
        label = $("<label/>").append(text);

    if (enabled === false) {
        // change default behavior
        var fnChange = function (e) {
            // do this before (not "return false" at end)
            e.preventDefault();
            e.stopPropagation();
            // unbind, uncheck, rebind
            $(this)
                .unbind("change", fnChange)
                .removeAttr("checked")
                .bind("change", fnChange);
        };
        input.bind("change", fnChange);
    } else {
        // default behavior
        label.attr("for", id);
        input.bind("change", data !== undefined ? data : {}, fn);
    }

    return $().add(input).add(label);
};

$.radio = function (name, value, checked, label, fn, data) {
    
    // text node or string/object?
    var text = label.nodeType === 3 ? label : document.createTextNode(label + "");
    
    return $().add(
        $("<input/>", {
            type: "radio",
            name: name,
            id: name + "-" + value,
            value: value,
            defaultChecked: !!checked,
            checked: !!checked
        })
        .css("border", "0px none") // for IE
        .bind("change", data, fn)
    ).add(
        $("<label/>", { "for": name + "-" + value }).append(text)
    );
};

$.button = function (options) {
    
    // options
    var opt = $.extend({
        title: "",
        click: $.noop,
        enabled: true,
        data: {},
        theme: "bright",
        css: {}
        // other options:
        // tabIndex, id, mousedown
    }, options || {});
    
    // class name
    var className = "ox-button" + (!opt.enabled ? " ox-button-disabled" : "");
    
    if (opt.theme === "dark") {
        // dark theme
        className += " dark";
    }
    
    if (ox.browser.Gecko) {
        // firefox
        className += " firefox";
    } else if (ox.browser.WebKit) {
        // webkit
        className += " webkit";
    }
    
    // create text node
    var text;
    if (opt.title.nodeType === 3) {
        // is text node!
        text = opt.title;
    } else {
        text = addTranslated(opt.title);
    }
    
    // create button
    var button = $("<button/>").addClass(className).append(
        $("<span/>").append(text)
    ).bind(
        "click", opt.data, opt.click
    );
    
    // add id?
    if (opt.id !== undefined) {
        button.attr("id", opt.id);
    }
    
    // add tabindex?
    if (opt.tabIndex !== undefined) {
        button.attr("tabindex", opt.tabIndex);
    }
    
    // IE padding fix
    if (IE <= 7 && /\S/.test(String(opt.title))) {
        button.css({ width: "auto", overflow: "visible" });
    }
    
    return button;
};

(function($){
    $.fn.extend({
        center: function () {
            return this.each(function() {
                var top = ($(window).height() - $(this).outerHeight()) / 2;
                var left = ($(window).width() - $(this).outerWidth()) / 2;
                $(this).css({ 
                    position: "absolute", margin: 0,
                    top: (top > 0 ? top : 0) + "px",
                    left: (left > 0 ? left : 0) + "px"
                });
            });
        }
    }); 
})(jQuery);
/**
 * 
 * All content on this website (including text, images, source
 * code and any other original works), unless otherwise noted,
 * is licensed under a Creative Commons License.
 * 
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 * 
 * Copyright (C) 2004-2009 Open-Xchange, Inc.
 * Mail: info@open-xchange.com 
 * 
 * @author Matthias Biggeleben <matthias.biggeleben@open-xchange.com>
 * 
 */

// DEVELOPMENT IN PROGRESS // SUBJECT TO PERMANENT CHANGE!

/** @ignore */
var oxWidgetFactory = {};

ox.gui.Widget = Class.extend({
/** @lends ox.gui.Widget.prototype */
    
    /** 
     * The basic widget class
     * @constructs
     * @param {String} id An optional unique id for the widget.
     */
    construct: function(id) {
    
        // widget ID
        this.id = this.autoId(id, "widget");
        
        // add to global index?
        if (ox.desktop && this.id) {
            ox.desktop.registerWidget(this);
            // remove "old" listeners - if id is reused
            ox.desktop.removeWidgetListener({ id: this.id });
        }
        
        /** The parent widget */
        this.parent = null;
        /** The widget's position if inside a container widget */
        this.position = 0;
        /** Layout parameters used by the layout manager **/
        this.layout = {};
        /** user data **/
        this.data = {};
        
        // can be cloned?
        this.clonable = this.clonable || false;
        
        // status
        this.statusLoaded = false;
        this.statusVisible = true;
        this.statusEnabled = true;
        this.statusFocused = false;
        this.statusDraggable = false;
        this.statusDropzone = false;
        
        this.requiresPaint = true;
        this.requiresRepaint = false;
        this.requiresResize = true;
        this.requiresNanny = false;
        this.requiresValidation = true;
        
        this.dom = {};
        /** The widget's DOM node */
        this.dom.node = null;
        /** Listeners */
        this.listeners = {};
        
        this.statusAllowContextMenu = false;
        this.statusAllowSelect = false;
        
        this.numHovers = 0;
        
        // create DOM node
        this.nodeCreate();
    },
    
    /**
     * The class name of the widget, e.g. ox.gui.Widget.
     * @returns (String) The class name.
     */
    getClass: function() { return "ox.gui.Widget"; },

    /**
     * @private
     */
    autoId: function(id, prefix) { 
        return id || ((prefix || "widget") + "-" + (ox.desktop.globalCounter++)); 
    },
    
    /**
     * @private
     * @param {String} The tag name of the widget's DOM node, e.g. DIV or SPAN.
     */
    nodeCreate: function(tagName) {
        // dom? (can happen if a class must call createNode before calling the super constructor)
        if (!this.dom) {
            this.dom = { node: null, placeholder: null };
        }
        // create node?
        if (!this.dom.node) {
            // get class name
            var widgetClass = this.getClass();
            // prototype in widget factory?
            if (this.clonable && oxWidgetFactory[widgetClass]) {
                // clone prototype
                this.dom.node = oxWidgetFactory[widgetClass].cloneNode(true);
            } else {
                this.dom.node = document.createElement(tagName || "div");
                this.dom.node.style.cursor = "default";
                // Selection
                this.nodePreventSelection();
                // Context Menu
                this.nodePreventContextMenu();
                // initialize node (can be overwritten by sub classes)
                this.nodeInit();
                   // can be cloned?
                if (this.clonable) {
                    oxWidgetFactory[widgetClass] = this.dom.node.cloneNode(true);
                }
            }
            // id
            if (this.id) { this.dom.node.id = this.id; }
            // class
            this.dom.node.setAttribute("oxWidgetClass", this.getClass());
            // add listeners
            this.nodeListeners();
        }
    },
    
    nodePreventContextMenu: function() {
        // FF, IE, Safari
        $(this.dom.node).bind('contextmenu', $handleContextMenu);
    },
    
    nodePreventSelection: function() {
        if (ox.browser.Gecko) {
            // Firefox
            this.dom.node.style.MozUserSelect = "-moz-none";
        } else {
            // this way for IE and others / prevents strange IE7 scrollbar flicker issue as well
            $(this.dom.node).bind('selectstart', $handleSelection);
        }
    },
    
    /**
     * @private
     */
    nodeInit: function() {
    },
    
    /**
     * @private
     */
    nodeListeners: function() {
    },
    
    /**
     * Add to container
     */
    addTo: function (parent) {
        if (parent && parent.add) {
            parent.add(this);
        }
        return this;
    },
    
    /**
     * Specifies the DOM node this widget will be added to.
     * @param Parent DOM node.
     */
    setParentDOMNode: function(parentDOMNode) {
        // keep visibility status
        var vStatus = this.statusVisible;
        // already in DOM?
        if (this.dom.node.parentNode) {
            this.removeFromDOM();
        }
        // append to DOM
        parentDOMNode.appendChild(this.dom.node);
        // restore visibility
        this.statusVisible = vStatus;
        // support chaining
        return this;
    },
    
    /**
     * Removes this widget from the DOM.
     */
    removeFromDOM: function() {
        // loop node object
        for (var name in this.dom) {
            // we cannot use jquery's remove here since it removes all event handlers
            // which is often not wanted
            var node = this.dom[name], p = node.parentNode;
            if (p) { p.removeChild(node); }
        }
        this.requiresValidation = true;
        this.requiresRepaint = true;
        // support chaining
        return this;
    },
    
    remove: function() {
        var p = this.parent;
        if (p) {
            p.removeChild(this);
        }
    },
    
    replaceBy: function(widget) {
        var p = this.parent;
        if (p) {
            p.replace(this, widget);
        }
    },
    
    /**
     * Paints the widget. Can be overridden by sub classes.
     */
    paint: function() {
    },
    
    clear: function() {
        $(this.dom.node).empty();
    },

    /**
     * Repaints the widget. Can be overridden by sub classes.
     * @param {function()} cont A continuation function which is called after
     * the repaint completes.
     */
    repaint: function (cont) {
        this.clear();
        this.paint(cont);
    },
    
    isShowing: function() {
        return this.statusVisible && (this.parent === null || this.parent.isShowing());
    },
    
    /**
     * Invalidates the widget. Layout managers will repaint the widget next time.
     */
    invalidate: function() {
        this.requiresValidation = true;
        if (this.parent) { this.parent.invalidate(); }
    },
    
    invalidateFamily: function() {
        this.requiresNanny = true;
        // trigger
        this.triggerWidgetEvent("familychanged");
        // bubble
        this.invalidateSize();
    },

    invalidateVisibility: function() {
        var p = this.parent;
        if (p) {
            p.requiresRepaint = true;
            p.invalidateSize();
        }
    },

    invalidateSize: function() {
        this.requiresResize = true;
        this.requiresValidation = true;
        if (this.parent) { this.parent.invalidateSize(); }
    },
    
    validate: function (force) {
        if (force || (this.requiresValidation && this.isShowing())) {
            // validate tree
            this.validateTree();
            // resize!
            this.resize();
        }
        return this;
    },
    
    validateTree: function() {
        
        if (this.requiresValidation && this.statusVisible) {
            // validate this
            if (this.requiresPaint || this.requiresNanny) {
                // paint!
                this.paint();
                this.requiresNanny = false;
                this.requiresPaint = false;
                this.requiresRepaint = false;
            }
            else if (this.requiresRepaint) {
                // repaint!
                this.repaint();
                this.requiresRepaint = false;
            }
            // done!
            this.requiresValidation = false;
            // trigger event
            this.triggerWidgetEvent("valid");
        }
    },
    
    /**
     * @returns {bool} True if the widget is valid.
     */
    isValid: function() { 
        return this.requiresValidation === false && 
            this.statusVisible === true && this.requiresPaint === false &&
            this.requiresNanny === false && this.requiresRepaint === false;
    },
    
    show: function() { return this.setVisible(true); },
    hide: function() { return this.setVisible(false); },
    
    setVisible: function(flag, cont) {
        var prev = this.statusVisible;
        flag = !!flag;
        // changed?
        if (flag != prev) {
            // update
            this.statusVisible = flag;
            // invalidate
            this.invalidateVisibility();
            // validate now?
            if (this.isShowing()) {
                this.validateTree();
            }
            if (this.statusVisible) {
                this.paintVisible(cont);
                ox.desktop.triggerEvent("visible", this, null);
            } else {
                this.paintInvisible(cont);
                ox.desktop.triggerEvent("invisible", this, null);
            }
        } else {
            ox.call(cont);
        }
        return this;
    },
    
    /**
     * @private
     */
    paintVisible: function(cont) {
        this.dom.node.style.display = "block";
        ox.call(cont);
    },
    
    /**
     * @private
     */
    paintInvisible: function(cont) {
        this.dom.node.style.display = "none";
        ox.call(cont);
    },
    
    setDraggable: function(flag, options) {
        var defaults = {
            type: "generic",
            handle: null,
            helper: "clone",
            customize: null,
            drop: "window",
            clone: function() { return this.clone(); }
        };
        this.dragOptions = $.extend(defaults, options);
        // bind functions?
        if ($.isFunction(this.dragOptions.helper)) {
            this.dragOptions.helper = $.proxy(this.dragOptions.helper, this);
        }
        if ($.isFunction(this.dragOptions.clone)) {
            this.dragOptions.clone = $.proxy(this.dragOptions.clone, this);
        }
        this.statusDraggable = !!flag;
        this.paintDraggable();
        return this;
    },
    
    /**
     * @private
     */
    paintDraggable: function() {
        var self = this;
        if (this.statusDraggable) {
            // add type && make draggable
            $(this.dom.node).attr("dragtype", this.dragOptions.type).draggable({
                addClasses: false,
                appendTo: 'body',
                scroll: false,
                zIndex: 65000,
                cursorAt: { left: 10, top: 10 },
                helper: this.dragOptions.helper,
                grid: [10, 10],
                distance: 10,
                iframeFix: true,
                revert: 'invalid',
                start: function(event, ui) {
                    // adjust css
                    $(ui.helper).css({
                        width: "auto", height: "auto", right: "auto", bottom: "auto"
                    });
                    // add class
                    ui.helper.addClass("oxDragHelper");
                    // customize?
                    if ($.isFunction(self.dragOptions.customize)) {
                        $.proxy(self.dragOptions.customize, self)(ui.helper);
                    }
                    // add reference to widget
                    ui.helper.data("widget", self);
                },
                stop: function(event, ui) {
                }
            });
            // handle?
            if (this.dragOptions.handle) {
                $(this.dom.node).draggable('option', 'handle', this.dragOptions.handle);
            }
            // has no dropzone?
            if (!this.statusDropzone) {
                // add dummy dropzone (accepts nothing)
                this.setDropzone(true);// { accept: ox.True });
            }
        } else {
            $(this.dom.node).removeAttr("dragtype").draggable('destroy');
        }
    },
    
    setDropzone: function(flag, options) {
        var defaults = {
            accept: ox.True
        };
        // process options
        this.dropOptions = $.extend(defaults, options);
        // make function
        var accept = this.dropOptions.accept;
        this.dropOptions.accept = $.isFunction(this.dropOptions.accept) ? this.dropOptions.accept : function(d) {
            return d.is(accept);
        };
        // set status
        this.statusDropzone = !!flag;
        this.paintDropzone();
        return this;
    },
    
    /**
     * @private
     */
    paintDropzone: function() {
        var self = this;
        if (this.statusDropzone) {
            $(this.dom.node).droppable({
                accept: this.dropOptions.accept,
                greedy: true,
                over: function(event, ui) {
                    // draggable? (exclude e.g. a normal window)
                    if(ui.helper.hasClass("oxDragHelper")) {
                        ui.helper.addClass("oxOverDropzone");
                    }
                },
                out: function(event, ui) {
                    // draggable? (exclude e.g. a normal window)
                    if(ui.helper.hasClass("oxDragHelper")) {
                        ui.helper.removeClass("oxOverDropzone");
                    }
                },
                drop: function(event, ui) {
                    // draggable? (exclude e.g. a normal window)
                    if(ui.helper.hasClass("oxDragHelper")) {
                        // get widget
                        var widget = ui.helper.data("widget");
                        var opt = widget.dragOptions;
                        if (opt.drop == "window" && $.isFunction(opt.clone)) {
                            var offset = ui.helper.offset();
                            var width = Math.max(400, Math.min(ui.helper.width() + 20, 800));
                            var height = Math.min(ui.helper.width() + 20, 300);
                            var win = ox.desktop.createWindow(self.autoId(null, widget.id+"-clone-window")).
                                setTitle("A clone").
                                setSize(width, height).
                                setPosition(offset.left, offset.top);
                            // clone
                            var ret = opt.clone(win);
                            if (ret) {
                                win.add(ret).show();
                            }
                        }
                    }
                }
            });
        } else {
            $(this.dom.node).droppable('destroy');
        }
    },
    
    clone: function(id) {
        // create new widget of same class
        var constructor = this.getConstructor();
        var clone = new constructor(id || (this.id+"-clone"));
        // copy layout information
        clone.layout = ox.cloneObject(this.layout);
        // copy customized functions
        for (var fn in this) {
            if (typeof(this[fn]) == 'function' && this[fn] != clone[fn]) {
                clone[fn] = this[fn];
            }
        }
        // copy listeners
        var listeners = this.getListeners();
        for (var i in listeners) {
            var listener = listeners[i];
            clone.addListener("widget:" + listener.type, listener.callback);
        }
        // custom handler
        this.cloneDeep(clone);
        // return clone
        return clone;
    },
    
    cloneDeep: function(clone) {
    },
    
//    /**
//     * Hides the widget
//     */
//    __hide: function() {
//        // has node?
//        if (this.statusVisible) {
//            this.paintInvisible();
//            // set status
//            this.statusVisible = false;
//            // trigger event
//            this.triggerWidgetEvent("hide", null);
//        }
//        // support chaining
//        return this;
//    },
    
    /**
     * Sets the CSS class name of the widget's DOM node.
     * @param className {String} The CSS class name.
     */
    setCSSClass: function(className) {
        $(this.dom.node).removeClass().addClass(className);
        return this;
    },

    /**
     * Adds a CSS class name to the widget's DOM node.
     * @param className {String} The CSS class name.
     */
    addCSSClass: function(className) {
        $(this.dom.node).addClass(className);
        return this;
    },

    /**
     * Removes a CSS class name from the widget's DOM node.
     * @param className {String} The CSS class name.
     */
    removeCSSClass: function(className) {
        $(this.dom.node).removeClass(className);
        return this;
    },

    /**
     * Specifies the CSS text of the widget's DOM node.
     * @param cssText {String} The CSS text, e.g. "background-color: yellow".
     */
    css: function() {
        var tmp = $(this.dom.node);
        tmp.css.apply(tmp, arguments);
        return this;
    },
    
    setStyle: function() {
        if (this.dom.node) {
            // pass to jQuery' CSS method
            var args = $.makeArray(arguments);
            switch (args.length) {
            case 1: $(this.dom.node).css(args[0]); break;
            case 2: $(this.dom.node).css(args[0], args[1]); break;
            }
        }
        return this;
        
//        var cssTextList = null;
//        // is string or array?
//        if (typeof(cssText) == "string") {
//            cssTextList = new Array(cssText);
//        } else {
//            cssTextList = cssText;
//        }
//        // loop styles
//        for (var i = 0; i < cssTextList.length; i++) {
//            // get key & value
//            var keyValue = cssTextList[i].split(/\:\s?/);
//            var name = keyValue[0], value = keyValue[1];
//            if (name.search(/-/) != -1) {
//                // CSS text syntax
//                // adjust key
//                name = name.toLowerCase().replace(/-[a-z]/, function(match) { return match.substr(1,1).toUpperCase(); });
//                // set DOM style
//                if (this.dom.node) { this.dom.node.style[name] = value || ""; } // IE needs explicit empty string
//            } else {
//                // DOM style syntax
//                if (this.dom.node) { this.dom.node.style[name] = value || ""; } // IE needs explicit empty string
//            }
//        }
//        // support chaining
//        return this;
    },
    
    /**
     * Adds an event listener.
     * @param {String} eventKey The event key, e.g. "dom:click", "dom:mousedown" as well
     * as "widget:..." for widget-specific events, and "ox:..." for global events.
     * @param {Object} target [Optional] The target of the event. Can be a DOM node or a widget.
     * @param {Object} callback A callback function. The callback will be bound to the widget, i.e.
     * "this" will refer to the widget.
     */
    addListener: function() {
        
        var opt = this.prepareListener.apply(this, arguments);
        
        // set listener for domain
        switch (opt.domain) {
            case "ox":
                // add global listener
                ox.desktop.addGlobalListener(opt);
                break;
            case "widget":
                // add widget listener
                opt.target = opt.target || this;
                ox.desktop.addWidgetListener(opt);
                break;
            case "dom":
                // add listener
                opt.target = opt.target || this.dom.node;
                ox.desktop.addDOMListener(opt);
                break;
            default:
                console.error("addListener() Unkown domain", opt.domain);
                break;
        }
        // support chaining
        return this;
    },
    
    removeListener: function() {
        
        var opt = this.prepareListener.apply(this, arguments);
        // set listener for domain
        switch (opt.domain) {
            case "ox":
                // add global listener
                ox.desktop.removeGlobalListener(opt);
                break;
            case "widget":
                // add widget listener
                opt.target = opt.target || this;
                ox.desktop.removeWidgetListener(opt);
                break;
            case "dom":
                // adjust target
                opt.target = opt.target || this.dom.node;
                // add listener
                ox.desktop.removeDOMListener(opt);
                break;
        }
        // support chaining
        return this;
    },
    
    getListeners: function() {
        return ox.desktop.getWidgetListeners(this);
    },
    
    /**
     * @private
     */
    prepareListener: function() {
        // get parameter
        var type, target, callback, args = arguments, domain, pair;
        switch(args.length) {
            // one parameter
            case 1:
                type = args[0]; target = null; callback = null;
                break;
            // two parameter
            case 2:
                type = args[0]; target = null; callback = args[1];
                break;
            // three parameter
            case 3:
                type = args[0]; target = args[1]; callback = args[2];
                break;
            default:
                return {};
        }
        
        // get domain of event (default is "dom")
        pair = type.search(/:/) != -1 ? type.split(":") : ["dom", type];
        domain = pair[0].toLowerCase();
        type = pair[1].toLowerCase();
        
        return {
            "type": type, "target": target, "callback": callback,
            "domain": domain, "scope": this
        };
    },
    
    /**
     * Triggers a global event.
     * @param eventName {String} The name of the event.
     * @param eventData {Object} Any data that will be send to listening widgets.
     */
    triggerEvent: function(eventName, eventData) {
        // is enabled?
        if (this.statusEnabled) {
            // yes, trigger event
            ox.desktop.triggerGlobalEvent(eventName, eventData !== undefined ? eventData : {});
        }
        // support chaining
        return this;
    },
    
    /**
     * Triggers a widget event.
     * @param eventName {String} The name of the event.
     * @param eventData {Object} Any data that will be send to listening widgets.
     */
    triggerWidgetEvent: function(eventName, eventData) {
        // is enabled?
        if (this.statusEnabled) {
            // yes, trigger event
            ox.desktop.triggerEvent(eventName, this, eventData !== undefined ? eventData : {});
        }
        // support chaining
        return this;
    },
    
    trigger: function() {
        
        var self = this;
        
        // internal helper
        var trigger = function (options) {
            var namespace = "", name = "";
            // namespace?
            var index = options.name.indexOf(":");
            if (index > 0) {
                namespace = options.name.substr(0, index);
                name = options.name.substr(index+1);
            } else {
                namespace = "global";
                name = options.name;
            }
            // go!
            switch (namespace) {
                case "widget": ox.desktop.triggerEvent(name, self, options.data || null, options.callback || $.noop); break;
                case "global": ox.desktop.triggerGlobalEvent(name, options.data || null, options.callback || $.noop); break;
            }
        };
        // get arguments
        var args = $.makeArray(arguments);
        // one argument?
        if (args.length == 1 && typeof(args[0]) == "string") {
            trigger({ name: args[0], data: null });
        } else if (args.length == 2 && typeof(args[0]) == "string" && typeof(args[1]) != "function") {
            trigger({ name: args[0], data: args[1] });
        } else if (args.length == 2 && typeof(args[0]) == "string" && typeof(args[1]) == "function") {
            trigger({ name: args[0], data: null, callback: args[1] });
        } else if (args.length == 3 && typeof(args[0]) == "string"  && typeof(args[1]) != "function" && typeof(args[2]) == "function") {
            trigger({ name: args[0], data: args[1], callback: args[2] });
        }
    },
    
    /**
     * Disables the widget. Still visible, but its appearance is changed and 
     * no events are processed.
     */
    disable: function() {
        // trigger event now! since this widget will be disabled next and
        // therefore it will be ignored by event handlers
        this.triggerWidgetEvent("disable", null);
        // set enabled to false
        this.statusEnabled = false;
        // render
        this.paintDisabled();
        // support chaining
        return this;
    },

    /**
     * @private
     */
    paintEnabled: function() {
        this.removeCSSClass("disabled");
    },

    /**
     * @private
     */
    paintDisabled: function() {
        this.addCSSClass("disabled");
    },
    
    setEnabled: function (state) {
        if (this.statusEnabled !== state) {
            if (state === true) {
                this.enable();
            } else {
                this.disable();
            }
        } 
    },
    
    /**
     * Enables the widget.
     * @see disable
     */
    enable: function() {
        // set enabled to true
        this.statusEnabled = true;
        this.paintEnabled();
        // trigger event
        this.triggerWidgetEvent("enable", null);
        // support chaining
        return this;
    },
    
    /**
     * @private
     */
    paintEnable: function() {
        this.removeCSSClass("disabled");
    },
    
    /**
     * @returns {bool} True if the widget is enabled.
     */
    isEnabled: function() {
        return this.statusEnabled;
    },
    
    /**
     * @returns {bool} True if the widget is visible.
     */
    isVisible: function() {
        return this.statusVisible;
    },
    
    /**
     * @returns {bool} True if the widget has the focus.
     */
    hasFocus: function() {
        return this.statusFocused;
    },
    
    setFocus: function(flag) {
        this.statusFocused = !!flag;
        if (this.statusFocused) {
            this.triggerWidgetEvent("focus", null);
        } else {
            this.triggerWidgetEvent("blur", null);
        }
    },
    
    override: function(name, fn) {
        if ($.isFunction(fn)) {
            this[name] = fn;
        }
        return this;
    },
    
    /**
     * Sets a layout parameter
     * @param {String} name The name of the parameter.
     * @param {String} value The value of the parameter.
     */
    setLayoutParam: function(name, value) {
        if (typeof name === "object") {
            // name -> properties
            $.extend(this.layout, name);
        } else {
            this.layout[name] = value;
        }
        // invalidates layout
        this.invalidate();
        // allow chaining
        return this;
    },
    
    resize: function() { },
    
    hasValue: function() { return false; },
    
    getSize: function() {
        var width = 0, height = 0;
        // is yet painted?
        if (!this.requiresPaint) {
            // get "real" size
            var $node = $(this.dom.node);
            width = $node.width(); 
            height = $node.height();
        } else {
            // get default size
            width = this.layout.width || this.layout.defaultWidth || 0;
            height = this.layout.height || this.layout.defaultHeight || 0;
        }
        return { "width": width, "height": height };
    },
    
    phantomize: function(invisible) {
        $(this.dom.node).phantomize(invisible);
    },
    
    materialize: function() {
        $(this.dom.node).materialize();
    },
    
    /**
     * Destroys the widget (and helps the garbage collector as much as possible)
     */
    destroy: function() {

        var p = this.parent;
        
        // unregister
        ox.desktop.unregisterWidget(this);
        
        // remove parent reference
        if (this.parent !== null) {
            this.parent.removeChild(this);
            this.parent = null;
        }
        // delete layout data
        this.layout = null;
        
        // remove widget listeners
        ox.desktop.removeListenerByScope(this);
        
        // clear DOM node (remove event handler, remove child nodes, remove node)
        this.removeFromDOM();
        // loop
        for (var id in this.dom) {
            $(this.dom[id]).unbind().empty().remove();
        }
        
        // remove DOM node reference
        this.dom.node = null;
        
        // if (p) { p.validate(); }
    },
    
    getData: function(key) {
        return this.data[key];
    },

    setData: function(key, value) {
        this.data[key] = value;
        return this;
    },
    
    /**
     * Should be called sometime during the initialization of subclass instances
     * to enable runtime language changes via i18nHandler().
     */
    enableI18n: function () {
        if (!this.i18nEnabled) {
            this.addListener("ox:LanguageChanged", this.i18nHandler);
            this.i18nEnabled = true;
        }
    },
    
    /**
     * Disables handling of runtime language changes if it was enabled
     * by enableI18n().
     */
    disableI18n: function () {
        if (this.i18nEnabled) {
            this.removeListener("ox:LanguageChanged", this.i18nHandler);
            this.i18nEnabled = false;
        }
    },
    
    /**
     * Subclasses should overwrite this method to handle runtime language
     * changes. The actual handling is controlled by enableI18n() and
     * disableI18n().
     */
    i18nHandler: function () {
    }
});

ox.gui.DebugWidget = ox.gui.Widget.extend({
/** @lends ox.gui.DebugWidget.prototype */
    
    getClass: function() { return "ox.gui.DebugWidget"; },
    
    /** 
     * A simple widget that can be used for debugging
     * @constructs
     * @extends ox.gui.Widget
     * @param {String} id An optional unique id for the widget.
     */
    construct: function(id) {
        // call super class constructor (inherit from Container)
        this._super(id);
        // local constructor
        this.currentState = "";
    },
    
    nodeInit: function() {
        this.dom.node.style.cssText = "background-color: lightyellow; border: 1px solid #fc0; padding: 5px; line-height: 11pt;";
    },

    paint: function() {
        this.dom.node.innerHTML = "I am a debug widget! My name is: <b>" + this.id + "</b>";
    }
});

ox.gui.Canvas = ox.gui.Custom = ox.gui.Widget.extend({
/** @lends ox.gui.Custom.prototype */
    
    getClass: function() { return "ox.gui.Custom"; },
    
    /** 
     * A simple widget
     * @constructs
     * @extends ox.gui.Widget
     * @param {String} id An optional unique id for the widget.
     */
    construct: function(paint, id) {
        // call super class constructor (inherit from Container)
        this._super(id);
        // override paint?
        if ($.isFunction(paint)) {
            this.paint = paint;
        }
    },
    
    nodeInit: function() {},

    paint: function() { }
});

ox.gui.Template = ox.gui.Widget.extend({
/** @lends ox.gui.Template.prototype */
    
    getClass: function() { return "ox.gui.Template"; },
    
    /** 
     * A simple template-based widget
     * @constructs
     * @extends ox.gui.Widget
     * @param {String} id An optional unique id for the widget.
     */
    construct: function(id) {
        
        // call super class constructor (inherit from Container)
        this._super(this.autoId(id, "template"));
        
        this.src = "";
        this.content = "";
        this.loading = false;
        
        this.data = {};
    },
    
    nodeInit: function() {
        $(this.dom.node).addClass("oxStretch");
    },
    
    setSrc: function(src) {
        this.src = src;
        return this;
    },
    
    paint: function() {
        var self = this;
        // already loading
        if (!this.loading) {
            this.loading = true;
            // load widget from external file
            $.ajax({
                method: "GET",
                url: this.src,
                success: function(response) {
                    self.loading = false;
                    // get content
                    self.content = response;
                    // create document fragment
                    var frag = self.getFragment(response);
                    // process & add fragment
                    self.dom.node.appendChild(self.prePaint(frag));
                    self.postPaint(self.dom.node);
                },
                error: function(response) {
                    self.loading = false;
                }
            });
        }
    },
    
    getFragment: function(html) {
        // create document fragment (take the long way, since a fragment does not "like" innerHTML)
        var frag = newfrag();
        var div = newnode("div");
        div.innerHTML = html;
        // append children
        var $c = div.childNodes;
        for (;$c.length;) { frag.appendChild($c[0]); }
        return frag;
    },
    
    postPaint: function(node) {
    },
    
    prePaint: function(frag) {
        return frag;
    },
    
    clear: function() {
        this.dom.node.innerHTML = "";
    },
    
    repaint: function() {
        // clear content
        this.clear();
        // create fragment
        var frag = this.getFragment(this.content);
        // process & add fragment
        this.dom.node.appendChild(this.prePaint(frag));
        this.postPaint(this.dom.node);
    }
});


ox.gui.Image = ox.gui.Widget.extend({
/** @lends ox.gui.Image.prototype */
    
    getClass: function() { return "ox.gui.Image"; },
    
    /** 
     * A simple image widget
     * @constructs
     * @extends ox.gui.Widget
     * @param {String} src The image source location.
     * @param {I18nString} An optional title (
     */
    construct: function(src, title) {
        // call super class constructor (inherit from Container)
        this._super(this.autoId(0, "image"));  
        this.src = src || "";
        this.title = title;
        this.image = $("<img/>");
        this.setSrc(this.src);
        this.setTitle(title);        
    },
        
    setSrc: function(src) {
        this.src = src;
        this.image.attr({ src: this.src });
        return this;
    },
    
    setTitle: function(title) {
        this.title = title;
        if (!this.title) {
            this.disableI18n();
            this.image.removeAttr("title");
        } else {
        	this.i18nHandler();
            this.enableI18n();
        }
        return this;
    },
    
    i18nHandler: function() {
        if (this.title) this.image.attr("title", expectI18n(this.title));
    },
    
    paint: function() {
        $(this.dom.node).append(this.image);
    },
    
    disable: function() {
        this._super();
        this.image.css({ opacity: .6 });
    }
});

ox.gui.Atomic = ox.gui.Widget.extend({
/** @lends ox.gui.Image.prototype */
    
    getClass: function() { return "ox.gui.Atomic"; },
    
    /** 
     * An atomic widget (plain DIV)
     * @constructs
     * @extends ox.gui.Widget
     */
    construct: function() {
        this._super(this.autoId(0, "atomic"));
    }
});
/**
 * 
 * All content on this website (including text, images, source
 * code and any other original works), unless otherwise noted,
 * is licensed under a Creative Commons License.
 * 
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 * 
 * Copyright (C) 2004-2009 Open-Xchange, Inc.
 * Mail: info@open-xchange.com 
 * 
 * @author Matthias Biggeleben <matthias.biggeleben@open-xchange.com>
 * @ignore
 */

// DEVELOPMENT IN PROGRESS // SUBJECT TO PERMANENT CHANGE!

ox.gui.Container = ox.gui.Widget.extend({
/** @lends ox.gui.Container.prototype */
    
    /** 
     * The basic container widget
     * @constructs
     * @extends ox.gui.Widget
     * @param {String} id An optional unique id for the widget.
     */
    construct: function(id) {

        // helps the layout manager finding the proper DOM node
        this.kindergarten = null;

        // the family
        /** An array of widgets added to this container */
        this.children = [];
        /** The first child */
        this.firstChild = null;
        /** The last child */
        this.lastChild = null;

        /** The layout manager of the container */
        this.layoutManager = null;

        // call super class constructor
        this._super(this.autoId(id, "container"));

        // reset target node
        this.kindergarten = this.dom.node;
    },

    getClass: function() { return "ox.gui.Container"; },
    
    nodeInit: function() {
        // set specific style
        this.setStyle({ overflow: "hidden" });
    },

    /**
     * Adds a widget to the container
     * @param {Widget} widget The widget to be added.
     */
    add: function(widget) {
        var children = this.getChildren();
        // add to list
        children.push(widget);
        // set position
        widget.position = children.length - 1;
        this.addPost(widget);
        // support chaining
        return this;
    },
    
    /*
     * @private
     */
    addPost: function (widget) {
        // is moved?
        if (widget.parent !== null) {
            widget.parent.removeChild(widget);
        }
        // first child?
        if (this.firstChild === null) {
            this.firstChild = widget;
        }
        // last child
        this.lastChild = widget;
        // set parent widget
        widget.parent = this;
        // invalidate
        this.invalidateFamily();
    },
    
    addRange: function(list) {
        var $i = 0, $l = list.length;
        for (; $i < $l; $i++) {
            this.add(list[$i]);
        }
        return this;
    },
    
    /**
     * Inserts a widget at a specific position
     * @param {Widget} widget The widget to be inserted.
     * @param {Int} index The position. Default=0.
     */
    insert: function(widget, index) {
        var children = this.getChildren();
        // adjust index
        index = index || 0;
        // add
        children.splice(index, 0, widget);
        // first child?
        if (index === 0) {
            this.firstChild = widget;
        }
        // set parent widget
        widget.parent = this;
        // set new positions
        var $c=this.getChildren(),$i=$c.length; for(;$i--;) { var child=$c[$i]; // fastest loop
            child.position = $i;
        }
        this.lastChild = $c[$c.length-1] || null;
        // invalidate
        this.invalidateFamily();
        // support chaining
        return this;
    },
    
    /**
     * Removes all children.
     */
    removeChildren: function() {
        this.firstChild = null;
        this.lastChild = null;
        // remove from DOM
        var $c=this.getChildren(),$i=$c.length; for(;$i--;) { var child=$c[$i]; // fastest loop
            child.removeFromDOM();
        }
        // remove
        var children = this.getChildren();
        children.splice(0, this.numChildren());
        // invalidate
        this.invalidateFamily();
        // support chaining
        return this;
    },
    
    moveChild: function(widget, index) {
        var $c=this.getChildren(),$i=$c.length,child; for(;$i--;) { child=$c[$i]; // fastest loop
            if (child == widget) {
                // remove child
                $c.splice($i, 1);
                // add child
                $c.splice(index, 0, widget);
                // set new positions
                $c=this.getChildren();$i=$c.length; for(;$i--;) { child=$c[$i]; // fastest loop
                    child.position = $i;
                }
                break;
            }
        }
    },
    
    removeChild: function(widget) {
        var $c=this.getChildren(),$i=$c.length,child; for(;$i--;) { child=$c[$i]; // fastest loop
            if (child == widget) {
                // remove from DOM
                child.removeFromDOM();
                // remove child
                $c.splice($i, 1);
                // set new positions
                $c=this.getChildren();$i=$c.length; for(;$i--;) { child=$c[$i]; // fastest loop
                    child.position = $i;
                }
                // invalidate
                this.invalidateFamily();
                break;
            }
        }
        return this;
    },
    
    /**
     * Removes this widget and all its children from the DOM.
     */
    removeFromDOM: function() {
        // remove all children
        var $c=this.getChildren(),$i=$c.length; for(;$i--;) { var child=$c[$i]; // fastest loop
            child.removeFromDOM();
        }
        // remove this
        ox.gui.Widget.prototype.removeFromDOM.call(this);
        // support chaining
        return this;
    },
    
    indexOf: function(widget) {
        var $c=this.getChildren(),$i=$c.length; for(;$i--;) { var child=$c[$i]; // fastest loop
            if (child == widget) {
                return $i;
            }
        }
        return -1;
    },
    
    hasChild: function(widget) {
        return this.indexOf(widget) != -1;
    },
    
    replace: function(formerWidget, newWidget) {
        var pos = this.indexOf(formerWidget);
        if (pos != -1) {
            // replace
            this.removeChild(formerWidget);
            this.insert(newWidget, pos);
            // destroy former widget
            formerWidget.destroy();
            return true;
        } else {
            return false;
        }
    },
    
    /**
     * Returns all children. Overridden by special widgets like ox.gui.MenuStrip.
     */
    getChildren: function() {
        return this.children; 
    },
    
    /**
     * Returns all visible children.
     */
    getVisibleChildren: function() {
        var i = 0, $c = this.getChildren(), $l = $c.length, tmp = [], child;
        for (; i < $l; i++) {
            child = $c[i];
            if (child.statusVisible === true) {
                tmp.push(child);
            }
        }
        return tmp; 
    },

    /**
     * Returns the number of children.
     */
    numChildren: function() {
        return this.getChildren().length;
    },

    /**
     * Determines whether or not the container has children.
     */
    hasChildren: function() {
        return this.getChildren().length > 0;
    },
    
    /*
     * Iterates over all children.
     * @param {Function} callback A callback function called for each child.
     * The function will receive three parameters: 1) The current child,
     * 2) the current index, 3) the number of children.
     */
    each: function(callback) {
        // bind callback to current class
        callback = $.proxy(callback, this);
        // get children
        var children = this.getChildren();
        var childCount = children.length;
        // loop children
        for (var i = 0; i < childCount; i++) {
            callback(children[i], i, childCount);
        }
    },
    
    /*
     * Iterates over all visible children.
     * @param {Function} callback A callback function called for each child.
     * The function will receive three parameters: 1) The current child,
     * 2) the current index, 3) the number of children.
     */
    eachVisible: function (callback) {
        if (callback) {
            // get children
            var children = this.getVisibleChildren();
            var i = 0, $l = children.length;
            // loop children
            for (; i < $l; i++) {
                callback.call(this, children[i], i, $l);
            }
        }
    },
    
    /**
     * Sets the layout manager
     * @param {ox.gui.LayoutManager} layoutManager The layout manager for this container.
     */
    setLayout: function(layoutManager) {
        this.layoutManager = layoutManager || new ox.gui.BlockFlowLayout(this);
        if (this.layoutManager.container === null) {
            this.layoutManager.container = this;
        }
        // allow chaining
        return this;
    },
    
    /**
     * @private
     */
    requireLayout: function() {
        if (this.layoutManager === null) {
            // add default manager
            this.layoutManager = new ox.gui.BlockFlowLayout(this);
        } else if (this.layoutManager.container === null) {
            // update reference
            this.layoutManager.container = this;
        }
    },
    
    /**
     * Clears the container
     * @private
     */
    clear: function() {
        // loop children
        var $c=this.getChildren(),$i=$c.length; for(;$i--;) { var child=$c[$i]; // fastest loop
            child.removeFromDOM();
        }
        // clear inner HTML
        if (this.dom.node) {
            this.dom.node.innerHTML = "";
        }
        // support chaining
        return this;
    },
    
//    __update: function() {
//        // loop children
//        var $c=this.getChildren(),$i=$c.length; for(;$i--;) { var child=$c[$i]; // fastest loop
//            if (!child.statusValid) {
//                child.update();
//            }
//        }
//        // update this widget
//        ox.gui.Widget.prototype.update.call(this);
//        // support chaining
//        return this;
//    },
    
    setKindergarten: function(DOMNode) {
        this.kindergarten = DOMNode;
        return this;
    },
    
    invalidateTree: function () {
        var $c=this.getChildren(),$i=$c.length,child; for(;$i--;) { child=$c[$i]; // fastest loop
            child.requiresValidation = true;
            if (child.invalidateTree) {
                child.invalidateTree();
            }
        }
        return this;
    },

    validateTree: function() {
        
        if (this.requiresValidation && this.statusVisible) {
            // take care of visible children
            var $c=this.getVisibleChildren(),$i=$c.length; for(;$i--;) { var child=$c[$i]; // fastest loop
                child.validateTree();
            }
            // validate this
            this.paint();
            // done!
            this.requiresValidation = false;
            // trigger event
            this.triggerWidgetEvent("valid");
        }
    },
    
    resize: function() {
        if (this.statusVisible) {
            // get visible children
            var $c=this.getVisibleChildren(),$i=$c.length; for(;$i--;) { var child=$c[$i]; // fastest loop
                child.resize();
            }
            // resize by layout manager
            if (this.layoutManager) {
                this.layoutManager.resize();
                this.triggerWidgetEvent("resize");
            }
        }
    },
    
    paint: function() {
        // no layout manager?
        this.requireLayout();
        // let the layout manager do the work
        this.layoutManager.doLayout();
        // support chaining
        return this;
    },
    
    repaint: function() {
        // no layout manager?
        this.requireLayout();
        // let the layout manager do the work
        this.layoutManager.update();
        // support chaining
        return this;
    },
    
    disable: function() {
        // disable children first
        var $c=this.getChildren(),$i=$c.length; for(;$i--;) { var child=$c[$i]; // fastest loop
            child.disable();
        }
        // call super class method
        ox.gui.Widget.prototype.disable.call(this);
        // support chaining
        return this;
    },
    
    enable: function() {
        // enable children first
        var $c=this.getChildren(),$i=$c.length; for(;$i--;) { var child=$c[$i]; // fastest loop
            child.enable();
        }
        // call super class method
        ox.gui.Widget.prototype.enable.call(this);
        // support chaining
        return this;
    },
    
    /**
     * Hides all widget's that implement hideHovers. (Don't ask)
     * @private
     */
    hideHovers: function() {
        // loop children
        var $c=this.getChildren(),$i=$c.length; for(;$i--;) { var child=$c[$i]; // fastest loop
            if (child.numHovers > 0) {
                child.hideHovers();
                child.numHovers = 0;
            }
        }
        // support chaining
        return this;
    },
    
    hasHover: function (flag) {
        // inc/dec
        this.numHovers = Math.max(0, this.numHovers + (flag === true ? 1 : -1));
        // bubble
        if (this.parent) {
            this.parent.hasHover(flag);
        }
    },
    
    getValues: function() {
        var values = {};
        // loop children
        var $c=this.getChildren(),$i=$c.length; for(;$i--;) { var child=$c[$i]; // fastest loop
            // container?
            if ($.isFunction(child.getValues)) {
                // get values
                values = $.extend(values, child.getValues());
            } 
            // has value?
            else if (child.hasValue()) {
                // get single value
                values[child.getName()] = child.getValue();
            }
        }
        return values;
    },
    
    getSerializedValues: function() {
        return $.param(this.getValues());
    },
    
    i18nHandler: function () {
        if (this.layoutManager) {
            this.layoutManager.i18nHandler();
        }
    },
    
    /**
     * Destroys the widget (and helps the garbage collector as much as possible)
     */
    destroy: function() {

        var p = this.parent;
        
        // remove DOM node now (for better performance)
        $(this.dom.node).remove();
        
        // destroy children (sorry for the wording)
        var $c=this.getChildren();
        if ($c) {
            var $i=$c.length; for(;$i--;) { var child=$c[$i]; // fastest loop
                child.destroy();
            }
        }
        
        // delete references
        this.children = [];
        this.firstChild = null;
        this.lastChild = null;
        this.kindergarten = null;
        
        // layout manager
        if (this.layoutManager) {
            // delete circular reference
            this.layoutManager.destroy();
            this.layoutManager = null;
        }
        
        ox.gui.Widget.prototype.destroy.call(this);
        
        // if (p) { p.validate(); }
    }
    
});
/**
 * 
 * All content on this website (including text, images, source
 * code and any other original works), unless otherwise noted,
 * is licensed under a Creative Commons License.
 * 
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 * 
 * Copyright (C) 2004-2009 Open-Xchange, Inc.
 * Mail: info@open-xchange.com 
 * 
 * @author Matthias Biggeleben <matthias.biggeleben@open-xchange.com>
 * @ignore
 */

// DEVELOPMENT IN PROGRESS // SUBJECT TO PERMANENT CHANGE!


ox.gui.LayoutManager = Class.extend({
/** @lends ox.gui.LayoutManager.prototype */
    
    getClass: function() { return "ox.gui.LayoutManager"; },
    
    /** 
     * The basic layout manager
     * @constructs
     * @param {ox.gui.Container} The container widget that should be managed
     * by the layout manager.
     */
    construct: function(container) {
        this.container = container || null;
        this.nannies = {};
        this.decorator = null;
        this.firstRun = true;
    },
    
    /**
     * This will layout the container widget.
     */
    doLayout: function() {
    },

    /**
     * This will only do updates on the container widget.
     */
    update: function() {
        this.doLayout();
    },

    /**
     * Handles a resize of the container's children or its parent container.
     */
    resize: function() {
    },
    
    /**
     * Handle i18n changes
     */
    i18nHandler: function () {
    },
    
    getNanny: function (id) {
        return typeof this.nannies[id] == "undefined" ? null : this.nannies[id];
    },
    
    createNanny: function (id, parentNode) {
        var nanny = this.nannies[id] = { node: $("<div/>"), processed: false };
        if (typeof parentNode != "undefined") {
            nanny.node.appendTo(parentNode);
        }
        return nanny;
    },
    
    setUnprocessed: function () {
        for (var id in this.nannies) {
            this.nannies[id].processed = false;
        }
    },

    removeUnprocessed: function () {
        for (var id in this.nannies) {
            if (this.nannies[id].processed === false) {
                this.nannies[id].node.remove();
                delete this.nannies[id];
            }
        }
    },
    
    destroy: function() {
        // container ref
        this.container = null;
        // decorator
        if (this.decorator) {
            this.decorator.remove();
        }
        // nannies
        for (var id in this.nannies) {
             this.nannies[id].node.remove();
             delete this.nannies[id];
        }
    },
    
    park: function (child) {
        if (child === undefined) {
            // move all children's DOM nodes
            // similar to remove but saves all event handlers
            if (this.container) {
                var park = $("<div/>");
                this.container.each(function (child) {
                    park.append(child.dom.node);
                });
            }
        } else if (child) {
            var park = $("<div/>").append(child.dom.node);
        }
    }
});

//-----------------------------------------------------------------------------

ox.gui.FlowLayout = ox.gui.LayoutManager.extend({
/** @lends ox.gui.FlowLayout.prototype */
    
    getClass: function() { return "ox.gui.FlowLayout"; },
    
    /** 
     * A typical flow layout manager. Actually, the browser will take care
     * of the layout.
     * @extends ox.gui.LayoutManager
     * @constructs
     * @param {ox.gui.Container} The container widget that should be managed
     * by the layout manager.
     */
    construct: function(container) {
        this._super(container);
    },
    
    doLayout: function() {
        
        // local vars
        var con = this.container;
        var targetDOMNode = con.kindergarten;
        
        // loop children
        var $i=0,$c=con.getChildren(),$l=$c.length; for (;$i<$l;$i++) { var child = $c[$i]; // fast loop
            // add to DOM & show
            //child.validate();
            targetDOMNode.appendChild(child.dom.node);
        }
    }
});

//-----------------------------------------------------------------------------

ox.gui.HBoxLayout = ox.gui.LayoutManager.extend({
    /** @lends ox.gui.HBoxLayout.prototype */
        
    getClass: function() { return "ox.gui.HBoxLayout"; },
    
    /** 
     * A HBox layout manager.
     * @extends ox.gui.LayoutManager
     * @constructs
     * @param {ox.gui.Container} The container widget that should be managed
     * by the layout manager.
     */
    construct: function(container) {
        this._super(container);
    },
    
    doLayout: function() {
        // local vars
        var con = this.container, tr = $("<tr/>"), 
            table = $("<table/>");
        $(con.kindergarten).append(table.append(tr));
        
        for (var attr in con.layout) {
            var tattr = "";
            switch (attr) {
                case "twidth":
                    table.attr("width", con.layout[attr]);
                    break;
                case "theight":
                    table.attr("height", con.layout[attr]);
                    break;
                default: break;
          }
      }
        
        // loop children
        var $i=0,$c=con.getChildren(),$l=$c.length; 
        for (;$i<$l;$i++) { 
            var child = $c[$i]; // fast loop
            var td = $("<td/>");
//            for (var attr in child.layout) {
//                switch (attr) {
//                    case "valign":
//                    case "align":
//                    case "width":
//                    case "height":
//                    case "nowrap":
//                        td.attr(attr, child.layout[attr]);
//                        break;
//                }
//            }
            tr.append(td.attr(child.layout || {}).append(child.dom.node));
        }
    }
});

// -----------------------------------------------------------------------------

ox.gui.BlockFlowLayout = ox.gui.LayoutManager.extend({
/** @lends ox.gui.BlockFlowLayout.prototype */
    
    getClass: function() { return "ox.gui.BlockFlowLayout"; },
    
    /** 
     * A vertical flow layout manager.
     * @extends ox.gui.LayoutManager
     * @constructs
     * @param {ox.gui.Container} The container widget that should be managed
     * by the layout manager.
     */
    construct: function(container) {
        this._super(container);
        this.inheritHeight = true;
    },
    
    doLayout: function() {
        
        // local vars
        var con = this.container;
        var targetDOMNode = con.kindergarten;
        var top = 0;
        
        // loop children
        var $i=0,$c=con.getChildren(),$l=$c.length; for (;$i<$l;$i++) { var child = $c[$i]; // fast loop
            // adjust child
            var div = newnode("div", {
                position: "absolute", 'top': top + "px", left: "0px", right: "0px"
            });
            // add to DOM & show
            //child.validate();
            div.appendChild(child.dom.node);
            targetDOMNode.appendChild(div);
            var h = $(child.dom.node).height();
            div.style.height = h + "px";
            top += h;
        }
        // windows e.g. do not inherit the height
        if (con.layout.inheritHeight) {
            targetDOMNode.style.height = top + "px";
        }
    },
    
    resize: function() {
        
        var con = this.container;
        var targetDOMNode = con.kindergarten;
        var kinder = targetDOMNode.childNodes;
        var top = 0;
        
        // loop children
        var $i=0,$c=con.getChildren(),$l=$c.length; for (;$i<$l;$i++) { var child = $c[$i]; // fast loop
            // resize child
            //child.resize();
            // adjust height & top
            if (kinder[$i]) {
                var h = $(child.dom.node).height();
                kinder[$i].style.top = top + "px";
                kinder[$i].style.height = h + "px";
                top += h;
            }
        }
        // windows e.g. do not inherit the height
        if (con.layout.inheritHeight) {
            targetDOMNode.style.height = top + "px";
        }
    }
});

// -----------------------------------------------------------------------------

ox.gui.ZeroLayout = ox.gui.LayoutManager.extend({
/** @lends ox.gui.ZeroLayout.prototype */
    
    getClass: function() { return "ox.gui.ZeroLayout"; },
    
    /** 
     * A zero layout manager. Does nothing.
     * @extends ox.gui.LayoutManager
     * @constructs
     * @param {ox.gui.Container} The container widget that should be managed
     * by the layout manager.
     */
    construct: function(container) {
        this._super(container);
    },
    
    doLayout: function() {
        // loop children
        var con = this.container, z = 0;
        var $i=0, $c=con.getChildren(), $l=$c.length, child, p;
        for (;$i<$l;$i++) {
            child=$c[$i]; // fast loop
            p = child.dom.node.parentNode;
            if (p === null || p.nodeType === 11) {
                con.kindergarten.appendChild(child.dom.node);
                child.dom.node.style.zIndex = z++;
            }
        }
    }
});

// -----------------------------------------------------------------------------

ox.gui.BorderLayout = ox.gui.LayoutManager.extend({
/** @lends ox.gui.BorderLayout.prototype */
    
    getClass: function() { return "ox.gui.BorderLayout"; },
    
    /** 
     * A typical border layout manager. Children can use "top", "right", "bottom",
     * "left", and "center".
     * @extends ox.gui.LayoutManager
     * @constructs
     * @param {ox.gui.Container} The container widget that should be managed
     * by the layout manager.
     */
    construct: function(container) {
        this._super(container);
        this.nannies = {};
        this.decorator = null;
    },
    
    getOrderedChildrenX: function() {
        // create proper order (top, bottom, left, right, center)
        var con = this.container;
        var children = [null, null, null, null, null];
        var $i=0, $c=con.getChildren(), $l=$c.length; for (;$i<$l;$i++) { var child=$c[$i]; // fast loop
            if (child.layout.position) {
                switch(child.layout.position) {
                case "top": children[0] = child; break;
                case "bottom": children[1] = child; break;
                case "left": children[2] = child; break;
                case "right": children[3] = child; break;
                case "center": children[4] = child; break;
                }
            }
        }
        return children;
    },
    
    getOrderedChildren: function() {
        // create proper order (top, bottom, left, right, center)
        var con = this.container;
        var center = [], nonCenter = [];
        var $i=0, $c=con.getChildren(), $l=$c.length; for (;$i<$l;$i++) { var child=$c[$i]; // fast loop
            if (child.layout.position) {
                switch(child.layout.position) {
                case "top":
                case "right":
                case "bottom":
                case "left":
                    nonCenter.push(child);
                    break;
                case "center":
                    center.push(child);
                    break;
                }
            }
        }
        return $.merge(nonCenter, center);
    },
    
    doLayout: function() {
        
        var con = this.container, kindergarten = con.kindergarten, nanny;
        var top = 0, bottom = 0, left = 0, right = 0, height = 0, width = 0;
        
        // stretch container
        con.addCSSClass("oxStretch");
        
        // add decorator DIV (stretch as well)
        if (this.decorator === null) {
            this.decorator = $("<div/>").addClass("oxStretch");
            if (con.layout.padding) {
                // padding becomes margin
                this.decorator.css("margin", con.layout.padding);
            }
            this.decorator.appendTo(kindergarten);
        }
        
        // mark nannies
        this.setUnprocessed();
        
        var numerize = function (value, defaultValue) {
            var n = parseInt(value, 10);
            return isNaN(n) ? (defaultValue || 0) : n;
        };
        
        // loop children
        var $i=0, $c=this.getOrderedChildren(), $l=$c.length;
        for (;$i<$l;$i++) { var child=$c[$i]; // fast loop
            if (child) {
                var css = null;
                switch (child.layout.position) {
                case "top":
                    height = numerize(child.layout.height, 25);
                    css = {
                        "top": top + "px", "left": left + "px", "right": right + "px", "height": height + "px"
                    };
                    top += height;
                    break;
                case "bottom":
                    height = numerize(child.layout.height, 25);
                    css = {
                        "bottom": bottom + "px", "left": left + "px", "right": right + "px", "height": height + "px"
                    };
                    bottom += height;
                    break;
                case "left":
                    width = numerize(child.layout.width, 25);
                    css = {
                        "top": top + "px", "bottom": bottom + "px", "left": left + "px", "width": width + "px"
                    };
                    left += width;
                    break;
                case "right":
                    width = numerize(child.layout.width, 25);
                    css = {
                        "top": top + "px", "bottom": bottom + "px", "right": right + "px", "width": width + "px"
                    };
                    right += numerize(width, 0);
                    break;
                case "center":
                    css = {
                        "top": top + "px", "left": left + "px", "right": right + "px", "bottom": bottom + "px"
                    };
                    break;
                }
                // add
                if (css !== null) {
                    // get nanny
                    nanny = this.getNanny(child.id);
                    // has no nanny?
                    if (nanny === null) {
                        // create nanny
                        nanny = this.createNanny(child.id, this.decorator);
                    }
                    // added?
                    if (nanny.node.get(0).firstChild != child.dom.node) {
                        nanny.node.empty().append(child.dom.node);
                    }
                    // update nanny
                    nanny.node.css(
                        $.extend({ position: "absolute", overflow: "hidden" }, css)
                    );
                    // mark as processed
                    nanny.processed = true;
                }
            }
        }
        
        // remove unprocessed nannies
        this.removeUnprocessed();
    },
    
    resize: function() {
        
        var con = this.container, kinderkarten = con.kindergarten;
        var top = 0, bottom = 0, left = 0, right = 0;
        var outerDIVs = con.kindergarten.childNodes;
        return;
    }
});
/**
 * 
 * All content on this website (including text, images, source
 * code and any other original works), unless otherwise noted,
 * is licensed under a Creative Commons License.
 * 
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 * 
 * Copyright (C) 2004-2009 Open-Xchange, Inc.
 * Mail: info@open-xchange.com 
 * 
 * @author Matthias Biggeleben <matthias.biggeleben@open-xchange.com>
 * @ignore
 */

// DEVELOPMENT IN PROGRESS // SUBJECT TO PERMANENT CHANGE!


ox.gui.StatefulContainer = ox.gui.Container.extend({
/** @lends ox.gui.StatefulContainer.prototype */
    
    getClass: function() { return "ox.gui.StatefulContainer"; },
    
    /** 
     * A container that manages its state.
     * @constructs
     * @extends ox.gui.Container
     * @param {String} id An optional unique id for the widget.
     */
    construct: function(id) {
        
        // call super class constructor (inherit from Container)
        this._super(this.autoId(id, "statContainer"));
        
        // local constructor
        this.currentState = "";
        this.currentChild = null;
        this.childIndex = {};
    },
    
    /**
     * Determines that state of the container.
     * @returns {String} A string representation of the current state.
     */
    getState: function() {
        var states = [], widget = this;
        // move down
        states = states.concat(this.getChildState(this));
        // move up
        while (widget) {
            // is a stateful container
            if (widget instanceof ox.gui.StatefulContainer && widget.currentState != "") {
                // add current
                states.push(widget.id + ":" + widget.currentState);
            }
            // climb up
            widget = widget.parent;
        }
        // return result
        return states.join(";");
    },
    
    /**
     * @private
     */
    getChildState: function(widget) {
        var states = [];
        if (widget instanceof ox.gui.Container || widget == ox.gui) {
            for (var i = 0; i < widget.children.length; i++) {
                var child = widget.children[i];
                if (child instanceof ox.gui.StatefulContainer && child.currentState != "") {
                    // add current
                    states.push(child.id + ":" + child.currentState);
                }
                states = states.concat(this.getChildState(child));
            }
        }
        return states;
    },
    
    /**
     * Sets the state of the container.
     * @param {String} A string representation of the new state.
     */
    setState: function(state) {
        var stateDescriptor = [];
        // is string?
        if (Object.isString(state)) {
            var pairs = state.split(';');
            for (var i = 0; i < pairs.length; i++) {
                var pair = pairs[i].split(':');
                var key = pair[0] || "", value = pair[1] || "";
                stateDescriptor[key.toLowerCase()] = value;
            }
        } else {
            stateDescriptor = state;
        }
        var lowerID = this.id.toLowerCase();
        if (stateDescriptor[lowerID] !== undefined) {
            this.currentState = stateDescriptor[lowerID];
            delete stateDescriptor[lowerID];
        }
        // loop children
        this.setStateRec(this, stateDescriptor);
    },
    
    /**
     * @private
     */
    setStateRec: function(widget, stateDescriptor) {
        // is container?
        if (widget instanceof ox.gui.Container || widget == ox.gui) {
            // loop children
            for (var i = 0; i < widget.children.length; i++) {
                var child = widget.children[i];
                // is a stateful container
                if (child instanceof ox.gui.StatefulContainer) {
                    child.setState(stateDescriptor);
                } else {
                    // recursion
                    this.setStateRec(child, stateDescriptor);
                }
            }
        }
    }
});
/**
 * 
 * All content on this website (including text, images, source
 * code and any other original works), unless otherwise noted,
 * is licensed under a Creative Commons License.
 * 
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 * 
 * Copyright (C) 2004-2009 Open-Xchange, Inc.
 * Mail: info@open-xchange.com 
 * 
 * @author Matthias Biggeleben <matthias.biggeleben@open-xchange.com>
 * @ignore
 */

// DEVELOPMENT IN PROGRESS // SUBJECT TO PERMANENT CHANGE!


ox.gui.ViewControl = ox.gui.StatefulContainer.extend({
/** @lends ox.gui.ViewControl.prototype */
    
    getClass: function () { return "ox.gui.ViewControl"; },
    
    /** 
     * A view controller. Allows only one child to be active and displayed.
     * @constructs
     * @extends ox.gui.StatefulContainer
     * @param {String} id An optional unique id for the widget.
     */
    construct: function (id) {
        
        // call super class constructor (inherit from Container)
        this._super(this.autoId(id, "viewControl"));
        
        this.setLayout(new ox.gui.ZeroLayout());
        this.addCSSClass("oxStretch");
        
        // local constructor
        this.activeView = null;
        this.lastView = null;
        this.views = {};
    },
    
    add: function (view) {
        // must be an instance of ox.gui.Widget
        if (view instanceof ox.gui.Widget) {
            // invisible
            view.statusVisible = true; // required to force paintInvisible (see next row)
            view.setVisible(false);
            // call super class method
            ox.gui.StatefulContainer.prototype.add.call(this, view);
            // first added view?
            if (!this.activeView) {
                this.activeView = view;
                this.activeView.setVisible(true);
                this.triggerWidgetEvent("viewchanged", this.activeView);
            }
            // add to index
            this.views[view.id] = view;
        } else {
            console.error("ox.gui.ViewControl.add(<view>)", "<view> must be an instance of ox.gui.Widget!");
        }
        // support chaining
        return this;
    },
    
    getState: function () {
        this.currentState = this.activeView.id;
        // call super class method
        return ox.gui.StatefulContainer.prototype.getState.call(this);
    },
    
    setState: function (state) {
        // call super class method
        ox.gui.StatefulContainer.prototype.setState.call(this, state);
        this.showView(this.currentState);
    },
    
    /**
     * Get the position of the active view.
     * @returns {Int} The position.
     */
    getPosition: function () {
        return this.activeView.position;
    },

    /**
     * Set the new active view by its position
     * @param {Int} position The position of the child.
     */
    setPosition: function (position) {
        // switch view
        if (position <= this.children.length - 1) {
            this.showView(this.children[position].id);
        }
    },
    
    /**
     * @private
     */
    hideActiveView: function () {
        if (this.activeView !== null) {
            // hide
            this.activeView.setVisible(false);
            this.currentState = "";
            this.validate();
        }
    },
    
    /**
     * Shows the given view.
     * @param {string} view The view to be shown (id).
     */
    showView: function (id) {
        // switch view?
        if (this.views[id] !== undefined && this.currentState !== id) {
            // hide active view
            this.hideActiveView();
            // remember last view
            this.lastView = this.activeView;
            // show view
            this.activeView = this.views[id];
            this.activeView.setVisible(true);
            this.currentState = id;
            // trigger event
            this.triggerWidgetEvent("viewchanged", this.activeView);
            // validate
            this.validate();
        }
    },
    
    showLastView: function () {
        if (this.lastView) {
            this.showView(this.lastView.id);
        }
    }
});

// -----------------------------------------------------------------------------

ox.gui.View = ox.gui.Container.extend({
/** @lends ox.gui.View.prototype */
    
    getClass: function() { return "ox.gui.View"; },
    
    /** 
     * A view container. A valid child for ox.gui.ViewController.
     * @constructs
     * @extends ox.gui.Container
     * @param {String} id An optional unique id for the widget.
     */
    construct: function(id) {
        this._super(id);
        this.addCSSClass("oxStretch");
        this.statusVisible = false;
    }    
});
/**
 * 
 * All content on this website (including text, images, source code and any
 * other original works), unless otherwise noted, is licensed under a Creative
 * Commons License.
 * 
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 * 
 * Copyright (C) 2004-2009 Open-Xchange, Inc. Mail: info@open-xchange.com
 * 
 * @author Matthias Biggeleben <matthias.biggeleben@open-xchange.com>
 * @ignore
 */

// DEVELOPMENT IN PROGRESS // SUBJECT TO PERMANENT CHANGE!

ox.gui.Desktop = ox.gui.StatefulContainer.extend({
/** @lends ox.gui.Desktop.prototype */
    
    getClass: function() { return "ox.gui.Desktop"; },
    
    /**
     * The topmost GUI control
     * 
     * @constructs
     * @extends ox.gui.StatefulContainer
     */
    construct: function() {
        
        this.globalCounter = 0;
        this.globalTabIndex = 1;
        this.listeners = {};
        this.globalListeners = {};
        
        this.registry = {
            "widget": {
                "list": {}, "observer": {}
            },
            "window": {
                "list": {}, "observer": {}
            }
        };

        this.themePath = "img";
        this.windowManager = null;

        this._super("ox.gui.desktop");
        
        // layout manager
        this.setLayout(new ox.gui.ZeroLayout(this));
    },
    
    nodeInit: function() {
        this._super();
        this.addCSSClass("oxContainer");
        this.addCSSClass("oxStretch");
    },
    
    paint: function() {
        // add special corners
// this.cornerTopRight = newnode("div", 0, {
// className: "oxDesktopCorner oxTopRight"
// });
// document.body.appendChild(this.cornerTopRight);
// this.cornerBottomRight = newnode("div", 0, {
// className: "oxDesktopCorner oxBottomRight"
// });
// document.body.appendChild(this.cornerBottomRight);
// // handler
// var Self = this;
// // $(this.cornerTopRight).mouseover(function() {
// // if (Self.windowManager) {
// // Self.windowManager.expose("hide");
// // }
// // });
// $(this.cornerBottomRight).mouseover(function() {
// if (Self.windowManager) {
// Self.windowManager.expose("expose");
// }
// });
// $(this.cornerBottomRight).mouseout(function() {
// if (Self.windowManager) {
// Self.windowManager.expose("reverse");
// }
// });
        ox.gui.StatefulContainer.prototype.paint.call(this);
    },

    getWindowManager: function() {
        return this.windowManager;
    },

    setWindowManager: function(windowManager) {
        this.windowManager = windowManager;
        return this;
    },
    
    createWindow: function(id, widgetClass, windowManager) {
        // class
        widgetClass = widgetClass ? widgetClass : ox.gui.Window;
        // create
        var w = new widgetClass(id);
        // implizit window manager?
        if (windowManager === undefined) {
            // use default window manager and add now
            if (this.windowManager) {
                this.windowManager.add(w);
                w.windowManager = this.windowManager;
            }
        } else {
            // use explicit window manager (do not add now)
            w.windowManager = windowManager;
        }
        return w;
    },
    
    /**
     * Gets a widget. The widget might not yet be created, so this function uses
     * a callback.
     */
    getWidget: function(id, callback) {
        return this.getObject("widget", id, callback);
    },
    
    /**
     * Gets a window. The window might not yet be created, so this function uses
     * a callback.
     */
    getWindow: function(id, callback) {
        return this.getObject("window", id, callback);
    },
    
    /**
     * Shows a window. The window might not yet be created, so this function
     * uses a callback internally.
     */
    showWindow: function(id) {
        this.getObject("window", id, function(window) {
            window.show();
        });
    },
    
    /**
     * @private
     */
    getObject: function(type, id, callback) {
        var reg = this.registry[type];
        // exists?
        if (reg.list[id] !== undefined) {
            ox.call(callback, reg.list[id]);
            return reg.list[id];
        } else {
            // create observer
            if (reg.observer[id] === undefined) {
                reg.observer[id] = [];
            }
            reg.observer[id].push(callback);
            return undefined;
        }
    },
    
    /**
     * Registers a widget in the global widget repository.
     */
    registerWidget: function(widget) {
        this.registerObject("widget", widget);
    },
    
    /**
     * Unregisters a widget
     */
    unregisterWidget: function(widget) {
        this.unregisterObject("widget", widget);
    },
    
    /**
     * Registers a window in the global window repository.
     */
    registerWindow: function(window) {
        this.registerObject("window", window);
    },
    
    /**
     * Unregisters a window
     */
    unregisterWindow: function(window) {
        this.unregisterObject("window", window);
    },
    
    /**
     * @private
     */
    registerObject: function(type, obj) {
        var reg = this.registry[type], id = obj.id || "";
        // add object
        reg.list[id] = obj;
        // any observers?
        if (reg.observer[id] !== undefined) {
            // loop observers
            var observers = reg.observer[id];
            for (var i = 0; i < observers.length; i++) {
                // callback
                observers[i](obj);
            }
            // delete observers
            delete reg.observer[id];
        }
    },
    
    /**
     * @private
     */
    unregisterObject: function(type, obj) {
        var reg = this.registry[type], id = obj.id || "";
        // remove object
        delete reg.list[id];
    },
    
    oops: function(e) {
        var message = Object.isString(e) ? e : (e.message || e) + "\n\n" + ox.desktop.oops.caller;
        alert("Oops! This sould not happen:\n\n" + message);
    },

    oopsFunc: function(functionSignature, message) {
        alert("Oops!\n" + functionSignature + "\n" + message);
    },
    
    setRootDOMNode: function(DOMNode) {
        this.setParentDOMNode(DOMNode);
        return this;
    },

    addDOMListener: function(options) {
        // get options
        var defaults = { 
            type: "click",
            target: document,
            callback: $.noop,
            scope: null
        };
        var opt = $.extend(defaults, options);
        
        // add listener
        $(opt.target).bind(opt.type, $.proxy(opt.callback, opt.scope || opt.target));
    },
    
    removeDOMListener: function(options) {
        // get options
        var defaults = { 
            type: "click",
            target: document,
            callback: $.noop,
            scope: null
        };
        var opt = $.extend(defaults, options);
        
        // add listener
        $(opt.target).unbind(opt.type, $.proxy(opt.callback, opt.scope || opt.target));
    },

    addGlobalListener: function(options) {
        // get options
        var defaults = { 
            type: "default",
            callback: $.noop,
            scope: this
        };
        var opt = $.extend(defaults, options);
        // lower case
        var types = opt.type.toLowerCase().split(/\s+/);
        delete opt.type;
        for (var i in types) {
            var type = types[i];
            // first listener?
            if (this.globalListeners[type] === undefined) {
                this.globalListeners[type] = [];
            }
            // add listener
            this.globalListeners[type].push({
                "callback": opt.callback,
                "scope": opt.scope
            });
        }
    },
    
    removeGlobalListener: function(options) {
        // get options
        var defaults = { 
            type: "default",
            callback: $.noop
        };
        var opt = $.extend(defaults, options);
        // lower case
        var type = opt.type.toLowerCase();
        var removeAll = opt.callback == $.noop;
        // queue exists?
        if (this.globalListeners[type]) {
            var queue = this.globalListeners[type], i = 0;
            // loop through queue
            while (i < queue.length) {
                if (removeAll || queue.callback == opt.callback) {
                    // remove from queue
                    queue.splice(i, 1);
                } else {
                    i++;
                }
            }
        }
    },

    addWidgetListener: function(options) {
        // get options
        var defaults = { 
            type: "valid",
            target: this,
            callback: $.noop,
            scope: this
        };
        var opt = $.extend(defaults, options);
        // lower case
        var types = opt.type.toLowerCase().split(/\s+/);
        delete opt.type;
        for (var i in types) {
            var type = types[i];
            var target = opt.target;
            // first listener?
            if (this.listeners[target.id] === undefined) {
                this.listeners[target.id] = [];
            }
            // add listener
            this.listeners[target.id].push({
                "callback": opt.callback,
                "type": type,
                "scope": opt.scope
            });
        }
    },
    
    removeWidgetListener: function(options) {
        // get options
        var defaults = { 
            type: null,
            target: this,
            callback: $.noop
        };
        var opt = $.extend(defaults, options);
        // remove all listeners? (i.e. no type given)
        if (!opt.type) {
            var id = opt.id !== undefined ? opt.id : opt.target;
            delete this.listeners[id];
        } else {
            // remove all callbacks of one type?
            var removeAll = opt.callback == $.noop;
            // queue exists?
            var target = opt.target;
            if (this.listeners[target.id]) {
                var queue = this.listeners[target.id], i = 0;
                // lower case
                var type = opt.type.toLowerCase();
                // loop through queue
                while (i < queue.length) {
                    var listener = queue[i];
                    if (listener.type == opt.type && (removeAll || listener.callback == opt.callback)) {
                        // remove from queue
                        queue.splice(i, 1);
                    } else {
                        i++;
                    }
                }
            }
        }
    },
    
    getWidgetListeners: function(widget) {
        var id, listeners, tmp = [];
        if (widget) {
            id = widget.id;
            listeners = this.listeners[id];
            if (listeners) {
                for (var i in listeners) {
                    tmp.push(listeners[i]);
                }
            }
        }
        return tmp;
    },
    
    getAllListeners: function() {
        
        var tmp = [], widget, queue, i, type, listener;
        
        // loop widgets
        for (widget in this.listeners) {
            queue = this.listeners[widget];
            // loop listeners
            for (i in queue) {
                listener = queue[i];
                tmp.push({ domain: "widget", type: listener.type, scope: listener.scope, callback: listener.callback });
            }
        }
        // loop globals
        for (type in this.globalListeners) {
            queue = this.globalListeners[type];
            // loop listeners
            for (i in queue) {
                listener = queue[i];
                tmp.push({ domain: "ox", type: type, scope: listener.scope, callback: listener.callback });
            }
        }
        return tmp;
    },
    
    removeListenerByScope: function(scope) {
        
        var id, queue, i, type, listener;
        
        // loop widgets
        for (id in this.listeners) {
            queue = this.listeners[id];
            // loop listeners
            i = 0;
            while (i < queue.length) {
                listener = queue[i];
                if (listener.scope == scope) { queue.splice(i, 1); } else { i++; }
            }
        }
        // loop globals
        for (type in this.globalListeners) {
            queue = this.globalListeners[type];
            // loop listeners
            for (i in queue) {
                listener = queue[i];
                if (listener.scope == scope) { queue.splice(i, 1); } else { i++; }
            }
        }
    },

    triggerEvent: function(type, source, data, callback) {
        // adjust source
        source = source || null;
        // lower case
        type = type.toLowerCase();
        var joint = new Join(callback || $.noop);
        var conts = [];
        var i, item;
        // event queue defined?
        if (this.listeners[source.id] !== undefined) {
            // get queue
            var queue = this.listeners[source.id], $l = queue.length;
            // loop through queue (to join)
            for (i = 0; i < $l; i++) {
                item = queue[i];
                if (type == item.type) {
                    conts[i] = joint.add($.noop);
                }
            }
            // loop again through queue
            for (i = 0; i < $l; i++) {
                item = queue[i];
                // match?
                if (type == item.type) {
                    // call back!
                    var ret = ($.proxy(item.callback, item.scope)({
                        "name": type, "source": source, "data": data, "cont": conts[i]
                    }));
                    // sync?
                    if (ret !== false) {
                        // no waiting
                        conts[i]();
                    }
                }
            }
            
        }
        // no conts?
        if (conts.length === 0) {
            ox.call(callback);
        }
    },
    
    triggerGlobalEvent: function(type, data) {
        // lower case
        type = type.toLowerCase();
        // event queue defined?
        if (this.globalListeners[type] !== undefined) {
            // get queue
            var queue = this.globalListeners[type];
            // loop through queue
            for (var i = 0; i < queue.length; i++) {
                var item = queue[i];
                // call back!
                ($.proxy(item.callback, item.scope)(data || null));
            }
        }
    },
    
    shutdown: function(options) {
        // options
        options = $.extend({
            animate: $.noop,
            complete: $.noop
        }, options);
        // destroy desktop
        this.destroy();
        // destroy ox namespace (keep function "call" so that final callbacks don't fail)
        window.ox = { call: ox.call };
        window.oxWidgetFactory = null;
        // animate
        options.animate(function () {
            // clear body
            $(document.body).css({ background: "#111 none" }).empty();
            // be last
            setTimeout(function () {
                // remove all scripts
                var headTag = document.getElementsByTagName('HEAD').item(0);
                var $c = headTag.childNodes, i = 0;
                while (i < $c.length) {
                    if ($c[i].tagName == "SCRIPT") {
                        // remove script
                        headTag.removeChild($c[i]);
                    } else {
                        i++;
                    }
                }
                // callback
                options.complete();
            }, 10);
        });
    },
    
    destroy: function() {
        // remove special event handler
        $(document.body).unbind();
        $(document).unbind();
        $(window).unbind();
        // remove main window controller
        this.windowManager = null;
        // garbage collection for IE: destroy all registered widgets
        if (ox.browser.IE) {
            var list = this.registry.widget.list;
            for (var id in list) list[id].destroy();
        }
        // super class
        ox.gui.Container.prototype.destroy.call(this);
    }
});


// ------------------------------------------------------------------------------

// create desktop object
ox.desktop = new ox.gui.Desktop();
$(window).unload(function() { ox.desktop.destroy(); });

// shortcuts
ox.$w = function(id) {
    return ox.desktop.registry.widget.list[id] || null;
};

// ------------------------------------------------------------------------------

/**
 * 
 * All content on this website (including text, images, source
 * code and any other original works), unless otherwise noted,
 * is licensed under a Creative Commons License.
 * 
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 * 
 * Copyright (C) 2004-2009 Open-Xchange, Inc.
 * Mail: info@open-xchange.com 
 * 
 * @author Matthias Biggeleben <matthias.biggeleben@open-xchange.com>
 * @ignore
 */

// DEVELOPMENT IN PROGRESS // SUBJECT TO PERMANENT CHANGE!


ox.gui.Selection = Class.extend({
/** @lends ox.gui.LayoutManager.prototype */
    
    getClass: function() { return "ox.gui.Selection"; },
    
    /** 
     * An abstract item selection
     * @constructs
     */
    construct: function() {
        
        this.multiple = true;
        
        this.selectedItems = {};
        this.observedItems = [];
        this.observedItemsIndex = {};
        this.preselectedItem = null;
        this.auto = true; // allows any form of automatic selection
        
        this.onChangeCallback = null;
        this.onRenameHandler = $.noop;
        this.clickHandler = $.noop;
        this.contextMenuHandler = null;
        this.lastSelectedItem = null;
        
        this.statusEnabled = true;
        this.focusIndex = 0;
        this.container = null;
        
        this.classFocus = "oxFocus";
        this.classSelected = "oxSelected";
        
        this.dispatcher = new ox.api.event.Dispatcher();
    },

    setMultiple: function(flag) {
        this.multiple = !!flag;
        return this;
    },

    onChange: function(fn) {
        this.onChangeCallback = fn;
        return this;
    },

    onRename: function(fn) {
        this.onRenameHandler = fn;
        return this;
    },

    triggerChangedEvent: function() {
        if (typeof this.onChangeCallback === "function") {
            this.onChangeCallback(this.getSelectedItems());
        }
    },
    
    observe: function (container) {
        
        this.container = container;
        this.focusIndex = 0;
        this.processNodes();
        
        var contains = function (node, list) {
            var i = 0, $l = list.length;
            for (; i < $l; i++) {
                if (node === list[i]) {
                    return true;
                }
            }
            return false;
        };
        
        // helper
        var self = this;
        var fnClick = function (e) {
            // get original target
            var node = e.originalTarget || e.target, id = undefined, selectable = false;
            var con = this.container, p;
            // due to a Firefox bug, we have to check if node is an ancenster of container
            // "Permission denied to access property * from a non-chrome context..."
            if (contains(node, $("*", con))) {
                // climb up and look for oxID
                try {
                    while (node) {
                        id = ox.util.firstOf(node.oxID, $(node).attr("oxid"));
                        selectable = ox.util.firstOf(node.oxSelectable, true);
                        if (id !== undefined) {
                            break;
                        }
                        p = node.parentNode;
                        node = p != con ? p : null;
                    }
                }
                catch (e) {
                }
            }
            // found the id?
            if (id !== undefined) {
                // default actions
                var click = function (option) {
                    if (option === "quiet") {
                        self.clickQuiet(id, e.ctrlKey, e.shiftKey);
                    } else if (option === "force") {
                        self.clickQuiet(id, e.ctrlKey, e.shiftKey);
                        self.triggerChangedEvent();
                    } else {
                        self.click(id, e.ctrlKey, e.shiftKey);
                    }
                };
                var context = function () {
                    if (self.contextMenuHandler !== null) {
                        // delay context menu to popup after typical hide methods
                        setTimeout(function () {
                            self.contextMenuHandler(id, e);
                        }, 10);
                    }
                };
                // extend event
                e.rightClick = e.which === 3 || (e.ctrlKey && ox.browser.MacOS);
                // call click handler
                var result = self.clickHandler({
                    id: id, 
                    event: e,
                    selectable: selectable,
                    click: click,
                    context: context
                });
                // execute default actions?
                if (result !== false) {
                    // left
                    if (selectable === true) {
                        click();
                    }
                    // right
                    if (e.rightClick) {
                        context();
                    }
                }
            } else {
                // auto select first element unless selection exists
                // self.selectOne();
            }
        };
        
        var fnFocus = $.proxy(function(e) {
            if (this.statusEnabled) {
                // use keydown, IE does not react on keypress with cursor keys
                $(document).bind("keydown", $.proxy(this.handleKey, this));
                // show focus
                this.paintFocus();
                this.selectFocus();
            }
        }, this);

        var fnBlur = $.proxy(function(e) {
            $(document).unbind("keydown", this.handleKey);
            // hide focus
            this.paintBlur();
        }, this);
        
        // bind
        $(container)
            .bind("mousedown", fnClick)
            //.bind("focus", fnFocus)
            //.bind("blur", fnBlur)
            // add tab index to receive the focus
            .attr("tabindex", -1); // tab events, but no visual tab
        
        return this;
    },
    
    setNodeFinder: function(fn) {
        this.findNodes = $.proxy(fn, this);
        return this;
    },
    
    findNodes: function() {
        return this.container.childNodes;
    },
    
    update: function() {
        this.processNodes();
    },
    
    /**
     * @private
     */
    processNodes: function() {
        // reset
        this.observedItems = [];
        this.observedItemsIndex = {};
        // loop
        var nodes = this.findNodes(), $l = nodes.length, node, id, foundLastItem = false;
        for (var i = 0; i < $l; i++) {
            node = nodes[i];
            id = ox.util.firstOf(node.oxID, $(node).attr("oxid"));
            // add to observation list
            if (id !== undefined && this.observedItemsIndex[id] === undefined) {
                var index = this.observedItemsIndex[id] = this.observedItems.length;
                this.observedItems.push({
                    "id": id, "node": node
                });
                foundLastItem = !foundLastItem && this.lastSelectedItem === id;
            }
        }
        // update selected items
        for (id in this.selectedItems) {
            if (this.observedItemsIndex[id] === undefined) {
                delete this.selectedItems[id];
            }
        }
        // delete last item?
        if (!foundLastItem) {
            this.lastSelectedItem = null;
        } else {
            this.clickQuiet(this.lastSelectedItem);
        }
        
        // preselection
        if (this.preselectedItem !== null) {
            this.clickQuiet(this.preselectedItem);
            this.preselectedItem = null;
            this.scrollIntoView();
        }
    },
    
    lookUp: function(index, item, cont) {
        cont(this.focusIndex > 0 ? this.focusIndex-1 : this.focusIndex);
    },
    
    lookDown: function(index, item, cont) {
        cont(this.focusIndex < this.numItems()-1 ? this.focusIndex+1 : this.focusIndex);
    },
    
    lookLeft: function(index, item, cont) {
        cont(index);
    },
    
    lookRight: function(index, item, cont) {
        cont(index);
    },
    
    /**
     * @private
     */
    handleKey: function(e) {
        var index = this.focusIndex, item = this.observedItems[index], id = item.id;
        var self = this, cont = function(newIndex, newId) {
            if (newIndex !== null) {
                // use index
                self.processChange(index, newIndex, e);
            } else {
                // use id
                newIndex = self.observedItemsIndex[newId];
                self.processChange(index, newIndex, e);
            }
        };
        switch (e.which) {
        case 37:
            // cursor right
            this.lookLeft(index, id, cont);
            return false;
        case 38:
            // cursor up
            this.lookUp(index, id, cont);
            return false;
        case 39:
            // cursor right
            this.lookRight(index, id, cont);
            return false;
        case 40:
            // cursor down
            this.lookDown(index, id, cont);
            return false;
        case 16:
            /* no break */
        case 32:
            // space/enter = select
            item = this.observedItems[this.focusIndex];
            if (item) {
                this.click(item.id, e.ctrlKey, e.shiftKey);
            }
            return false;
        case 113:
            // F2 / rename
            item = this.observedItems[this.focusIndex];
            if (item) {
                this.onRenameHandler(item.id, item);
            }
            return false;
        }
    },
    
    disable: function () {
        if (this.statusEnabled === true) {
            this.statusEnabled = false;
            $(document).unbind("keydown", this.handleKey);
            for (var id in this.selectedItems) {
                this.deselect(id);
            }
        }
    },
    
    enable: function () {
        if (this.statusEnabled === false) {
            this.statusEnabled = true;
            if (this.lastSelectedItem !== null) {
                this.clickQuiet(this.lastSelectedItem, true);
            }
            $(this.container).focus();
        }
    },
    
    /**
     * @private
     */
    processChange: function(oldFocusIndex, newFocusIndex, e) {
        if (oldFocusIndex != newFocusIndex && newFocusIndex !== undefined) {
            // remove old focus
            this.paintBlur();
            // set new focus
            this.focusIndex = newFocusIndex;
            // paint new focus
            this.paintFocus();
            if (!e.ctrlKey) {
                this.selectFocus();
            }
        }
    },
    
    paintBlur: function() {
        var item = this.observedItems[this.focusIndex], self = this;
        if (item) {
            $(item.node).removeClass(this.classFocus);
        }
    },
    
    paintFocus: function() {
        var item = this.observedItems[this.focusIndex];
        if (item) {
            $(item.node).addClass(this.classFocus).intoViewport(this.container);
        }
    },
    
    selectFocus: function() {
        var item = this.observedItems[this.focusIndex];
        if (item) {
            
            this.click(item.id);
        }
    },
    
    selectOne: function () {
        if (this.numSelected() === 0) {
            var item = this.observedItems[0];
            if (item) {
                this.click(item.id);
            }
        }
    },
    
    clickQuiet: function(id, multiple, range) {
        this.click(id, multiple, range, true);
    },
    
    click: function(id, multiple, range, noEvent) {
        if (this.observedItemsIndex[id] !== undefined) {
            // get previous selection length
            var pre = { l: this.numSelected(), item: this.lastSelectedItem };
            // multiple?
            if (!this.multiple || !multiple) {
                // clear selection
                this.clear();
            }
            // update focus
            var self = this, updateFocus = function () {
                // remove old focus
                self.paintBlur();
                // set focus index
                self.focusIndex = self.observedItemsIndex[id];
                // paint new focus
                self.paintFocus();
            };
            // range?
            if (range && this.lastSelectedItem) {
                // select range
                this.selectRange(this.lastSelectedItem, id);
                updateFocus();
            } else {
                // add single item to selection
                this.toggle(id);
                if (this.lastSelectedItem !== id) {
                    updateFocus();
                    // remember last item
                    this.lastSelectedItem = id;
                }
            }
            // get current selection length
            var post = { l: this.numSelected(), item: id };
            // trigger event?
            if (noEvent !== true && (pre.l != post.l || pre.item != post.item)) {
                this.triggerChangedEvent();
            }
        }
    },

    selectRange: function(lastId, newId) {
        // get indexes
        var fromIndex = this.observedItemsIndex[lastId];
        var toIndex = this.observedItemsIndex[newId];
        // swap?
        if (fromIndex > toIndex) {
          var tmp = fromIndex; fromIndex = toIndex; toIndex = tmp;
        }
        // loop
        for (var i = fromIndex; i <= toIndex; i++) {
            var item = this.observedItems[i];
            this.select(item.id);
        }
    },
    
    getItemById: function (id) {
        var index = this.observedItemsIndex[id];
        return this.observedItems[index];
    },
    
    scrollIntoView: function () {
        var items = this.getSelectedItems();
        if (items.length > 0) {
            $(items[0].node).intoViewport(this.container); //.scrollIntoView(true); // top align
            if (ox.browser.IE) {
                // fix IE7 issue
                window.scrollTo(0,0);
            }
        }
    },

    preselect: function (id) {
        this.preselectedItem = id;
    },
    
    select: function (id) {
        // is not selected?
        if (!this.isSelected(id)) {
            // get item
            var item = this.getItemById(id);
            if (item !== undefined) {
                // select
                this.selectedItems[id] = item;
                this.selectedItems[id].selectionId = id;
                var node = $(item.node).addClass(this.classSelected);
                this.dispatcher.trigger("selected", { id: id, item: item, node: node });
            }
        }
    },

    deselect: function (id) {
        // is selected?
        if (this.isSelected(id)) {
            delete this.selectedItems[id];
            var item = this.getItemById(id);
            if (item) {
                // deselect
                var node = $(item.node).removeClass(this.classSelected);
                this.dispatcher.trigger("deselected", { id: id, item: item, node: node });
            }
        }
    },

    isSelected: function(id) {
        return this.selectedItems[id] !== undefined;
    },

    toggle: function(id) {
        if (this.isSelected(id)) {
            this.deselect(id);
        } else {
            this.select(id);
        }
    },
    
    clear: function (quiet) {
        // deselect items
        for (var id in this.selectedItems) {
            this.deselect(id);
        }
        // trigger event
        if (quiet !== true) {
            this.triggerChangedEvent();
        }
    },

    numItems: function() {
        return this.observedItems.length;
    },
    
    numSelected: function() {
        var count = 0;
        for (var id in this.selectedItems) { count++; }
        return count;
    },

    getSelectedItems: function() {
        var list = [];
        for (var id in this.selectedItems) {
            list.push(this.selectedItems[id]);
        }
        return list;
    },

    getSelection: function() {
        var list = [];
        for (var id in this.selectedItems) {
            list.push(id);
        }
        return list;
    },
    
    override: function(name, fn) {
        if ($.isFunction(fn)) {
            this[name] = fn;
        }
        return this;
    },
    
    setClickHandler: function (handler) {
        this.clickHandler = handler;
    },
    
    setContextMenuHandler: function (handler) {
        this.contextMenuHandler = handler;
    },
    
    triggerContextMenu: function (id, e) {
        var self = this;
        if (self.contextMenuHandler !== null) {
            setTimeout(function () {
                self.contextMenuHandler(id, e);
            }, 10);
        }
    }
});
/**
 * 
 * All content on this website (including text, images, source code and any
 * other original works), unless otherwise noted, is licensed under a Creative
 * Commons License.
 * 
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 * 
 * Copyright (C) 2004-2009 Open-Xchange, Inc. Mail: info@open-xchange.com
 * 
 * @author Matthias Biggeleben <matthias.biggeleben@open-xchange.com>
 * @ignore
 */

// DEVELOPMENT IN PROGRESS // SUBJECT TO PERMANENT CHANGE!
/*
 * jslint white: true, browser: true, devel: true, evil: true, forin: true,
 * undef: true, eqeqeq: true, immed: true
 */

/* global ox, $, newnode, newtab */

ox.gui.Tree = ox.gui.Widget.extend(function() {
    /** @lends ox.gui.Tree.prototype */

    // local namespace
    var TreeNode = function(node, data, tree) {
        this.data = data;
        this.id = node.id;
        this.parent = node.parent;
        this.name = node.name;
        this.type = node.type;
        this.hasChildren = node.hasChildren;
        this.children = [];
        this.parentNode = null;
        this.dom = {
            node : null,
            kindergarten : null
        };
        this.depth = 0;
        this.position = 0;
        this.statusOpen = false;
        this.loading = false;
        this.tree = tree;
        this.isCut = false;
        this.selectable = true;
        // add to index
        if (tree.get(this.id) !== undefined) {
            // adopt children
            var i = 0, $c = tree.get(this.id).children, $l = $c.length;
            for (; i < $l; i++) {
                this.add($c[i]);
            }
            $c = [];
        }
        tree.set(this.id, this);
    };

    /**
     * Add child node
     * 
     * @param node
     */
    TreeNode.prototype.add = function (node) {
        node.parentNode = this;
        node.depth = this.depth + 1;
        node.position = this.children.length;
        this.children.push(node);
        this.hasChildren = true;
    };

    /**
     * Remove node
     * 
     * @param node
     */
    TreeNode.prototype.remove = function() {
        // remove all children
        var i = 0, $c = this.children, $l = $c.length;
        for (; i < $l; i++) {
            $c[i].remove();
        }
        // unregister DnD
        if (this.dndSource) {
            unregisterSource(this.dom.node, this.dndSource);
        }
        if (this.dndTarget) {
            unregisterTarget(this.dom.node, this.dndTarget);
        }
        // remove dom nodes
        $(this.dom.node).remove();
        $(this.dom.kindergarten).remove();
        // remove from index
        delete this.tree.nodeIndex[String(this.id)];
        // remove from parents lists
        if (this.parentNode) {
            var p = this.parentNode;
            $l = p.children.length;
            for (i = 0; i < $l; i++) {
                if (p.children[i] === this) {
                    p.children.splice(i, 1);
                    break;
                }
            }
        }
        // update selection
        this.tree.selection.update();
    };

    /**
     * Paint header of a tree node
     */
    TreeNode.prototype.paintHeader = function(div) {

        // skip root node?
        var skipRoot = this === this.tree.rootNode && this.tree.showRootNode === false;
        
        // create images
        this.dom.toggler = newnode("img");
        this.dom.icon = newnode("img");
        
        // paint node?
        if (!skipRoot) {
            // calculate width of spacer (try to rely on parent's depth)
            this.depth = this.parentNode ? this.parentNode.depth + 1 : this.depth;
            var width = 20 * (this.depth - (this.tree.showRootNode ? 0 : 1));
            // create nodes
            var tab = newtab({ emptyCells: "show", width: "100%", tableLayout: "fixed" }, {
                className : "oxTreeNode"
            }, [ newnode("tbody", 0, 0, [ this.dom.tr = newnode("tr", 0, 0, [
                    this.dom.spacer = newnode("td", { width: width + "px" }),
                    newnode("td", 0, {
                        className : "oxTreeNodeToggler"
                    }, [ this.dom.toggler ]), newnode("td", 0, {
                        className : "oxTreeNodeIcon"
                    }, [ this.dom.icon ]),
                    this.dom.title = newnode("td", 0, 
                            { className : "oxTreeNodeTitle" }) ]) ]) ]);
            
            // kindergarten
            this.dom.kindergarten = newnode("div", {
                display : "none"
            });

            // replace or add?
            if (this.dom.node === null && div !== undefined) {
                // add node
                div.appendChild(tab);
                // add kindergarten
                div.appendChild(this.dom.kindergarten);
            } else {
                // replace
                $(this.dom.node).replaceWith(tab);
            }

            // set (new) dom node
            this.dom.node = tab;

            // add events
            var self = this;
            $(this.dom.toggler).bind("click", function() {
                self.toggle();
                return false;
            }).bind("mousedown", function() {
                return false;
            });
            $(this.dom.tr).bind("dblclick", function() {
                self.toggle();
            });
            $(this.dom.tr).bind("click", function() {
                clickNode(self);
                function clickNode(node) {
                    if (node.selectable === false) {
                        node.open(function() {
                            var next = node.next();
                            if (next) {
                                if (next.selectable) {
                                    self.tree.selection.click(next.id);
                                } else {
                                    clickNode(next);
                                }
                            }
                        });
                    }
                }
            });

            // dnd
            this.enableDnD();

        } else {
            // node
            this.dom.node = newnode("div");
            // kindergarten
            this.dom.kindergarten = newnode("div");
            // add nodes
            div.appendChild(this.dom.node);
            div.appendChild(this.dom.kindergarten);
        }
    };

    /**
     * Update tree node
     */
    TreeNode.prototype.update = function () {
        // update
        this.dom.toggler.src = ox.gui.themePath + (!this.hasChildren ? "img/dummy.gif" : (this.statusOpen ? "img/folderminus.png" : "img/folderplus.png"));
        // customize
        var defaults = {
            src : ox.gui.themePath + "icons/16/folder_closed.gif",
            name : this.name,
            html : undefined
        };
        $.extend(defaults, this.tree.customize(this) || {});
        // add id (for selection)
        this.dom.node.oxSelectable = this.selectable;
        this.dom.node.oxID = this.id;
        // icon
        this.dom.icon.src = defaults.src;
        // name as data
        var title = $(this.dom.title);
        // html?
        if (defaults.html !== undefined) {
            // DOM nodes?
            if (typeof defaults.html === "string") {
                if (debug) console.warn("InnerHTML is evil", getStackTrace());
                // string: add via html()
                title.html(defaults.html);
            } else if (defaults.html.nodeType === 1) {
                // add DOM nodes
                title.empty().get(0).appendChild(defaults.html);
            } else {
                // add indirect
                title.empty().append($(defaults.html));
            }
        } else {
            // text node?
            if (typeof defaults.name === "object" && defaults.name.nodeType === 3) {
                // clear & append text node
                title.empty().append(defaults.name);
            } else {
                // set plain text
                title.text(defaults.name);
            }
        }
        
        this.dom.kindergarten.style.display = this.statusOpen ? "block" : "none";
        // hide toggler when node has no children
        $(this.dom.toggler).parent().css({ visibility : this.hasChildren ? "" : "hidden" });
    };
    
    /**
     * Paint tree node
     * 
     * @param {Object} div DOM node
     * @param {function()} cont A continuation function
     */
    TreeNode.prototype.paint = function (div, cont) {

        // skip root node?
        var skipRoot = this === this.tree.rootNode && this.tree.showRootNode === false;

        // has DOM node yet?
        if (this.dom.node === null) {
            // get proper parent node
            if (div === undefined && !this.parentNode) {
                if (cont) {
                    cont();
                }
                return;
            }
            // paint header
            this.paintHeader(div || this.parentNode.dom.kindergarten);
        }

        if (!skipRoot) {
            this.update();
        }

        // auto open?
        if ((this.tree.openNodes[this.id] || skipRoot) && this.statusOpen === false) {
            // open!
            delete this.tree.openNodes[this.id];
            this.open(cont);
        } else {
            // join
            var join = new Join(cont || $.noop);
            var lock = join.add();
            // draw children
            var i = 0, $c = this.children, $l = $c.length;
            for (; i < $l; i++) {
                $c[i].paint(this.dom.kindergarten, join.add());
            }
            lock();
        }
    };

    TreeNode.prototype.repaint = function() {
        // repaint
        var self = this, repaint = function () {
            // paint
            self.paint();
            // update selection
            self.tree.selection.update();
        };
        // paint running?
        if (this.tree.painter.running === true) {
            this.tree.painter.queue.push(repaint);
        } else {
            repaint();
        }
    };

    /**
     * Depending on the DnD settings of the corresponding tree, enables
     * source and/or target handling for DnD on this node.
     */
    TreeNode.prototype.enableDnD = function() {
        if (!this.dom.node) {
            return;
        }
        var self = this;
        if (this.tree.dndSource) {
            this.tree.dndSource(this.data, function(type, callback) {
                self.dndSource = registerSource(self.dom.node, type,
                        callback, null, null, foldertreedisable,
                        defaultdisabledremove);
            });
        }
        if (this.tree.dndTarget) {
            this.tree.dndTarget(this.data, function(callbacks) {
                var cbs = {};
                for ( var i in callbacks) {
                    cbs[i] = (function(cb) {
                        return function(a, b, c, d, e, f) {
                            legacyOut(a, b, c, d, null, f);
                            cb(a, b, c, d, e, f);
                        };
                    })(callbacks[i]);
                }
                self.dndTarget = registerTarget(self.dom.node, cbs, null,
                        null, legacyIn, legacyIn, legacyOut, true, true);
            });
        }
        var TImeOutOpen;
        function legacyIn(e, type, objects, position, targetNode) {
            foldertreeenable(e, type, objects, position, targetNode, self.data);
            var mynode = e.currentTarget || e.srcElement || false;
            if (!mynode.className.match(/dndOver/)) {
                mynode.className += " dndOver";
            }
            function setOpenFN() {
                self.open();
            }
            if (TImeOutOpen) {
                clearTimeout(TImeOutOpen);
            }
            return (TImeOutOpen = setTimeout(setOpenFN, 1000));
        }
        function legacyOut(e, type, objects, position, OutOpen, dropNode) {
            if (TImeOutOpen) {
                clearTimeout(TImeOutOpen);
            }
            if (dropNode) {
                dropNode.className = removeClass(dropNode.className, "dndOver");
            } else if (e) {
                var mynode = e.currentTarget || e.srcElement || false;
                if (mynode) {
                    mynode.className = removeClass(mynode.className, "dndOver");
                }
            }
        }
    };

    /**
     * Toggle open/close
     */
    TreeNode.prototype.toggle = function() {
        if (this.statusOpen) {
            this.close();
        } else {
            this.open();
        }
    };

    /**
     * Close node
     */
    TreeNode.prototype.close = function () {
        // remove from open list
        delete this.tree.openNodes[this.id];
        // visual close?
        if (this.statusOpen) {
            this.statusOpen = false;
            this.update();
            // trigger event
            this.tree.trigger("widget:close", {
                id : this.id,
                node : this
            });
        }
    };

    /**
     * Open node
     */
    TreeNode.prototype.open = function (cont) {
        if (!this.statusOpen) {
            var self = this;
            var showKindergarten = function() {
                self.statusOpen = true;
                self.paint(self.dom.kindergarten, function() {
                    self.tree.selection.update();
                    if (cont) cont();
                });
                // trigger event
                self.tree.trigger("widget:open", {
                    id : self.id,
                    node : self
                });
            };
            if (this.hasChildren && this.children.length === 0) {
                // busy
                $(this.dom.kindergarten).css( {
                    height : "30px",
                    display : "block"
                }).addClass("oxBusySmall");
                this.loadChildren(function (data) {
                    // apply grep
                    data = $.grep(data, self.tree.grep);
                    // idle
                    $(self.dom.kindergarten).css( {
                        height : ""
                    }).removeClass("oxBusySmall");
                    // no error?
                    if (data !== false) {
                        // add
                        self.tree.addMultiple(data);
                        // show
                        showKindergarten();
                    } else {
                        if (cont) {
                            cont();
                        }
                    }
                });
            } else {
                showKindergarten();
            }
        }
    };

    /**
     * Get number of children
     */
    TreeNode.prototype.numChildren = function() {
        return this.children.length;
    };

    /**
     * Load children (uses loadChildren implementation of ox.gui.tree)
     */
    TreeNode.prototype.loadChildren = function (cont) {
        if (!this.loading) {
            this.loading = true;
            var self = this;
            this.tree.loadChildren(this.id, function (data) {
                self.loading = false;
                cont(data);
            });
        }
    };

    /**
     * Get first child (useful for selections)
     */
    TreeNode.prototype.first = function() {
        return this.children[0] || null;
    };

    /**
     * Get last child (useful for selections)
     */
    TreeNode.prototype.last = function() {
        var c = this.children;
        return c.length > 0 ? c[c.length - 1] : null;
    };

    /**
     * Cursor move: left (useful for selections)
     */
    TreeNode.prototype.left = function() {
        // look for leftmost child
        return this.statusOpen && this.numChildren() ? this.last().left() : this;
    };

    /**
     * Cursor move: right (useful for selections)
     */
    TreeNode.prototype.right = function() {
        // look for rightmost child
        return this.statusOpen && this.numChildren() ? this.first().right() : this;
    };

    /**
     * Cursor move: previous sibling (useful for selections)
     */
    TreeNode.prototype.prevSibling = function() {
        var p = this.parentNode, pos = this.position;
        return p && pos > 0 ? p.children[pos - 1] : null;
    };

    /**
     * Cursor move: next sibling (useful for selections)
     */
    TreeNode.prototype.nextSibling = function() {
        var p = this.parentNode, pos = this.position;
        return p && pos < p.children.length ? p.children[pos + 1] : null;
    };

    /**
     * Cursor move: next/down (useful for selections)
     */
    TreeNode.prototype.next = function(skipChildren) {
        // open and has children?
        if (!skipChildren && this.statusOpen && this.children.length > 0) {
            // return first child
            return this.first();
        } else {
            // has parent?
            if (this.parentNode) {
                var sibling = this.nextSibling();
                if (sibling) {
                    // return next sibling
                    return sibling;
                } else {
                    // traverse
                    return this.parentNode.next(true);
                }
            } else {
                return null;
            }
        }
    };

    /**
     * Cursor move: previous/up (useful for selections)
     */
    TreeNode.prototype.prev = function() {
        // root node?
        if (!this.parentNode) {
            return null;
        } else {
            var sibling = this.prevSibling();
            if (sibling) {
                return sibling.left();
            } else {
                // return parent node
                return this.parentNode;
            }
        }
    };

    /**
     * Start inline edit
     * 
     * @return
     */
    TreeNode.prototype.startEdit = function (success, abort) {
        
        var self = this;
        
        if (success === undefined) {
            success = function(value) {
                // trigger event
                self.tree.trigger("widget:edited", {
                    node : self,
                    data : self.data,
                    id : self.data.id,
                    value : value
                }, function() {
                    // enable selection
                    self.tree.selection.enable();
                });
            };
        }
        
        if (abort === undefined) {
            abort = function() {
                // repaint
                self.repaint();
                // enable selection
                self.tree.selection.enable();
            };
        }
        
        this.tree.selection.disable();
        
        var node = $(this.dom.title);
        ox.gui.util.inlineEdit.call(this, node, this.name || "", success, abort);
    };

    this.getClass = function() {
        return "ox.gui.Tree";
    };

    /** 
     * A tree view widget.
     * @constructs
     * @extends ox.gui.Widget
     * @param {String} id An optional unique id for the widget.
     * @example
     
        var div = jQuery("<div/>").css({
            position: "absolute", top: 50, right: 50, bottom: 50, left: 50, border: "5px solid #ccc",
            backgroundColor: "white"
        }).appendTo(document.body);
        
        // new tree
        var t = new ox.gui.Tree();
        // add to DOM manually (not via layout managers)
        t.setParentDOMNode(div[0]);
        // add root node
        t.setRootNode({ id: 0, parent: null, name: "Root", module: "root", hasChildren: true });
        // add nodes
        t.add({ id: 1, parent: 0, name: "Subfolder #1" });
        t.add({ id: 2, parent: 0, name: "Subfolder #2" });
        t.add({ id: 3, parent: 1, name: "Subfolder #2A" });
        t.add({ id: 4, parent: 1, name: "Subfolder #2B" });
        // set css class for selected elements
        t.selection.classSelected = "background-color-PMG-selection-elements";
        // go!
        t.paint();
     */
    this.construct = function(id) {

        // call super class constructor (inherit from Container)
        this._super(this.autoId(id, "tree"));

        // root node
        this.rootNode = null;
        // index of all tree nodes
        this.nodeIndex = {};
        // open nodes
        this.openNodes = {};
        // show root node
        this.showRootNode = true;
        // load root
        this.loadRootNode = false;
        this.loadRootNodeId = 0;
        // cut&paste node
        this.cutNode = null;
        // lock
        this.painter = {
            running: false,
            queue: []
        };

        // internal ref
        var tree = this;

        // internal selection
        var s = this.selection = new ox.gui.Selection();
        s.setMultiple(false);
        s.setNodeFinder(function() {
            return $(".oxTreeNode", this.container);
        });
        s.observe(this.dom.node);
        s.override("lookUp", function(index, id, cont) {
            // get previous node
            var prev = tree.get(id).prev();
            cont(null, prev !== null ? prev.id : null);
        });
        s.override("lookDown", function(index, id, cont) {
            // get next node
            var next = tree.get(id).next();
            cont(null, next !== null ? next.id : null);
        });
        s.override("lookRight", function(index, id, cont) {
            var node = tree.get(id);
            // has children?
            if (node.hasChildren) {
                // open?
                if (node.statusOpen) {
                    // select first child
                    cont(null, node.first().id);
                    return;
                } else {
                    // open node
                    node.open();
                }
            }
            cont(index);
        });
        s.override("lookLeft", function(index, id, cont) {
            var node = tree.get(id);
            // has children?
                if (node.hasChildren && node.statusOpen) {
                    // close node
                node.close();
                cont(index);
            } else if (node.parentNode) {
                // select parent node
                cont(null, node.parentNode.id);
            } else {
                cont(index);
            }
        });
        s.onChange(function(selection) {
            if (selection.length) {
                tree.trigger("widget:select", selection[0].id);
            }
        });
        s.onRename(function(id, item) {
            tree.startEdit(id);
        });
    };

    /**
     * Internal use
     */
    this.nodeInit = function() {
        this.setStyle( {
            background : "white",
            overflow : "auto"
        });
        this.addCSSClass("oxStretch");
    };

    /**
     * Set root node
     */
    this.setRootNode = function(data) {
        this.rootNode = new TreeNode(this.nodeParser(data), data, this);
        this.rootNode.parent = null;
        return this;
    };

    /**
     * Node parser. Derives the necessary information from a data object.
     * Internally, a node has the following properties: - id: Unique
     * identifier - parent: ID of the parent node - name: Name/Title/Label -
     * type: Can be used to customize a node visually - hasChildren:
     * True/False. If true but a node has no children, loadChildren is
     * called.
     */
    this.setNodeParser = function(fn) {
        this.nodeParser = fn;
        return this;
    };

    /**
     * Default node parser
     */
    this.nodeParser = function(data) {
        return {
            id : String(data.id !== undefined ? data.id : 1),
            parent : data.parent !== undefined ? String(data.parent) : null,
            name : data.name || "default",
            type : "default",
            hasChildren : data.hasChildren ? true : false
        };
    };
    
    /**
     * Node filter (must return true/false)
     */
    this.grep = function (data) {
        return true;
    };

    /**
     * Enables Drag & Drop behaviour in this tree. Node-specific behaviour
     * is defined by two callbacks, which are called for each node.
     * Node-independent behaviour modifications during dragging (e. g.
     * scrolling and opening of nodes) are implemented by the tree and
     * enabled automatically when this function is called. Once enabled, the
     * D&D behaviour of a tree cannot be disabled.
     * 
     * @param {function(Object),
     *            function(String, function)} [source] A function which is
     *            called with each tree node's data and a continuation
     *            function as parameters. The parameters to the continuation
     *            function will be used as the type and callback parameters
     *            to registerSource.
     * @param {function(Object,
     *            function(Object))} [target] A function which is called
     *            with each tree node's data and a continuation function as
     *            parameters. The parameter to the continuation functionwill
     *            be used as the callbacks parameter to registerTarget.
     */
    this.enableDnD = function(source, target) {
        this.dndSource = source;
        this.dndTarget = target;
        for ( var i in this.nodeIndex) {
            this.nodeIndex[i].enableDnD();
        }
        var node = this.dom.node, interval, scrollSpeed = 0, yMax;
        function canScroll() {
            return scrollSpeed < 0 && node.scrollTop > 0 ||
                   scrollSpeed > 0 && node.scrollTop < yMax;
        }
        var RANGE = 20; // Height of the sensitive area in px.
        var MAX = 1; // Maximal scrolling speed in px/ms.
        var scale = MAX / RANGE;
        // The speed is specified in px/ms. A range of 1 to 10 results
        // in a speed of 100 to 1000 px/s.
        function scroll(speed) {
            scrollSpeed = speed;
            if (canScroll()) {
                var t0 = (new Date).getTime(), y0 = node.scrollTop;
                if (interval !== undefined) clearInterval(interval);
                interval = setInterval(function() {
                    if (canScroll()) {
                        var dt = (new Date).getTime() - t0;
                        var y = y0 + scrollSpeed * dt;
                        if (y < 0) y = 0;
                        else if (y > yMax) y = yMax;
                        else {
                            node.scrollTop = y;
                            return;
                        }
                        node.scrollTop = y;
                    }
                    clearInterval(interval);
                    interval = undefined;
                }, 10);
            } else {
                if (interval !== undefined) clearInterval(interval);
                interval = undefined;
            }
        }
        $(node).mousemove(function(e) {
            if (!isDragging()) return;
            if (e.target != node) stopDnDPropagation(e);
            var y = e.pageY - $(node).offset().top;
            yMax = node.scrollHeight - node.clientHeight;
            if (y < RANGE) {
                scroll((y - RANGE) * scale);
            } else if (node.clientHeight - y < RANGE) {
                scroll((RANGE - node.clientHeight + y) * scale);
            } else {
                scroll(0);
            }
        }).mouseleave(function(e) { scroll(0); });
    };

    /**
     * Add new node. Use data binding. Calls internal nodeParser which can
     * be customized. See setNodeParser for details.
     * 
     * @param {Object}
     *            data
     */
    this.add = function (data) {
        var node = this.nodeParser(data), parent = node.parent;
        // looks like another root node?
        if (parent !== null) {
            // missing parent?
            if (this.get(parent) === undefined) {
                // create dummy node
                (new TreeNode( {
                    id : String(parent),
                    parent : null,
                    name : "dummy",
                    type : "default",
                    hasChildren : true
                }, {}, this));
            }
            // add node
            this.get(parent).add(new TreeNode(node, data, this));
        }
        return node;
    };

    this.get = function(id) {
        return this.nodeIndex[String(id)];
    };

    this.set = function(id, obj) {
        this.nodeIndex[String(id)] = obj;
    };

    /**
     * Remove node
     * 
     * @param {string}
     *            id
     */
    this.remove = function(id) {
        // exists?
        if (this.get(id) !== undefined) {
            this.get(id).remove();
        }
    };

    /**
     * Add multiple data objects
     */
    this.addMultiple = function(list) {
        var i = 0, $l = list.length;
        for (; i < $l; i++) {
            this.add(list[i]);
        }
    };

    /**
     * Default implementation of loadChildren
     */
    this.loadChildren = function(id, cont) {
        cont();
    };

    /**
     * Default implementation of loadNode
     */
    this.loadNode = function(id, cont) {
        cont();
    };

    /**
     * Customizes the TreeNode objects before they are painted.
     * Customization of a TreeNode happens by returning an options object
     * with some or all of the following fields:
     * <dl>
     * <dt>src</dt>
     * <dd>A string which specifies the icon URL. The default is the closed
     * folder icon of the current theme. Usually, the icon depends at least
     * on node.statusOpen.</dd>
     * <dt>name</dt>
     * <dd>A value which is attached to the DOM node via jQuery's .data()
     * method using the key &quot;name&quot;. If the field html is not
     * specified, it is also used as the displayed text (via jQuery's
     * .text() method). The default is the name of the node.</dd>
     * <dt>html</dt>
     * <dd>A value which is used as the displayed text of the node. It is
     * set via jQuery's .html() method. Although not documented in jQuery,
     * DOM nodes can (and should) be used instead of strings.</dd>
     * </dl>
     * 
     * @param {TreeNode}
     *            node The node to customize.
     * @type Object
     * @return An object with customization options.
     */
    this.customize = function(node) {
        return {};
    };

    /**
     * Get root node
     */
    this.getRootNode = function() {
        return this.rootNode;
    };

    /**
     * Remove all nodes
     */
    this.clearNodes = function (includeRoot) {
        // clear index
        this.nodeIndex = {};
        // not include root?
        var root = this.rootNode;
        if (root !== null && includeRoot !== true) {
            // restore root node
            this.set(root.id, root);
            // remove children
            root.children = [];
            // reset DOM nodes
            root.dom = {
                node : null
            };
            // set closed
            root.statusOpen = false;
        } else {
            // clear root node
            this.rootNode = null;
        }
    };

    /**
     * Repaint
     */
    this.repaint = function (fn) {
        var self = this;
        var cont = function () {
            self.triggerWidgetEvent("painted");
            ox.util.call(fn);
        };
        if (this.painter.running === false) {
            // remember open nodes & selected item
            var open = this.getOpen();
            var item = this.selection.getSelection();
            // clear nodes except root
            this.clearNodes(false);
            // restore open nodes
            this.setOpen(open);
            // paint
            this.clear();
            this.paint(cont);
            // select previous item
            if (item.length > 0) {
                this.select(item[0]);
            }
        } else {
            // connect to running paint
            this.painter.queue.push(cont);
        }
    };

    /**
     * Paint
     */
    this.paint = function (fn) {
        // wrap continuation
        var self = this;
        var cont = function () {
            self.triggerWidgetEvent("painted");
            ox.util.call(fn);
        };
        var done = function () {
            self.painter.running = false;
            // call all back
            var i = 0, fn = self.painter.queue, $l = fn.length;
            for (; i < $l; i++) {
                ox.util.call(fn[i]);
            }
            // clear queue
            self.painter.queue = [];
        };
        // paint root node
        var paintRoot = function () {
            if (self.rootNode) {
                self.rootNode.paint(self.dom.node, done);
                self.selection.update();
            }
        };
        // something to do?
        if (this.painter.running === false && (this.loadRootNode === true || this.rootNode)) {
            this.painter.running = true;
            this.painter.queue.push(cont);
            // load root?
            if (this.loadRootNode === true) {
                this.loadNode(this.loadRootNodeId, function (data) {
                    // reset root node
                    self.setRootNode(data);
                    self.loadRootNode = false;
                    paintRoot();
                });
            } else {
                paintRoot();
            }
        } else {
            // nothing to do
            done();
        }
    };

    /**
     * @private
     */
    var cloneChildNodes = function(node, clone) {
        var $i = 0, $c = node.children, $l = $c.length;
        for (; $i < $l; $i++) {
            clone.add($c[$i].data);
            cloneChildNodes($c[$i], clone);
        }
    };

    /**
     * @private
     */
    var openChildNodes = function(original, clone) {
        for ( var id in original.nodeIndex) {
            if (clone.get(id)) {
                clone.get(id).statusOpen = original.get(id).statusOpen;
            }
        }
    };

    /**
     * @private -- historical (clone demo)
     */
    this.cloneDeep = function(clone) {
        if (this.rootNode) {
            // set root node
            clone.setRootNode(this.rootNode.data);
            // add all child nodes
            cloneChildNodes(this.rootNode, clone);
            // toggle open/close for all child nodes
            openChildNodes(this, clone);
        }
    };

    /**
     * Set/add open nodes. Defined by an array of IDs
     */
    this.setOpen = function(list, flag) {
        var i = 0, $l = list.length;
        for (; i < $l; i++) {
            if (flag === undefined || flag === true) {
                this.openNodes[list[i]] = true;
            } else {
                delete this.openNodes[list[i]];
            }
        }
        return this;
    };

    /**
     * Get current open nodes.
     * 
     * @returns {Array} IDs of the open nodes
     */
    this.getOpen = function() {
        var tmp = [], id;
        for (id in this.nodeIndex) {
            if (this.get(id).statusOpen === true) {
                tmp.push(id);
            }
        }
        return tmp;
    };

    /**
     * Select a node
     */
    this.select = function(id) {
        if (this.get(id) !== undefined) {
            this.selection.select(id);
        }
    };

    /**
     * Trigger inline edit of a node
     */
    this.startEdit = function (id, ok, abort) {
        if (this.get(id) !== undefined) {
            this.get(id).startEdit(ok, abort);
        }
    };

    /**
     * Mark a node as "cut" (cut&paste)
     * @param {String} [id] The ID of the cut node. If not specified or invalid,
     * any previously cut node will be marked as not cut.
     */
    this.cut = function(id) {
        // previous cut node?
        if (this.cutNode) {
            this.cutNode.isCut = false;
            this.cutNode.paint();
        }
        if (id !== undefined && this.get(id) !== undefined) {
            // set new node
            this.cutNode = this.get(id);
            this.cutNode.isCut = true;
            this.cutNode.paint();
        } else {
            this.cutNode = null;
        }
    };
});
/**
 * 
 * All content on this website (including text, images, source
 * code and any other original works), unless otherwise noted,
 * is licensed under a Creative Commons License.
 * 
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 * 
 * Copyright (C) 2004-2009 Open-Xchange, Inc.
 * Mail: info@open-xchange.com 
 * 
 * @author Matthias Biggeleben <matthias.biggeleben@open-xchange.com>
 * @author Stefan Preuss <stefan.preuss@open-xchange.com>
 * @ignore
 */

// DEVELOPMENT IN PROGRESS // SUBJECT TO PERMANENT CHANGE!

ox.gui.Button = ox.gui.Widget.extend({
/** @lends ox.gui.Button.prototype */
    
    getClass: function() { return "ox.gui.Button"; },
    
    /** 
     * A button widget.
     * @constructs
     * @extends ox.gui.Widget
     * @param {I18nString} text The text of the button.
     * @param {Function} clickListener The default callback ("dom:click").
     */
    construct: function(id, text, clickListener) {
    
        this.clonable = true;
        
        // call super class constructor (inherit from Container)
        this._super(this.autoId(id, "ox.gui.Button"));
        
        this.text = text;
        this.textNode = $("<span/>");
        
        // enable & handle
        this.enableI18n();
        this.i18nHandler();
        
        // dom:click
        if (clickListener) {
            this.addListener("dom:click", clickListener);
        }
    },
    
    i18nHandler: function() {
        this.textNode.attr("title", expectI18n(this.text))
                     .text(expectI18n(this.text));
    },
    
    paint: function() {

        $(this.dom.node).append($("<div/>").addClass("fakeButton background-color-default cursorPointer logoutground").
                append($("<div/>").addClass("fakeButtonBottom").
                        append($("<div/>").addClass("fakeButtonLeft").
                                append($("<div/>").addClass("fakeButtonRight").
                                        append($("<div/>").addClass("fakeButtonTopLeft").
                                                append($("<div/>").addClass("fakeButtonTopRight").
                                                        append($("<div/>").addClass("fakeButtonBottomLeft").
                                                                append($("<div/>").addClass("fakeButtonBottomRight").append(this.textNode)))
                                                )
                                        )
                                )
                        )
                )                
        );
        
        if (!this.isEnabled()) {
            this.paintDisabled();
        }
    },
    
    paintVisible: function() {
        this.dom.node.style.display = "inline";
    },
    
    paintEnabled: function() {
        if (this.dom.node.firstChild) {
            this.dom.node.firstChild.disabled = false;
        }
    },

    paintDisabled: function() {
        if (this.dom.node.firstChild) {
            this.dom.node.firstChild.disabled = true;
        }
    },

    /**
     * Sets the global event to fire when the button is clicked.
     * @param {String} eventName The name of the event.
     */
    setEvent: function(eventName) {
        this.addListener("dom:click", function() {
            this.triggerEvent(eventName);
        });
    },

    /**
     * Sets the widget event to fire when the button is clicked.
     * @param {String} eventName The name of the event.
     */
    setWidgetEvent: function(eventName, eventSource) {
        this.addListener("dom:click", function() {
            var source = eventSource || this;
            source.triggerWidgetEvent(eventName);
        });
    }
});


ox.gui.SearchField = ox.gui.Widget.extend(function () {
    
    var searchImg, searchMenuImg, cancelImg, textChanged;
        
    this.getClass = function() {
        return "ox.gui.SearchField"; 
    };
        
    this.construct = function(id, options) {
        
        // call super class constructor (inherit from Container)
        this._super(this.autoId(id, "ox.gui.SearchField"));
        
        var Self = this;
        
        this.options = $.extend({
            defaultText: _("Please enter your search"),
            width: 188
        }, options);
        
        this.firstPaint = true;
        
        this.p = {};
        
        this.p.input = $("<input/>").attr({
            type: "text",
            value: expectI18n(_("Please enter your search")),
            role: "textbox"
        }).
        css({ border: "none", color: "silver", width: this.options.width + "px", background: "transparent",
            outline: "none", display: "inline-block", height: "17px", marginLeft: "3px",
            marginTop: "0px" }).
        bind("keyup focusin focusout", this.options, function (event) {
            if (event.type == "focusin" && !Self.p.textChanged) {
                $(this).attr({ value: "" }).css({ color: "black" });
            } else if (event.type == "focusout" && $(this).attr("value") == "") {
                $(this).attr({ value: expectI18n(event.data.defaultText) }).css({ color: "silver" });
                $(Self.p.cancelImg).css({ visibility: "hidden" });
                Self.p.textChanged = false;
            } else {
                switch(event.which) {
                    case 27:
                        Self.p.textChanged = false;
                        $(this).attr("value", "");
                        $(this).blur();
                        break;
                    case 13:
                        Self.trigger("widget:search", { value: $(this).attr("value") });
                    default:
                        Self.p.textChanged = true;
                        $(Self.p.cancelImg).css({ visibility: "visible" });
                }
            }
        });
        
        this.p.input.bind("mousedown", function(event) {
            // stop propagation, required for panel implementation
            event.stopPropagation();
        });
        
        this.p.searchMenuImg = $("<div/>").
        css({ "float": "left", height: "1.6em", width: "16px", verticalAlign: "middle", textAlign: "center", backgroundColor: "#65769B" }).
        append(
                $("<img/>").attr({ src: getFullImgSrc("img/arrows/search_arrow.png"), align: "absmiddle" })
        );
        
        this.p.searchImg = $("<img/>").
        attr({ src: getFullImgSrc("icons/16/search.png"), align: "absmiddle" }).
        css({ cursor: "pointer", "float": "right", marginTop: "2px", marginRight: "3px", marginLeft: "3px" }).
        click(function() {
            // fire only search if something really changed
            if (Self.p.textChanged && Self.p.input.attr("value") != "") {
                Self.trigger("widget:search", { value: Self.p.input.attr("value") });
            } else {
                triggerEvent('OX_Delete_Search');
            }
            return false;
        });
        
        this.searchMenu = new PulldownMenu(this.p.searchMenuImg[0]);
        
        this.searchMenu.setFields = function(fields, names, checked, closeWindow) {
            this.removeAllItems();
            this.onChange = emptyFunction;
            if (fields && fields.length) {
                this.checked = checked;
                for (var i = 0; i < fields.length; i++) {
                    var item = new MenuItem(names[i], function() {
                        Self.searchMenu.checked[this.searchField] = this.getChecked();
                        Self.searchMenu.onChange();
                        return closeWindow || false;
                    });
                    item.searchField = fields[i];
                    item.setChecked(checked[fields[i]]);
                    Self.searchMenu.addItem(item);
                }
                Self.p.searchMenuImg.css({ display: "" });
            } else {
                Self.p.searchMenuImg.css({ display: "none" });
            }
        };
        
        this.p.cancelImg = $("<img/>").attr({ id: "delete_search", src: getFullImgSrc("img/search_x.png"), align: "absmiddle" }).
        css({ marginTop: "2px", marginRight: "5px", cursor: "pointer", visibility: "hidden", "float": "right" }).
        bind("click", this.options, function (event) {
            //$(Self.p.input).attr({ value: expectI18n(event.data.defaultText) }).css({ color: "silver" });
            $(this).css({ visibility: "hidden" });
            $(Self.p.input).attr("value", "");
            $(Self.p.input).blur();
            Self.p.textChanged = false;
            // todo: move to own trigger event  
            triggerEvent('OX_Delete_Search');
        });
    };

    this.paint = function() {
        
        var Self = this;
        
        // paint it once only
        // otherwise, you need to call "empty" (as before) which, however, kills lots of event handlers
        if (this.firstPaint) {
            var input = IE ? $('<form onsubmit="return false"></form>').
                                 append(this.p.input)
                           : this.p.input;
            $(this.dom.node).addClass("border-color-design").
            css({
                backgroundColor: "white",
                border: "1px inset silver",
                borderRadius: "5px 5px 5px 5px",
                MozBorderRadius: "5px",
                WebkitBorderRadius: "5px",
                lineHeight: "1.5em"
            }).append(
                $("<div/>")
                .css({ overflow: "hidden", height: "18px", whiteSpace: "nowrap" })
                .append(this.p.searchMenuImg)
                .append(this.p.searchImg)
                .append(this.p.cancelImg)
                .append(input)
            );
            if (this.options.disabled) {
                this.disable();
            }
            this.firstPaint = false;
        }
        
        // adjust width
        this.p.input.css("width", this.options.width + "px");
    };
    
    this.disable = function() {
        this._super();
        this.options.disabled = true;
        if (this.p.input !== undefined) {
            this.p.input.attr({ disabled: "disable", value: "" });
            this.p.searchImg.css({ cursor: "default", opacity: ".2" });
            $(this.dom.node).css({ backgroundColor: "transparent" });
        }
        return this;
    };
    
    this.enable = function() {
        this._super();
        if (this.p.input !== undefined) {
        	$(this.dom.node).css({ backgroundColor: "white" });
        	this.p.searchImg.css({ cursor: "pointer", opacity: "" });
            this.p.input.attr({
                disabled: null, 
                value: expectI18n(this.options.defaultText) 
            }).css({ color: "silver" });
            this.p.cancelImg.css({ visibility: "hidden" });
            this.p.textChanged = false;
        }
        return this;
    };
    
    this.getValue = function() {
        return this.p.input.value();
    };
    
    this.setValue = function(value) {
        this.p.input.focus();
        this.p.input.value(value);
    };
});

ox.gui.Bar = ox.gui.Widget.extend({
	    
	    getClass: function() { return "ox.gui.Bar"; },
	    
	    /**
	     * Constructor
	     */
	    construct: function(options) {
	        this._super(this.autoId(0, "bar")); 
	        
	        // default options
	        this.options = $.extend({
	            max: 100,
	            width: "100%",
	            height: "10px",
	            showText: true,
	            textFormat: 'percentage',
	            title: "%1$s / %2$s",
	            boxBGImage: getFullImgSrc('img/pg_bg.png'),
	            barImage: {
	        		0: getFullImgSrc('img/pg_green.png'),
	        		75: getFullImgSrc('img/pg_orange.png'),
	        		90: getFullImgSrc('img/pg_red.png')
	        	}
	        }, options);
	        
	        this.barNode = $("<div>")
	            .css({
	                background: 'url(' + this.options.barImage["0"] + ')',
	                position: "absolute",
	                width: 0,
	                left: 0,
	                top: 0,
	                bottom: 0
	            });
	        this.barText = $("<span>")
	            .css({
	                position: "relative",
	                lineHeight: "1em",
	                fontSize: "9pt",
	                margin: "0px 5px"
	            });
	        this.descriptionNode = $("<div>")
	            .css({
	                whiteSpace: "nowrap",
	                fontSize: "7pt", 
	                position: "absolute",
	                lineHeight: "10px",
	                top: "0px",
	                left: "3px"
	            });
	        
	        this.enableI18n();
	        this.i18nHandler();
	    },
	    
	    i18nHandler: function() {
	        if (this.options.description) {
	            this.descriptionNode.text(expectI18n(this.options.description));
	        }
	    },
	    
	    paint: function() {
	    	$(this.dom.node).empty();
	    	
	    	$(this.dom.node).
	    	append($("<table/>")
	    	        .attr({
	    	            width: "100%",
	    	            cellSpacing: "0",
	    	            cellPadding: "0"
	    	        })
	    	        .append($("<tr/>")
	    	                .append($("<td/>")
	    	                        .css({
	    	                            width: this.options.width
	    	                        })
	    	                        .append(
	    	                                $("<div/>")
	    	                                .css({
	    	                                    background: 'url(' + this.options.boxBGImage + ')',
	    	                                    height: this.options.height,
	    	                                    border: "1px solid gray", position: "relative",
	    	                                    margin: "0px 5px", overflow: "hidden"
	    	                                })
	    	                                .append(this.barNode)
	    	                                .append(this.descriptionNode)
	    	                        )
	                        ).append($("<td/>")
	                                .css({ 
	                                    whiteSpace: "nowrap"
	                                })
	                                .append(this.barText)
                            )
                    )
            );
	    },
	    
	    /**
	     * Sets the max. value used for drawing the bar
	     * @param {Integer}
	     */
	    setMax: function(max) {
	    	this.options.max = max;
	    },
	    
	    /**
	     * Updates the progress by the given value
	     * @param {Integer} Used value
	     */
	    progress: function(used) {
	    	// calculate position
	    	var percent = Math.round((used / this.options.max) * 100);
	    	this.barNode.css("width", percent + "%");
	    	
	    	// default bar image
	    	var img = this.options.barImage["0"];
	    	// iterate through given steps and grep image
	    	for (var i in this.options.barImage) {
	    		if (i > percent) break;
	    		img = this.options.barImage[i];
	    	}

	    	// should no text be shown?
	    	this.barText.css({ display: this.options.showText ? "" : "none" });
	    	
	    	// sets the text shown on the right
	    	if (ox.util.isFunction(this.options.textFormat)) {
	    	    this.barText.text(this.options.textFormat(used, this.options.max));
	    	} else if (this.options.textFormat === "percentage") {
	    	    this.barText.text(percent + "%");
	    	} else {
	    	    this.barText.text(used + " / " + this.options.max);
	    	}
	    	
	    	// sets the title
	    	if (ox.util.isFunction(this.options.title)) {
	    	    $(this.dom.node).attr("title", this.options.title(used, this.options.max));
	    	} else {
	    	    $(this.dom.node).attr("title", format(this.options.title, used, this.options.max));
	    	}
	    	
	    	// re-set bar image
	    	this.barNode.css("background", 'url(' + img + ')');
	    }
});
/**
 * 
 * All content on this website (including text, images, source
 * code and any other original works), unless otherwise noted,
 * is licensed under a Creative Commons License.
 * 
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 * 
 * Copyright (C) 2004-2009 Open-Xchange, Inc.
 * Mail: info@open-xchange.com 
 * 
 * @author Matthias Biggeleben <matthias.biggeleben@open-xchange.com>
 * @ignore
 */

// DEVELOPMENT IN PROGRESS // SUBJECT TO PERMANENT CHANGE!

/*jslint white: true, browser: true, devel: true, evil: true, forin: true, undef: true, eqeqeq: true, immed: true */

/*global jQuery */

(function ($) {
    
    var Draggable = function (node, options) {
        
        // vars
        var helper = null, fast = null;
        var offset = { top: 0, left: 0 };
        var delta = { top: 0, left: 0 };
        var origin = { top: 0, left: 0 };
        var dropzone = null;
        
        // defaults
        var o = $.extend({
            helper: "clone",
            zIndex: 65000,
            revert: true,
            opacity: 1,
            shadow: false,
            handle: undefined,
            over: function (flag, helper) {
                helper.css("backgroundColor", flag ? "#a6c37d" : "white");
            },
            out: function (helper) {
                helper.css("backgroundColor", "white");
            },
            scope: {},
            // events
            start: $.noop,
            stop: $.noop,
            drag: $.noop
        }, options || {});
        
        // set scope
        o.hover = $.proxy(o.hover, node);
        
        var remove = function () {
            if (helper !== null) {
                helper.remove();
                helper = null;
                fast = null;
            }
        };
        var revert = function () {
            if (o.revert === true) {
                helper.empty().css({
                    MozBoxShadow: "",
                    border: "1px solid #ccc"
                }).animate(
                    { top: offset.top + "px", left: offset.left + "px" },
                    500,
                    remove
                );
            } else {
                remove();
            }
        };
        
        // event handlers
        var move = (function () {
            
            var px = 0, py = 0, x, y, abs = Math.abs;
            
            return function (e) {
                // use fast access
                x = e.pageX + delta.left;
                y = e.pageY + delta.top;
                if (abs(px - x) >= 5 || abs(py - y) >= 5) {
                    fast.top = y + "px";
                    fast.left = x + "px";
                    px = x;
                    py = y;
                    o.drag(e, x, y);
                }
            };
        }());
        
        var hover = function (e) {
            switch (e.type) {
            case "mouseover":
                var zone = $(this).data("dropzone"), accept = zone.accept(o.scope);
                o.over(accept, helper);
                dropzone = accept ? zone : null;
                break;
            case "mouseout":
                dropzone = null;
                o.out(helper);
                break;
            }
        };
        var drag = function (e) {
            // unbind
            $(document).unbind("mousemove", drag);
            // create helper
            if (o.helper === "clone") {
                // get offset
                offset = node.offset();
                // get delta
                if (o.cursorAt) {
                    delta = {
                        top: o.cursorAt.top,
                        left: o.cursorAt.left
                    };
                } else {
                    delta = {
                        top: offset.top - origin.top,
                        left: offset.left - origin.left
                    };
                }
                // create clone
                var clone = node.clone(false).css({
                    position: "static",
                    top: "",
                    left: "",
                    right: "",
                    bottom: "",
                    width: node.width() + "px",
                    height: node.height() + "px",
                    margin: 0
                });
                var css = {
                    position: "absolute", 
                    top: (e.pageY - delta.top) + "px",
                    left: (e.pageX - delta.left) + "px",
                    zIndex: o.zIndex,
                    opacity: o.opacity
                };
                if (o.shadow === true) {
                    css.MozBoxShadow = "2px 2px 20px 3px #000";
                    css.webkitBoxShadow = "2px 2px 20px 3px #000";
                } else {
                    css.MozBoxShadow = "none";
                    css.webkitBoxShadow = "none";
                }
                clone.find("iframe").remove();
                helper = $("<div/>").css(css).append(clone);
            } else {
                helper = node;
            }
            // get fast access
            fast = helper[0].style;
            // event
            o.start(e, { helper: helper, original: node });
            // initial move
            move(e);
            // replace in DOM
            helper.appendTo(document.body);
            // activate drop zones
            $(".dropzone").bind("mouseover mouseout", hover);
            // bind
            $(document).bind("mousemove", move);
        };
        var stop = function (e) {
            // unbind handlers
            $(".dropzone").unbind("mouseover mouseout", hover);
            $(document).unbind("mousemove", move);
            $(document).unbind("mousemove", drag);
            $(document).unbind("mouseup", stop);
            // revert?
            if (helper !== null) {
                if (dropzone === null) {
                    o.out(helper);
                    revert();
                } else {
                    remove();
                    dropzone.drop(o.scope);
                }
                // event
                o.stop(e);
            }
        };
        var start = function (e) {
            // remember origin
            origin = { top: e.pageY, left: e.pageX };
            // bind events
            $(document).bind("mousemove", drag);
            $(document).bind("mouseup", stop);
        };

        // disable
        this.disable = function () {
            (o.handle ? node.find(o.handle) : node).unbind("mousedown", start);
            return this;
        };
        
        // enable
        this.enable = function () {
            (o.handle ? node.find(o.handle) : node).bind("mousedown", start);
            return this;
        };
    };
    
    $.fn.dndDraggable = function (options) {
        
        return this.each(function () {
            // vars
            var node = $(this), instance = node.data("draggable");
            // action?
            if (typeof options === "string") {
                if (instance) {
                    switch (options) {
                    case "destroy":
                        instance.disable();
                        node.removeData("draggable");
                        break;
                    case "disable":
                        instance.disable();
                        break;
                    case "enable":
                        instance.enable();
                        break;
                    }
                }
            } else {
                // has instance?
                if (instance) {
                    node.draggable("destroy");
                }
                // new instance
                node.data("draggable", new Draggable(node, options).enable());
            }
        });
    };
    
    var Dropzone = function (node, options) {
        
        // defaults
        var o = $.extend({
            accept: function () {
                return true;
            },
            drop: function (scope) {
                console.log("Drop!", scope);
            }
        }, options || {});
        
        this.accept = function (scope) {
            return o.accept(scope);
        };
        
        this.drop = function (scope) {
            o.drop(scope);
        };
        
        // disable
        this.disable = function () {
            node.removeClass("dropzone");
            return this;
        };
        
        // enable
        this.enable = function () {
            node.addClass("dropzone");
            return this;
        };
    };
    
    $.fn.dropzone = function (options) {
        
        return this.each(function () {
            // vars
            var node = $(this), instance = node.data("dropzone");
            // action?
            if (typeof options === "string") {
                if (instance) {
                    switch (options) {
                    case "destroy":
                        instance.disable();
                        node.removeData("dropzone");
                        break;
                    case "disable":
                        instance.disable();
                        break;
                    case "enable":
                        instance.enable();
                        break;
                    }
                }
            } else {
                // has instance?
                if (instance) {
                    node.dropzone("destroy");
                }
                // new instance
                node.data("dropzone", new Dropzone(node, options).enable());
            }
        });
    };
    
}(jQuery));
/**
 * 
 * All content on this website (including text, images, source
 * code and any other original works), unless otherwise noted,
 * is licensed under a Creative Commons License.
 * 
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 * 
 * Copyright (C) Open-Xchange Inc., 2006-2012
 * Mail: info@open-xchange.com 
 * 
 * @author Matthias Biggeleben <matthias.biggeleben@open-xchange.com>
 * @ignore
 */

// DEVELOPMENT IN PROGRESS // SUBJECT TO PERMANENT CHANGE!


ox.gui.Menu = ox.gui.Container.extend({
/** @lends ox.gui.Menu.prototype */
    
    getClass: function() { return "ox.gui.Menu"; },
    
    /** 
     * The basic Menu class
     * @constructs
     * @extends ox.gui.Container
     * @param {String} id An optional unique id for the widget.
     */
    construct: function(id) {

        // call super class constructor (inherit from Container)
        this._super(id);
        
        // active?
        this.statusActive = false;
        
        // current menu item
        this.openHover = null;
    },
    
    /**
     * @private
     */
    getTopOfPropagation: function() {
        var current = this;
        while (current.parent && current.parent.openHover) {
            current = current.parent;
        }
        return current;
    },

    /**
     * @private
     */
    propagateOpenHover: function (item) {
        // local handler?
        var p = this.hoverParent || this.parent;
        if (p.openHover !== undefined) {
            // another item open?
            if (p.openHover && p.openHover != item) {
                p.openHover.closeHover();
            }
            // set open menu item
            p.openHover = item;
        } else if ($.isFunction(p.propagateOpenHover)) {
            // a parent widget will take care
            p.propagateOpenHover(item);
        }
    },
    
    closeHover: function () {
    },
    
    /**
     * @private
     */
    isParentActive: function() {
        // return (and inject) state
        return (this.parent.statusActive = !!this.parent.statusActive);
    }
});

//------------------------------------------------------------------------------

// define within a closure to manage private static vars

(function () {
    
    // radio buttons
    var radios = {};

    ox.gui.MenuItem = ox.gui.Menu.extend({
    /** @lends ox.gui.MenuItem.prototype */
        
        getClass: function() { return "ox.gui.MenuItem"; },
        
        /** 
         * The basic MenuItem class
         * @constructs
         * @extends ox.gui.Menu
         * @param {Object} options Options
         * @param {String} [options.id] Widget ID
         * @param {String} [options.title] Menu item title
         * @param {Array} [options.icons] Menu icons
         * @param {String} [options.pathPrefix] Optional non-theme prefix for icon paths
         * @param {Boolean} [options.big] Appear with a large icon
         * @param {function ()} [options.action] Default action. Must return true to close the menu.
         * @param {Boolean} [options.enabled] Enabled/Disabled
         * @param {Array} [options.requires] Internal descriptions to enable/disable items
         * @param {String} [options.behavior] button/checkbox/radio/split
         * @param {String} [options.group] Group name/id for radio buttons
         */
        construct: function (options) {
            
            // options
            this.options = $.extend({
                id: undefined,
                title: noI18n(""),
                icons: [],
                pathPrefix: undefined,
                big: false,
                action: $.noop,
                enabled: true,
                requires: [],
                behavior: "button",
                group: "",
                widget: this,
                parts: "both" // text/icon/both
            }, options || {});
            
            // can be cloned
            this.clonable = true;
            
            // call super class constructor (inherit from Menu)
            this._super(this.options.id);
            
            // re-get id
            this.options.id = this.id;
    
            // set text, icon etc.
            this.text = this.options.title;
            this.icon = this.options.icons.length > 0 ? this.options.icons[0] : "";
            this.iconDisabled = this.options.icons.length > 1 ? this.options.icons[1] : "";
            this.bigIcon = !!this.options.big;
            this.statusEnabled = !!this.options.enabled;
            this.forceSmallIcon = false;
    
            // menu strip
            this.menuStrip = null;
            this.menuStripAlign = "bottom";
            
            // click callback
            this.onClickCallback = null;
            
            // show/hide timeouts
            this.timeoutShow = null;
            this.timeoutHide = null;
            
            // radio?
            if (this.options.behavior === "radio") {
                // get group
                var group = this.options.group + "";
                // add to observer
                if (radios[group] === undefined) {
                    radios[group] = [];
                }
                radios[group].push(this);
            }
        },
        
        i18nHandler: function () {
            this.paint();
        },
        
        nodeInit: function() {
            // styles
            this.addCSSClass("menu-item");
            //this.css({ cursor: "pointer" });
        },
        
        nodeListeners: (function() {
           
            var fnOver = function (e) {
                if (this.isEnabled()) {
                    this.addCSSClass("menu-item-over");
                    if (this.options.behavior !== "split" && this.isParentActive() && this.isEnabled()) {
                        this.activate();
                    }
                }
            };
            
            var fnDown = function (e) {
                // fix event
                e = $.event.fix(e || window.event);
                // "hard" mousedown
                if (this.options.mousedown) {
                    this.options.mousedown(this.options, e);
                }
                if (this.isEnabled()) {
                    this.addCSSClass("menu-item-down");
                }
            };
            
            var fnUp = function (e) {
                if (this.isEnabled()) {
                    this.removeCSSClass("menu-item-down");
                }
            };
            
            var fnOut = function (e) {
                if (this.isEnabled()) { 
                    this.removeCSSClass("menu-item-down");
                }
                window.clearTimeout(this.showTimeout);
                this.removeCSSClass("menu-item-over");
            };
            
            var fnClick = function (e) {
                // fix event
                e = $.event.fix(e || window.event);
                // is enabled?
                if (this.isEnabled()) {
                    // action taken
                    var action = false;
                    // is radio or checkbox?
                    if (this.options.behavior === "radio") {
                        // check
                        this.setChecked(true);
                        action = true; // consider as action taken
                    } else if (this.options.behavior === "checkbox") {
                        // toggle state
                        this.setChecked(!this.stateChecked);
                        action = true; // consider as action taken
                    }
                    // has individual action?
                    if (this.options.action !== $.noop) {
                        // extend event
                        e.widget = this;
                        // action
                        action = this.options.action(e) || action; // the order is important!
                    }
                    // action taken?
                    if (action === true) {
                        // hide hovers
                        ox.desktop.hideHovers();
                    } else {
                        // activate
                        this.toggleActive();
                    }
                    // avoid bubbling
                    e.stopPropagation();
                }
            };
            
            if (ox.browser.IE) {
                // fix strange IE/jQuery bug
                // TODO: review this issue with more time
                return function () {
                    var node = this.dom.node;
                    node.onmouseover = $.proxy(fnOver, this);
                    node.onmousedown = $.proxy(fnDown, this);
                    node.onmouseup = $.proxy(fnUp, this);
                    node.onmouseout = $.proxy(fnOut, this);
                    node.onclick = $.proxy(fnClick, this);
                };
            } else {
                return function () {
                    var node = $(this.dom.node), click = false, self = this;
                    node.bind("mouseover", $.proxy(fnOver, this));
                    node.bind("mouseout", $.proxy(fnOut, this));
                    node.bind("mousedown", function (e) {
                        click = true;
                        $.proxy(fnDown, self)(e);
                    });
                    node.bind("mouseup", function (e) {
                        $.proxy(fnUp, self)(e);
                        if (click) {
                            // for some unknown reason, webkit rarely misses the "click" event
                            // since "down" and "up" are always working, we use this silly workaround
                            $.proxy(fnClick, self)(e);
                            click = false;
                        }
                    });
                };
            }
            
        }()),
        
        validate: function (force) {
            // render both the widget as well as the menuStrip
            if (this.menuStrip) {
                // copy flag
                this.menuStrip.requiresValidation = !!(this.requiresValidation || force);
                // validate menu strip
                this.menuStrip.validate();
            }
            // render item
            ox.gui.Widget.prototype.validateTree.call(this);
        },
        
        paintVisible: function (cont) {
            this.dom.node.style.display = "block";
            this.paintState();
            ox.call(cont);
        },
        
        paintEnabled: function() {
            this.paintState();
        },
        
        paintDisabled: function() {
            this.paintState();
        },
        
        getPath: function(path) {
            var prefix = this.options.pathPrefix;
            return prefix === undefined ? ox.gui.themePath + path
                                        : urlify(path);
        },
        
        paint: function () {
            
            // enable i18n (on first paint)
            this.enableI18n();
            
            // get empty jQuery node
            var node = $(this.dom.node).empty(),
                // ref to this
                self = this;
            
            // remove keyboard shortcuts
            var text = tooltip = (this.text || "").replace(/&/, "");
            node.attr({ title: String(tooltip || text) });
            
            // big icon?
            if (this.bigIcon && !this.forceSmallIcon) {
                
                // popup below the item
                this.menuStripAlign = "bottom";
                
                // center align
                node.css({ textAlign: "center" });
                
                // adjust line breaks
                var matches = text.match(/([^\s]+\s|[^\s]+$)/g);
                var escaped = "";
                if (matches && matches.length > 1) {
                    var splitIndex = Math.ceil(matches.length/2);
                    var half1 = matches.slice(0, splitIndex);
                    var half2 = matches.slice(splitIndex);
                    text = half1.join("") + "\n" + half2.join("") + "\u200B";
                    escaped = text.replace(/</g, "&lt;").replace(/\s*\n\s*/g, "<br/>");
                } else {
                    escaped = text.replace(/</g, "&lt;").replace(/\s*\n\s*/g, "<br/>") + "<br/>&nbsp;";
                }
                
                // add icon?
                if (this.icon) {
                    var icon = $("<img/>", {
                        src: this.getPath(this.icon), 
                        alt: String(tooltip || text)
                    });
                    node.append(icon.addClass("menu-item-icon-large"));
                    this.dom.icon = icon[0];
                }
                
                // add text
                var div = $("<div/>")
                    .css("whiteSpace", "nowrap")
                    .html(escaped).appendTo(node);
                
                // arrow?
                if (this.hasChildren()) {
                    div.append(
                        $("<img/>", { src: ox.gui.themePath + "img/arrows/menu_arrow_down.gif" }).
                            addClass("menu-item-arrow")
                    );
                }
                
            } else {
                
                // left align
                node.css({
                    textAlign: "left",
                    whiteSpace: "nowrap"
                });
                
                var parts = this.options.parts;
                var hasIcon = this.icon && (parts === "icon" || parts === "both");
                var hasText = parts === "text" || parts === "both";
                
                // add icon?
                if (hasIcon) {
                    var icon = $("<img/>", {
                        src: this.getPath(this.icon),
                        alt: String(tooltip || text)
                    });
                    node.append(icon.addClass("menu-item-icon"));
                    this.dom.icon = icon[0];
                }
                
                // split button?
                if (this.options.behavior === "split") {
                    // add text
                    if (hasText) {
                        node.append(
                            $("<span/>").text("\u00a0\u00a0" + text + "\u00a0")
                        );
                    }
                    // arrow?
                    if (this.hasChildren()) {
                        node.append(
                             $("<img/>", {
                                src: ox.gui.themePath + "img/arrows/menu_arrow_down.gif",
                                alt: ""
                             })
                            .addClass("menu-item-arrow menu-item-split")
                            .bind("mousedown mouseup", function (e) {
                                e.stopPropagation();
                            })
                            .bind("click", function (e) {
                                // activate
                                self.toggleActive();
                                return false;
                            })
                        );
                    }
                } else {
                    
                    // add text
                    if (hasText) {
                        node.append(
                            $("<span/>").text(hasIcon ? "\u00a0" + "\u00a0" + text : text)
                        );
                    }
                    
                    // arrow?
                    if (this.hasChildren()) {
                        node.append(
                            $("<img/>", { src: ox.gui.themePath + "img/arrows/menu_arrow_down.gif", alt: "" }).
                                addClass("menu-item-arrow")
                        );
                    }
                }
            }
            // consider current state
            this.paintState();
        },
        
        /**
         * Toggle active status.
         */
        toggleActive: function() {
            if (this.statusActive) {
                this.deactivate();
            } else {
                this.activate();
            }
        },
        
        /**
         * Activates the menu.
         */
        activate: function() {
            this.statusActive = true;
            this.open();
        },
        
        /**
         * Deactivates the menu.
         */
        deactivate: function() {
            this.statusActive = false;
            this.close();
            // loop children
            this.each(function(child) {
                if (child.deactivate) {
                    child.deactivate();
                }
            });
        },
        
        closeHover: function () {
            this.deactivate();
        },
        
        /**
         * Defines the default callback (dom:click).
         * @param {Function} callback The callback function
         */
        onClick: function(callback) {
            this.onClickCallback = callback;
        },
        
        hide: function() {
            this._super();
            this.deactivate();
        },
        
        hideHovers: function () {
            this.deactivate();
        },
        
        /**
         * Opens the menu item's menu strip (if it has children)
         */
        open: function (delay) {
            // propagate
            this.propagateOpenHover(this);
            // has children?
            if (this.hasChildren()) {
                // strip initialized?
                if (!this.menuStrip) {
                    this.menuStrip = new ox.gui.MenuStrip();
                    this.menuStrip.parent = this;
                }
                if (!this.menuStrip.statusOpen) {
                    // adjust delay
                    delay = delay || 50;
                    // stop running timeout
                    if (this.showTimeout != null) {
                        window.clearTimeout(this.showTimeout);
                    }
                    // show strip
                    var self = this;
                    this.showTimeout = window.setTimeout(function() {
                        // open strip
                        self.menuStrip.open();
                    }, delay);
                }
            }
        },
        
        /**
         * Closes the menu item's menu strip.
         */
        close: function() {
            if (this.menuStrip) {
                this.menuStrip.close();
            }
        },
        
        /**
         * Set state: checked (for checkbox behavior)
         */
        setChecked: function (state) {
            // changed?
            if (state !== this.stateChecked) {
                // is radio to be checked?
                if (state === true && this.options.behavior === "radio") {
                    // disable other items
                    var i = 0, items = radios[this.options.group] || [], $l = items.length, item;
                    for (; i < $l; i++) {
                        item = items[i];
                        if (item !== this) {
                            item.setChecked(false);
                        }
                    }
                }
                // set new state
                this.stateChecked = state;
                // paint
                this.paintState();
            }
        },
        
        paintState: function () {
            
            // invisible?
            if (this.statusVisible === false) {
                // done
                return;
            }
            
            // get node
            var node = $(this.dom.node);
            
            // checked?
            if (this.stateChecked === true) {
                // add class (overwrites disabled)
                node.addClass("menu-item-checked");
                node.removeClass("menu-item-disabled");
                return;
            } else {
                node.removeClass("menu-item-checked");
            }
            
            // disabled?
            if (this.statusEnabled === false) {
                // add class
                node.addClass("menu-item-disabled");
            } else {
                node.removeClass("menu-item-disabled");
            }
        },
        
        /**
         * Sets the icon dynamically.
         * @param {String} icon The new URL of the icon, relative to a theme.
         */
        setIcon: function(icon) {
            this.icon = icon;
            if (this.dom.icon) this.dom.icon.src = this.getPath(icon);
        },
        
        /**
         * Sets the title dynamically.
         * @param {String} title The new title of the menu item.
         */
        setTitle: function(title) {
            this.options.title = title;
            this.text = title;
            this.paint();
        }
    });
    
}());

//------------------------------------------------------------------------------

ox.gui.MainMenu = ox.gui.Menu.extend({
/** @lends ox.gui.MainMenu.prototype */
    
    getClass: function() { return "ox.gui.MainMenu"; },
    
    /** 
     * A main menu
     * @constructs
     * @extends ox.gui.Menu
     * @param {String} id An optional unique id for the widget.
     */
    construct: function(id) {
        
        // call super class constructor (inherit from Menu)
        this._super(id);
        
        // set layout manager
        this.setLayout(new MenuBarLayout(this));
    },
    
    add: function(item) {
        // set align
        item.menuStripAlign = "bottom";
        // call super class method
        ox.gui.Container.prototype.add.call(this, item);
    }
});


//------------------------------------------------------------------------------

(function () {
    
    ox.gui.MenuStrip = ox.gui.Menu.extend({
    /** @lends ox.gui.MenuStrip.prototype */
        
        getClass: function() { return "ox.gui.MenuStrip"; },
        
        /** 
         * A menu strip
         * @constructs
         * @extends ox.gui.Menu
         * @param {String} id An optional unique id for the widget.
         */
        construct: function(id) {
            
            this.clonable = true;
            this.isHover = true;
            
            // call super class constructor
            this._super(id);
            
            // set layout manager
            this.setLayout(new ox.gui.MenuStripLayout(this));
            
            this.statusOpen = false;
            this.statusActive = true; // always active
            
            this.inDOM = false;
            
            // mouse event handlers
            var self = this;
            this.fnLocalEvent = $.proxy(function (e) {
                e.stopPropagation();
            }, this);
            this.fnGlobalEvent = $.proxy(function (e) {
                this.parent.deactivate();
                this.close(); // make sure it really closes in all cases
            }, this);
            
            var overlays = $();
            
            this.fixIFrames = function (flag) {
                // show?
                if (flag) {
                    // get all visible iframes
                    var iframes = $("iframe:visible"), i = 0, $i = iframes.length;
                    var iframe, offset, width, height, fix;
                    // loop
                    for (; i < $i; i++) {
                        // get
                        iframe = iframes.eq(i);
                        // get dimensions
                        offset = iframe.offset();
                        width = iframe.outerWidth();
                        height = iframe.outerHeight();
                        // add overlay
                        overlays = overlays.add($("<div/>").css({
                            position: "absolute",
                            top: offset.top,
                            left: offset.left,
                            width: width,
                            height: height,
                            zIndex: 5000,
                            backgroundColor: "white",
                            opacity: 0.001
                        }).text(" ").appendTo(document.body));
                    }
                } else {
                    // hide overlays
                    overlays.remove();
                    overlays = $();
                }
            };
        },
        
        nodeInit: function() {
            this.addCSSClass("menustrip");
        },
        
        nodeListeners: function() {
            // add listener
            this.addListener("dom:resize", window, function(e) {
                this.parent.deactivate();
            });
        },
        
        getChildren: function() { return this.parent.getChildren(); },
        
        /**
         * @private
         */
        setPosition: function() {
            // get parent
            var parent = this.parent;
            // get position of current menu
            var parenNode = $(parent.dom.node);
            var parentPos = parenNode.offset();
            var parentDim = {
                width: parenNode.outerWidth(true),
                height: parenNode.outerHeight(true)
            };
            // get menu dimensions
            var node = $(this.dom.node);
            var dim = {
                width: node.outerWidth(true) || 0,
                height: node.outerHeight(true) || 0
            };
            // get maximal positions
            var maxLeft = $(document).width() - dim.width - 10, maxTop = $(document).height() - dim.height - 10;
            // adjust position of menu strip
            var style = this.dom.node.style;
            switch (parent.menuStripAlign) {
                case "right":
                    style.left = Math.max(0, Math.min(parentPos.left + parentDim.width - 5, maxLeft)) + "px";
                    style.top  = Math.max(0, Math.min(parentPos.top, maxTop)) + "px";
                    break;
                case "bottom":
                    style.left = Math.max(0, Math.min(parentPos.left, maxLeft)) + "px";
                    style.top  = Math.max(0, Math.min(parentPos.top + parentDim.height, maxTop)) +"px";
                    break;
            }
            // be on top
            var p = parent.parent;
            if (p && p.menuStrip) {
                style.zIndex = (p.menuStrip.dom.node.style.zIndex || 10001) + 1;
            } else {
                style.zIndex = 10001;
            }
        },
        
        /**
         * Opens the menu strip
         */
        open: function (delay) {
            if (!this.statusOpen) {
                // set status
                this.statusOpen = true;
                // has children?
                if (this.hasChildren()) {
                    // in DOM?
                    if (!this.inDOM) {
                        // add to DOM
                        document.body.appendChild(this.dom.node);
                    }
                    // apply iframe fix
                    this.fixIFrames(true);
                    // set strip position?
                    this.setPosition();
                    this.validate(true);
                    // bind mouse event
                    $(this.dom.node).bind("click", this.fnLocalEvent);
                    $(document.body).bind("click", this.fnGlobalEvent);
                    // show strip
                    this.dom.node.style.display = "block";
                    this.dom.node.style.visibility = "visible";
                    // register
                    this.hasHover(true);
                }
            }
        },
        
        /**
         * Closes the menu strip
         */
        close: function () {
            this.statusOpen = false;
            // remove iframe fix
            this.fixIFrames(false);
            // unbind mouse event
            $(this.dom.node).unbind("click", this.fnLocalEvent);
            $(document.body).unbind("click", this.fnGlobalEvent);
            // hide
            this.dom.node.style.display = "none";
            this.dom.node.style.visibility = "hidden";
            this.hasHover(false);
        }
    });
    
}());

//------------------------------------------------------------------------------


ox.gui.ContextMenu = ox.gui.Menu.extend({
/** @lends ox.gui.ContextMenu.prototype */
    
    getClass: function() { return "ox.gui.ContextMenu"; },
    
    /** 
     * A context menu / right mouse button menu.
     * @constructs
     * @extends ox.gui.Menu
     * @param {String} id An optional unique id for the widget.
     */
    construct: function(id) {
        // call super class constructor (inherit from Menu)
        this._super(id);
    },
    
    paint: function() {
        this.dom.node.innerHTML = "[ContextMenu] " + this.id;
    }
});


//------------------------------------------------------------------------------

/** @ignore */
function MenuBarLayout(targetDOMNode) {
    this.targetDOMNode = targetDOMNode || null;
}

/** @ignore */
MenuBarLayout.prototype.doLayout = function(container) {
    // set target DOM node
    var targetDOMNode = this.targetDOMNode || container.dom.node;
    // clear
    targetDOMNode.innerHTML = "";
    var table = document.createElement("table");
    var tbody = document.createElement("tbody"); // IE needs an explicit
                                                    // table body
    var tr = document.createElement("tr");
    // loop children
    for(var i = 0; i < container.children.length; i++) {
        var child = container.children[i];
        // add to DOM & show
        var td = document.createElement("td");
        child.setParentDOMNode(td);
        child.validate();
        tr.appendChild(td);
    }
    tbody.appendChild(tr);
    table.appendChild(tbody);
    targetDOMNode.appendChild(table);
}


//------------------------------------------------------------------------------


ox.gui.MenuStripLayout = ox.gui.LayoutManager.extend({
/** @lends ox.gui.MenuStripLayout.prototype */
    
    getClass: function () { return "ox.gui.MenuStripLayout"; },
    
    /** 
     * A menu strip layout manager.
     * @extends ox.gui.LayoutManager
     * @constructs
     * @param {ox.gui.Container} The container widget that should be managed
     * by the layout manager.
     */
    construct: function (container) {
        this._super(container);
    },
    
    doLayout: function () {
        var target = this.container.dom.node;
        // loop children
        this.container.each(function (child) {
            child.validate();
            target.appendChild(child.dom.node);
        });
    }
});
/**
 * 
 * All content on this website (including text, images, source
 * code and any other original works), unless otherwise noted,
 * is licensed under a Creative Commons License.
 * 
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 * 
 * Copyright (C) Open-Xchange Inc., 2006-2012
 * Mail: info@open-xchange.com 
 * 
 * @author Matthias Biggeleben <matthias.biggeleben@open-xchange.com>
 * @ignore
 */

// DEVELOPMENT IN PROGRESS // SUBJECT TO PERMANENT CHANGE!


ox.gui.TabControl = ox.gui.StatefulContainer.extend({
/** @lends ox.gui.TabControl.prototype */
    
    getClass: function() { return "ox.gui.TabControl"; },
    
    /** 
     * A tab control widget. Manages tab pages.
     * @constructs
     * @extends ox.gui.StatefulContainer
     * @param {String} id An optional unique id for the widget.
     */
    construct: function (id) {
        
        // call super class constructor
        this._super(id);
        
        // set layout manager (for tabs; not pages)
        this.setLayout(new ox.gui.TabControlLayout(this));
        
        // expand/hover
        this.expanded = true;
        this.expandable = false;
        this.inHover = false;
        this.lastClickedChild = null;
        
        // create tab container
        this.tabContainer = document.createElement("div");
        this.tabContainer.className = "oxTabControl";
        // create tab page container
        this.tabPageContainer = document.createElement("div");
        this.tabPageContainer.className = "oxTabControlPageContainer";
        
        // create hover
        var target = $("#everything");
        this.dom.hover = document.createElement("div");
        this.dom.hover.className = "oxTabControlPageHover";
        target.append(this.dom.hover);
        this.dom.hoverUnderlay = document.createElement("div");
        this.dom.hoverUnderlay.className = "oxTabControlPageHoverUnderlay";
        target.append(this.dom.hoverUnderlay);
        
        // event handler
        var self = this;
        $(this.dom.hoverUnderlay).add(document.body).bind("click", function (e) {
            if (self.inHover && self.expanded) {
                // collapse
                self.collapse();
            }
        });

        // set target node
        this.defaultTargetDOMNodeForChildren = this.tabPageContainer;
        
        this.enableI18n();
    },
    
    i18nHandler: function() {
        this.layoutManager.doLayout();
    },
    
    nodeInit: function() {
        // stretch DOM node
        this.setStyle("overflow: visible");
    },
    
    nodeListeners: function() {
        
        var fnWheel = function (e) {
            var p = 0 + this.currentChild.position;
            var children = this.getChildren();
            p = Math.max(0, Math.min(p - e.deltaSign, children.length-1));
            this.showChild(children[p].id);
            return false;
        };
        
        // wheel event (turned off)
        // this.addListener("dom:mousewheel", this.dom.hover, fnWheel);
        
        // double click
        this.addListener("dom:dblclick", this.tabContainer, function (e) {
            // only on upper bar
            if (e.target === this.tabContainer) {
                this.toggleHover();
                return false;
            }
        });
    },
    
    add: function (widget) {
        // must be an instance of Widget
        if (widget.id && widget instanceof ox.gui.Widget) {
            // invisible
            widget.statusVisible = true; // required to force paintInvisible (see next row)
            widget.setVisible(false);
            // call super class method
            ox.gui.StatefulContainer.prototype.add.call(this, widget);
            // first added page?
            if (!this.currentChild && !this.inHover) {
                this.currentChild = widget;
                this.currentState = widget.id;
                this.currentChild.setVisible(true);
                this.triggerWidgetEvent("tabchanged", this.currentChild);
            }
            // add to index
            this.childIndex[widget.id] = widget;
            
        } else {
            ox.desktop.oopsFunc(this.getClass()+"(<widget>)", "<widget> must be an instance of ox.gui.Widget and it MUST have an id!");
        }
        // support chaining
        return this;
    },
    
    hideCurrentChild: function () {
        if (this.currentChild != null) {
            // hide
            this.currentChild.hide();
            this.currentState = "";
            // hide hovers, e.g. menustrips
            this.hideHovers();
        }
    },
    
    setPosition: function (position) {
        if (typeof position === "number" && position > 0 && this.children.length > position) {
            // show child
            this.showChild(this.children[position].id);
        }
    },
    
    showChild: function (childID) {
        // switch to child
        if (this.childIndex[childID] !== undefined && this.currentState != childID) {
            this.hideCurrentChild();
            this.currentChild = this.childIndex[childID];
            this.currentState = childID;
            // update tabs
            this.layoutManager.update(this);
            // show child
            this.currentChild.show();
        }
    },
    
    /**Highest
     * Toggles expand/collapse.
     */
    toggle: function() {
        if (this.expandable) {
            if (this.expanded) {
                this.collapse();
            } else {
                this.expand();
            }
        }
    },
    
    /**
     * Expands the tab page container.
     */
    expand: function() {
        if (this.expandable && !this.expanded) {
            if (this.inHover) {
                // adjust dimensions
                // get offset of tab control
                var o = $(this.dom.node).offset();
                // add height of tab control
                o.top += $(this.tabContainer).height();
                // move hover to proper position
                $(this.dom.hover).css({
                    top: o.top + "px",
                    left: o.left + "px"
                });
                // move underlay to proper position
                $(this.dom.hoverUnderlay).css({
                    top: o.top + "px"
                }).show();
                // show hover
                $(this.dom.hover).show();
                this.hasHover(true);
            }
            // show container
            $(this.tabPageContainer).show();
            // set flag
            this.expanded = true;
            // update tabs
            this.layoutManager.update(this);
            // trigger event
            this.triggerWidgetEvent("tabexpanded", this);
            // trigger event
            this.triggerWidgetEvent("tabtoggled", this);
        }
    },

    /**
     * Collapses the tab page container.
     */
    collapse: function() {
        if (this.expandable && this.expanded) {
            // hide
            $(this.dom.hoverUnderlay).hide();
            $(this.tabPageContainer).hide();
            $(this.dom.hover).hide();
            // set flag
            this.expanded = false;
            // update tabs
            this.layoutManager.update(this);
            // no hovers
            this.hasHover(false);
            // trigger event
            this.triggerWidgetEvent("tabcollapsed", this);
            // trigger event
            this.triggerWidgetEvent("tabtoggled", this);
            
        }
    },
    
    /**
     * Toggle "hover mode".
     */
    toggleHover: function (flag, event) {
        // adjust params
        flag = flag === undefined ? !this.inHover : flag;
        event = event === undefined || event === true;
        // change?
        if (flag !== this.inHover) {
            if (flag === false) {
                // unregister
                this.hasHover(false);
                // move
                $(this.tabPageContainer).
                    removeClass("oxTabControlPageContainerInHover").
                    appendTo(this.dom.node);
                this.inHover = false;
                // hide underlay
                $(this.dom.hoverUnderlay).hide();
                // trigger event
                if (event) {
                    this.triggerWidgetEvent("togglehover", false);
                }
                // expand
                this.expand();
            } else {
                // register
                this.hasHover(true);
                // append to hover
                $(this.tabPageContainer).
                    addClass("oxTabControlPageContainerInHover").
                    appendTo(this.dom.hover);
                this.inHover = true;
                // trigger event
                if (event) {
                    this.triggerWidgetEvent("togglehover", true);
                }
                // collapse
                this.collapse();
            }
        }
    },
    
    hideHovers: function () {
        if (this.inHover) {
            // collapse
            this.collapse();
            // hide underlay
            $(this.dom.hoverUnderlay).hide();
        }
        // continue
        ox.gui.StatefulContainer.prototype.hideHovers.call(this);
    }
});

//------------------------------------------------------------------------------

ox.gui.TabPage = ox.gui.Container.extend({
/** @lends ox.gui.TabPage.prototype */
    
    getClass: function() { return "ox.gui.TabPage"; },
    
    /** 
     * A tab page
     * @constructs
     * @extends ox.gui.Container
     * @param {String} id An optional unique id for the widget.
     * @param {String} title The title of the tab page.
     */
    construct: function(id, title) {
        // call super class constructor (inherit from Container)
        this._super(id);
        
        // tab title
        this.title = title || _("No title");
    },
    
    initNode: function() {
        this._super();
        this.setStyle("overflow: hidden");
    }
});

//------------------------------------------------------------------------------

ox.gui.TabControlLayout = ox.gui.LayoutManager.extend({
/** @lends ox.gui.TabControlLayout.prototype */
    
    getClass: function() { return "ox.gui.TabControlLayout"; },
    
    /** 
     * @constructs
     * @extends ox.gui.LayoutManager
     */
    construct: function(container) {
        this._super(container);
        this.tds = [];
        this.firstRun = true;
    },
    
    doLayout: function() {

        var layoutmanager = this;
        var tabControl = this.container;
        
        // clear
        tabControl.tabContainer.innerHTML = "";
        layoutmanager.tds = [];
        
        // add "toggle" button
        var button = document.createElement("div");
        button.className = tabControl.inHover ? "oxTabControlToggleReverse" : "oxTabControlToggle";
        tabControl.tabContainer.appendChild(button);
        
        // add listener
        tabControl.addListener("dom:click", button, function(e) {
            if (this.expanded) {
                this.collapse();
            } else {
                this.showChild(this.children[0].id);
                this.expand();
            }
            this.lastClickedChild = null;
            this.toggleHover();
            return false;
        });
        
        tabControl.addListener("widget:togglehover", function (e) {
            button.className = e.data ? "oxTabControlToggleReverse" : "oxTabControlToggle";
        });
        
        // create table
        var table = document.createElement("table");
        table.border = table.cellPadding = table.cellSpacing = 0;
        var tbody = document.createElement("tbody");
        var tr = document.createElement("tr");
        // loop children
        tabControl.each(function(child, i, childCount) {
            // add to DOM & show
            var td = document.createElement("td");
            var className = i < childCount - 1 ? "oxTab" : "oxTabLast";
            className += child == tabControl.currentChild ? " oxTabCurrent" : "";
            td.className = className;
            // add a dead link (safari needs this for tab index)
            var a = document.createElement("a");
            var title = escapeHTML(expectI18n(child.title));
            a.innerHTML = title.replace(/&amp;(.)/, '$1');
            a.href = "#";
            a.tabIndex = ox.gui.globalTabIndex++;
            td.appendChild(a);
            tr.appendChild(td);
            // save for faster updates
            layoutmanager.tds.push(td);

            // click handler
            var fnClick = function (e) {
                var tab = this.parent;
                // we have to use "lastClickedChild" here, since this click event 
                // interferes with "dom:onFocus" (see above)
                if (tab.inHover && tab.lastClickedChild == this && tab.expanded) {
                    tab.collapse();
                } else {
                    // todo: abstract solution for tracking
                    track({
                        type: "click",
                        dest: "tab",
                        data: { id: this.id, title: this.title },
                        text: "Change to tab \"" + this.title.replace(/&/, "") + "\" (" + this.id + ")"
                    });
                    tab.showChild(this.id);
                    tab.expand();
                }
                tab.lastClickedChild = this;
                return false;
            };
            child.addListener("dom:click", td, fnClick);
//            child.addListener("dom:mouseover", td, function (e) {
//                var tab = this.parent;
//                // change if hover
//                if (tab.inHover) {
//                    tab.showChild(this.id);
//                    tab.expand();
//                }
//            });
        });
        tbody.appendChild(tr);
        table.appendChild(tbody);
        tabControl.tabContainer.appendChild(table);

        // add listener
        tabControl.addListener("dom:click", document, function(e) {
            if (this.inHover && this.expanded) {
                this.lastClickedChild = null; // dom:focus workaround (see above)
                this.collapse();
            }
        });
        
        // add tab pages
        tabControl.each(function(child) {
            tabControl.tabPageContainer.appendChild(child.dom.node);
        });
        
        if (this.firstRun) {
            // add container
            tabControl.dom.node.appendChild(tabControl.tabContainer);
            // which parent node?
            if (tabControl.expandable && tabControl.inHover) {
                tabControl.dom.hover.appendChild(tabControl.tabPageContainer);
            } else {
                tabControl.dom.node.appendChild(tabControl.tabPageContainer);
            }
            this.firstRun = false;
        }
        
        // show current tab page
        if (tabControl.currentChild) {
            tabControl.currentChild.show();
        }
    },
    
    update: function() {

        // get table cells
        var tds = this.tds;
        var tabControl = this.container;
        
        // loop children
        tabControl.each(function(child, i, childCount) {
            // exists?
            if (tds[i] !== undefined) {
                // determine class name
                var className = i < (childCount - 1) ? "oxTab" : "oxTabLast";
                // current?
                if (tabControl.expanded && child == tabControl.currentChild) {
                    className += " oxTabCurrent";
                    tds[i].className = className;
                    try {
                        tds[i].firstChild.focus();
                    }
                    catch (e) {
                        // IE might say that this element cannot be focused if it's invisible
                    }
                } else {
                    tds[i].className = className;
                }
            }
        });
    }
});
/**
 * 
 * All content on this website (including text, images, source code and any
 * other original works), unless otherwise noted, is licensed under a Creative
 * Commons License.
 * 
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 * 
 * Copyright (C) Open-Xchange Inc., 2006-2012 Mail: info@open-xchange.com
 * 
 * @author Matthias Biggeleben <matthias.biggeleben@open-xchange.com>
 * @ignore
 */

// DEVELOPMENT IN PROGRESS // SUBJECT TO PERMANENT CHANGE!

ox.gui.PanelMenu = ox.gui.Container.extend( {
/** @lends ox.gui.PanelMenu.prototype */

    getClass : function() {
        return "ox.gui.PanelMenu";
    },

    /**
     * The basic container widget
     * 
     * @constructs
     * @extends ox.gui.Container
     * @param {String} id An optional unique id for the widget.
     * @param {String} title The title of the panel menu. Important when using as TabMenu.
     */
    construct : function(id, title) {

        // call super class constructor (inherit from Container)
        this._super(this.autoId(id));
    
        // title (used for TabMenu)
        this.title = title || noI18n("");
    
        // rows
        this.rows = 3;
    
        // dynamic resizing
        this.dynamicResizing = false;
        this.hasCollapsedSections = false;
    
        // set layout manager
        this.setLayout(new ox.gui.PanelMenuLayout(this));
    
        // add widget listener
        this.addListener("widget:show", function() {
            this.layoutManager.resize();
        });
    },
    
    nodeInit: function() {
        this._super();
        this.setStyle("overflow: hidden");
    },

    /**
     * @private
     */
    propagateOpenHover: function(item) {
        // local handler?
        var p = this.parent;
        if (p.openHover !== undefined) {
            // another item open?
            if (p.openHover && p.openHover != item) {
                p.openHover.deactivate();
            }
            // set open menu item
            p.openHover = item;
        } else if ($.isFunction(p.propagateOpenHover)) {
            // a parent widget will take care
            p.propagateOpenHover(item);
        }
    },
    
    setTitle: function(title) {
        this.title = title;
        if (this.parent) this.parent.layoutManager.doLayout();
    }
});

// ------------------------------------------------------------------------------

ox.gui.PanelMenuSection = ox.gui.Container.extend( {
/** @lends ox.gui.PanelMenuSection.prototype */

    getClass : function() {
        return "ox.gui.PanelMenuSection";
    },

    /**
     * The basic container widget
     * 
     * @constructs
     * @extends ox.gui.Container
     * @param {I18nString}
     *            title The title of the menu section.
     */
    construct : function(text, id) {

        // call super class constructor (inherit from Container)
        this._super(id);

        this.text = text || noI18n("");

        // dynamic resizing
        this.collapsed = false;
        this.dropdown = null;
        this.maxWidth = 0;

        // set layout manager
        this.setLayout(new ox.gui.PanelMenuSectionLayout(this));
    },

    add : function() {
        var target = this.collapsed ? this.dropdown : this;
        ox.gui.Container.prototype.add.apply(target, arguments);
    },

    insert : function() {
        var target = this.collapsed ? this.dropdown : this;
        ox.gui.Container.prototype.insert.apply(target, arguments);
    },

    /**
     * @private
     */
    propagateOpenHover: function(item) {
        // local handler?
        var p = this.parent;
        if (p && p.openHover !== undefined) {
            // another item open?
            if (p.openHover && p.openHover != item) {
                p.openHover.deactivate();
            }
            // set open menu item
            p.openHover = item;
        } else if (p && $.isFunction(p.propagateOpenHover)) {
            // a parent widget will take care
            p.propagateOpenHover(item);
        }
    },

    /**
     * Expands the menu section (if collapsed).
     */
    expand : function() {
        if (this.collapsed) {
            this.removeChildren();
            // move children
            this.dropdown.each(function(child) {
                child.forceSmallIcon = false;
                if (child.bigIcon) {
                    child.menuStripAlign = "bottom";
                    child.statusValid = false;
                } else {
                    child.menuStripAlign = "right";
                }
                ox.gui.Container.prototype.add.call(this.parent, child);
            });
            this.dropdown.removeChildren();
            this.collapsed = false;
            this.statusValid = false;
            this.validate();
        }
    },

    /**
     * Collapses the menu section.
     */
    collapse : function() {
        if (!this.collapsed && this.hasChildren()) {
            // first icon
            var icon = this.getChildren()[0].icon;
            if (this.dropdown == null) {
                this.dropdown = new ox.gui.MenuItem(this.text || _("Dropdown"),
                        icon, true);
            }
            // move children
            // 1st: copy items with big icons
            this.each(function(child) {
                if (child.bigIcon) {
                    child.forceSmallIcon = true;
                    child.menuStripAlign = "right";
                    child.statusValid = false; // big icons only
                    this.dropdown.add(child);
                }
            });
            // 2nd: copy items with small icons
            this.each(function(child) {
                if (!child.bigIcon) {
                    child.forceSmallIcon = true;
                    child.menuStripAlign = "right";
                    this.dropdown.add(child);
                }
            });
            // remove child from section
            this.removeChildren();
            // add dropdown as new child
            ox.gui.Container.prototype.add.call(this, this.dropdown);
            this.collapsed = true;
            this.dropdown.statusValid = false;
            this.statusValid = false;
            this.validate();
        }
    }
});

// ------------------------------------------------------------------------------

ox.gui.PanelMenuLayout = ox.gui.LayoutManager.extend( {
/** @lends ox.gui.PanelMenuLayout.prototype */

    /**
     * A layout manager for the panel menu.
     * 
     * @extends ox.gui.LayoutManager
     * @constructs
     * @param {ox.gui.Container} The container widget that should be managed by the layout manager.
     */
    construct: function (container) {
        this._super(container);
        
        this.table = null;
        this.tbody = null;
        this.tfoot = null;
        
        this.maxWidth = 0;
    },

    getClass : function () {
        return "ox.gui.PanelMenuLayout";
    },

    doLayout : function () {

        // local vars
        var container = this.container;
        var showNames = this.container.parent.options.showSectionNames;
        var target = $(this.container.dom.node);
        var self = this;
        
        // create table
        if (this.table === null) { 
            this.table = $("<table/>", { border: 0, cellPadding: 0, cellSpacing: 0 })
                .addClass("panelMenu")
                .append(
                    $("<tbody/>").append(this.tbody = $("<tr/>"))
                )
                .append(
                    $("<tfoot/>").append(this.tfoot = $("<tr/>"))
                );
        }
        
        // park children
        this.park();
        
        // clear table body
        this.tbody.empty();
        
        // loop children
        container.eachVisible(function (child, i, childCount) {
            // add to DOM & show
            var td = $("<td/>");
            // set class name
            td.addClass("panelSectionContainer" + (i === childCount - 1 ? " last" : ""));
            // add child
            td.appendTo(self.tbody).append(child.dom.node);
        });
        
        // loop children (again)
        this.tfoot.empty();
        if (showNames === true) {
            container.eachVisible(function (child) {
                $("<td/>").text(expectI18n(child.text)).
                    addClass("panelMenuSectionName").
                    appendTo(self.tfoot);
            });
        }
        
        // use fixed height for empty tabs
        // add now, so that we can implement dynamic resizing
        target.css("height", "64px").append(this.table);
        
        // add listener
        container.addListener("dom:resize", window, function(e) {
            // get size
            var manager = this.layoutManager;
            manager.maxWidth = $(this.dom.node).width();
            // too large/might be too small
            this.layoutManager.resize();
        });
        
        // find topmost, invalid parent
        var current = container;
        while (current.parent && !current.parent.statusValid) {
            current = current.parent;
        }
        
        // and wait for it
        container.addListener("widget:valid", current, function(event) {
            // get size
            var manager = this.layoutManager;
            manager.maxWidth = $(this.dom.node).width();
            // get size of sections
            this.eachVisible(function(child) {
                child.maxWidth = child.dom.node.clientWidth;
            });
            // activate dynamic resizing (once all parent nodes are
            // rendered)
            this.dynamicResizing = true;
            this.layoutManager.resize();
        });
        
        // don't resize twice
        if (container != current) {
            container.layoutManager.resize();
        }
    },

    resize : function() {

        // disable resize (needs some polishing)
        return;
        
        // shortcut
        var container = this.container;
        var children = container.getVisibleChildren();
        var maxWidth = this.maxWidth;

        if (container.dynamicResizing
                && (this.table.clientWidth > maxWidth || container.hasCollapsedSections)) {
            // try to expand (from left to right)
            if (container.hasCollapsedSections
                    && this.table.clientWidth < maxWidth) {
                for ( var i = 0; i < children.length; i++) {
                    var child = children[i];
                    if (child.collapsed
                            && (this.table.clientWidth + child.maxWidth) < maxWidth) {
                        child.expand();
                    }
                }
            }
            // collapse sections (right to left)
            if (this.table.clientWidth >= maxWidth) {
                for ( var i = children.length - 1; i >= 0; i--) {
                    var child = children[i];
                    // now check size
                    if (this.table.clientWidth >= maxWidth) {
                        child.collapse();
                    } else {
                        break;
                    }
                }
            }
        }
        // collapsed sections?
        for ( var i = children.length - 1; i >= 0; i--) {
            if (children[i].collapsed) {
                container.hasCollapsedSections = true;
                break;
            }
        }
    }
});

// ------------------------------------------------------------------------------

ox.gui.PanelMenuSectionLayout = ox.gui.LayoutManager.extend( {
/** @lends ox.gui.PanelMenuLayout.prototype */

    /**
     * A layout manager for a panel menu section.
     * 
     * @extends ox.gui.LayoutManager
     * @constructs
     * @param {ox.gui.Container} The container widget that should be managed by the layout manager.
     */
    construct: function (container) {
        this._super(container);
        this.table = null;
        this.tbody = null;
    },

    getClass: function () {
        return "ox.gui.PanelMenuSectionLayout";
    },

    doLayout: function () {
        
        // create table
        if (this.table === null) {
            this.table = $("<table/>", { border: 0, cellPadding: 0, cellSpacing: 0 }).
                addClass("panelMenuSection").
                append(
                    this.tbody = $("<tbody/>")
                );
        }
        
        this.renderPartial();
        
        this.table.appendTo(this.container.dom.node);
    },

    /**
     * @private
     */
    renderPartial : function() {
    
        // park children
        var park = $("<div/>");
        this.container.each(function (child) {
            park.append(child.dom.node);
        });
        
        // clear table body
        this.tbody.empty();
        
        // get big & small icons
        var bigIcons = [], smallIcons = [];
        // loop children (big icons only)
        this.container.each(function (child) {
            if (child.bigIcon) {
                bigIcons.push(child);
            } else {
                smallIcons.push(child);
            }
        });
        
        var hasBigIcons = bigIcons.length > 0;
        var rows = 2;
        var columns = Math.ceil(smallIcons.length / rows);
        
        // create rows
        var trs = [], r = 0;
        for (; r < rows; r++) {
            trs.push($("<tr/>").appendTo(this.tbody));
        }
        
        // vars
        var i, c, r, td;
        
        // loop children (big icons)
        for (i = 0; i < bigIcons.length; i++) {
            var child = bigIcons[i];
            td = $("<td/>", { rowSpan: rows }).
                addClass("panelMenuItem large").
                append(child.dom.node);
            // append to row
            trs[0].append(td);
        }
        
        // create cells for small icons
        for (c = 0, i = 0; c < columns; c++) {
            for (r = 0; r < rows; r++, i++) {
                td = $("<td/>").addClass("panelMenuItem").css({
                    height: Math.floor(100 / rows) + "%"
                });
                // bottom?
                if (r === rows - 1) {
                    td.addClass("bottom");
                }
                trs[r].append(td);
                // small icon?
                if (smallIcons[i]) {
                    td.append(smallIcons[i].dom.node);
                } else {
                    td.html("&nbsp;");
                }
            }
        }
        
//        park.empty();
//        park = null;
    }
});

// ------------------------------------------------------------------------------

ox.gui.TabMenu = ox.gui.TabControl.extend( {
/** @lends ox.gui.TabMenu.prototype */

    getClass : function() {
        return "ox.gui.TabMenu";
    },

    /**
     * A TabMenu container. A combination of TabControl and PanelMenu.
     * 
     * @constructs
     * @extends ox.gui.TabControl
     * @param {String} id An optional unique id for the widget.
     */
    construct : function(options) {

        // defaults
        this.options = $.extend( {
            id : undefined,
            action : $.noop,
            showSectionNames : true,
            inHover: false
        }, options || {});
    
        // call super class constructor
        this._super(this.options.id);
    
        // set layout manager (for tabs; not pages)
        this.setLayout(new ox.gui.TabMenuLayout(this));
    
        // expand/hover
        this.expanded = true;
        this.expandable = true;
        this.inHover = this.options.inHover;
    
        // required to work with ox.gui.menuItem
        this.openHover = null;
        
        // apply "inHover"
        if (this.options.inHover) {
            // mark as collapsed
            this.expanded = false;
            // move container
            $(this.tabPageContainer).
                addClass("oxTabControlPageContainerInHover").
                appendTo(this.dom.hover);
        }
    }
});

ox.gui.TabMenuLayout = ox.gui.TabControlLayout.extend({
/** @lends ox.gui.TabMenuLayout.prototype */
    
    getClass: function() { return "ox.gui.TabMenuLayout"; }
    
});

/**
 * 
 * All content on this website (including text, images, source
 * code and any other original works), unless otherwise noted,
 * is licensed under a Creative Commons License.
 * 
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 * 
 * Copyright (C) Open-Xchange Inc., 2006-2012
 * Mail: info@open-xchange.com 
 * 
 * @author Matthias Biggeleben <matthias.biggeleben@open-xchange.com>
 * 
 */

ox.gui.morph = function () {

    // widget pool
    ox.widgets = {};
    
    // set topmost layout manager
    ox.desktop.setLayout(new ox.gui.ZeroLayout());
    
    // move sidepanel's children
    var sidenode = $("#sidesplit");
    
    // init
    ox.gui.initController();
    ox.gui.initToppanel();
    ox.gui.initSidepanel(sidenode);
    ox.gui.initFolderTree();
    ox.gui.initConfigTree();
    // toolbar
    if (true) {
        // init toolbar controller
        var $p = ox.gui.initToolBar(); // returns some shared private functions/vars.
        ox.gui.initToolBarCore(); // init the core toolbar, depends on initToolBar 
        ox.gui.initToolBarPortal($p);
        ox.gui.initToolBarMail($p);
        ox.gui.initToolBarCalendar($p);
        ox.gui.initToolBarContacts($p);
        ox.gui.initToolBarTasks($p);
        ox.gui.initToolBarInfoStore($p);
        ox.gui.initToolBarConfig($p);
        ox.gui.initToolBarMessaging($p);
        ox.gui.initToolBarPermissions($p);
        
        $p.postInit();
    }
    
    // i18n
    register("LanguageChanged", function () {
        // event forward
        ox.desktop.triggerGlobalEvent("LanguageChanged");
    });
};
/**
 * 
 * All content on this website (including text, images, source
 * code and any other original works), unless otherwise noted,
 * is licensed under a Creative Commons License.
 * 
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 * 
 * Copyright (C) Open-Xchange Inc., 2006-2012
 * Mail: info@open-xchange.com 
 * 
 * @author Stefan Preuss <stefan.preuss@open-xchange.com>
 * 
 */

ox.gui.initToppanel = function () {
        
    /*
     * Module Icons
     */
    //var today = new Date(now()).getUTCDate();
    var modules = ox.api.ui.listModules();
    var moduleNodes = {}, selectedModule;
    
    $("#header").css({
        height: "30px",
        backgroundImage: "url('" + getFullImgSrc("img/toolbar/toolbar30.png") + "')"
    }).empty();

    
    $("#notheader").css({
        top: "30px"
    });
    
    var topPanel = ox.widgets.toppanel = new ox.gui.Container("ox-top").
        addTo(ox.desktop).
        setParentDOMNode($("#header")[0]).
        css({
            position: "absolute", right: "18px", 
            bottom: "0px", left: "0px", top: "0px"
        }).
        setLayout(new ox.gui.BorderLayout());
    
    ox.widgets.toppanel.hide = function() {
        this.setVisible(false);
        jQuery("#header").hide();
        jQuery("#notheader").css({ top: "0px" });
    };
    
    ox.widgets.toppanel.show = function() {
        this.setVisible(true);
        jQuery("#notheader").css({ top: "30px" });
        jQuery("#header").show();
    };
    
    var moduleContainer = ox.widgets.moduleNavigation = new ox.gui.Container("ox-modules").
        addTo(topPanel).
        setLayout(new ox.gui.ZeroLayout()).
        setLayoutParam({ position: "center" });
   
    
    var changeIconState = ox.UIController.changeIconState = function (name) {
        
        // visual selection
        var select = function (name) {
            $(moduleNodes[name].dom.node).
                css({
                    borderLeft: "1px inset silver",
                    borderRight: "1px inset silver"
                }).
                addClass("background-color-PMG-selection-elements");
        };
        
        var deselect = function (name) {
         // fixed: changed from null to "" due to ie problem
            $(moduleNodes[name].dom.node).
                css({ border: "" }).removeClass("background-color-PMG-selection-elements");
        };
        
        // if module selection hasn't change, return!
        if (selectedModule === name) {
            return;
        }
        
        // remove styles on all nodes first except for expert tree (if active)
        for (var i in moduleNodes) {
            deselect(i);
        }
        
        // update module selection
        if (ox.api.ui.moduleExists(name)) {
            // set selection
            select(name);
            selectedModule = name;
        }
    };
    
    // TODO rename to something not global or move into ox.UIController.
    expertSelection = new ox.gui.Custom(function() {
        var input = this.input = $("<input/>").css({ "float": "left", border: "0px none" }).
        attr({ type: "checkbox", id: "expert-tree" }).
        bind("click", function(event) {
            var flag = $(this).is(":checked");
            track({
                type: "click",
                dest: "expert_mode",
                data: { expert: flag },
                text: flag ? "Change to expert mode" : "Change to non-expert mode"
            });
            ox.UIController.setExpertTree(flag);
        });
        
        $(this.dom.node).
        append(input).
        append($("<div/>").css({ "float": "left" }).
                append($("<label/>").attr({ "for": "expert-tree" }).
                        append(addTranslated(_("Expert mode"))))
        );
        // initial state
        if (ox.UIController.showExpertTree) {
            input.attr("checked", "checked");
        }
    });
    
    
    ox.UIController.dispatcher.bind("expertchanged", function (e) {
        // change status only if != current status
        if (expertSelection && expertSelection.input && 
                expertSelection.input.attr("checked") != e.enabled) {
            expertSelection.input.attr("checked", e.enabled);
        }
    });
    
    ox.UIController.bind("modulechange", function (e) {
        changeIconState(e.module === "messaging" ? "mail" : e.module);
    });
    
    // loop
    var i = 0, $l = modules.length;
    var factory = function (module) {
        
        // set icons if not already done
        if (module.icons === undefined) {
            module.icons = ["icons/24/mod_" + module.name + ".png"];
        }
        
        // special handling for calendar
        if (module.name === "calendar") {
            module.icons[0] = "img/calendar/dayicons/mod_calendar-" + (new Date(now())).getUTCDate() + ".png";
        }
        
        var img = new ox.gui.Image().
        	addCSSClass("ox-top-module-icons top-icon-" + module.name).
            setSrc(getFullImgSrc(module.icons[0])).
            setTitle(module.title);
                
        // module activated?
        if (module.disabled) {
            
            // show for upsell?
            if (ox.upsell.isVisible(module.name)) {
                
                // disable image
                img.disable();
                
                // append lock icon
                jQuery(img.dom.node).append(
                      jQuery("<div>").addClass("top-icon-disabled")
                      .css({ 
                          backgroundImage: "url(" + getFullImgSrc("icons/16/lock.png") + ")",
                          position: "absolute",
                          width: "16px",
                          height: "16px",
                          bottom: "2px",
                          right: "0px"
                      })
                );
                
            	// trigger upsell if feature is disabled
            	img.addListener("click", function (e) {
                    triggerEvent("Feature_Not_Available", "modules/" + module.name, window);
            	});
            	
            	// add image to container
                moduleContainer.add(img);
                moduleNodes[module.name] = img;
            }
        	
        } else {
            
        	// bind click to icon
            img.css({ cursor: "pointer" }).
            addListener("click", function (e) {
                
                track({
                    type: "click",
                    dest: "module_bar",
                    data: { module: module.name, folder: "auto" },
                    text: "Change to module " + module.name
                });
                
                ox.UIController.setModule({ module: module.name, folder: "auto" });
            });
            
            // add image to container
            moduleContainer.add(img);
            moduleNodes[module.name] = img;
        }
    };
    
    for (; i < $l; i++) {
        factory(modules[i]);
    }

    // adding checkbox to toggle expert
    if (ox.api.config.get("ui.global.expert.mode.configurable", true) === true) {
        moduleContainer.add(
                expertSelection.
                setLayoutParam({ position: "center" }).
                setCSSClass("top-icon-expert-selection").
                css({ margin: "5px 0 0 20px", "float": "left" })
        );
    }

    //---- Logout button

    new ox.gui.Button("ox-topbar-logout", _("Logout"), function() {
        triggerEvent("Logout");
    }).addTo(topPanel).
    setLayoutParam({ position: "right", width: 120, valign: "top", align: "right" }).
    css({ position: "absolute", top: "5px", right: "0px" });
    
    //----- search bar
    ox.widgets.searchField = {
        active: function() {
            if (this[ox.api.ui.getActiveModule()]) {
                return this[ox.api.ui.getActiveModule()];
            } else {
                return this.top;
            }
        }
    };
    
    ox.widgets.searchField.top = new ox.gui.SearchField("ox-topbar-search", { disabled: true }).
    addTo(topPanel).
    setLayoutParam({ position: "right", width: "250px" }).
    css({ position: "absolute", top: "5px", bottom: "4px", left: "0px", right: "0px" }).
    addListener("widget:search", function (e) {
        // vars
        var module = ox.UIController.getModule(),
            id = "",
            tmp = this.searchMenu.checked;
            trigger = function () {
                triggerEvent("OX_New_Search", ox.api.ui.getActiveModule(), e.data.value);
            };
        // module with criteria?
        if (module === "mail" || module === "contacts") {
            // any criteria selected?
            for (id in tmp) {
                if (tmp[id] === true) {
                    // trigger search
                    trigger();
                    break;
                }
            }
        } else {
            // trigger search for module without criteria
            trigger();
        }
    });
    
    
    //----- buttons
    
    var buttons = new ox.gui.Container("ox-buttons").
    addTo(topPanel).
    setLayout(new ox.gui.HBoxLayout()).
    setLayoutParam({ position: "right", width: "80px" }).
    css({ position: "absolute", top: "3px", right: "10px" });
    
    
    var loading = new ox.gui.Image().
    addCSSClass("ox-top-module-buttons").
    css({ cursor: "pointer" }).
    setLayoutParam({ width: "20px", align: "right" }).
    setSrc(getFullImgSrc("img/toolbar/tb_loading.gif")).
    setTitle(_("Refresh")).
    addListener("click", function (e) {
        track({
            type: "click",
            dest: "refresh_button",
            text: "User clicked on the refresh button"
        });
        triggerEvent("OX_Refresh");
    });
    
    buttons.add(loading);
    
    setLoading = function(enabled) {
        loading.setSrc(getFullImgSrc(enabled ? 
                "img/toolbar/ox_animated.gif" : "img/toolbar/tb_loading.gif"));
        loading.setTitle(enabled ? _("Loading...") : _("Refresh"));
    };
    
    var cursor = null;
    setLoadingCursor = function(enabled) {
        if (IE) return; // Changing the cursor triggers mouseup in IE
        if (enabled) {
            if (!cursor){
                cursor = setMouseCursor("wait");
            }
        } else if (cursor) {
            removeMouseCursor(cursor);
            cursor = null;
        }
    };
    
    // Listen to legacy event.
    register("Loading", function(isLoading) {
        setLoading(enabled);
        setLoadingCursor(enabled);
    });
    
    var help = new ox.gui.Image().
    addCSSClass("ox-top-module-buttons").
    css({ cursor: "pointer" }).
    setLayoutParam({ width: "20px", align: "right" }).
    setSrc(getFullImgSrc("img/toolbar/help.gif")).
    setTitle(_("Help"));

    buttons.add(help);

    var _faq_guid = null;
    var faq = new MenuItem(_("FAQ"),
            function() {
                var uri = ox.api.config.get("ui.global.help.faq_path");
                var win = window.open(uri, "faq");
                win.focus();
            }
    );
    faq.setVisible(ox.api.config.contains("ui.global.help.faq_path"));
    
    var menu = globalContextMenus.help = new PulldownMenu(help.dom.node);
    menu.addItem(new MenuItem(_("Help"),
        function() { triggerEvent("OX_Show_Help", help_location); }));
    menu.addItem(faq);
    menu.addItem(new MenuItem(_("Error messages"),
        function() { ox.UINotifier.showReport(); }));
    menu.addItem(new MenuItem(_("About"),
        function() { triggerEvent("OX_Show_About"); }));

};
/**
 *
 * All content on this website (including text, images, source
 * code and any other original works), unless otherwise noted,
 * is licensed under a Creative Commons License.
 *
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 *
 * Copyright (C) Open-Xchange Inc., 2006-2012
 * Mail: info@open-xchange.com
 *
 * @author Matthias Biggeleben <matthias.biggeleben@open-xchange.com>
 *
 */

ox.gui.initSidepanel = function (sidenode) {

    var newFolderIcon = null;

    // prepare parent node
    $("<div/>", { id: "ox-sidesplit" }).
    addClass("oxStretch").
    append($("<div/>").
            css({ position: "absolute", top: "24px", right: "-12px",
                width: "12px", height: "24px" }).addClass("sp-toggle").
                append(
                        $("<img/>").attr("src", getFullImgSrc("img/arrows/arrow_double_gray_right.gif")).
                        css({ marginTop: "8px"})
                ).click(function() {
                    ox.widgets.sidepanel.toggle();
                    delete ox.widgets.sidepanel.userToggle;
                }).hide()
    ).appendTo(sidenode[0]);

    // create new sidepanel container
    var sidepanel = ox.widgets.sidepanel = new ox.gui.Container("ox-sidepanel-container").
        setLayout(new ox.gui.BorderLayout()).
        css({ background: "white" }).
        addTo(ox.desktop).
        setParentDOMNode(sidenode[0]);

    // initial sidebar width
    sidepanel.oldWidth = 229; // 19.1em

    // function to trigger sidebar toggle
    $("#sidepanel_dragpoint_background").mousedown(function(e) {
        return false;
    });
    sidepanel.toggle = function(state) {
        // set toggle state
        if (state !== undefined) {
            if (state === this.toggled) {
                return; // nothing changed, bye
            }
            this.toggled = state;
        } else {
            this.toggled = !this.toggled;
        }

        $("div.sp-toggle").css({ display: this.toggled ? "" : "none"});
        $(".sp-toggle-img img").css({ display: this.toggled ? "none" : ""});

        // hide sidepanel drag bar
        $("#sidepanel_dragpoint").css({ display: this.toggled ? "none" : ""});

        // hide newFolderIcon when avialable and toggled (nice-to-have :-))
        if (newFolderIcon !== null) {
            newFolderIcon.setVisible(!this.toggled);
        }

        // remember old width of the sidebar before toggle
        if (this.toggled) {
            this.oldWidth = $("#sidesplit").outerWidth();
        }

        // resize split
        resizeSplit("sidesplit", this.toggled ? "3px" : this.oldWidth + "px");

        // trigger global event
        triggerEvent("OX_Sidepanel_Collapse", this.toggled);
    };

    // hide when landing page is portal
    if ((ox.api.config.get("gui.global.landing_page.module") || "portal") === "portal") {
    	sidepanel.toggle(true);
    }

    // view
    var view = ox.widgets.sidepanelViewControl = new ox.gui.ViewControl("ox-side-view").
        setLayoutParam({ position: "center" }).
        css({ background: "white" }).
        setCSSClass("oxStretch");

    // register for resize
    resizeEvents.register("Resized", function () {
        sidepanel.resize();
    });

    // view hash
    var v = ox.widgets.sidepanelViews = {};

    var meta = {
        "portal": {
            title: _("Portal")
        },
        "mail": {
            title: _("E-Mail")
        },
        "calendar": {
            title: _("Calendar"),
            module: "calendar",
            icon: getFullImgSrc("icons/16/calendar.png"),
            sections: {
                "private": _("Private calendars"),
                "shared": _("Shared calendars"),
                "public": _("Public calendars")
            }
        },
        "contacts": {
            title: _("Contacts"),
            module: "contacts",
            icon: getFullImgSrc("icons/16/contacts.png"),
            sections: {
                "private": _("Private contacts"),
                "shared": _("Shared contacts"),
                "public": _("Public contacts")
            }
        },
        "tasks": {
            title: _("Tasks"),
            module: "tasks",
            icon: getFullImgSrc("icons/16/tasks.png"),
            sections: {
                "private": _("Private tasks"),
                "shared": _("Shared tasks"),
                "public": _("Public tasks")
            }
        },
        "infostore": {
            title: _("Infostore"),
            sections: {
                "private": _("Private folders"),
                "all": _("All folders")
            }
        },
        "configuration": {
            title: _("Configuration")
        },
        "foldertree": {
            title: _("Folders")
        }
    };

    // mixin
    ox.gui.util = {};

    ox.gui.util.createFolder = function (options) {

        // options
        var opt = $.extend({
            widget: null,
            success: $.noop,
            folder: 0,
            module: ox.UIController.getModule(),
            data: {}
        }, options);

        // get free title for new folder
        ox.api.folder.getFreeTitle(opt.folder, function (title) {

            // default data
            var defaultData = {
                folder_id: opt.folder,
                id: "",
                module: opt.module,
                own_rights: 1883275524,
                permissions: [],
                standard_folder: 0,
                subscribed: true,
                subfolders: false,
                title: title,
                total: 0,
                unread: 0
            };

            if (opt.module === "mail") {
                defaultData.id = opt.folder + "/" + defaultData.title;
                defaultData.subscr_subflds = true;
                defaultData.tmpFolder = true;
            } else {
                defaultData.id = "-1";
            }

            // default folder data
            opt.data = $.extend(defaultData, opt.data);

            // mark folder as special
            opt.data.$ = { creating: true };

            // get tree & expert mode
            var tree = ox.api.config.get("modules.folder.tree", 0),
                expert = ox.UIController.showExpertTree,
                cache, cleanup, removeFromCache, cont, ok, abort,
                widget = opt.widget,
                parent, revert = false;

            // handlers
            cleanup = function () {
                // re-enable selection
                widget.selection.enable();
                // bind
                ox.api.folder.dispatcher.bind("modify", widget.onModify);
                // avoid leaks
                widget = opt.widget = opt.data = null;
                opt = null;
            };

            ok = function (title) {
                // remove from cache
                removeFromCache();
                // bind
                ox.api.folder.dispatcher.bind("modify", widget.onModify);
                // create folder
                ox.api.folder.create({
                    folder: opt.folder,
                    data: {
                        title: title,
                        module: opt.data.module
                    },
                    success: function (data) {
                        // setting a selection here will not work!
                        // tree repaint is required, so doing this later
                        cleanup();
                    },
                    error: abort
                });
            };
            
            abort = function () {
                removeFromCache();
                if (revert) {
                    parent.subfolders = false;
                    cache = ox.api.cache["folder" + tree];
                    cache.setComplete(opt.folder, false);
                }
                widget.repaint(function () {
                    widget.selection.clickQuiet(opt.folder);
                    cleanup();
                });
            };

            cont = function () {
                // widget can be simple view or tree view, so look for setOpen
                if (widget.setOpen) {
                    widget.setOpen([opt.folder]);
                }
                // repaint
                widget.repaint(function () {
                    setTimeout(function () {
                        widget.selection.clickQuiet(opt.data.id);
                        widget.startEdit(opt.data.id, ok, abort);
                    }, 10);
                });
            };

            if (widget !== null) {
                // reload/load parent's subfolders
                ox.api.folder.getSubFolders({
                    folder: opt.folder,
                    success: function () {
                        // unbind
                        ox.api.folder.dispatcher.unbind("modify", widget.onModify);
                        // get cache
                        cache = ox.api.cache["folder" + tree];
                        // parent folder now has subfolders
                        if ((parent = cache.get(opt.folder))) {
                            if (parent.subfolders === false) {
                                parent.subfolders = true;
                                cache.setComplete(opt.folder, true);
                                revert = true;
                            }
                        }

                        // get proper cache
                        if (opt.module === "mail" || opt.module === "infostore" || expert === true) {
                            // prepend to folder cache
                            cache.prepend(opt.data);
                            removeFromCache = function () {
                                cache.remove(opt.data.id);
                            };
                            cont();
                        } else {
                            // insert into flat folder cache
                            var flatCache = ox.api.cache.folderFlat.get(opt.module);
                            if (flatCache) {
                                (function () {
                                    var section, id, i, $i, item;
                                    for (id in flatCache) {
                                        section = flatCache[id];
                                        for (i = 0, $i = section.length; i < $i; i++) {
                                            item = section[i];
                                            if (item.id === opt.folder) {
                                                section.splice(i + 1, 0, opt.data);
                                                removeFromCache = function () {
                                                    var pos = $.inArray(opt.data, section);
                                                    section.splice(pos, 1);
                                                };
                                                cont();
                                                return;
                                            }
                                        }
                                    }
                                }());
                            }
                        }
                    }
                });
            }
        });
    };

    ox.gui.util.inlineEdit = function (node, value, success, abort, selector) {
        // disable selection?
        if (this.tree && this.tree.selection) {
            this.tree.selection.disable();
        } else if (this.selection) {
            this.selection.disable();
        }
        // this
        var self = this,
            innerNode,
            // input field
            input = $("<input/>", { type: "text", value: "" }).
                bind("keydown", function (e) {
                    e.stopPropagation();
                }).
                bind("selectstart", function (e) {
                    e.stopPropagation();
                }),
            // handler
            unbind = function (value) {
                $("body").unbind("mousedown", fnOk);
                input.unbind("blur keydown selectstart mousedown").remove();
                innerNode.text(value || ""); // bug #20138
                input = innerNode = null;
            },
            fnAbort = function (e) {
                unbind();
                ox.util.call(abort);
            },
            fnOk = function (e) {
                var value = input.val();
                // empty?
                if ($.trim(value) !== "") {
                    unbind(value);
                    ox.util.call(success, value);
                } else {
                    // empty value is same as abort
                    fnAbort(e);
                }
            };
        // replace node content
        input.
            css({
                width: "100%",
                border: "1px solid #555",
                borderWidth: "1px 0px 1px 0px",
                background: "#eee"
            }).
            val(value || "").
            bind("mousedown", function () {
                return false;
            }).
            bind("keydown", function (e) {
                switch (e.which) {
                case 13: // enter
                    fnOk(e);
                    return false;
                case 27: // escape
                    fnAbort(e);
                    return false;
                }
            }
        );
        // bind ok
        input.bind("blur", fnOk);
        $("body").bind("mousedown", fnOk);
        // add to node
        innerNode = selector !== undefined ? $(selector, node) : $(node);
        innerNode.empty().append(input);
        // focus input field
        input.focus();
        input.select();
    };

    var addRootFolder = configGetKey("modules.folder.tree") == 1;
    // Icon for adding new folders to the root level of the advanced tree when tree=1
    if (addRootFolder) {
        newFolderIcon = new ox.gui.Image().
        setSrc(getFullImgSrc("img/folder/newfolder.gif")).setTitle(_("New folder")).
        css({ verticalAlign: "middle", cursor: "pointer", marginLeft: "5px", marginTop: "4px" }).
        setVisible(false).
        setLayoutParam({ position: "left", width: 4 });
        if (newFolderIcon.image) {
            // adjust width and height for IE
            // todo: needs to be done in stylesheet
            jQuery(newFolderIcon.image).css({ width: "16px", height: "16px" });
        }

        // enable only when expertview is on
        ox.UIController.dispatcher.bind("modulechange expertchanged", function (e) {
            var enabled = (ox.UIController.showExpertTree === true || ox.UIController.getModule() === "mail");
            newFolderIcon.setVisible(enabled);
            newFolderIcon.layout.width = enabled ? "30px" : "4px"; // change width
            ox.widgets.sidepanel.title.paint(); // trigger paint
        });
    }

    // flat view
    (function () {

        /*
         * Context Menu
         * ------------------------------------------------------------------------------------
         */

        if (addRootFolder) {
            var rootMenu = globalContextMenus.newFolderTypeExpert = new PulldownMenu(newFolderIcon.dom.node);
            rootMenu.getContext = constant({
                data: { id: 1 },
                widget: ox.widgets.sidepanelViews.folderTree
            });
        }

        globalContextMenus.newFolderType = new PopupMenu();

        var PIM = { calendar: 1, tasks: 1, contacts: 1, system: 1 };
        var mail = { mail: 1 };
        if (addRootFolder) {
            PIM["mail"] = 1;
            mail = PIM;
        }
        var items = {
            mail:      { name: _("E-Mail"),    when: mail },
            calendar:  { name: _("Calendar"),  when: PIM },
            tasks:     { name: _("Tasks"),     when: PIM },
            contacts:  { name: _("Contacts"),  when: PIM },
            infostore: { name: _("InfoStore"), when: { infostore: 1 } }
        };

        for (var i in items) {
            // enabled or at least visible for upsell?
            if (ox.upsell.isVisible(i) && ox.upsell.isVisible(i + "-new-folder")) {

                (function (type) {

                    function createHandler(context) {
                        // Disable the default repaint and manually trigger it later.
                        // This allows to specify a callback for the repaint.
                        var widget = context.widget !== undefined ? context.widget : ox.widgets.sidepanelViews.folderTree;
                        // non-expert and module mail needs to be another widget
                        if (type === "mail" && ox.UIController.showExpertTree === false
                            && ox.api.config.get("modules.folder.tree", 0) === 1) {
                            widget = ox.widgets.sidepanelViews.mailTree;
                        }
                        // create
                        ox.gui.util.createFolder({
                            widget: widget,
                            module: type,
                            folder: context.data.id
                        });
                    }
                    var def = items[type];
                    var key = "modules." + type + ".module";
                    var item = new MenuItem(def.name, createHandler);
                    item.onShow = function () {
                        // get context
                        var ctx = this.getContext();
                        // get module
                        if (ox.UIController.showExpertTree) {
                            // show?
                            if (ctx.data && ctx.data.id === "2" && type === "mail") {
                                this.setEnabled(false); // no mail in public folders
                            } else {
                                this.setEnabled((ctx.type in def.when) && configGetKey(key)); // all
                            }
                        } else {
                            // exclude unified mail (subfolders too)
                            if (ox.api.folder.is("unifiedmail", ctx.data)) {
                                this.setEnabled(false);
                            } else {
                                // allow current type only
                                this.setEnabled(type === ox.UIController.getModule());
                            }
                        }
                    };
                    globalContextMenus.newFolderType.addItem(item);
                    if (addRootFolder) {
                        item = new MenuItem(def.name, createHandler);
                        item.onShow = function() {
                            if (ox.UIController.showExpertTree) {
                                this.setEnabled(("system" in def.when) && configGetKey(key));
                            } else {
                                this.setEnabled(type === "mail");
                            }
                        };
                        rootMenu.addItem(item);
                    }
                })(i);
            }
        }

        var menu = new ContextMenu();

        // create global refs
        globalContextMenus.simpleFolderMenu = menu;

        // new folder
        var new_folder = new MenuItem(_("New folder"), globalContextMenus.newFolderType);
        new_folder.setIcon("img/folder/newfolder.gif", "img/folder/newfolder_d.gif");
        new_folder.onShow = function () {
        	var ctx = this.getContext(),
        		canCreate = false;
        	// check for the create folder permission
        	if (ctx.data && ctx.data.id) {
            	canCreate = ox.api.folder.can("create", 
            			ox.api.folder.get({ folder: ctx.data.id }));
        	}
        	
            // check if creating folder is allowed for current module
            var module = ox.UIController.getModule();
            this.setEnabled(ox.upsell.isVisible(module + "-new-folder") 
            		&& canCreate);
        };
        menu.addItem(new_folder);

        // send as link
        var link = new MenuItem(_("Send as link"), function (context) {
            // get folder first
            ox.api.folder.get({
                folder: context.data.id,
                success: function (data) {
                    // use global fn
                    infostore_sendLink([{ data: {
                        folder_id: data.id,
                        module: data.module
                    }}]);
                }
            });
        });
        menu.addItem(link);

        // subscribe
        var subscribe = new MenuItem(_("Subscribe folder"), function (context) {
            SubscribeFolder.openWindow({ selected: ox.api.account.derive(context.data.id, true) });
        });
        menu.addItem(subscribe);

        // "Extended" (submenu)
        var extended = globalContextMenus.extended_folder_menu = new PopupMenu();
            // Import
            var import_data = new MenuItem(_("Import"), function (context) {
                ox.api.folder.get({
                    folder: context.data.id,
                    success: function (data) {
                        // set global var
                        importFolderDestination = data;
                        triggerEvent("OX_Switch_View", "configuration/import");
                    }
                });
            });
            extended.addItem(import_data);
            // Export (Submenu)
            var export_submenu = new PopupMenu();
                var export_submenu_ical = new MenuItem(_("iCalendar"),
                    function (ctx) {
                        ox.api.folder.exportContacts(ctx.data.id, "ICAL");
                    });
                export_submenu.addItem(export_submenu_ical);
                var export_submenu_vcard = new MenuItem(_("vCard"),
                    function (ctx) {
                        ox.api.folder.exportContacts(ctx.data.id, "VCARD");
                    });
                export_submenu.addItem(export_submenu_vcard);
                var export_submenu_csv = new MenuItem(_("CSV"), function (ctx) {
                    ox.api.folder.exportContacts(ctx.data.id, "CSV");
                });
                export_submenu.addItem(export_submenu_csv);
                var export_contact = new MenuItem(_("Export"), export_submenu);
                extended.addItem(export_contact);
        // add to menu
        menu.addItem( new MenuItem(_("Extended"), extended));

        // separator
        menu.addItem(new MenuSeparator());

        // delete
        var del = new MenuItem(_("Delete"), function (context) {
            removeFolder(context.data.id);
        });
        del.setIcon("img/menu/delete_folder.gif", "img/menu/delete_folder_d.gif");
        menu.addItem(del);

        // rename
        var rename = new MenuItem(_("Rename"), function (context) {
            // start edit
            context.widget.startEdit(context.data.id);
        });
        rename.setIcon("img/folder/renamefolder.gif", "img/folder/renamefolder_d.gif");
        menu.addItem(rename);

        var cut = new MenuItem(_("Cut"), function (context) {
            // cut node
            context.widget.cut(context.data.id);
        });
        cut.setIcon("img/folder/cutfolder.gif", "img/folder/cutfolder_d.gif");
        menu.addItem(cut);

        var paste = new MenuItem(_("Paste"), function (context) {
            ox.api.folder.get({
                folder: context.data.id,
                success: function(target) {
                    ox.api.folder.move(context.widget.cutNode.data, target);
                    context.widget.cut();
                }
            });
        });
        paste.setIcon("img/folder/pastefolder.gif", "img/folder/pastefolder_d.gif");
        menu.addItem(paste);

        var folder_read = new MenuItem(_("Mark folder read"), function (context) {
            // mark every mail as read
            triggerEvent("OX_Folder_Mark_Read", context.data.id);
        });
        folder_read.setIcon("icons/16/mark_as.png", "icons/16/mark_as.png");
        menu.addItem(folder_read);

        var empty = new MenuItem(_("Empty folder"), function (context) {
            // clear folder
            var text = "Are you sure you want to delete all E-Mails from this folder? The deleted E-Mails will be moved to the Trash folder."; /*i18n*/
            // permanently delete message or is folder the trash folder?
            var accounts = ox.api.account.all(), trash = false;
            for (var i=0; i < accounts.length; i++) {
                if (accounts[i].trash_fullname === context.data.id) {
                    trash = true;
                    break;
                }
            }
            if (configGetKey("mail.deletemail") || trash === true) {
                text = "Are you sure you want to permanently delete all E-Mails from this folder? The deleted E-Mails will be irrevocably lost."; /*i18n*/
            }
            accounts = trash = null;
            function cbok () {
                // select folder to clear to avoid a removed subfolder still being selected
                ox.UIController.setFolder({
                    folder: context.data.id,
                    module: "default",
                    success: function () {
                        // clear folder now
                        ox.api.folder.clear(context.data.id, function () {
                            // update folder / sub folders
                            ox.api.http.pause();
                            ox.api.folder.get({
                                folder: context.data.id,
                                cache: false
                            });
                            ox.api.folder.getSubFolders({
                                folder: context.data.id,
                                cache: false
                            });
                            ox.api.folder.get({
                                folder: configGetKey("modules.mail.defaultFolder.trash"),
                                cache: false
                            });
                            ox.api.http.resume(function () {
                                // trigger
                                triggerEvent("OX_Folder_Cleared", context.data.id);
                                // trigger modify
                                ox.api.folder.dispatcher.trigger("modify");
                            });
                        });
                    }
                });
            }
            newConfirm(_("Empty folder"),_(text),AlertPopup.YESNO,null,null,cbok,null,null);

        });
        empty.setIcon("img/folder/empty_folder.gif", "img/folder/propertiesfolder_d.gif");
        menu.addItem(empty);

        // separator
        menu.addItem(new MenuSeparator());

        var props = new MenuItem(_("Properties"), function (context) {
            ox.api.folder.viewProperties(context.data.id, false);
        });
        props.setIcon("img/folder/propertiesfolder.gif", "img/folder/propertiesfolder_d.gif");
        menu.addItem(props);

        menu.onShow = function (context) {
            // get context
            var id = context.data.id, api = ox.api.folder;
            // get folder
            api.get({
                folder: id,
                success: function (folder) {

                    // new folder
                    new_folder.setEnabled((folder.own_rights & 127) >= 4);

                    // module system and name default is the parent root mail folder
                    context.type = api.is("account", folder) ? "mail": folder.module;

                    // subscribe
                    subscribe.setEnabled(api.can("subscribe", folder));
                    subscribe.setVisible(api.is("mail", folder));

                    // send link (anything but system and mail)
                    link.setEnabled(folder.own_rights & (127 << 7) && folder.module != "mail" && ("infostore_sendLink" in window));
                    link.setVisible(!api.is("system", folder) && context.type !== "mail");

                    // delete
                    del.setEnabled(api.can("delete", folder));

                    // rename
                    rename.setEnabled(api.can("rename", folder));

                    cut.setVisible(context.widget.cut !== undefined);
                    if (cut.visible) {
                        cut.setEnabled(del.enabled &&
                            (!context.widget.cutNode ||
                                context.widget.cutNode.id != id));
                    }

                    paste.setVisible(context.widget.cut !== undefined);
                    if (paste.visible) {
                        paste.disable();
                        if (context.widget.cutNode) {
                            api.canMove(context.widget.cutNode.data, folder,
                                function() { paste.enable(); },
                                function() { paste.disable(); });
                        }
                    }

                    // empty
                    empty.setVisible(api.can("empty", folder));

                    // mark folder read
                    folder_read.setVisible(api.is("mail", folder));

                    // import
                    import_data.setEnabled(api.can("import", folder));

                    // export
                    export_contact.setEnabled(api.can("export", folder));
                    export_submenu_ical.setVisible(api.is("calendar|tasks", folder));
                    export_submenu_vcard.setVisible(api.is("contacts", folder));
                    export_submenu_csv.setVisible(api.is("contacts", folder));
                }
            });
        };

        /*
         * Tree views
         * ------------------------------------------------------------------------------------
         */
        ox.gui.TypedFolderTree = ox.gui.Tree.extend({

            construct: function (id, options) {

                // super constructor
                ox.gui.Tree.prototype.construct.call(this, id);

                // options
                this.options = $.extend({
                    type: "mail",
                    icon: "mail.png",
                    root: "1",
                    tree: ox.api.config.get("modules.folder.tree", "0"),
                    showRootNode: false,
                    loadRootNode: false
                }, options || {});

                this.css({ padding: "10px 0px 10px 0px" });

                this.firstPaint = true;

                // show root node
                this.showRootNode = this.options.showRootNode;
                this.loadRootNode = this.options.loadRootNode;
                this.loadRootNodeId = this.options.root;

                var TYPE = this.options.type;
                var self = this;

                // node parser
                this.setNodeParser(function (data) {
                    // root node fix
                    if (data) { // fix to prevent error on console TODO fix it real
                        var parent = data.id === self.options.root ? null : data.folder_id;
                    }

                    return {
                        "id": data.id, "parent": parent, "name": data.title,
                        "type": data.module, "hasChildren": data.subfolders
                    };
                });

                this.addListener("widget:select", function(e) {
                    var api = ox.api.folder;
                    // get folder data
                    api.get({
                        folder: e.data,
                        success: function (data) {
                            // can read?
                            if (api.can("read", data)) {
                                // mail?
                                if (TYPE === "mail") {
                                    if (api.is("mail", data)) {
                                        // change folder
                                        ox.UIController.setFolder({ folder: data.id, module: "mail" });
                                    } else if (api.is("messaging", data)) {
                                        // change folder AND module
                                        ox.UIController.setFolder({ folder: data.id, module: "messaging" });
                                    }
                                } else {
                                    // change folder
                                    ox.UIController.setFolder({ folder: data.id });
                                }
                            }
                        }
                    });
                });

                this.selection.onRename(function (id, item) {
                    // get folder
                    ox.api.folder.get({
                        folder: id,
                        success: function (data) {
                            // can rename?
                            if (ox.api.folder.can("rename", data)) {
                                // start edit
                                self.startEdit(id);
                            }
                        }
                    });
                });

                /**
                 * Context Menu via Left Click
                 */
                this.selection.dispatcher.bind("selected", function(e) {
                    // add context button
                    if (e.node.find(".context-button").length === 0) {
                        var button = $.button({
                            title: "\u00a0\u00a0"
                        }).bind("click", function(event) {
                            self.selection.triggerContextMenu(e.id, event);
                            return false;
                        });
                        if (IE <= 7) {
                            button.css("width", "");
                        }
                        if (IE) {
                            button = $('<form onsubmit="return false"></form>')
                                .css("margin", "0")
                                .append(button);
                        }
                        var td = $("<td/>").css({
                            width: "20px",
                            textAlign: "right",
                            paddingRight: "5px"
                        }).addClass("context-button").append(button);
                        if (IE <= 7) {
                            td.css({ "width": "20px" });
                        }
                        e.node.find("tr").append(td);
                    } else {
                        e.node.find(".context-button").show();
                    }
                });
                this.selection.dispatcher.bind("deselected", function(e) {
                    // remove context button
                    e.node.find(".context-button").hide();
                });

                this.addListener("widget:edited", function (e) {
                    // rename folder
                    ox.api.folder.rename({
                        folder: e.data.id,
                        title: e.data.value,
                        success: function (data) {
                            // get current tree id
                            var ID = self.id + "/" + ox.api.config.get("modules.folder.tree", 0);
                            // has parent node already been opened?
                            if (configGetKey(["gui", "tree", ID, "state", e.data.id]) === true) {
                                // remove previous open/close state
                                configRemoveKey(["gui", "tree", ID, "state", e.data.id]);
                                // update user config
                                configSetKey(["gui", "tree", ID, "state", data.id], true);
                                // set to open
                                self.setOpen([data.id]);
                            }
                            // continue
                            ox.util.call(e.cont, data);
                        },
                        error: function () {
                            // repaint
                            self.repaint();
                            // select
                            self.selection.select(e.data.id);
                            // continue
                            ox.util.call(e.cont);
                        }
                    });
                    return false;
                });

                var displayName = "";
                this.customize = function (node) {

                    var messagingServices = {
                        "com.openexchange.messaging.facebook": {
                            icon: "facebook.png"
                        },
                         "com.openexchange.messaging.rss": {
                            icon: "rss.png"
                        },
                        "com.openexchange.messaging.twitter": {
                            icon: "twitter.png"
                        }
                    };

                    var data = node.data, api = ox.api.folder, id;
                    var name = data.title;

                    // clear all custom styles
                    $(node.dom.title).parent().css({ fontWeight: "" }).
                        removeClass("font-color-disabled font-style-low");
                    
                    $(node.dom.icon).css({ backgroundImage: "" });

                    // add style
                    var processUnread = function (meta) {
                        if (data.unread !== null && data.unread > 0) {
                            $(node.dom.title).parent().css({ fontWeight: "bold" });
                            meta.name = meta.name;
                            meta.html = document.createTextNode(meta.name +
                                " (" + data.unread + ")");
                        }
                        return meta;
                    };

                    // top folder?
                    if ((this.options.tree === 0 && data.folder_id === this.options.root) ||
                            (ox.api.folder.is("account", data)) ) {
                        src = this.options.icon;
                        if (ox.api.folder.is("account", data)) {
                            // auto-open external accounts
                            node.selectable = false;
                            
                            // click on root of mailaccounts should always select the INBOX
                            // need to temporary overide next
                            var account = ox.api.account.get(ox.api.account.derive(data.id));
                            if (account) {
                                var prevnext = node.next;
                                node.next = function(skipChildren) {
                                    if (this.statusOpen && this.children.length > 0) {
                                        // iterate through children to search for INBOX node
                                        for (var i = 0; i < this.children.length; i++) {
                                            if (this.children[i].id === account.inbox_fullname) {
                                                return this.children[i];
                                            }
                                        }
                                    }
                                    // use default
                                    prevnext();
                                };
                            }
                        }
                    }
                    else {
                        var src = "folder_closed" /* default icon */, accountId, a = {};
                        // get account id
                        accountId = /^default(\d*)\b/.exec(data.id);
                        if ($.isArray(accountId)) {
                            // default0?
                            if (accountId[1] === "0") {
                                // use config
                                var f = config.mail.folder;
                                a.inbox_fullname = f.inbox;
                                a.trash_fullname = f.trash;
                                a.sent_fullname = f.sent;
                                a.drafts_fullname = f.drafts;
                                a.spam_fullname = f.spam;
                            } else {
                                // get account data from cache
                                a = ox.api.cache.account.get(accountId[1]);
                                id = "default" + accountId[1];
                                // unified inbox, e.g. behaves like an account but is not
                                if (!a) {
                                    a = {
                                        inbox_fullname: id + "/INBOX",
                                        trash_fullname: id + "/Trash",
                                        sent_fullname: id + "/Sent",
                                        drafts_fullname: id + "/Drafts",
                                        spam_fullname: id + "/Spam"
                                    };
                                }
                                // inbox still not set?
                                if (a.inbox_fullname === undefined) {
                                    a.inbox_fullname = id + "/INBOX";
                                }
                            }
                        }

                        // special mail folder?
                        switch (data.id) {
                        case a.inbox_fullname: src = "inbox.png"; break;
                        case a.trash_fullname: src = "garbage.png"; break;
                        case a.sent_fullname: src = "outbox.png"; break;
                        case a.drafts_fullname: src = "draft.png"; break;
                        case a.spam_fullname: src = "spam.png"; break;
                        case a.confirmed_ham_fullname: src = "ham.png"; break;
                        case a.confirmed_spam_fullname: src = "spam.png"; break;
                        default:
                            // normal mail folder
                            // can read?
                            if ((api.is("system", data) || api.can("read", data)) && node.isCut !== true) {
                                // ok
                                src = node.statusOpen && node.hasChildren ? "folder_opened.png" : "folder_closed.png";
                                $(node.dom.title.parentNode).removeClass("font-color-disabled");
                            } else {
                                // cannot read
                                src = node.statusOpen && node.hasChildren ? "folder_opened_dis.png" : "folder_closed_dis.png";
                                $(node.dom.title.parentNode).addClass("font-color-disabled");
                            }
                        }
                        // tweaking infostore icons
                        switch (data.module) {
                            case "infostore":
                                if (data.folder_id === "10"){
                                    src = "user";
                                }else{
                                    src = "infostore";
                                }
                                // can read?
                                if (!api.can("read", data)) {
                                    // don't visually change system folders
                                    if (data.folder_id === "1") {
                                        src = src + "_dis.gif"; // disable symbol
                                        $(node.dom.title.parentNode).addClass("font-color-disabled font-style-low"); // gray italic
                                    } else {
                                        src = src + ".png";
                                    }
                                    node.selectable = false;
                                } else {
                                    src = src + ".png";
                                    node.selectable = true;
                                }
                            case "messaging":
                                var service = /^(.*?):/.exec(data.id);
                                if (service && messagingServices[service[1]]){
                                   src = messagingServices[service[1]].icon;
                                }
                                break;
                        }
                        // mark published folders
                        if (ox.api.folder.is("published", data)) {
                            // add shared symbol to original
                            $(node.dom.icon).css({ backgroundImage: "url(" + ox.gui.themePath + "icons/16/" +src+ ")" });
                            src = "shared_globe.png";
                        }
                    }

                    if (data.id === "default0") {
                        // replace with user name
                        var identifier = config.identifier;
                        var user = internalCache.getUsers([identifier], function (data) {
                            displayName = data[identifier].display_name;
                        });
                        if (user && user[identifier]) {
                            name = displayName = user[identifier].display_name;
                        } else {
                            name = displayName;
                        }
                    }

                    return processUnread({
                        src: ox.gui.themePath + "icons/16/" + src,
                        name: name
                    });
                };

                this.selection.classSelected = "background-color-PMG-selection-elements";

                var fnToggle = function () {
                    // get id
                    var ID = this.id + "/" + ox.api.config.get("modules.folder.tree", 0);
                    // loop over open nodes
                    var state = {};
                    $.each(self.getOpen(), function (i, id) {
                        state[id] = true;
                    });
                    // update user config
                    configSetKey(["gui", "tree", ID, "state"], state);
                };
                this.addListener("widget:open", fnToggle);
                this.addListener("widget:close", fnToggle);

                // click handler
                this.selection.setClickHandler(function (e) {

                    // selection disabled
                    if (self.selection.statusEnabled === false) {
                        return false;
                    }

                    // context menu?
                    if (e.event.rightClick) {
                        // tracking
                        track({
                            type: "context_menu",
                            data: { id: e.id },
                            text: "Right click on folder " + e.id
                        });
                        // process
                        e.click("quiet");
                        e.context();
                    } else if (e.selectable === true && e.id !== "-1") {
                        // viewing properties?
                        if (currentpath[0] === "configuration") {
                            // change back to folder's module
                            ox.UIController.setFolder({
                                folder: e.id,
                                module: "default"
                            });
                        } else {
                            // normal click
                            // track
                            ox.api.folder.get({
                                folder: e.id,
                                success: function (data) {
                                    track({
                                        type: "click",
                                        dest: "tree",
                                        data: { id: data.id, title: data.title },
                                        text: "Select " + data.module + " folder \"" + data.title + "\" (" + e.id + ")"
                                    });
                                }
                            });
                            // process
                            if (ox.UIController.getFolder() !== e.id) {
                                e.click("force");
                            } else {
                                e.click();
                            }
                        }
                    }

                    // prevent default
                    return false;
                });

                // context menu
                this.selection.setContextMenuHandler(function (id, e) {
                    // show folder context menu
                    var menu = globalContextMenus.simpleFolderMenu;
                    menu.display(e.pageX, e.pageY, {
                        data: { id: id },
                        api: true,
                        simple: true,
                        module: TYPE,
                        widget: self
                    });
                });

                // add root node?
                if (this.loadRootNode === false) {
                    this.setRootNode({ id: this.options.root, folder_id: null, title: "Root", module: "root", subfolders: true });
                }

                this.onModify = function (data) {
                    var finalOpenNodes = [];

                    // close invalid nodes
                    if (data.why === "folder.remove" || data.why === "folder.move") {
                        // is mail?
                        if (ox.api.folder.is("mail", data.folder)) {
                            // get id
                            var key = data.oldId;
                            // close invalid nodes
                            $.each(ox.util.keys(self.openNodes).concat(self.getOpen()), function (i, id) {
                                if ((new RegExp("^" + key)).test(id)) {
                                    // get tree node
                                    var node = self.get(id);
                                    if (node) {
                                        // required to re-open later
                                        finalOpenNodes.push(id.replace(key, data.id));
                                        // close
                                        node.close();
                                    }
                                }
                            });
                        }
                        // unset current folder
                        ox.UIController.unsetFolder();

                    } else if (data.why !== "folder.refresh") {

                        // affects current mail folder?
                        var folder = ox.UIController.getFolder("mail"),
                            tree = ox.api.config.get("modules.folder.tree", 0),
                            cache = ox.api.cache["folder" + tree];
                        // is removed?
                        if (folder !== undefined && !cache.contains(folder)) {
                            // unset folder
                            ox.UIController.unsetFolder("mail");
                        }
                    }

                    if (data.why === "folder.update.local") {
                        // repaint affected node only
                        var node = self.get(data.id);
                        if (node) {
                            node.repaint();
                        }
                    } else {
                        if (data.why === "folder.move") {
                            ox.UIController.setFolder({
                                folder: data.id,
                                module: "default",
                                success: function () {
                                    // reopen node
                                    if (self.setOpen) self.setOpen(finalOpenNodes);
                                    self.repaint(function() {
                                        // clean up
                                        finalOpenNodes = null;
                                    });
                                }
                            });
                        } else {
                            // repaint full tree
                            self.repaint();
                        }
                    }
                };
            },

            loadChildren: function (id, cont) {
                // step #2: get sub folders
                var self = this;
                var type = this.options.type;
                var getFolders = function () {
                    ox.api.folder.getSubFolders({
                        folder: id,
                        tree: ox.api.config.get("modules.folder.tree", "0"),
                        success: function (data) {
                            // mail only
                            data = $.grep(data, function (elem, i) {
                                if (type === "mail") {
                                    // mail AND messaging
                                    var isMessaging = elem.module === "messaging",
                                        isMail = elem.module === "mail",
                                        isMailFolder = elem.tmpFolder === true || /^default/.test(elem.id),
                                        isSubscribed = elem.subscribed === true || elem.subscr_subflds === true,
                                        isSystem = elem.module === "system",
                                        isProperFolder = isMailFolder || isMessaging;
                                    return isProperFolder && isSubscribed && (isMail || isMessaging || isSystem);
                                } else {
                                    return (elem.subscribed === true || elem.subscr_subflds === true) && (elem.module === type || elem.module === "system");
                                }
                            });
                            cont(data);
                        },
                        error: function () {
                            cont(false);
                            // return true; // prevent global error reporting
                        }
                    });
                };
                // mail enabled?!?
                if (ox.api.config.get("modules.mail.module") === true) {
                    // step #1: get accounts (cached after first request)
                    ox.api.account.all(getFolders);
                } else {
                    getFolders();
                }
            },

            loadNode: function (id, cont) {
                // get single folder
                var self = this;
                ox.api.folder.get({
                    folder: id,
                    tree: ox.api.config.get("modules.folder.tree", "0"),
                    success: function (data) {
                        ox.util.call(cont, data);
                    }
                });
            },

            // prevent flicker
            clear: function () {
            },

            expandToFolder: function (folderID) {

                // get open nodes
                var open = this.getOpen();

                // this
                var self = this;

                var getParentIDs = function (data, callback) {
                    // loop over folder
                    var i = 1, $l = data.length, list = [], id;
                    for (; i < $l; i++) {
                        id = data[i].id;
                        // not already open?
                        if ($.inArray(id, open) === -1) {
                            list.push(id);
                        }
                    }
                    callback(list);
                };

                var expandTreeNodes = function (list) {

                    // need repaint?
                    if (list.length > 0) {
                        // yes
                        self.setOpen(list);
                        self.repaint(function () {
                            self.selection.clickQuiet(folderID);
                        });
                    } else {
                        // no, select directly
                        self.selection.clickQuiet(folderID);
                    }
                };

                // get all parents of folder via api
                ox.api.folder.getParents({
                    folder: folderID,
                    success: function (data){
                        getParentIDs(data, expandTreeNodes);
                    }
                });
            },

            paint: function (cont) {
                if (this.painter.running === false) {

                    this.painter.running = true;

                    // first paint?
                    if (this.firstPaint === true) {
                        // get folder state id
                        var ID = this.id + "/" + ox.api.config.get("modules.folder.tree", 0);
                        // get open folders
                        if (configContainsKey(["gui", "tree", ID, "state"])) {
                            this.setOpen(ox.util.keys(configGetKey(["gui", "tree", ID, "state"])));
                        } else {
                            this.setOpen(this.options.open || []);
                        }
                        // visually busy
                        $(this.dom.node).css({ background: "" }).addClass("oxBusySmall");
                        // done
                        this.firstPaint = false;
                    }

                    // unbind
                    ox.api.folder.dispatcher.unbind("modify", this.onModify);

                    // this
                    var self = this;

                    // continuation
                    var step2 = function () {
                        // get ids of open nodes (exclude external accounts)
                        var list = $.grep(ox.util.keys(self.openNodes), function (key) {
                            return /^(\d+|default0)/.test(key);
                        });
                        // pause
                        ox.api.http.pause();
                        var pssst = function () {
                            return true; // prevent global error
                        };
                        // loop
                        var i = 0, $l = list.length;
                        for (; i < $l; i++) {
                            ox.api.folder.getSubFolders({
                                folder: list[i],
                                tree: self.options.tree,
                                error: pssst
                            });
                        }
                        // resume
                        ox.api.http.resume(
                            function () {
                                // paint
                                $(self.dom.node).empty().removeClass("oxBusySmall");
                                self.painter.running = false;
                                ox.gui.Tree.prototype.paint.call(self, function () {
                                    // clear open nodes hash
                                    self.openNodes = {};
                                    // select id
                                    if (self.selection !== undefined) {
                                        self.selection.enable();
                                        self.selection.preselect(currentFolder); // before paint
                                        self.selection.clickQuiet(currentFolder); // after paint
                                    }
                                    // bind
                                    ox.api.folder.dispatcher.bind("modify", self.onModify, window);
                                    // continue
                                    ox.util.call(cont);
                                });
                            },
                            pssst
                        );
                    };

                    // consider current folder (not in nested windows)
                    var currentFolder = ox.UIController ? ox.UIController.getFolder() : undefined;
                    if (currentFolder !== undefined) {
                        // get path
                        ox.api.folder.getParents({
                            folder: currentFolder,
                            tree: this.options.tree,
                            success: function (data) {
                                // remove current folder
                                data.shift();
                                // get ids
                                var list = $.map(data, function (folder) {
                                    return folder.id;
                                });
                                // leads to root folder?
                                if ($.inArray(self.options.root, list) > -1) {
                                    // mark as open
                                    self.setOpen(list);
                                }
                                // continue
                                step2();
                            }
                        });
                    } else {
                        // continue
                        step2();
                    }
                } else {
                    this.painter.queue.push(cont);
                }
            }
        });

        /*
         * Mail folder tree
         */
        var tree = ox.api.config.get("modules.folder.tree", 0);

        v.mail = new ox.gui.Container("ox-sidepanel-mail").
        setLayout(new ox.gui.BorderLayout());

        var mail_tree = ox.widgets.sidepanelViews.mailTree = new ox.gui.TypedFolderTree("ox-sidepanel-mail-tree", {
            type: "mail",
            icon: "mail.png",
            root: "1",
            tree: tree,
            open: tree === 0 ? ["default0"] : []
        }).setLayoutParam({ position: "center" });

        mail_tree.enableDnD(ox.gui.initSidepanel.dndSource, ox.gui.initSidepanel.dndTarget);

        v.mail.add(mail_tree);

        // check for plugin
        if (ox.api.config.get("modules.mail.module") === true &&
        		ox.mail.addAccount.available()) {
            // add extension
            mail_tree.addListener("widget:painted", function () {
                var node = $(this.dom.node);
                if (node.find("a.add-account").length === 0) {
                    node.append(
                        $('<div>')
                        .css({ lineHeight: '2em', margin: '0 15px 0 15px' })
                        .append(
                            $('<a>', { href: '#' })
                            .addClass('common-link add-account')
                            .append(addTranslated(_("Add E-Mail account ...")))
                            .bind('click', function (e) {
                                e.preventDefault();
                                // enabled?
                                if (ox.api.config.get("modules.mailaccount.module", false)) {
                                    ox.mail.addAccount();
                                } else {
                                    triggerEvent("Feature_Not_Available", "configuration/mail/accounts/new", window, "foldertree");
                                }
                            })
                        )
                    );
                }
            });
        }

        /*
         * Infostore folder tree
         */
        v.infostore = new ox.gui.Container("ox-sidepanel-infostore").
        setLayout(new ox.gui.BorderLayout());

        var infostore_tree = ox.widgets.sidepanelViews.infostoreTree = new ox.gui.TypedFolderTree("ox-sidepanel-infostore-tree", {
            type: "infostore",
            icon: "infostore.png",
            root: "9",
            open: ["9", "10", ox.api.config.get("folder.infostore")]
        }).setLayoutParam({ position: "center" });
        infostore_tree.enableDnD(ox.gui.initSidepanel.dndSource, ox.gui.initSidepanel.dndTarget);

        v.infostore.add(infostore_tree);

        /*
         * Flat views
         * ------------------------------------------------------------------------------------
         */
        var flatView = ox.gui.Custom.extend({

            construct: function (paint, id, module) {

                // super constructor
                ox.gui.Custom.prototype.construct.call(this, paint, id);

                // module
                this.module = module;
                this.folders = {};

                var self = this;

                // selection
                var s = this.selection = new ox.gui.Selection();
                s.setMultiple(false);
                s.setNodeFinder(function () {
                    return $(".ox-sidepanel-flat-folder", this.container);
                });
                s.classSelected = "background-color-PMG-selection-elements";
                s.onChange(function (items) {
                    if (self.isVisible() && items.length) {
                        var id = String(items[0].id);
                        setTimeout(function () {
                            // change folder
                            ox.UIController.setFolder({ folder: id });
                        }, 10);
                    }
                });
                s.onRename(function (id, item) {
                    // get folder
                    ox.api.folder.get({
                        folder: id,
                        success: function (data) {
                            // can rename?
                            if (ox.api.folder.can("rename", data)) {
                                // start edit
                                self.startEdit(id);
                            }
                        }
                    });
                });
                s.observe(self.dom.node, "ox-sidepanel-view-wrapper");

                // click handler
                s.setClickHandler(function (e) {

                    // selection disabled
                    if (self.selection.statusEnabled === false) {
                        return false;
                    }

                    // context menu?
                    if (e.event.rightClick) {
                        // event
                        track({
                            type: "context_menu",
                            dest: "tree_flat",
                            data: { id: e.id },
                            text: "Right click on folder " + e.id + "in flat view"
                        });
                        // process
                        e.click("quiet");
                        e.context();
                    } else if (e.selectable === true) {
                        // viewing properties?
                        if (currentpath[0] === "configuration") {
                            // change back to folder's module
                            ox.UIController.setFolder({
                                folder: e.id,
                                module: "default"
                            });
                        } else if (e.id !== "-1") {
                            // normal click
                            // track
                            ox.api.folder.get({
                                folder: e.id,
                                success: function (data) {
                                    track({
                                        type: "click",
                                        dest: "tree_flat",
                                        data: { id: data.id, title: data.title },
                                        text: "Select " + data.module + " folder \"" + data.title + "\" (" + e.id + ")"
                                    });
                                }
                            });
                            // process
                            if (ox.UIController.getFolder() !== e.id) {
                                e.click("force");
                            } else {
                                e.click();
                            }
                        }
                    }

                    // prevent default
                    return false;
                });

                s.setContextMenuHandler(function (id, e) {
                    // show folder context menu
                    var menu = globalContextMenus.simpleFolderMenu;
                    menu.display(e.pageX, e.pageY, {
                        data: { id: id },
                        api: true,
                        simple: true,
                        module: module,
                        widget: self
                    });
                });

                this.addListener("widget:edited", function (e) {
                    // update folder
                    ox.api.folder.update({
                        folder: e.data.id,
                        data: { title: e.data.value },
                        success: function(data) {
                            ox.api.cache.folderFlat.remove(data.module);
                            // select
                            self.selection.select(data.id);
                            // continue
                            ox.util.call(e.cont);
                        },
                        error : function() {
                            // repaint
                            self.paint();
                            // select
                            self.selection.select(e.data.id);
                            // continue
                            ox.util.call(e.cont);
                        }
                    });
                    return false;
                });

                // track visibility
                this.addListener("widget:invisible", function () {
                    s.disable();
                });
                this.addListener("widget:visible", function () {
                    s.enable();
                });

                // set class now (IE7 issues; flickering scrollbar)
                $(this.dom.node).addClass("ox-sidepanel-view");

                this.onModify = function (data) {
                    // close invalid nodes
                    if (data.why === "folder.remove" || data.why === "folder.move") {
                        // unset current folder
                        ox.UIController.unsetFolder();
                    }

                    if (data.why === "folder.update.local") {
                        // repaint affected node only
                        var node = self.get(data.id);
                        if (node) {
                            node.repaint();
                        }
                    } else {
                        if (data.why === "folder.move") {
                            ox.UIController.setFolder({
                                folder: data.id,
                                module: "default",
                                success: function () {
                                    // reopen node
                                    if (self.setOpen) self.setOpen([ data.id ]);
                                    self.repaint();
                                }
                            });
                        } else {
                            // repaint full tree
                            self.repaint();
                        }
                    }
                };

                /**
                 * Context Menu via Left Click
                 */
                s.dispatcher.bind("selected", function (e) {
                    // add context button
                    if (e.node.find(".context-button").length === 0) {
                        var button = $.button({
                            title: "\u00a0\u00a0"
                        }).css({
                            position: "absolute",
                            right: "5px",
                            zIndex: 2
                        }).bind("click", function(event) {
                            s.triggerContextMenu(e.id,  event);
                            return false;
                        })
                        .addClass("context-button");
                        if (IE) {
                            button = $('<form onsubmit="return false"></form>')
                                .css("margin", "0")
                                .append(button);
                        }
                        e.node.prepend(button);
                    } else {
                        e.node.find(".context-button").show();
                    }
                });
                s.dispatcher.bind("deselected", function (e) {
                    // remove context button
                    e.node.find(".context-button").hide();
                });
            }
        });

        flatView.prototype.startEdit = function (id, success, abort) {
            // get
            if (this.folders[id] !== undefined) {

                var self = this;

                // unbind
                ox.api.folder.dispatcher.unbind("modify", this.onModify);

                if (success === undefined) {
                    success  = function (value) {
                        // trigger event
                        self.trigger(
                            "widget:edited",
                            {
                                node: self,
                                id: id,
                                value: value
                            },
                            function () {
                                // repaint (enables selection)
                                self.repaint();
                            }
                        );
                    };
                }

                if (abort === undefined) {
                    abort = function () {
                        // repaint (enables selection)
                        self.repaint();
                    };
                }

                this.selection.disable();

                var node = this.folders[id].node;
                ox.gui.util.inlineEdit.call(
                    this, node, node.data("title"), success, abort, "span"
                );
            }
        };

        var sections = {
            "private": {
                title: _("Private folders")
            },
            "public": {
                title: _("Public folders")
            },
            "shared": {
                title: _("Shared folders")
            }
        };

        // "Global address book"

        var paintFlatView =  function (cont) {
            var self = this;

            // unbind
            ox.api.folder.dispatcher.unbind("modify", this.onModify);

            function addDnD(folder, node, out) {
                if (self.dndSource) {
                    self.dndSource(folder, function (type, callback) {
                        out.dndSource = registerSource(node, type, callback,
                            null, null, foldertreedisable, defaultdisabledremove);
                    });
                }
                if (self.dndTarget) {
                    self.dndTarget(folder, function (callbacks) {
                        var cbs = {};
                        for (var i in callbacks) {
                            cbs[i] = (function (cb) {
                                return function (a, b, c, d, e, f) {
                                    legacyOut(a, b, c, d, null, f);
                                    cb(a, b, c, d, e, f);
                                };
                            })(callbacks[i]);
                        }
                        out.dndTarget = registerTarget(node, cbs, null, null,
                            legacyIn, legacyIn, legacyOut, true);
                    });
                }
                function legacyIn(e, type, objects, position, targetNode) {
                    foldertreeenable(e, type, objects, position, targetNode, folder);
                    var mynode = e.currentTarget || e.srcElement || false;
                    if (!mynode.className.match(/dndOver/)) {
                        mynode.className += " dndOver";
                    }
                }
                function legacyOut(e, type, objects, position, OutOpen, dropNode) {
                    if (dropNode) {
                        dropNode.className = removeClass(dropNode.className, "dndOver");
                    } else if (e) {
                        var mynode = e.currentTarget || e.srcElement || false;
                        if (mynode) {
                            mynode.className = removeClass(mynode.className, "dndOver");
                        }
                    }
                }
            }

            var paintFolders = function (folders, node, section) {

                // sort by title, default folder always first
                var id = String(ox.api.config.get("folder."+ self.module));
                // sort (skip shared folder; have special order)
                if (section !== "shared") {
                    folders.sort(function (a, b) {
                        return a.id === id ? -1 : a.title.toLowerCase() > b.title.toLowerCase() ? +1 : -1;
                    });
                }
                // add owner
                var addOwner = function (folder, node) {
                    if (ox.api.folder.is("shared", folder)) {
                        var displayName = folder["com.openexchange.folderstorage.displayName"] || "";
                        node.append("<br/>");
                        node.append(
                            $("<span/>").css({ color: "#666", fontStyle: "italic" }).
                                text("(" + displayName + ")")
                        );
                    }
                };
                // loop
                var i = 0, $l = folders.length, folder, div, count = 0;
                for (i = 0; i < $l; i++) {
                    // get
                    folder = folders[i];
                    // read permisssion?
                    if (ox.api.folder.can("read", folder)) {
                        var img = $("<img/>").attr({ src: meta[self.module].icon }).
                        css({ backgroundRepeat: "no-repeat", position: "absolute", left: "3px" });
                        if (ox.api.folder.is("published", folder)) {// published folders
                            // add shared symbol to original
                            img.css({"background-image" : "url(" + img.attr("src") + ")" });
                            img.attr("src", ox.gui.themePath + "icons/16/shared_globe.png");
                        }

                        // add nodes
                        node.append(
                            // div
                            div = $("<div/>", { oxID: folder.id }).
                                data("title", _(folder.title || "")).
                                addClass("ox-sidepanel-flat-folder").
                                css({ position: "relative" }).
                            append(img).
                            // folder name
                            append(
                                $("<span/>").text(folder.title || "")
                            )
                        );
                        // add owner?
                        addOwner(folder, div);
                        // add to index
                        var out = self.folders[folder.id] = {
                            id: folder.id,
                            node: div
                        };
                        // support DND
                        addDnD(folder, div[0], out);
                        // count
                        count++;
                    }
                }
                return count;
            };

            setTimeout(function () {

                // clear
                for (var i in self.folders) {
                    f = self.folders[i];
                    if (f.dndSource) unregisterSource(f.node[0], f.dndSource);
                    if (f.dndTarget) unregisterSource(f.node[0], f.dndTarget);
                    delete self.folders[i];
                }

                $(self.dom.node).empty();

                // add spinner
                if (!ox.browser.IE) { // IE7 flicker issue
                    $(self.dom.node).addClass("busy-spinner-white");
                }

                // get visible folders via API (flat list)
                ox.api.folder.getAllVisible(self.module, function (data) {

                    // remove spinner
                    if (!ox.browser.IE) { // IE7 flicker issue
                        $(self.dom.node).removeClass("busy-spinner-white");
                    }

                    // add sections
                    for (var id in sections) {
                        $(self.dom.node).append(
                            self.dom[id] = $("<div/>").addClass("ox-sidepanel-section").append(
                                $("<div/>").addClass("ox-sidepanel-section-title").append(
                                    addTranslated(meta[self.module].sections[id])
                                )
                            )
                        );
                        // paint folders
                        var count = paintFolders(data[id] || [], self.dom[id], id);
                        // remove section?
                        if (count === 0) {
                            self.dom[id].remove();
                        }
                    }

                    // enable selection
                    self.selection.enable();
                    self.selection.update();
                    self.selection.clickQuiet(ox.UIController.getFolder());

                    // bind to folder api
                    ox.api.folder.dispatcher.bind("modify", self.onModify, window);

                    // continuation
                    ox.util.call(cont);
                });

            }, 10);
        };

        // No source, since dragging folders without hierarchy makes no sense
        flatView.prototype.dndTarget = ox.gui.initSidepanel.dndTarget;

        // calendar
        v.calendar = new flatView(paintFlatView, "ox-sidepanel-calendar", "calendar");

        // contacts
        v.contacts = new flatView(paintFlatView, "ox-sidepanel-contacts", "contacts");

        // tasks
        v.tasks = new flatView(paintFlatView, "ox-sidepanel-tasks", "tasks");

    }());


    /*
     * Module title
     */
    var title = ox.widgets.sidepanel.title = new ox.gui.Container("ox-sidebar-title").
    setLayout(new ox.gui.BorderLayout()).
    css({ height: "26px" }).
    setLayoutParam({ position: "top", height: 27 }).
    addCSSClass("ox-sidepanel-title topheader-color").
    addTo(sidepanel).
    add(
        new ox.gui.Custom(
            function () {
                $(this.dom.node).empty().
                    append(addTranslated(this.title));
            },
            "ox-sidepanel-title"
        ).
        setLayoutParam({ position: "center" }).
        css({ marginLeft: "6px" }).
        addListener("widget:viewchanged", view, function (e) {
            // make sure folder & config are shown
            $("#folder_tree").show();
            // change title
            var match = e.data.id.match(/(\w+)$/);
            this.title = meta[match[1]].title;
            if (newFolderIcon !== null) {
                newFolderIcon.setVisible(match[1] !== "configuration");
            }
            this.paint();
        })
    ).add(
            new ox.gui.Image().
            css({ cursor: "pointer" }).
            addCSSClass("sp-toggle-img").
            setLayoutParam({ position: "right", width: 13 }).
            setSrc(getFullImgSrc("img/arrows/arrow_double_white_left.png")).
            setTitle(_("Collapse sidepanel")).
            addListener("click", function (e) {
                ox.widgets.sidepanel.toggle();
                ox.widgets.sidepanel.userToggle = true;
            })
    );

    // add new folder image only if required
    if (addRootFolder) {
        title.insert(newFolderIcon, 0);
    }

    // add view to sidepanel
    sidepanel.add(view);

    /*
     * Module views
     */
    v.portal = new ox.gui.Widget("ox-sidepanel-portal").addTo(view);
    v.mail.addTo(view);
    v.calendar.addTo(view);
    v.contacts.addTo(view);
    v.tasks.addTo(view);
    v.infostore.addTo(view);
    v.folder = new ox.gui.Container("ox-sidepanel-foldertree").addTo(view).
        setLayout(new ox.gui.BorderLayout());


    //--- quota

    var quotaContainer = { mail: [], filestore: [] }, quotaReq = [];

    // store to widgets container for public access
    if (!ox.widgets.sidepanel.quota) {
        ox.widgets.sidepanel.quota = {};
    }

    // Since all bars are needed twice - in the simple and in the expert view -
    // the bars are created by functions.

    function createMailBars(container, qContainer) {
        var bar = ox.widgets.sidepanel.quota.mail = new ox.gui.Bar({ textFormat: function(used, max) {
            return bytesToString(used) + " / " + bytesToString(max);
        }, title: function(used, max) {
            return _("E-Mail") + " " + bytesToString(used) + " / " +
                   bytesToString(max);
        }, description: _("E-Mail") }).
            setLayoutParam({ position: "bottom", height: 18 }).hide();
        container.add(bar);

        var bar2 = ox.widgets.sidepanel.quota.mail_count = new ox.gui.Bar({ textFormat: "fraction", description: _("E-Mail") }).
            setLayoutParam({ position: "bottom", height: 18 }).hide();
        container.add(bar2);

        qContainer.push({ id: "mail_storage",
            field_map: { usage:"use", quota:"quota" }, bar: bar });
        qContainer.push({ id: "mail_count",
            field_map: { usage:"countuse", quota:"countquota" }, bar: bar2 });
    }

    function createInfoStoreBars(container, qContainer) {
        var bar = ox.widgets.sidepanel.quota.infostore = new ox.gui.Bar({ textFormat: function(used, max) {
            return bytesToString(used) + " / " + bytesToString(max);
        }, title: function(used, max) {
            return _("Infostore") + " " + bytesToString(used) + " / " +
                   bytesToString(max);
        }, description: _("Infostore") }).
            setLayoutParam({ position: "bottom", height: 18 }).hide();
        container.add(bar);

        qContainer.push({ id: "filestore",
            field_map: { usage:"use", quota:"quota" }, bar: bar });
    }

    // register quota bars for available modules
    if (ox.api.config.get("modules.infostore.module")) {
        createInfoStoreBars(v.infostore, quotaContainer.filestore);
        createInfoStoreBars(v.folder, quotaContainer.filestore);
    	quotaReq.push({ module:"quota", action: "filestore" });
    }


    if (ox.api.config.get("modules.mail.module")) {
        createMailBars(v.mail, quotaContainer.mail);
        createMailBars(v.folder, quotaContainer.mail);
        quotaReq.push({ module:"quota", action: "mail" });
    }

    // fetches quota from the back-end
    function checkQuota() {
    	// helper to remove quota object with given ID from container
    	function removeFromContainer(id) {
    		for (var i in quotaContainer) {
                for (var ia in quotaContainer[i]) {
                    if (quotaContainer[i][ia].id == id) {
                    	if (quotaContainer[i][ia].bar) {
                    		var parent = quotaContainer[i][ia].bar.parent;
                    		quotaContainer[i][ia].bar.destroy();
                    		//ox.widgets.sidepanelViews.mail.validate();
                    		parent.validate();
                    	}
                        delete(quotaContainer[i][ia]);
                    }
                }
            }
    	}

    	// fire put
    	ox.api.http.PUT({
            module: "multiple",
            data: quotaReq,
            appendColumns: false,
            success: function (data) {
    			// now its gonna be tricky
    			// iterate through server results
	    		for (var i=0; i < data.length; i++) {
	    			// iterate through container
	                for (ia in quotaContainer[quotaReq[i].action]) {
	                	// getting quota object mapped to results
	                	var quota = quotaContainer[quotaReq[i].action][ia];
	                	if (!quota) continue;
	                	// no results for this request? remove it!
	                	if (data[i] == undefined || data[i].data == undefined) {
	             		   removeFromContainer(quota.id);
	                 	   continue;
	                    }
	             	   	// getting quota information (count and usage)
	                    var tQuota = data[i].data[quota.field_map.quota];
	                    var tUsage = data[i].data[quota.field_map.usage];
	                    // no results for this request? remove it!
	                    if (tQuota == undefined || tQuota == -1024 || tQuota == -1) {
	                    	removeFromContainer(quota.id);
	                        continue;
	                    }
	                    // store quota information to container
	                    quota.data = { quota: tQuota, usage: tUsage };

	                    // paint
	                    quota.bar.setMax(tQuota);
	                    quota.bar.progress(tUsage);
	                    if (quota.bar.isVisible() == false) {
	                        quota.bar.show();
	                    }
	                }
	            }
        	}
    	});
    };
    // receive information the first time
    checkQuota();
    // update quota information on refresh
    register("OX_Refresh", checkQuota);
    // register to empty folder
    register("OX_Folder_Cleared", function(folder_id) {
        // user empty a folder, this might be the either the trash
        // or user activated direct delete. if so we need to force
        // a quota update
        if (ox.api.config.get("mail.deletemail") === true
                || ox.api.config.get("mail.folder.trash") === folder_id) {
            checkQuota();
        }
    });

    //--- mini-cal

    var minical = ox.widgets.miniCalendar = new ox.gui.Widget("ox-sidepanel-view-minical");

    minical.expanded = ox.api.config.get("gui.global.minicalendar.expanded", false);

    minical.hide = function() {
        this.setVisible(false);
        this.layout.height = this.getHeight();
        this.invalidate();
        ox.widgets.sidepanel.validate();
    };

    minical.getHeight = function () {
        return this.statusVisible ? (this.expanded ? (13 * pxPerEm + 6) : (2 * pxPerEm + 4)) : 0;
    };

    minical.show = function() {
        // disabled means not visible at all
        if (this.isEnabled() === false) {
            return;
        }
        this.setVisible(true);
        this.layout.height = this.getHeight();
        this.invalidate();
        ox.widgets.sidepanel.validate();
    };

    minical.refreshState = function () {
        this.toggle(ox.api.config.get("gui.global.minicalendar.expanded", false));
    };

    minical.toggle = function (state) {
        // disabled means not visible at all
        if (this.isEnabled() === false) {
            return;
        }
        this.expanded = !this.expanded;
        if (typeof state == "boolean") {
            this.expanded = state;
        }

        // make changes permanent
        if (this.expanded === false || ox.UIController.getModule() !== "calendar") {
            ox.api.config.set("gui.global.minicalendar.expanded", this.expanded);
        }

        $("img.mini-toggle", this.dom.node)
            .attr("src", getFullImgSrc("img/" + (this.expanded ? "minus.gif" : "plus.gif")));

        this.layout.height = this.getHeight();
        this.invalidate();
        ox.widgets.sidepanel.validate();
    };

    minical.paint = function () {

        var Self = this;
        $(this.dom.node).empty().addClass("oxStretch");

        var content = $("<div/>").
            css({
                height: "11.3em",
                width: "18em",
                overflow: "hidden",
                position: "absolute"
            }).
            append($("<div/>").attr("id", "mini-content").addClass("calminimainCSS"));

        var header = $("<table/>")
            .addClass("headercontenttable border-color-design")
            .css({
                height: "2em",
                backgroundImage: "url('" + getFullImgSrc("img/toolbar/toolbar30.png") + "')"
            })
            .append(
                $("<tr/>").append(
                    $("<td/>").css({ width: "50%" }).text(" ")
                )
                .append(
                    $("<td/>").attr("align", "right").css({ cursor: "pointer", width: "10px" }).
                    append($("<img/>").attr("src", getFullImgSrc("img/arrows/arrow_darkgrey_left.gif"))
                    ).bind("click", function() {
                        oMiniCalendar.back(1);
                    })
                )
                .append(
                    $("<td/>").attr("align", "center").append(
                        $("<div/>").addClass("mini-month").attr("id", "mini-month")
                    )
                )
                .append(
                    $("<td/>").attr("align", "left").css({ cursor: "pointer", width: "10px" }).
                    append($("<img/>").attr("src", getFullImgSrc("img/arrows/arrow_darkgrey_right.gif"))
                    ).bind("click", function() {
                        oMiniCalendar.next(1);
                    })
                )
                .append(
                    $("<td/>").css({ width: "50%"}).attr("align", "left").append(
                        $("<div/>").addClass("mini-year").attr("id", "mini-year")
                    )
                ).append(
                    $("<td/>").css({ width: "20px"}).attr("align", "right").append(
                        $("<img/>").addClass("mini-toggle").
                        attr("src", getFullImgSrc("img/minus.gif")).
                        css({ cursor: "pointer" }).
                        click(function() {
                            ox.widgets.miniCalendar.userToggle = true;
                            ox.widgets.miniCalendar.toggle();
                        })
                    )
                )
            );

        $(this.dom.node).append(header);
        $(this.dom.node).append(content);

        oMiniCalendar = new cAnimMiniCalendar(
            fn_minicalendar_action, null,
            [ $("#mini-content", content)[0], $(".mini-month", header)[0], $(".mini-year", header)[0]]
        );
        b3xViewLoaded = false;
    };

    function fn_minicalendar_action() {

        oMiniCalendar.setSelected(this.id);
        activeDay=oMiniCalendar.selectedDate;
        activeMonth=oMiniCalendar.selectedMonth;
        activeYear=oMiniCalendar.selectedYear;

        // notify
        track({
            type: "click",
            dest: "mini_calender",
            data: { year: activeYear, month: activeMonth, day: activeDay },
            text: "Select date in mini calendar " + activeYear + "-" + activeMonth + "-" + activeDay
        });
        triggerEvent("OX_Mini_Calendar_Date_Picked", activeYear, activeMonth, activeDay);

        if(currentpath[0] == "calendar") {
            if (currentpath[1] != "team" && currentpath[2] == "month") {
                // this routine is not required in team view; might do too much in other views anyway...
                loadMonthView();
            }
            ox.UIController.setModule({
                module: "calendar",
                view: currentpath.join("/"),
                folder:"auto",
                force: "module"
            });
        } else if (configContainsKey("folder.calendar")) {
           ox.UIController.setModule({
               module: "calendar",
               folder:"auto",
               view:"calendar/calendar/day",
               force: "module"
           });
        }
    }

    function updateMiniCalData() {
        oMiniCalendar.update();
    }

    register("OX_Refresh_Mini_Calendar", updateMiniCalData);
    register("OX_Refresh", updateMiniCalData);

    minical
        .css({
            background: "white",
            border: "1px solid",
            borderWidth: "1px 0px 0px 0px"
        })
        .addCSSClass("border-color-design")
        .setLayoutParam({ position: "bottom", height: (13 * pxPerEm + 6) })
        .addTo(sidepanel);
};


// Two functions shared between ox.gui.FolderTree and ox.gui.TypedFolderTree
ox.gui.initSidepanel.dndSource = function(data, cont) {
    cont("folder/" + data.type, constant(data.id));
};
ox.gui.initSidepanel.dndTarget = function(subfolder, cont) {
    function folderHandler(e, t, data, p, n, d) {
        ox.api.folder.get({
            folder: data,
            success: function(data) {
                ox.api.folder.move(data, subfolder);
            }
        });
    }
    var tObj = {};
    if (   (subfolder.own_rights & 127) >= 4 // folder permissions >= create subfolders
        || subfolder.module == 'mail')
    {
        if (subfolder.type == 5) {
            if (subfolder.id == 1) {
                tObj["folder/1"] = folderHandler;
            } else if (subfolder.id in { 2: 1, 10: 1, 15: 1 }) {
                tObj["folder/2"] = folderHandler;
            } else {
                tObj["folder/" + subfolder.type] = folderHandler;
            }
        } else {
            if ( (subfolder.type == 7 || subfolder.type == 1) && configGetKey("modules.folder.tree") == 1) {
                tObj["folder/1"] = tObj["folder/7"] = folderHandler;
            } else {
                tObj["folder/" + subfolder.type] = folderHandler;
            }
        }
    }

    /*
     * check if user has writeObject permissions and if it's not a
     * shared folder of module calendar, tasks or contacts. move operations
     * into a shared folder of the named modules will be declined by the server
     */
    var tmp_module = subfolder.module;
    if (   ox.api.folder.can("write", subfolder) === true // folder permissions >= create objects
        && !(subfolder.type == 3 && tmp_module == "tasks"))
    {
        /*
         * user has writeObject permissions on this folder, register event
         * which handles move operations
         */
        tObj[subfolder.module] = function(e,dragType,data,mouseposition,targetNode,dropNode) {
            if (subfolder.type == 2) {
                for (var i in data.currentObjects) {
                    if (data.currentObjects[i].private_flag) {
                        return;
                    }
                }
            }
            data.endMoving(subfolder.id);
        };
    } else {
        /*
         * user doesn't have writeObject permissions or it's a shared folder
         * so register dummy event
         */
        tObj[subfolder.module] = $.noop;
    }

    if (subfolder.module != "contacts") {
        tObj.contacts = $.noop;
    }
    if (subfolder.module != "tasks") {
        tObj.tasks = $.noop;
    }
    if (subfolder.module != "calendar") {
        tObj.calendar = $.noop;
    }
    if (subfolder.module != "infostore") {
        tObj.infostore = $.noop;
    }
    if (subfolder.module != "mail") {
        tObj.mail = $.noop;
    }
    if (subfolder.module == "contacts") {
        tObj.mailaddress = function (e, t, objects) {
            createNewContactfromMail(objects, subfolder.id);
        };
    } else {
        tObjmailaddress = $.noop;
    }
    cont(tObj);
};
/**
 *
 * All content on this website (including text, images, source
 * code and any other original works), unless otherwise noted,
 * is licensed under a Creative Commons License.
 *
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 *
 * Copyright (C) Open-Xchange Inc., 2006-2012
 * Mail: info@open-xchange.com
 *
 * @author Matthias Biggeleben <matthias.biggeleben@open-xchange.com>
 * @author Alexander Quast <alexander.quast@open-xchange.com>
 *
 */

/*
 * Tree views
 * ------------------------------------------------------------------------------------
 */
ox.gui.FolderTree = ox.gui.Tree.extend({

    construct: function (id, options) {

        // super constructor
        ox.gui.Tree.prototype.construct.call(this, id);

        // options
        this.options = $.extend({
            tree: ox.api.config.get("modules.folder.tree") || 0
        }, options || {});

        // root
        if (this.options.root === undefined) {
            this.options.root = this.options.tree === 1 ? "1" : "0";
        }

        // grep
        if (ox.util.isFunction(this.options.grep)) {
            this.grep = this.options.grep;
        }

        var TYPE = this.options.type;
        var self = this;

        this.css({ padding: "10px 0px 10px 0px" });

        // node parser
        this.setNodeParser(function (data) {
            // root node fix
            if (data){ // fix to prevent error on console TODO fix it real
                var parent = data.id === self.options.root ? null : data.folder_id;
            }

            return {
                "id": data.id, "parent": parent, "name": data.title,
                "type": data.module, "hasChildren": data.subfolders
            };
        });

        // this tree displays all folders
        this.customize = function (node) {

            var messagingServices = {
                    "com.openexchange.messaging.facebook": {
                        icon: "facebook.png"
                    },
                     "com.openexchange.messaging.rss": {
                        icon: "rss.png"
                    },
                    "com.openexchange.messaging.twitter": {
                        icon: "twitter.png"
                    }
            };

            var data = node.data, api = ox.api.folder, id;
            var name = data.title;

            // clear all custom styles
            $(node.dom.title).parent().css({ fontWeight: "" });
            $(node.dom.icon).css({ backgroundImage: "" });

            // add style
            var processUnread = function (meta) {
                if (data.unread !== null && data.unread > 0) {
                    $(node.dom.title).parent().css({ fontWeight: "bold" });
                    meta.name = meta.name;
                    // inner-HTML is evil, so wrap it all in a text node
                    meta.html = newtext(meta.name + " (" + data.unread + ")");
                }
                return meta;
            };

            // top folder?
            if ((this.options.tree === 0 && data.folder_id === this.options.root) ||
                    (ox.api.folder.is("account", data)) ) {
                src = this.options.icon;
                if (ox.api.folder.is("account", data)) {
                    // auto-open external accounts
                    node.selectable = false;
                    
                    // click on root of mailaccounts should always select the INBOX
                    // need to temporary overide next
                    var account = ox.api.account.get(ox.api.account.derive(data.id));
                    if (account) {
                        var prevnext = node.next;
                        node.next = function(skipChildren) {
                            if (this.statusOpen && this.children.length > 0) {
                                // iterate through children to search for INBOX node
                                for (var i = 0; i < this.children.length; i++) {
                                    if (this.children[i].id === account.inbox_fullname) {
                                        return this.children[i];
                                    }
                                }
                            }
                            // use default
                            prevnext();
                        };
                    }
                }
            }
            
            var src = "folder_closed" /* default icon */, accountId, a = {};
            // get account id
            accountId = /^default(\d*)\b/.exec(data.id);

            if ($.isArray(accountId)) {
                // default0?
                if (accountId[1] === "0") {
                    // use config from api!
                    var config = ox.api.config.get();
                    var f = config.mail.folder;
                    if (f) { // might be undefined when mail module is down
                        a.inbox_fullname = f.inbox;
                        a.trash_fullname = f.trash;
                        a.sent_fullname = f.sent;
                        a.drafts_fullname = f.drafts;
                        a.spam_fullname = f.spam;
                    }
                } else {
                    // get account data from cache
                    a = ox.api.cache.account.get(accountId[1]);
                    id = "default" + accountId[1];
                    // unified inbox, e.g. behaves like an account but is not
                    if (!a) {
                        a = {
                            inbox_fullname: id + "/INBOX",
                            trash_fullname: id + "/Trash",
                            sent_fullname: id + "/Sent",
                            drafts_fullname: id + "/Drafts",
                            spam_fullname: id + "/Spam"
                        };
                    }
                    // inbox still not set?
                    if (a.inbox_fullname === undefined) {
                        a.inbox_fullname = id + "/INBOX";
                    }
                }
            }
            
            // iterate through all modules (mail, calendar, system etc..)
            switch (data.module){
            // "src" is the icon to be displayed
            case "system" :
                if (ox.api.folder.is("account", data)) { // is mail account?
                    src= "mail";
                } else {
                    switch(data.id) {
                        case "1":
                            src = "user";
                            break; // show user symbol for static id 1
                        case "2":
                            src = "public";
                            node.selectable = false;
                            break; // show user public folder
                        case "3":
                            src = "shared";
                            node.selectable = false;
                            break; // show shared folder id
                    }
                  }
                if(/u:/.test(data.id)) {
                    src = "user";
                }
                break;
            case "mail" :   // special mail folder?

                switch (data.id) {
                case a.inbox_fullname: src = "inbox";break;
                case a.trash_fullname: src = "garbage";break;
                case a.sent_fullname: src = "outbox"; break;
                case a.drafts_fullname: src = "draft"; break;
                case a.spam_fullname: src = "spam"; break;
                case a.confirmed_ham_fullname: src = "ham"; break;
                case a.confirmed_spam_fullname:  src = "spam"; break;
                default:
                    // special mail folders
                    // TODO nicening, remove unneeded else block
                    if ((api.is("system", data) || api.can("read", data)) && node.isCut !== true) {
                        // ok
                        src = node.statusOpen && node.hasChildren ? "folder_opened" : "folder_closed";
                    } else {
                        // cannot read
                        src = node.statusOpen && node.hasChildren ? "folder_opened" : "folder_closed";
                    }
                }
            break;
            case "contacts":
                src = "contacts";
                $(node.dom.title.parentNode).removeClass("font-style-low");

                break;
            case "tasks":
                src = "tasks";
                $(node.dom.title.parentNode).removeClass("font-style-low");
                break;

            case "calendar":
                src = "calendar";
                $(node.dom.title.parentNode).removeClass("font-style-low");
                break;

            case "infostore":
                if (data.folder_id === "10"){
                    src = "user";
                } else {
                    src = "infostore";
                    $(node.dom.title.parentNode).removeClass("font-style-low");

                    if (data.id === "9" || data.id === "10" || data.id === "14" || data.id === "15") {

                        node.selectable = false;
                    }
                }
                break;
            case "messaging":
                  var service = /^(.*?):/.exec(data.id);
                  if (service){
                     src = messagingServices[service[1]].icon;
                  }
                break;
            default:
                // nothing yet
                break;
            }
            // complete image source to grayish or colored icons
            // also text-style to gray or gray italic
            if (this.options.type === undefined  || (this.options.type == "") || (data.module === this.options.type )
                    || (data.folder_id === "0") || (data.module === "system" && src ==="mail" && this.options.type === "mail")) {

                // sufficient permissions?
                var sufficientPermissions = api.can("read", data);
                // explicit permission check?
                if (this.options.minPerms !== undefined) {
                    var perms = data.own_rights;
                    var mask = this.options.minPerms || 0;
                    if (   (perms       & 127) < (mask       & 127)
                        || (perms >>  7 & 127) < (mask >>  7 & 127)
                        || (perms >> 14 & 127) < (mask >> 14 & 127)
                        || (perms >> 21 & 127) < (mask >> 21 & 127)
                        || (perms >> 28 &   1) < (mask >> 28 &   1)) {
                        // insufficient permissions
                        sufficientPermissions = false;
                    }
                }
                // folder has no read and write access
                if (!sufficientPermissions && data.module !== "system" &&
                    data.module !== "infostore" || node.isCut)
                {
                    src = src + "_dis.png"; // disable symbol
                    $(node.dom.title.parentNode).addClass("font-color-disabled font-style-low"); // gray italic
                } else {
                    if(data.module != "messaging"){
                        src = src + ".png"; //standard icon, messaging has own png
                    }

                    $(node.dom.title.parentNode).removeClass("font-color-disabled"); // remove gray text

                    if (ox.api.folder.is("published", data)) {// published folders
                        // add shared symbol to original
                        $(node.dom.icon).css({"background-image" : "url(" + ox.gui.themePath + "icons/16/" +src+ ")" });
                        src = "shared_globe.png" ;
                    }
                }
            } else {
                if(data.module != "messaging"){
                    src = src + "_dis.png"; //standard icon, messaging has own png
                }
                $(node.dom.title.parentNode).addClass("font-color-disabled"); // for module selection also gray out every unwanted folders
            }

            if (data.id === "default0") {
                // Return a text node which is asynchronously filled with
                // the user's display name.
                var titlenode = newtext(name);
                internalCache.getUsers([config.identifier], function(users) {
                    titlenode.data = users[config.identifier].display_name;
                });
                return {
                    src: ox.gui.themePath + "icons/16/" + src,
                    html: titlenode
                };
            } else {
                return processUnread({
                    src: ox.gui.themePath + "icons/16/" + src,
                    name: name
                });
            }
        };

        this.selection.classSelected = "background-color-PMG-selection-elements";
        this.showRootNode = false;

        // add root node
        this.setRootNode({
            id: this.options.root,
            folder_id: null,
            title: "Root",
            module: "root",
            subfolders: true
        });

        var ID = this.id + "/" + this.options.tree;

        var fnToggle = function () {
            // loop over open nodes
            var state = {};
            $.each(self.getOpen(), function (i, id) {
                state[id] = true;
            });
            // update user config
            configSetKey(["gui", "tree", ID, "state"], state);
        };
        this.addListener("widget:open", fnToggle);
        this.addListener("widget:close", fnToggle);

        // get open folders
        if (configContainsKey(["gui", "tree", ID, "state"])) {
            this.setOpen(ox.util.keys(configGetKey(["gui", "tree", ID, "state"])));
        } else {
            this.setOpen(this.options.open || []);
        }

        this.onModify = function (data) {
            var finalOpenNodes = [];

            // close invalid nodes
            if (data.why === "folder.remove" || data.why === "folder.move") {
                // is mail?
                if (ox.api.folder.is("mail", data.folder)) {
                    // get id
                    var key = data.oldId;
                    // close invalid nodes
                    $.each(ox.util.keys(self.openNodes).concat(self.getOpen()), function (i, id) {
                        if ((new RegExp("^" + key)).test(id)) {
                            // get tree node
                            var node = self.get(id);
                            if (node) {
                                // required to re-open later
                                finalOpenNodes.push(id.replace(key, data.id));
                                // close
                                node.close();
                            }
                        }
                    });
                }
                // unset current folder
                ox.UIController.unsetFolder();

            } else if (data.why !== "folder.refresh") {

                // affects current mail folder?
                var folder = ox.UIController.getFolder("mail"),
                    tree = ox.api.config.get("modules.folder.tree", 0),
                    cache = ox.api.cache["folder" + tree];
                // is removed?
                if (folder !== undefined && !cache.contains(folder)) {
                    // unset folder
                    ox.UIController.unsetFolder("mail");
                }
            }

            if (data.why === "folder.update.local") {
                // repaint affected node only
                var node = self.get(data.id);
                if (node) {
                    node.repaint();
                }
            } else {
                if (data.why === "folder.move") {
                    ox.UIController.setFolder({
                        folder: data.id,
                        module: "default",
                        success: function () {
                            // reopen node
                            if (self.setOpen) self.setOpen(finalOpenNodes);
                            self.repaint(function() {
                                // clean up
                                finalOpenNodes = null;
                            });
                        }
                    });
                } else {
                    // repaint full tree
                    self.repaint();
                }
            }
        };
    },
    
    destroy: function () {
        ox.api.folder.dispatcher.unbind("modify", this.onModify);
        return ox.gui.Tree.prototype.destroy.call(this);
    },

    loadChildren: function (id, cont) {
        var self = this;
        ox.api.folder.getSubFolders({
            folder: id,
            tree: this.options.tree,
            success: function (data) {
                // subscribed folders only
                data = $.grep(data, function (elem, i) {
                    return elem.subscribed === true || elem.subscr_subflds === true;
                });
                cont(data);
            },
            error: function (error) {
                cont(false);
            }
        });
    },

    expandToFolder: function (folderID) {

        // get open nodes
        var open = this.getOpen();

        // this
        var self = this;

        var getParentIDs = function (data, callback) {
            // loop over folder
            var i = 1, $l = data.length, list = [], id;
            for (; i < $l; i++) {
                id = data[i].id;
                // not already open?
                if ($.inArray(id, open) === -1) {
                    list.push(id);
                }
            }
            callback(list);
        };

        var expandTreeNodes = function (list) {

            // need repaint?
            if (list.length > 0) {
                // yes
                self.setOpen(list);
                self.repaint(function () {
                    self.selection.clickQuiet(folderID);
                });
            } else {
                // no, select directly
                self.selection.clickQuiet(folderID);
            }
        };

        // get all parents of folder via api
        ox.api.folder.getParents({
            folder: folderID,
            success: function (data){
                getParentIDs(data, expandTreeNodes);
            }
        });
    },

    paint: function (cont) {
        if (this.painter.running === false) {

            this.painter.running = true;

            // first paint?
            if (this.firstPaint === true) {
                // get folder state id
                var ID = this.id + "/" + ox.api.config.get("modules.folder.tree", 0);
                // get open folders
                if (configContainsKey(["gui", "tree", ID, "state"])) {
                    this.setOpen(ox.util.keys(configGetKey(["gui", "tree", ID, "state"])));
                } else {
                    this.setOpen(this.options.open || []);
                }
                // visually busy
                $(this.dom.node).css({ background: "" }).addClass("oxBusySmall");
                // done
                this.firstPaint = false;
            }

            // unbind
            ox.api.folder.dispatcher.unbind("modify", this.onModify);

            // this
            var self = this;

            // continuation
            var step2 = function () {
                // get ids of open nodes (exclude external accounts)
                var list = $.grep(ox.util.keys(self.openNodes), function (key) {
                    return /^(\d+|default0)/.test(key);
                });
                // pause
                ox.api.http.pause();
                var pssst = function () {
                    return true; // prevent global error
                };
                // loop
                var i = 0, $l = list.length;
                for (; i < $l; i++) {
                    ox.api.folder.getSubFolders({
                        folder: list[i],
                        tree: self.options.tree,
                        error: pssst
                    });
                }
                // resume
                ox.api.http.resume(
                    function () {
                        // paint
                        $(self.dom.node).empty().removeClass("oxBusySmall");
                        self.painter.running = false;
                        ox.gui.Tree.prototype.paint.call(self, function () {
                            // clear open nodes hash
                            self.openNodes = {};
                            // select id
                            if (self.selection !== undefined && self.selection.auto !== false) {
                                self.selection.preselect(currentFolder); // before paint
                                self.selection.clickQuiet(currentFolder); // after paint
                            }
                            // bind
                            ox.api.folder.dispatcher.bind("modify", self.onModify, window);
                            // continue
                            ox.util.call(cont);
                        });
                    },
                    pssst
                );
            };

            // consider current folder (not in nested windows)
            var currentFolder = ox.UIController ? ox.UIController.getFolder() : undefined;
            if (currentFolder !== undefined) {
                // get path
                ox.api.folder.getParents({
                    folder: currentFolder,
                    tree: this.options.tree,
                    success: function (data) {
                        // remove current folder
                        data.shift();
                        // get ids
                        var list = $.map(data, function (folder) {
                            return folder.id;
                        });
                        // leads to root folder?
                        if ($.inArray(self.options.root, list) > -1) {
                            // mark as open
                            self.setOpen(list);
                        }
                        // continue
                        step2();
                    }
                });
            } else {
                // continue
                step2();
            }
        } else {
            this.painter.queue.push(cont);
        }
    }
});

// extend API
ox.api.ui.selectFolder = (function () {

    return function (options) {
        var opt = $.extend({
            root: ox.api.config.get("modules.folder.tree", 0),
            callback: jQuery.noop
        }, options || {});

        // clear
        removeChildNodes(opt.node);
        
        // create instance (again)
        var tree = new ox.gui.FolderTreePopup(opt);
        tree.setParentDOMNode(opt.node);
        tree.selection.auto = false;
        tree.paint();
        return tree;
    };
}());

/*
 * Tree view for popup folder tree (like move objects)
 * ------------------------------------------------------------------------------------
 */

ox.gui.FolderTreePopup = ox.gui.FolderTree.extend({

    construct: function (options) {

        var opt = $.extend({
            destroy: true
        }, options || {});

        ox.gui.FolderTree.prototype.construct.call(this, opt.id || null, opt);

        var callback = opt.callback, self = this;

        this.addListener("widget:select", function (e) {
            var api = ox.api.folder;
            api.get({
                folder: e.data,
                success: function (data) {
                    // typed tree?
                    if (opt.type !== undefined) {
                        if (opt.type === data.module) {
                            callback(data);
                            if (opt.destroy === true) {
                                self.destroy();
                            }
                        }
                    } else if (data.module !== "system") {
                        callback(data);
                        if (opt.destroy === true) {
                            self.destroy();
                        }
                    }
                }
            });
        });
    }
});

ox.gui.initFolderTree = function () {

   var tree = ox.widgets.folderTree = new ox.gui.FolderTree("ox-sidepanel-foldertree", {
       type: "", root: ox.api.config.get("modules.folder.tree") === 1 ? "1" : "0"
   });

   tree.enableDnD(ox.gui.initSidepanel.dndSource,
                  ox.gui.initSidepanel.dndTarget);

   ox.widgets.sidepanelViews.folderTree = tree;

   // add listeners
   tree.addListener("widget:select", function(e) {
       var api = ox.api.folder;
       var self = this;
       // get folder data
       api.get({
           folder: e.data,
           success: function (data) {
               // TODO: check whether "someone" meant folder_id === 1?
               if (data.id !== 2 && data.id !== 3 && data.id !== 9 && data.id !== 10 && data.id !== 14 && data.id !== 15) {
                   //not system?
                   if (!api.is("system", data) && api.can("read", data)) {
                       // "normal" folder
                       ox.UIController.setModule({ folder: data.id, module: data.module });
                   } else if (e.data === "1") {
                       // jump to portal
                       ox.UIController.setModule({ module: "portal" });
                   } else if(e.data === "6"){
                       // GAL fix
                       ox.UIController.setModule({ folder: "6", module: "contacts" });
                   }
               }
           }
       });
   });

   tree.selection.onRename(function (id, item) {
       // get folder
       ox.api.folder.get({
           folder: id,
           success: function (data) {
               // can rename?
               if (ox.api.folder.can("rename", data)) {
                   // start edit
                   tree.startEdit(id);
               }
           }
       });
   });

   /**
    * Context Menu via Left Click
    */
   tree.selection.dispatcher.bind("selected", function(e) {
       // add context button
       if (e.node.find(".context-button").length === 0) {
            var button = $.button({
                title: "\u00a0\u00a0"
            }).bind("click", function(event) {
                tree.selection.triggerContextMenu(e.id,  event);
                return false;
            });
            if (IE <= 7) {
                button.css("width", "");
            }
            if (IE) {
                button = $('<form onsubmit="return false"></form>')
                    .css("margin", "0")
                    .append(button);
            }
            var td = $("<td/>").css({
                width: "20px",
                textAlign: "right",
                paddingRight: "5px"
            }).addClass("context-button").append(button);
            if (IE <= 7) {
                td.css({ "width": "20px" });
            }
            e.node.find("tr").append(td);
       } else {
           e.node.find(".context-button").show();
       }
   });
   tree.selection.dispatcher.bind("deselected", function(e) {
       // remove context button
       e.node.find(".context-button").hide();
   });

   tree.addListener("widget:edited", function (e) {
       // update folder
       var self = this;
       ox.api.folder.update({
           folder: e.data.id,
           data: { title: e.data.value },
           success: function (data) {

               // get current tree id
               var ID = self.id + "/" + ox.api.config.get("modules.folder.tree", 0);
               // has parent node already been opened?
               if (configGetKey(["gui", "tree", ID, "state", e.data.id]) === true) {
                   // remove previous open/close state
                   configRemoveKey(["gui", "tree", ID, "state", e.data.id]);
                   // update user config
                   configSetKey(["gui", "tree", ID, "state", data.id], true);
                   // set to open
                   self.setOpen([data.id]);
               }

                ox.api.folder.refresh({ folders: [e.data.data.folder_id],
                                        success: e.cont });
           },
           error : function () {
               // repaint
               self.repaint();
               // select
               self.selection.select(e.data.id);
               // continue
               ox.util.call(e.cont);
           }
       });
       return false;
   });

   // click handler
   tree.selection.setClickHandler(function (e) {

       // selection disabled
       if (this.statusEnabled === false) {
           return false;
       }

       // context menu?
       if (e.event.rightClick) {

           // tracking
           track({
               type: "context_menu",
               dest: "tree",
               data: { id: e.id },
               text: "Right click on folder " + e.id
           });
           // cont
           e.click("quiet");
           e.context();
       } else if (e.selectable === true && e.id !== "-1") {
           // viewing properties?
           if (currentpath[0] === "configuration") {
               // change back to folder's module
               ox.UIController.setFolder({
                   folder: e.id,
                   module: "default"
               });
           } else {
               // normal click
               // track
               ox.api.folder.get({
                   folder: e.id,
                   success: function (data) {
                       track({
                           type: "click",
                           dest: "tree",
                           data: { id: data.id, title: data.title },
                           text: "Select " + data.module + " folder \"" + data.title + "\" (" + e.id + ")"
                       });
                   }
               });
               if (ox.UIController.getFolder() !== e.id) {
                   e.click("force");
               } else {
                   e.click();
               }
           }
       }

       // prevent default
       return false;
   });

   // context menu
   tree.selection.setContextMenuHandler(function (id, e) {
       // show folder context menu
       var menu = globalContextMenus.simpleFolderMenu;
       menu.display(e.pageX, e.pageY, {
           data: { id: id },
           api: true,
           simple: true,
           module: tree.options.type,
           widget: tree
       });
   });

   // do not show root node
   tree.showRootNode = false;

   // add
   ox.widgets.sidepanelViews.folder.insert(tree, 0);
   tree.setLayoutParam({ position: "center" });

   /*
    EXAMPLE USAGE
    ***********************************************************

    masterFolderTree = new ox.gui.FolderTree(null, {
        type: "mail", root: "0"
    });

    var testKnoten = $("<div>").css({
        position: "absolute", top: "30px", right: "250px", bottom: "50px", left: "350px", zIndex: 65000,
        border: "5px solid #fc0", backgroundColor: "white"
    }).appendTo(document.body);

    masterFolderTree.setParentDOMNode(testKnoten[0]);

    masterFolderTree.paint();

    END EXAMPLE ***************************************
 */

   // check for plugin
   if (ox.api.config.get("modules.mail.module") === true &&
		   ox.mail.addAccount.available()) {
       // add extension
       tree.addListener("widget:painted", function () {
           var node = $(this.dom.node);
           if (node.find("a.add-account").length === 0) {
               node.append(
                   $('<div>')
                   .css({ lineHeight: '2em', margin: '0 15px 0 15px' })
                   .append(
                       $('<a>', { href: '#' })
                       .addClass('common-link add-account')
                       .append(addTranslated(_("Add E-mail account ...")))
                       .bind('click', function (e) {
                           e.preventDefault();
                           // enabled?
                           if (ox.api.config.get("modules.mailaccount.module", false)) {
                               ox.mail.addAccount();
                           } else {
                               triggerEvent("Feature_Not_Available", "configuration/mail/accounts/new", window, "foldertree");
                           }
                       })
                   )
               );
           }
       });
   }
};
/**
 * 
 * All content on this website (including text, images, source
 * code and any other original works), unless otherwise noted,
 * is licensed under a Creative Commons License.
 * 
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 * 
 * Copyright (C) Open-Xchange Inc., 2006-2012
 * Mail: info@open-xchange.com 
 * 
 * @author Matthias Biggeleben <matthias.biggeleben@open-xchange.com>
 * @author Stefan Preuss <stefan.preuss@open-xchange.com>
 * 
 */

ox.gui.initConfigTree = function () {
	
	// register in global repository
	var configTree = ox.widgets.configTree = new ox.gui.Tree("ox-sidepanel-configuration").
	css({ padding: "10px 0px 10px 0px" }).
	addTo(ox.widgets.sidepanelViewControl);
	
	ox.widgets.sidepanelViews.configuration = configTree;
	
    configTree.showRootNode = false;
    
    configTree.customize = function (node) {
        var icon;
        if (node.data.innerNode) {
            node.selectable = false;
            icon = "/img/folder/settings_folder_" + (node.statusOpen ? "open.gif" : "closed.gif");
        } else {
            icon = "/img/folder/settings.gif";
        }
        return { src: ox.gui.themePath + icon, html: node.data.domNode || addTranslated(node.name) };
    };
    
    configTree.addListener("widget:select", function (event) {    	
        var data = this.nodeIndex[event.data].data;
        if (data.onClick) {
            data.onClick();
        } else {
            configuration_askforSave(function () {
                triggerEvent("OX_Switch_View", event.data);
            });
        }
    });
	
    // must initialize the config tree right now, since plugins might call for it
    configTree.setRootNode({ id: "configuration", name: "Configuration", /*i18n*/ innerNode: true });
    configTree.add({ parent: "configuration", id: "configuration/import", name: "Import" /*i18n*/ });
    configTree.add({ parent: "configuration", id: "configuration/settings", name: "Options" /*i18n*/ });
    configTree.add({ parent: "configuration", id: "configuration/tags", name: "Categories" /*i18n*/ });

    // Module Portal
    if (configGetKey("modules.portal.module")) {
        configTree.add({ parent: "configuration", id: "configuration/portal", name: "Start Page", /*i18n*/ innerNode: true });
        configTree.add({ parent: "configuration/portal", id: "configuration/portal/layout", name: "Layout" /*i18n*/ });
        if (ox.api.config.get("modules.uwaWidgets.enabled") === true) {
            configTree.add({ parent: "configuration/portal", id: "configuration/portal/external", name: "UWA Modules" /*i18n*/ });
        }
    }

    // Module Mail
    if (configGetKey("modules.mail.module") === true || configGetKey("modules.mail.module") !== null && configGetKey("modules.mailaccount.module")) {
        configTree.add({ parent: "configuration", id: "configuration/mail", name: "E-Mail", /*i18n*/ innerNode: true });
        if (configGetKey("modules.mail.module")) {
            configTree.add({ parent: "configuration/mail", id: "configuration/mail/preferences", name: "Preferences" /*i18n*/ });
            configTree.add({ parent: "configuration/mail", id: "configuration/mail/signatures", name: "Signatures" /*i18n*/ });
        }
    }

    // Module Calendar
    if (configGetKey("modules.calendar.module")) {
        configTree.add({ parent: "configuration", id: "configuration/calendar", name: "Calendar", /*i18n*/ innerNode: true });
        configTree.add({ parent: "configuration/calendar", id: "configuration/calendar/preferences", name: "Preferences" /*i18n*/ });
        if (configGetKey("modules.calendar.calendar_teamview")) {
            configTree.add({ parent: "configuration/calendar", id: "configuration/calendar/teams", name: "Teams" /*i18n*/ });
        }
    }

    // Module Contacts
    if (configGetKey("modules.contacts.module")) {
        configTree.add({ parent: "configuration", id: "configuration/contacts", name: "Contacts", /*i18n*/ innerNode: true });
        configTree.add({ parent: "configuration/contacts", id: "configuration/contacts/preferences", name: "Preferences" /*i18n*/ });
    }

    // Module Tasks
    if (configGetKey("modules.tasks.module")) {  
        configTree.add({ parent: "configuration", id: "configuration/tasks", name: "Tasks", /*i18n*/ innerNode: true });
        configTree.add({ parent: "configuration/tasks", id: "configuration/tasks/preferences", name: "Preferences" /*i18n*/ });
    }

    // Module InfoStore
    if (configGetKey("modules.infostore.module")) {
        configTree.add({ parent: "configuration", id: "configuration/infostore", name: "InfoStore", /*i18n*/ innerNode: true });
        configTree.add({ parent: "configuration/infostore", id: "configuration/infostore/preferences", name: "Preferences" /*i18n*/ });
    }
    
    configTree.selection.classSelected = "background-color-PMG-selection-elements";
    
};
/**
 * 
 * All content on this website (including text, images, source
 * code and any other original works), unless otherwise noted,
 * is licensed under a Creative Commons License.
 * 
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 * 
 * Copyright (C) Open-Xchange Inc., 2006-2012
 * Mail: info@open-xchange.com 
 * 
 * @author Matthias Biggeleben <matthias.biggeleben@open-xchange.com>
 * 
 */

ox.gui.initController = function () {

    // set to true for debugging module/folder switch issues
    if (false) {
        ox.UIController.dispatcher.bind("modulechange", function (e) {
            console.warn("modulechange", e);
        });
        ox.UIController.dispatcher.bind("folderchange", function (e) {
            console.warn("folderchange", e);
        });
    }
    
    // change view on module change
    ox.UIController.dispatcher.bind("modulechange", function (e) {
        
        var module = e.module;
        var folder = e.folder;
        var view;
        
        // search field (clear first; mail & contacts set fields afterwards)
        ox.widgets.searchField.active().searchMenu.setFields();
        
        // sidepanel & search bar
        if (module === "portal") {
            ox.widgets.miniCalendar.show();
            ox.widgets.searchField.top.disable();
            view = ox.UIController.showExpertTree ? "foldertree" : "portal";
            ox.widgets.sidepanelViewControl.showView("ox-sidepanel-" + view);
            ox.widgets.sidepanel.toggle(!ox.UIController.showExpertTree);
        } else if (module === "configuration") {
            ox.widgets.miniCalendar.hide();
            ox.widgets.searchField.top.disable();
            ox.widgets.sidepanel.toggle(false);
        } else {
            // collapsed by user?
            if (ox.widgets.sidepanel.userToggle === true) {
                ox.widgets.sidepanel.toggle(true);
            } else {
                ox.widgets.sidepanel.toggle(false);
            }
            ox.widgets.miniCalendar.show();
            // search field
            ox.widgets.searchField.top.enable();
        }
        
        // toggle calendar always when module calendar
        if (module === "calendar") {
            ox.widgets.miniCalendar.toggle(true);
        } else {
            ox.widgets.miniCalendar.refreshState();
        }
        
        var module = ox.UIController.getModule();
        var folder = ox.UIController.getFolder();
        
        // SPAM folder?
        if (module === "mail" && folder === ox.api.folder.isStdFolder("spam", folder)) {
            // SPAM is always in list view
            changeView(ox.api.config.get("gui.mail.view_spam", "mail/list/unthreaded"), "newfolder");
        }
        else if (e.view !== undefined) {
            // change to specific view
            changeView(e.view, module === "configuration" ? "samefolder" : "newfolder");
        }
        else if (menulastviews[module] !== undefined) {
            // change to module
            changeView(menulastviews[module], module === "configuration" ? "samefolder" : "newfolder");
        }
        else {
            alert("Unknown Module: " + module);
        }
        
        // jump to or from configuration? (doesn't trigger folder change since config has no default folder)
        if (e.change === "module" || e.change == "both") {
            if (module === "configuration") {
                // jump in
                ox.widgets.sidepanelViewControl.showView("ox-sidepanel-configuration");
                ox.widgets.configTree.selection.click(menulastviews[module], false, false, true);
            } else if (e.previous.module === "configuration") {
                // jump away
                ox.widgets.sidepanelViewControl.showView("ox-sidepanel-" 
                        + (ox.UIController.showExpertTree ? "foldertree" : module));
            }
        }
        
        // resize toolbar
        ox.ToolBarController.initialResize();
    });
    
    // change view on folder change
    ox.UIController.dispatcher.bind("folderchange", function (e) {
        
        var module = ox.UIController.getModule();
        var folder = ox.UIController.getFolder();
        
        if (e.change === "folder") {
            
            // SPAM folder?
            if (module === "mail" && ox.api.folder.isStdFolder("spam", folder)) {
                // SPAM is always in list view
                changeView(ox.api.config.get("gui.mail.view_spam", "mail/list/unthreaded"), "newfolder");
            }
            else if (e.view !== undefined) {
                // change to specific view
                changeView(e.view, module === "configuration" ? "samefolder" : "newfolder");
            }
            else if (menulastviews[module] !== undefined) {
                // change to module
                changeView(menulastviews[module], e.change === "folder" || e.change === "both" ? "newfolder" : "samefolder");
            }
            else {
                alert("Unknown Module: " + module);
            }
        }
        
        // update sidepanel
        var id = ox.UIController.showExpertTree && module !== "configuration" ? "foldertree" : module;
        var viewControl = ox.widgets.sidepanelViewControl;
        viewControl.showView("ox-sidepanel-" + id);
        // update proper widget
        var views = ox.widgets.sidepanelViews;
        var map = {
            "mail": views.mailTree,
            "calendar": views.calendar,
            "contacts": views.contacts,
            "tasks": views.tasks,
            "infostore": views.infostoreTree,
            "foldertree": views.folderTree
        };
        if (map[id] !== undefined) {
            if (map[id].expandToFolder) {
                // trees
                map[id].expandToFolder(folder);
            } else {
                // flat views
                map[id].selection.clickQuiet(folder);
            }
        }
    });
};
/**
 * 
 * All content on this website (including text, images, source
 * code and any other original works), unless otherwise noted,
 * is licensed under a Creative Commons License.
 * 
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 * 
 * Copyright (C) Open-Xchange Inc., 2006-2012
 * Mail: info@open-xchange.com 
 * 
 * @author Matthias Biggeleben <matthias.biggeleben@open-xchange.com>
 * 
 */

/*jslint bitwise: false, nomen: false, onevar: false, plusplus: false, regexp: false, white: true, browser: true, devel: true, evil: true, forin: true, undef: true, eqeqeq: true, immed: true */

/*global ox, jQuery, $, equals, register, triggerEvent, menuswitchMailView */

ox.gui.initToolBar = function () {

    /*
     * namespaces for toolbars (for easy global access)
     */
    ox.widgets.toolBar = {
        viewControl: undefined, // view controller that switched between all toolbar instances, e.g. mail, calendar, contacts.
        views: {}, // all toolbar instances
        meta: {}, // all meta data
        tabs: {}, // all tab instances, e.g. "mail-edit"
        sections: {}, // all section instances
        menuItems: {} // all menu item instances
    };
    
    /**
     * Short description of meta objects
     * title {String} The title/name/label of a menu item
     * icons {Array} Icons of that menu item. The first one is the enabled icon, the second is the disabled one.
     * big {Boolean} True if the icon should be a large one
     * requires {Array|function} Either an array of requirements (see ability object; below) or a function.
     * 
     * Example for requires: 
     *     requires: ["some", "modify"]
     *     This says that the menu items need some selected elements (> 0) and write privileges.
     *     requires: ["one"]
     *     A menu item is only enabled if one element is selected (=== 1)
     *     
     * Using a function: [function (ability, selection, cont)]
     *     When using a function, it is called with the ability object as first
     *     and the current selection as second parameter.
     *     Must either return true or call the cont function with the parameter
     *     true to enable the menu item.
     *     
     */
    
    /**
     * Ability object
     * @example
     * ability = {
     *   "read": true,
     *   "modify": false,
     *   "delete": false,
     *   "some": true,
     *   "one": false,
     *   "none": false,
     *   "multiple": true
     * };
     */
    
    // upsell namespace
    var upsell = ox.api.window.core.ox.upsell;
    
    /*
     * Common meta data
     */
    var meta = ox.widgets.toolBar.meta = {
        /*
         * Create
         */
        create: {
            "mail": {
                title: _("New E-Mail"),
                abbr: _("E-Mail"),
                icons: ["mod_mail.png"],
                big: true,
                requires: ["mail"],
                mousedown: function (options) {
                    if (ox.api.ui.getModule("mail").disabled) {
                        triggerEvent("Feature_Not_Available", "modules/mail", window);
                    }
                },
                visible: upsell.visible("mail")
            },
            "calendar": {
                title: _("New appointment"),
                abbr: _("Appointment"),
                icons: ["mod_calendar.png"],
                big: true,
                requires: ["calendar"],
                mousedown: function(options) {
                    if (ox.api.ui.getModule("calendar").disabled) {
                        triggerEvent("Feature_Not_Available", "modules/calendar", window);
                    }
                },
                visible: upsell.visible("calendar")
            },
            "contact": {
                title: _("New contact"),
                abbr: _("Contact"),
                icons: ["mod_contacts.png"],
                big: true,
                requires: ["contacts"],
                mousedown: function(options) {
                    if (ox.api.ui.getModule("contacts").disabled) {
                        triggerEvent("Feature_Not_Available", "modules/contacts", window);
                    }
                },
                visible: upsell.visible("contacts")
            },
            "distributionlist": {
                title: _("New distribution list"),
                abbr: _("Distribution list"),
                icons: ["distributionlist.png"],
                big: true,
                requires: ["contacts"],
                mousedown: function(options) {
                    if (ox.api.ui.getModule("contacts").disabled) {
                        triggerEvent("Feature_Not_Available", "modules/contacts", window);
                    }
                },
                visible: upsell.visible("contacts")
            },
            "task": {
                title: _("New task"),
                abbr: _("Task"),
                icons: ["mod_tasks.png"],
                big: true,
                requires: ["tasks"],
                mousedown: function(options) {
                    if (ox.api.ui.getModule("tasks").disabled) {
                        triggerEvent("Feature_Not_Available", "modules/tasks", window);
                    }
                },
                visible: upsell.visible("tasks")
            },
            "file": {
                title: _("New Infoitem"),
                abbr: _("Infoitem"),
                icons: ["mod_infostore.png"],
                big: true,
                requires: ["infostore"],
                mousedown: function(options) {
                    if (ox.api.ui.getModule("infostore").disabled) {
                        triggerEvent("Feature_Not_Available", "modules/infostore", window);
                    }
                },
                visible: upsell.visible("infostore")
            }
        },
        /*
         * View
         */
        view: {
            "vsplit": {
                title: _("V-Split"),
                icons: ["v_split_view.png"],
                big: true,
                behavior: "radio",
                group: "split-views",
                checked: function () {
                    return currentpath[1] === "vsplit";
                }
            },
            "hsplit": {
                title: _("H-Split"),
                icons: ["h_split_view.png"],
                big: true,
                behavior: "radio",
                group: "split-views",
                checked: function () {
                    return currentpath[1] === "hsplit";
                }
            },
            "list": {
                title: _("List"),
                icons: ["list_view.png"],
                big: true,
                behavior: "radio",
                group: "split-views",
                checked: function () {
                    return currentpath[1] === "list";
                }
            }
        }
    };
    
    /*
     * Flags
     */
    meta.flags = {};
    meta.flags["0"] = {
        title: _("Remove flag"),
        icons: ["tag_remove.png"],
        big: true,
        requires: ["some", "modify"]
    };
    for (var i = 1; i <= 10; i++) {
        meta.flags["" + i] = {
            number: i,
            //#. %d is the flag number
            title: format(_("Flag #%d"), i),
            icons: ["tag_" + i + ".png"],
            big: true,
            requires: ["some", "modify"],
            behaviour: "radio",
            group: "flags-radio",
            checked : function (ability,selection) {
                // show radio state just for one seleted item
                if (currentpath[0] === "mail" && ability.one) {
                    return selection[0].color_label == this.number;
                } else false;
            }
        };
    }
    
    meta.socialOX = {
            refresh: {
                title: _("Refresh"),
                icons: [ "img/toolbar/tb_loading.gif" ],
                group: "social-ox",
                requires: function() {
                    return false;
                },
                mousedown: function (options, e) {
                    if (ox.api.config.get(["modules","com.openexchange.subscribe"]) !== true) {
                        corewindow.triggerEvent("Feature_Not_Available", "modules/" + activemodule + "/social/refresh", window);
                    }
                }
            },
            publish: {
                title: _("Publish"),
                icons: ["dummy.gif"],
                group: "social-ox",
                requires: function() {
                    return (ox.api.config.get(["modules","com.openexchange.publish"]) === true);
                },
                action: function() { 
                    triggerEvent("Feature_Publications");
                    return true;
                },
                mousedown: function (options, e) {
                    if (ox.api.config.get(["modules","com.openexchange.publish"]) !== true) {
                        corewindow.triggerEvent("Feature_Not_Available", "modules/" + activemodule + "/social/publish", window);
                    }
                }
            },
            subscribe: {
                title: _("Subscribe"),
                icons: ["dummy.gif"],
                group: "social-ox",
                requires: function() {
                    return (ox.api.config.get(["modules","com.openexchange.subscribe"]) === true);
                },
                mousedown: function (options, e) {
                    if (ox.api.config.get(["modules","com.openexchange.subscribe"]) !== true) {
                        corewindow.triggerEvent("Feature_Not_Available", "modules/" + activemodule + "/social/subscribe", window);
                    }
                }
            }
    };
    
    
    /*
     * Helper: Mass adds menu items based on their meta data
     */
    
    // full list of toolbar menu items
    var items = ox.widgets.toolBar.menuItems = {};
    
    /**
     * Mass add menu items
     * @param {Object} The target widget
     * @param {Object} Meta data (usually some subset)
     * @param {String} A prefix that is added to all IDs
     * @param {Array} An array of all IDs that should be added
     * @example:
     *     add(section, meta.flags, "mail-flag",
     *         ["0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "10"]
     *     );
     *     This one adds menu items defined in meta.flags to a section (widget).
     *     All added items get the prefix "mail-flag-". The IDs to add are defined by the forth parameter.
     */
    var add = function (target, namespace, prefix, ids) {
        // prefix
        prefix = prefix !== "" ? prefix + "-" : "";
        // default action
        var fnAction = function (e) {
            // call controller
            return ox.ToolBarController.trigger(e.widget.id);
        };
        // loop
        var i = 0, $l = ids.length, id, options;
        var j, icons, $k,icon;
        for (; i < $l; i++) {
            id = ids[i];
            if ((options = namespace[id])) {
                // set id
                options.id = prefix + id;
                // selection name
                if (options.selection === undefined) {
                    options.selection = "default";
                }
                // set default action (unless has own)
                if (options.action === undefined) {
                    options.action = fnAction;
                }
                // fix icon paths
                if (options.icons !== undefined) {
                    for (j = 0, icons = options.icons, $k = icons.length; j < $k; j++) {
                        // contains no dash?
                        if (!/[\/]/.test(icons[j])) {
                            icons[j] = "icons/" + (options.big === true ? "24/" : "16/") + icons[j];
                        }
                    }
                }
                // add as menu item
                target.add((items[options.id] = new ox.gui.MenuItem(options)));
            } else {
                console.error("Cannot find meta data for", id, namespace);
            }
        }
    };
    
    /**
     * ToolBarController
     */
    ox.ToolBarController = (function () {
        
        // get menu items
        var getItems = function (namespace, selection, keysOnly) {
            // grep
            var id, tmp = [], m, regex = new RegExp("^" + namespace + "-"), item;
            for (id in items) {
                m = id.match(regex);
                if (m && m.length) {
                    item = items[id];
                    if (selection === undefined || (selection === item.options.selection)) {
                        tmp.push(keysOnly === true ? id : item);
                    }
                }
            }
            return tmp;
        };
        
        // satisfies requirements
        var satisfies = function (requires, ability, selection, cont) {
            // is individual function?
            if (ox.util.isFunction(requires)) {
                var result = requires(ability, selection, cont);
                if (result !== undefined) {
                    cont(result);
                }
            } 
            else if (requires === undefined || requires.length === 0) {
                // no requirements, so... true
                cont(true);
            } else {
                // loop over requirements
                var i = 0, $l = requires.length, result = true;
                for (; i < $l && result; i++) {
                    result = result && (!! ability[requires[i]]); // !! = get boolean value, might be undefined
                }
                cont(result);
            }
        };
        
        // apply
        var applySelection = function (name, ability, selection, folders) {
            
            // get affected menu items (look for current module)
            var list = getItems(ox.UIController.getModule(), name);
            
            // helper
            var isFunction = ox.util.isFunction;
            var asyncEnable = function (item) {
                return function (flag) {
                    item.setEnabled(flag);
                };
            };
            // loop over items and enable or disable them properly
            var i = 0, $l = list.length, item, options;
            for (; i < $l; i++) {
                // get item
                item = list[i];
                options = item.options;
                // proper selection?
                if (options.selection === name || (options.selection === undefined && name === "default")) {
                    // checked?
                    if (options.checked !== undefined) {
                        if (isFunction(options.checked)) {
                            item.setChecked(options.checked(ability, selection));
                        } else {
                            item.setChecked(!!options.checked);
                        }
                    } 
                    // visibility
                    if (options.visible !== undefined) {
                        item.setVisible(options.visible(ability, selection));
                    }
                    // requirements satisfied?
                    satisfies(options.requires, ability, selection, asyncEnable(item));
                }
            }
        };
        
        // prepare a modules object for extending abilities later
        var modules = { };
        var tmp = ox.api.ui.listModules();
        for (var i in tmp) {
            modules[tmp[i].name] = !tmp[i].disabled;
        }
        
        // get ability (requires that all affected folders are cached)
        var getAbility = function (selection) {
            // get id of current user
            var myself = ox.api.config.get("identifier");
            // selection length
            var $l = selection.length;
            /**
             * Ability object
             */
            var ability = {
                "read": true,
                "modify": true,
                "delete": true,
                "none": $l === 0,
                "some": $l > 0,
                "one": $l === 1,
                "multiple": $l > 1
            };
            
            $.extend(ability, modules);
            
            // shortcut
            var api = ox.api.folder;
            // helper
            var getRight = function (folder, owner, offset) {
                // get bits
                var bits = api.derive("bits", folder, offset);
                // check
                if (bits === 0) {
                    // no permissions
                    return false;
                } else if (bits === 1) {
                    // only own objects
                    return owner === myself;
                } else {
                    // all objects or admin
                    return true;
                }
            };
            // loop over selection
            var i = 0, item, folder, id;
            for (; i < $l; i++) {
                // get item
                item = selection[i];
                // get folder
                if (item) {
                    id = item.folder || item.folder_id;
                    folder = id === undefined ? undefined : ox.api.folder.get({ folder: id });
                }
                // get owner (i.e. created_by)
                if (item && folder) {
                    // get abilities
                    ability.read = ability.read && getRight(folder, item.created_by, 7); // read
                    ability.modify = ability.modify && getRight(folder, item.created_by, 14); // write
                    ability["delete"] = ability["delete"] && getRight(folder, item.created_by, 21); // delete
                } else {
                    // folder unknown
                    ability.unknown = true;
                    ability.read = false;
                    ability.modify = false;
                    ability["delete"] = false;
                    break;
                }
            }
            return ability;
        };
        
        // default tabs
        var defaultTab = {};
        
        // switch to default tab (on selection change)
        var switchToDefaultTab = function () {
            // loop over tab controls
            var id, d;
            for (id in defaultTab) {
                // get default
                d = defaultTab[id];
                // visible?
                if (d.widget.isVisible()) {
                    d.widget.setPosition(d.position);
                }
            }
        };
        
        // current selection state
        var current = {};
        
        // clear on view change
        ox.api.event.register("OX_View_Changed", function () {
            current = {};
        });
        
        // process selection
        var processSelection = function (name, hasFolders, selection, force) {
            // current view
            var view = ox.UIController.getView();
            // has changed?
            if (force === true || current[view + ":" + name] === undefined || !equals(current[view + ":" + name].selection, selection)) {
                // vars
                var i = 0, $l = selection.length, obj, ids = {};
                // get all affected folders?
                if (hasFolders === true) {
                    for (; i < $l; i++) {
                        obj = selection[i];
                        ids[obj.folder || obj.folder_id] = true;
                    }
                }
                // switch to default tab (if at least one element is selected)
                if ($l > 0) {
                    switchToDefaultTab();
                }
                // remember state
                current[view + ":" + name] = {
                    ids: ox.util.keys(ids),
                    count: $l,
                    selection: selection,
                    hasFolders: hasFolders,
                    name: name
                };
                // continuation
                var cont = function (data) {
                    // turn result into an array
                    var folders = ox.util.values(data);
                    // get ability
                    var ability = getAbility(selection);
                    // apply selection
                    applySelection(name, ability, selection, folders);
                };
                // fetch folders?
                if (hasFolders === true) {
                    ox.api.folder.getMultiple({
                        list: current[view + ":" + name].ids,
                        success: cont
                    });
                } else {
                    cont({});
                }
            }
        };
        
        // toolbar handlers
        var handlers = [];
        
        return {
            
            /**
             * Add handler
             * @param {function (id)} Handler. Just called with the id (string) of the clicked menu item.
             * A handler returns true, if it feels responsible for that item.
             * @returns {Boolean} True/False/Undefined. Just true matters here.
             */
            addHandler: function (fn) {
                if (ox.util.isFunction(fn)) {
                    // enqueue
                    handlers.push(fn);
                } else {
                    console.error("ToolBarController.addHandler() Not a function: ", fn);
                }
            },
            
            /**
             * Trigger the event processing. All registered handlers will be called in order to find one that
             * can handle the id.
             * @param {String} id
             * @returns {Boolean} True/False. Just False matters here, as it says that nothing happed.
             * This is used by nested menu items, so that they know that no action was taken and do need
             * need to close all hovering elements.
             */
            trigger: function (id) {
                
                // make sure it's a string
                id = String(id);
                
                // get current selections
                var name, obj, selections = {}, view = ox.UIController.getView(), len = (view || "").length;
                for (name in current) {
                    if (name.substr(0, len) === view) {
                        obj = current[name];
                        selections[obj.name] = obj.selection;
                    }
                }
                
                // event
                if (ox.widgets.toolBar.menuItems[id] !== undefined) {
                    corewindow.track({
                        type: "click",
                        dest: "menu_item",
                        data: { 
                            id: id, 
                            title: String(ox.widgets.toolBar.menuItems[id].text)
                        },
                        text: "Click on menu item \"" + ox.widgets.toolBar.menuItems[id].text + "\" (" + id + ")",
                        module: ox.UIController.getModule(),
                        view: ox.UIController.getView()
                    });
                }
                
                // loop over handlers
                var i = 0, $l = handlers.length, result = false;
                for (; i < $l; i++) {
                    // call handler
                    result = handlers[i](id, selections);
                    if (result === true) {
                        return true;
                    }
                }
                
                // when reaching this point, no action has been taken, so...
                return false;
            },
            
            /**
             * Get items
             * @param {String} namespace Namespace. Works like a prefix that runs through a grep 
             * in order to find all matching menu items
             * @returns {Array} A list of all menu item instanced that meet the namespace prefix
             */
            getItems: function (namespace, selection) {
                return getItems(namespace, selection, false);
            },
            
            /**
             * Get item IDs
             * @param namespace {String} Namespace. See getItems()
             * @returns {Array} A list of all menu item IDs
             */
            getItemIds: function (namespace, selection) {
                return getItems(namespace, selection, true);
            },
            
            /**
             * Get ability based on a given selection
             * @param {Array} selection A selection array
             * @returns {Object} An object describing the abilities, like READ, WRITE, or possible actions respectively
             */
            getAbility: function (selection) {
                return getAbility(selection);
            },
            
            /**
             * Apply current selection to current toolbar
             */
            applyCurrentSelection: function () {
                // loop over current selections
                var name, item;
                for (name in current) {
                    // get item
                    item = current[name];
                    // call internal fn
                    processSelection(item.name, item.hasFolders, item.selection, true);
                }
            },
            
            /**
             * Get current selection
             */
            getCurrentSelection: function (name) {
                if (name === undefined) {
                    return current;
                } else {
                    // get current view
                    var view = ox.UIController.getView();
                    return current[view + ":" + name];
                }
            },
            
            /**
             * Clear current selection
             */
            clearCurrentSelection: function () {
                for (var name in current) {
                    $.extend(current[name], {
                        ids: [],
                        count: 0,
                        selection: []
                    });
                }
            },
            
            /**
             * process selection
             */
            processSelection: function (name, hasFolders, selection, force) {
                // call internal fn
                return processSelection(name, hasFolders, selection, force);
            },
            
            /**
             * Set default tab
             */
            setDefaultTab: function (tabControl, position) {
                if (tabControl !== undefined) {
                    defaultTab[tabControl.id] = {
                        widget: tabControl,
                        position: position
                    };
                }
            },
            
            showDefaultTab: function () {
                switchToDefaultTab();
            },
            
            initialResize: function () {
                var collapsed =  ox.api.window.isNested ? false : ox.api.config.get("gui.menu.collapsed", false);
                // get module
                var ctrl = ox.UIController;
                var module = ctrl.getModule() || ctrl.initialModule;
                // get height
                var height = collapsed || (module === "portal" && currentpath[0] !== "configuration") ? "26px" : "90px";
                jQuery("#toolbar_container, #window_toolbar_container").height(height);
                resizeSplit("all_of_menu", height);
            },
            
            resizeCurrent: function (e) {
                // state
                var collapsed = e.data === true;
                // resize
                var height = collapsed ? "26px" : "90px";
                $("#toolbar_container, #window_toolbar_container").height(height);
                resizeSplit("all_of_menu", height);
            },
            
            resize: function (e) {
                // get views
                var views = ox.widgets.toolBar.views;
                // state
                var collapsed = e && e.data === true;
                // resize
                var height = collapsed ? "26px" : "90px";
                $("#toolbar_container, #window_toolbar_container").height(height);
                resizeSplit("all_of_menu", height);
                // store state
                ox.api.config.set("gui.menu.collapsed", collapsed);
                // loop over all tab controls
                for (var id in views) {
                    if (views[id] !== this && id !== "portal") {
                        views[id].toggleHover(collapsed, false);
                    }
                }
            }
        };
    }());
    
    var resizeCurrent = function (e) {
        ox.ToolBarController.resizeCurrent(e);
    };

    var resize = function (e) {
        ox.ToolBarController.resize(e);
    };

    var postInit = function () {
        /*
         * Bind to all tab controls
         */
        var views = ox.widgets.toolBar.views;
        // loop over all tab controls
        for (var id in views) {
            // bind
            views[id].addListener("widget:togglehover", id === "portal" ? resizeCurrent : resize);
        }
    };
    
    // return shared vars/functions
    return {
        // resize handler
        resize: resize,
        // mass "add" handler
        add: add,
        // post init handler
        postInit: postInit,
        // default options
        getOptions: function (options) {
            return $.extend({
                // section names
                showSectionNames: false,
                // open in hover (main window: use user config; sub windows: always show toolbar)
                inHover: ox.api.window.isNested ? false : ox.api.config.get("gui.menu.collapsed", false)
            }, options || {});
        }
    };
};
/**
 * 
 * All content on this website (including text, images, source
 * code and any other original works), unless otherwise noted,
 * is licensed under a Creative Commons License.
 * 
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 * 
 * Copyright (C) Open-Xchange Inc., 2006-2012
 * Mail: info@open-xchange.com 
 * 
 * @author Matthias Biggeleben <matthias.biggeleben@open-xchange.com>
 * 
 */

/*jslint bitwise: false, nomen: false, onevar: false, plusplus: false, regexp: false, white: true, browser: true, devel: true, evil: true, forin: true, undef: true, eqeqeq: true, immed: true */

/*global ox, jQuery, $, equals, register, triggerEvent, menuswitchMailView */

ox.gui.initToolBarCore = function () {
    /*
     * Add common handler
     */
    
    // event hash
    var events = {
        // create new element
        "create-mail": "OX_NEW_MAIL",
        "create-calendar": "OX_Before_Create_Calendar",
        "create-contact": "OX_Before_Create_Contact",
        "create-distributionlist": "OX_Before_Create_Distributionlist",
        "create-task": "OX_Before_Create_Task",
        "create-file": "OX_Before_Create_InfoStore"
    };
    
    ox.ToolBarController.addHandler(function (id) {
        
        // regex matches
        var m;
        
        // is create? (strip prefix first)
        m = id.match(/^\w+-(create-\w+)(-clone)?$/);
        if (m && m.length && events[m[1]] !== undefined) {
            // trigger event
            triggerEvent(events[m[1]]);
            return true;
        }
        
        // is flag/mark?
        m = id.match(/^\w+-flag-(\d+)$/);
        if (m && m.length) {
            // trigger event
            triggerEvent("OX_Add_Flag", m[1]);
            return true;
        }
    });
    
    /*
     * Add root DOM element (just a flying DIV right now)
     */
    var root = $("#toolbar_container")[0];
    
    /*
     * Add top-level view controller (containing all toolbars)
     */
    var control = ox.widgets.toolBar.viewControl = new ox.gui.ViewControl("toolbar-view-control").
        removeCSSClass("oxStretch").
        setParentDOMNode(root).
        addTo(ox.desktop);
    
    /*
     * Bind to UIController
     */
    ox.UIController.dispatcher.bind("modulechange", function (e) {
        // change toolbar to match current module
        var view = ox.widgets.toolBar.views[e.module];
        if (view !== undefined) {
            // change view
            control.showView(view.id);
            // jump to default tab
            ox.ToolBarController.showDefaultTab();
        }
    });
    
    /*
     * Bind to default selection
     */
    ox.api.event.register("Selected", function (selection) {
        // process selection
        ox.ToolBarController.processSelection("default", true, selection);
        // validate top-level control
        ox.widgets.toolBar.viewControl.validate();
    });
    
    // init
    ox.ToolBarController.processSelection("default", true, []);
};
/**
 * 
 * All content on this website (including text, images, source
 * code and any other original works), unless otherwise noted,
 * is licensed under a Creative Commons License.
 * 
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 * 
 * Copyright (C) Open-Xchange Inc., 2006-2012
 * Mail: info@open-xchange.com 
 * 
 * @author Viktor Pracht <viktor.pracht@open-xchange.com>
 */

ox.gui.initToolBarPortal = function ($p) {

    /*
     * Add widgets
     */
    
    // vars
    var tabs = ox.widgets.toolBar.tabs;
    var sections = ox.widgets.toolBar.sections;
    var items = ox.widgets.toolBar.menuItems;
    var toolbar, tab, section;
    
    // fn
    var add = $p.add;
    
    // create toolbar instance
    toolbar = ox.widgets.toolBar.views.portal = new ox.gui.TabMenu(
        $p.getOptions({
            id: "toolbar-portal",
            showSectionNames: false,
            inHover: true
        })
    );
    
    // new
    tab = tabs.portalCreate = new ox.gui.PanelMenu(
        "toolbar-portal-create", _("&New")).addTo(toolbar);
    section = sections.portalCreate = new ox.gui.PanelMenuSection(_("New"), "portalCreateSection").addTo(tab);
    add(section, ox.widgets.toolBar.meta.create, "portal-create",
        ["mail", "calendar", "contact", "distributionlist", "task", "file"]);
    
    // add to top-level element
    toolbar.addTo(ox.widgets.toolBar.viewControl);
    
    // first tab is default
    ox.ToolBarController.setDefaultTab(toolbar, 0);
};
/**
 * 
 * All content on this website (including text, images, source
 * code and any other original works), unless otherwise noted,
 * is licensed under a Creative Commons License.
 * 
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 * 
 * Copyright (C) Open-Xchange Inc., 2006-2012
 * Mail: info@open-xchange.com 
 * 
 * @author Matthias Biggeleben <matthias.biggeleben@open-xchange.com>
 * 
 */

/*jslint bitwise: false, nomen: false, onevar: false, plusplus: false, regexp: false, white: true, browser: true, devel: true, evil: true, forin: true, undef: true, eqeqeq: true, immed: true */

/*global ox, jQuery, $, triggerEvent, menuswitchMailView */

ox.gui.initToolBarMail = function ($p) {
    
    var upsell = ox.api.window.core.ox.upsell;
    
    /*
     * Meta data
     */
    var meta = $.extend(ox.widgets.toolBar.meta, {
        /*
         * Mail
         */
        mail: {
            "compose": {
                title: _("New E-Mail"),
                icons: ["mod_mail.png"],
                big: true,
                visible: function () {
                    // competes with "edit draft" (could show both if support for 1024x768 is stopped
                    return !isStdFolder("drafts");
                }
            },
            "quick-read": {
                title: _("Mark as read"),
                icons: ["mark_as_done.png"],
                big: true,
                requires: ["multiple", "modify"],
                visible: function (ability) {
                    return ability.multiple;
                }
            },
            "reply": {
                title: _("Reply"),
                icons: ["reply.png"],
                big: true,
                requires: ["one"],
                visible: function (ability) {
                    return !ability.multiple;
                }
            },
            "reply-all": {
                title: _("Reply all"),
                icons: ["reply_all.png"],
                big: true,
                requires: ["one"],
                visible: function (ability) {
                    return !ability.multiple;
                }
            },
            "forward": {
                title: _("Forward"),
                icons: ["forward.png"],
                big: true,
                requires: ["some"]
            },
            "delete": {
                title: _("Delete"),
                icons: ["delete.png"],
                big: true,
                requires: ["some", "delete"]
            },
            "edit": {
                title: _("Edit draft"),
                icons: ["draft.png"],
                big: true,
                requires: function (ability) {
                    // current folder is drafts folder?
                    return ability.one && isStdFolder("drafts");
                },
                visible: function () {
                    // current folder is drafts folder?
                    return isStdFolder("drafts");
                }
            },
            "copy": {
                title: _("Copy"),
                icons: ["mail_copy.png"],
                requires: ["some"],
                visible: upsell.visible("mail-copy-move")
            },
            "move": {
                title: _("Move"),
                icons: ["mail_move.png"],
                requires: ["some", "delete"],
                visible: upsell.visible("mail-copy-move")
            },
            "save": {
                title: _("Save"),
                icons: ["send_as_link.png"],
                requires: ["some"]
            },
            "view-source": {
                title: _("Show source"),
                icons: ["mail_source.png"],
                requires: ["one"]
            },
            
            "print": {
                title: _("Print"),
                icons: ["print.png"],
                requires: ["some"]
            },
            "mark-as": {
                title: _("Mark as"),
                icons: ["mark_as.png"],
                requires: ["some"]
            },
            "mark-as-read": {
                title: _("Read"),
                requires: ["some", "modify"]
            },
            "mark-as-unread": {
                title: _("Unread"),
                requires: ["some", "modify"]
            },
            "mark-as-answered": {
                title: _("Answered"),
                requires: ["some", "modify"]
            },
            "mark-as-unanswered": {
                title: _("Unanswered"),
                requires: ["some", "modify"]
            },
            "mark-as-deleted": {
                title: _("Deleted"),
                requires: ["some", "modify"]
            },
            "mark-as-undeleted": {
                title: _("Undeleted"),
                requires: ["some", "modify"]
            },
            /*Attachments*/
            "attachment-open": {
                title: _("Open attachment"),
                icons: ["attachment_open.png"],
                selection: "mail-attachment",
                requires: ["one"]
            },
            "attachment-save": {
                title: _("Save as"),
                icons: ["send_as_attachment.png"],
                selection: "mail-attachment",
                requires: ["one"]
            },
            "attachment-save-infostore": {
                title: _("Save in InfoStore"),
                icons: ["save.png"],
                selection: "mail-attachment",
                requires: ["one", "infostore"],
                mousedown: function(options, e) {
                    if (ox.api.config.get("modules.infostore.module") == false) {
                        corewindow.triggerEvent("Feature_Not_Available", "modules/mail/save_to_infostore", window);
                    }
                },
                visible: ox.upsell.visible("infostore")
            }
        }
    });
    
    // SPAM buttons
    (function () {
        
        // get some config parameter
        var spamButton = ox.api.config.get("mail.spambutton", false);
        var spamFolder = ox.api.config.get("mail.folder.spam");
        
        meta.mail.spam = {
            title: _("Spam"),
            icons: ["spam.png"],
            requires: function (ability, selection) {
                var folder = ox.UIController.getFolder();
                var accountId = ox.api.folder.derive("accountId", folder);
                return ability.some && spamButton === true && accountId === 0 && folder !== spamFolder;
            },
            visible: function (ability, selection) {
                var folder = ox.UIController.getFolder();
                var accountId = ox.api.folder.derive("accountId", folder);
                return spamButton === true && accountId === 0 && folder !== spamFolder;
            }
        };
        
        meta.mail.ham = {
            title: _("Not Spam"),
            icons: ["ham.png"],
            requires: function (ability, selection) {
                var folder = ox.UIController.getFolder();
                var accountId = ox.api.folder.derive("accountId", folder);
                return ability.some && spamButton === true && accountId === 0 && folder === spamFolder;
            },
            visible: function (ability, selection) {
                return spamButton === true && ox.UIController.getFolder() === spamFolder;
            }
        };
        
    }());
    
    
    // toggles attachment section
    var toggleAttachmentSection = function (visible) {
        if (currentpath[0] === "mail") {
            // get parent tab
            var tab = ox.widgets.toolBar.tabs.mailEdit;
            var section = tab.children[2];
            if (section.isVisible() != visible) {
                section.setVisible(visible);
                tab.validate();
            }
        }
    };
    
    
    /*
     * Add widgets
     */
    
    // vars
    var tabs = ox.widgets.toolBar.tabs;
    var sections = ox.widgets.toolBar.sections;
    var items = ox.widgets.toolBar.menuItems;
    var toolbar, tab, section;
    
    // fn
    var add = $p.add;
    
    // create toolbar instance
    toolbar = ox.widgets.toolBar.views.mail = new ox.gui.TabMenu(
        $p.getOptions({
            id: "toolbar-mail",
            showSectionNames: false
        })
    );
    
    // new
    tab = tabs.mailCreate = new ox.gui.PanelMenu("toolbar-mail-create",
        //#. '&' before a letter indicates a shortcut, i.e. Alt+N for '&New'
        _("&New")).addTo(toolbar);
    // section
    section = sections.mailCreate = new ox.gui.PanelMenuSection(_("New"), "mailCreateSection").addTo(tab);
    // add items
    add(section, meta.create, "mail-create",
        ["mail", "calendar", "contact", "distributionlist", "task", "file"]
    );
    
    // Bind to attachment selection
    ox.api.event.register("SubSelected", function (selection) {
        ox.ToolBarController.processSelection("mail-attachment", false, selection, true);
        //toggleAttachmentSection(selection.length != 0);
    });
    
    // edit
    tab = tabs.mailEdit = new ox.gui.PanelMenu("toolbar-mail-edit",
        //#. '&' before a letter indicates a shortcut, i.e. Alt+E for '&Edit'
        _("&Edit")).addTo(toolbar);
    // section compose
    section = sections.mailCompose = new ox.gui.PanelMenuSection(_("Compose"), "mailComposeSection").addTo(tab);
    // add items
    add(section, meta.mail, "mail-compose", ["compose", "edit"]);
    // section
    section = sections.mailEdit = new ox.gui.PanelMenuSection(_("Edit"), "mailEditSection").addTo(tab);
    // add items
    add(section, meta.mail, "mail-edit",
        ["quick-read", "reply", "reply-all", "forward", "delete",
         "print", "mark-as",
         "copy", "move",
         "save", "view-source",
         "spam", "ham"
        ]
    );
    // add to "mark as"
    add(items["mail-edit-mark-as"], meta.mail, "mail-edit",
        ["mark-as-read", "mark-as-unread",
         "mark-as-answered", "mark-as-unanswered",
         "mark-as-deleted", "mark-as-undeleted"
        ]
    );
        
    
    // attachment (tab for 1024x768 support)
    tab = tabs.mailAttachment = new ox.gui.PanelMenu("toolbar-mail-attachment",
        //#. '&' before a letter indicates a shortcut, i.e. Alt+V for '&View'
        _("&Attachment")).addTo(toolbar);
    section = sections.mailAttachment = new ox.gui.PanelMenuSection(_("Attachment"), "mailAttachmentSection").addTo(tab);
    // add items
    add(section, meta.mail, "mail-attachment",
        [ "attachment-open", "attachment-save", "attachment-save-infostore" ]
    );
    
    // views
    tab = tabs.mailView = new ox.gui.PanelMenu("toolbar-mail-view",
        //#. '&' before a letter indicates a shortcut, i.e. Alt+V for '&View'
        _("&View")).addTo(toolbar);
    // section
    section = sections.mailView = new ox.gui.PanelMenuSection(_("View"), "mailViewSection").addTo(tab);
    // add items
    add(section, meta.view, "mail-view",
        ["vsplit", "hsplit", "list"]
    );

    // Tagging (Flags & Categories)
    if (upsell.isVisible("mail-flags")) {
        tab = tabs.mailTagging = new ox.gui.PanelMenu("toolbar-mail-tag",
            //#. '&' before a letter indicates a shortcut, i.e. Alt+F for '&Flags'
            _("&Flags")).addTo(toolbar);
        // section
        section = sections.mailFlags = new ox.gui.PanelMenuSection(_("Flags"), "mailFlagsSection").addTo(tab);
        // add items
        add(section, meta.flags, "mail-flag",
           ["0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "10"]
        );
    }
    
    // Add section search when embbedded mode is on
    if (corewindow.embedded) {
        ox.widgets.searchField.mail = new ox.gui.SearchField("ox-topbar-search-mail").
        addListener("widget:search", function(e) {
            triggerEvent("OX_New_Search", ox.api.ui.getActiveModule(), e.data.value);
        });
        
        tab = tabs.mailSearch = new ox.gui.PanelMenu("toolbar-mail-search",
                _("&Search")).addTo(toolbar);
        // section
        section = sections.mailSearch = new ox.gui.PanelMenuSection(_("Search"), "mailSearchSection").addTo(tab);
        // add items
        section.add(ox.widgets.searchField.mail);
    }
    
    
    /*
     * Add handler
     */
    
    // event hash
    var events = {
        // edit mail
        "mail-compose-compose": "OX_NEW_MAIL",
        "mail-compose-compose-clone": "OX_NEW_MAIL",
        "mail-compose-edit": "OX_Mail_Edit_Draft",
        "mail-edit-reply": "OX_Mail_Reply",
        "mail-edit-reply-all": "OX_Mail_ReplyAll",
        "mail-edit-forward": "OX_Mail_Forward",
        "mail-edit-delete": "OX_Mail_Delete",
        "mail-edit-copy": "OX_Mail_Copy",
        "mail-edit-move": "OX_Mail_Move",
        "mail-edit-save": "OX_Mail_SaveMail",
        "mail-edit-view-source": "OX_Mail_Show_Source",
        "mail-edit-print": "OX_Print",
        "mail-edit-spam": "OX_Mail_Spam",
        "mail-edit-ham": "OX_Mail_Ham",
        "mail-attachment-attachment-save": "OX_Attachment_Save",
        "mail-attachment-attachment-save-infostore": "OX_Attachment_SaveInfoStore"
    };
    
    // mapping of mail flags
    var mailFlags = {
        "read": "seen", 
        "unread": "unseen",
        "answered": "answer", 
        "unanswered": "answered",
        "deleted": "delete", 
        "undeleted": "undelete"
    };
    
    // add handler function (must return true when action taken)
    ox.ToolBarController.addHandler(function (id) {
        
        // regex matches
        var m;
        
        // defined by event?
        if (events[id] !== undefined) {
            // trigger
            triggerEvent(events[id]);
            return true;
        }
        
        // is mail mark?
        m = id.match(/^(mail-edit-quick|mail-edit-mark-as)-(\w+)$/);
        if (m && m.length) {
            // trigger event
            triggerEvent("OX_Mail_Flag", mailFlags[m[2]]);
            return true;
        }
        
        // individual handlers
        switch (id) {
        case "mail-view-vsplit":
            menuswitchMailView('vsplit');
            return true;
        case "mail-view-hsplit":
            menuswitchMailView('hsplit');
            return true;
        case "mail-view-list":
            menuswitchMailView('list');
            return true;
        case "mail-attachment-attachment-open":
            triggerEvent(menunestedmessage ? "OX_Attachment_Open_Nested" : "OX_Attachment_Open");
            return true;
        }
    });
    
    // add to top-level element
    toolbar.addTo(ox.widgets.toolBar.viewControl);
    
    // "edit" tab is default
    ox.ToolBarController.setDefaultTab(toolbar, 1);
};
/**
 *
 * All content on this website (including text, images, source
 * code and any other original works), unless otherwise noted,
 * is licensed under a Creative Commons License.
 *
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 *
 * Copyright (C) Open-Xchange Inc., 2006-2012
 * Mail: info@open-xchange.com
 *
 * @author Matthias Biggeleben <matthias.biggeleben@open-xchange.com>
 *
 */

/*jslint bitwise: false, nomen: false, onevar: false, plusplus: false, regexp: false, white: true, browser: true, devel: true, evil: true, forin: true, undef: true, eqeqeq: true, immed: true */

/*global ox, jQuery, $, triggerEvent, menuswitchMailView */

ox.gui.initToolBarCalendar = function ($p) {

    var upsell = ox.api.window.core.ox.upsell;

    /*
     * Meta data
     */
    var meta = $.extend(ox.widgets.toolBar.meta, {

        /*
         * Calendar
         */
        calendar: {
            "compose": {
                title: _("New Appointment"),
                icons: ["mod_calendar.png"],
                big: true
            },
            "edit": {
                title: _("Edit"),
                icons: ["edit.png"],
                big: true,
                requires: ["one", "modify"]
            },
            "delete": {
                title: _("Delete"),
                icons: ["delete.png"],
                big: true,
                requires: ["some", "delete"]
            },
            "print": {
                title: _("Print"),
                icons: ["print.png"],
                big: true,
                requires: function(ability, selection) {
                    switch(currentpath[1]) {
                        case "team":
                            return false;
                        case "detail":
                            return ability.one;
                        case "list":
                            return true;
                        default:
                            if (currentpath[2] === "custom") {
                                return false;
                            } else {
                                return Boolean(ox.api.config.get("modules.calendar.printing"));
                            }
                    }
                }
            },
            "move": {
                title: _("Move"),
                icons: ["calendar_move.png"],
                big: true,
                requires: ["some", "delete"],
                visible: upsell.visible("calendar-copy-move")
            },
            "confirmation-change": {
                title: _("Confirmation"),
                icons: ["confirmation_change.png"],
                big: true,
                requires: function (ability, selection, cont) {
                    if (ability.some && !AcceptDeny.visible) {
                        // expect the current folder to be already loaded
                        var folder = ox.api.folder.get({
                            folder: ox.UIController.getFolder()
                        });
                        // take care when user is in share
                        var myself = folder.type == 3 ? folder.created_by
                                : ox.api.config.get("identifier");
                        var anyPrivate = false, allParticipant = true;
                        if ("title" in selection[0]) {
                            for (var i = 0; i < selection.length; i++) {
                                check(i, selection[i]);
                            }
                            done();
                        } else if (convertAppointmentArray != identity &&
                                   storageCache.current)
                        {
                            storageCache.current.newIterate(selection,
                                emptyFunction, check, done);
                        } else {
                            var collection = { columns: ["private_flag", "users",
                                                         "participants"], objects: [] };
                            for (var i in selection) {
                                var myObj = selection[i];
                                collection.objects.push(
                                        OXCalendarMapping.createKeyFromObject(
                                          { module: "calendar",
                                            folder_id: myObj.folder || myObj.folder_id,
                                            id: myObj.id,
                                            recurrence_position: myObj.recurrence_position || 0 }
                                      )
                                );
                            }

                            OXCache.newRequest(null, "calendar", collection, null, function(data) {
                                var oldConvert = convertAppointmentArray;
                                convertAppointmentArray = identity;
                                for (var i = 0; i < data.objects.length; i++) {
                                    check(0, data.objects[i]);
                                }
                                convertAppointmentArray = oldConvert;
                                done();
                            });
                        }
                    } else {
                        cont(false);
                    }
                    function check(n, item) {
                        item = convertAppointmentArray(item);
                        anyPrivate = anyPrivate || item.private_flag;
                        
                        // loop over users and check if current user is listed
                        var users = item.users || item.participants;
                        var $l = users.length;
                        var participant = false;
                        for (var i = 0; i < $l; i++) {
                            if (users[i].id === myself) {
                                participant = true;
                            }
                        }
                        allParticipant = allParticipant && participant;
                    }
                    function done() { cont(!anyPrivate && allParticipant); }
                }
            },
            /*Attachments*/
            "attachment-open": {
                title: _("Open attachment"),
                icons: ["attachment_open.png"],
                selection: "attachments",
                requires: function(ability) {
                    return ability.one;
                }
            },
            "attachment-save": {
                title: _("Save as"),
                icons: ["send_as_attachment.png"],
                selection: "attachments",
                requires: function(ability) {
                    return ability.one;
                }
            },
            "attachment-save-infostore": {
                title: _("Save in InfoStore"),
                icons: ["save.png"],
                selection: "attachments",
                requires: function(ability) {
                    return ability.one;
                },
                mousedown: function (options, e) {
                    if (ox.api.config.get("modules.infostore.module") == false) {
                        corewindow.triggerEvent("Feature_Not_Available", "modules/calendar/save_to_infostore", window);
                    }
                }
            },

            /* Views */
            "calendar": {
                title: _("Calendar"),
                icons: ["mod_calendar.png"],
                big: true,
                behavior: "radio",
                group: "calendar-view-type",
                checked: function () {
                    return currentpath[1] === "calendar";
                }
            },
            "team": {
                title: _("Team"),
                icons: ["teamchange.png"],
                big: true,
                behavior: "radio",
                group: "calendar-view-type",
                requires: function (ability) {
                    return ox.api.config.get("modules.calendar.calendar_teamview");
                },
                checked: function () {
                    return currentpath[1] === "team";
                },
                mousedown: function (options, e) {
                    if (ox.api.config.get("modules.calendar.calendar_teamview") == false) {
                        corewindow.triggerEvent("Feature_Not_Available", "modules/calendar/team", window);
                    }
                }
            },
            "list": {
                title: _("List"),
                icons: ["list_view.png"],
                big: true,
                behavior: "radio",
                group: "calendar-view-type",
                checked: function () {
                    return currentpath[1] === "list";
                }
            },
            "detail": {
                title: _("Detail"),
                icons: ["detail_view.png"],
                big: true,
                requires: ["one"],
                behavior: "radio",
                group: "calendar-view-type",
                checked: function () {
                    return currentpath[1] === "detail";
                }
            },
            // Timeranges
            "day": {
                 title: _("Day"),
                 icons: ["dummy.gif"],
                 behavior: "radio",
                 group: "calendar-view-interval",
                 checked : function () {
                    return currentpath[2] === "day";
                }
            },
            "workweek": {
                 title: _("Work Week"),
                 icons: ["dummy.gif"],
                 behavior: "radio",
                 group: "calendar-view-interval",
                 checked : function () {
                     return currentpath[2] === "workweek";
                 }
            },
            "month": {
                 title: _("Month"),
                 icons: ["dummy.gif"],
                 behavior: "radio",
                 group: "calendar-view-interval",
                 checked : function () {
                     return currentpath[2] === "month";
                 }
            },
            "week": {
                title: _("Week"),
                icons: ["dummy.gif"],
                behavior: "radio",
                group: "calendar-view-interval",
                checked : function () {
                    return currentpath[2] === "week";
                }
            },
            "custom": {
                 title: _("Custom"),
                 icons: ["dummy.gif"],
                 behavior: "radio",
                 group: "calendar-view-interval",
                 checked : function () {
                     return currentpath[2] === "custom";
                 }
            },
            /* team members */
            "add": {
                 title: _("Add"),
                 icons: ["member_add.png"],
                 selection: "team-member",
                 requires: function (ability) {
                     // in teamview?
                     return currentpath[1] === "team";
                 }
            },
            "remove": {
                 title: _("Remove"),
                 icons: ["member_remove.png"],
                 selection: "team-member",
                 requires: function (ability) {
                    // teamview and "some" selection
                    return currentpath[1] === "team" && ability.some;
                 }
            },
            "choose-team": {
                 title: _("Choose Team"),
                 icons: ["teamchange.png"],
                 selection: "team-member",
                 requires: function (ability) {
                    // teamview and teams exists
                    return currentpath[1] === "team" &&
                        (ox.api.config.get("gui.calendar.teams") || []).length > 0;
                 }
            },
            "create-team": {
                 title: _("Create Team"),
                 icons: ["teamchange.png"],
                 selection: "team-member",
                 requires: function (ability) {
                        // teamview and "some" selection
                        return currentpath[1] === "team";
                 }
            }
        },
        teams: {}
    });

    // toggles attachment section
    var toggleAttachmentSection = function () {
        if (currentpath[0] === "calendar") {
            // get parent tab
            var tab = ox.widgets.toolBar.tabs.calendarEdit;
            var section = tab.children[3];
            var visible = section.isVisible();
            // in team view?
            if (currentpath[2] === "attachment") {
                if (!visible) {
                    section.setVisible(true);
                    tab.validate();
                }
            } else if (visible) {
                section.setVisible(false);
                tab.validate();
            }
        }
    };

    // hide/show teamview section
    var toggleTeamviewSection = function () {
        if (currentpath[0] === "calendar") {
            // get parent tab
            var tab = ox.widgets.toolBar.tabs.calendarEdit;
            var section = tab.children[2];
            var visible = section.isVisible();
            // in team view?
            if (currentpath[1] === "team") {
                if (!visible) {
                    section.setVisible(true);
                    tab.validate();
                }
            } else if (visible) {
                section.setVisible(false);
                tab.validate();
            }
        }
    };

    // Add existing teams to metadata
    var addTeams = function () {
        var teams = ox.api.config.get("gui.calendar.teams");
        if (teams) {
            meta.teams = {};
            var teamItems = [];
            var i = 0, $l = teams.length;
            for (; i < $l; i++) {
                teamItems.push("team-#" + i);
                meta.teams["team-#" + i] = {
                    title: teams[i].name,
                    behavior: "radio",
                    group: "calendar-teamview-teams-internal"
                };
            }
            // add
            add(items["calendar-teamview-choose-team"], meta.teams, "calendar-teamview", teamItems);
        }
    };

    register("OX_Calendar_Teamlist_Changed", function () {
        // get menu item
        var item = ox.widgets.toolBar.menuItems["calendar-teamview-choose-team"];
        // remove children
        item.removeChildren();
        // add teams
        addTeams();
        // repaint
        item.validate();
    });

    /*
     * Add widgets
     */

    // vars
    var tabs = ox.widgets.toolBar.tabs;
    var sections = ox.widgets.toolBar.sections;
    var items = ox.widgets.toolBar.menuItems;
    var toolbar, tab, section;

    // fn
    var add = $p.add;

    // create toolbar instance
    toolbar = ox.widgets.toolBar.views.calendar = new ox.gui.TabMenu(
        $p.getOptions({
            id: "toolbar-calendar",
            showSectionNames: false
        })
    );

    // new
    tab = tabs.calendarCreate = new ox.gui.PanelMenu("toolbar-calendar-create",
        //#. '&' before a letter indicates a shortcut, i.e. Alt+N for '&New'
        _("&New")).addTo(toolbar);
    // section
    section = sections.calendarCreate = new ox.gui.PanelMenuSection(_("New"), "calendarCreateSection").addTo(tab);
    // add items
    add(section, meta.create, "calendar-create",
        ["mail", "calendar", "contact", "distributionlist", "task", "file"]
    );

    // edit
    tab = tabs.calendarEdit = new ox.gui.PanelMenu("toolbar-calendar-edit",
        //#. '&' before a letter indicates a shortcut, i.e. Alt+E for '&Edit'
        _("&Edit")).addTo(toolbar);
    // section "new"
    section = sections.calendarCompose = new ox.gui.PanelMenuSection(_("New"), "calendarComposeSection").addTo(tab);
    // add items
    add(section, meta.calendar, "calendar-edit",
        ["compose"]
    );
    // section "edit"
    section = sections.calendarEdit = new ox.gui.PanelMenuSection(_("Edit"), "calendarEditSections").addTo(tab);
    // add items
    add(section, meta.calendar, "calendar-edit",
        ["edit", "delete", "confirmation-change", "print", "move"]
    );

    // section teams
    section = sections.calendarTeam = new ox.gui.PanelMenuSection(_("Teams"), "calendarTeamSection").addTo(tab);
    // add items
    add(section, meta.calendar, "calendar-teamview",
        ["add","remove","choose-team","create-team"]
    );

    // section attachments only visible in detail view
    if (upsell.isVisible("infostore") && upsell.isVisible("calendar-attachments")) {
        section = sections.calendarAttachment = new ox.gui.PanelMenuSection(_("Attachments"), "calendarAttachmentSections").addTo(tab);
        add(section, meta.calendar, "calendar-attachments",
            ["attachment-open","attachment-save","attachment-save-infostore"]
        );
    }

    // add existing teams to chooseTeam
    addTeams();

    // views
    tab = tabs.calendarView = new ox.gui.PanelMenu("toolbar-calendar-view",
        //#. '&' before a letter indicates a shortcut, i.e. Alt+V for '&View'
        _("&View")).addTo(toolbar);

    // section View
    section = sections.calendarView = new ox.gui.PanelMenuSection(_("View"), "calendarViewSection").addTo(tab);
    // add items
    add(section, meta.calendar, "calendar-view",
        ["calendar", "team", "list", "detail"]
    );

    // section Interval
    section = sections.calendarInterval = new ox.gui.PanelMenuSection(_("Time Range"), "calendarIntervalSection").addTo(tab);
    // add items
    add(section, meta.calendar, "calendar-view-interval",
        ["day", "workweek", "month", "week", "custom"]
    );

    // Tagging (Flags & Categories)
    if (upsell.isVisible("calendar-flags")) {
        tab = tabs.calendarTagging = new ox.gui.PanelMenu("toolbar-calendar-tag",
            //#. '&' before a letter indicates a shortcut, i.e. Alt+F for '&Flags'
            _("&Flags")).addTo(toolbar);
        // section
        section = sections.calendarFlags = new ox.gui.PanelMenuSection(_("Flags"), "calendarFlagSection").addTo(tab);
        // add items
        add(section, meta.flags, "calendar-flag",
           ["0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "10"]
        );
    }

    // Social OX
    if (ox.api.config.get(["modules","com.openexchange.subscribe"]) === true ||
            ox.api.config.get("ui.upsell.features.social") === true) {
        tab = tabs.calendarSocial = new ox.gui.PanelMenu("toolbar-calendar-socialOX",
            _("&Social")).addTo(toolbar);
        section = sections.calendarSocialOX = new ox.gui.PanelMenuSection(_("Social"), "calendarSocialOXSections").addTo(tab);
        add(section, meta.socialOX, "calendar-socialOX", [ "refresh", "subscribe" ]);
    }

    // Add section search when embbedded mode is on
    if (corewindow.embedded) {
        ox.widgets.searchField.calendar = new ox.gui.SearchField("ox-topbar-search-calendar").
        addListener("widget:search", function(e) {
            triggerEvent("OX_New_Search", ox.api.ui.getActiveModule(), e.data.value);
        });

        tab = tabs.calendarSearch = new ox.gui.PanelMenu("toolbar-calendar-search",
                _("&Search")).addTo(toolbar);
        // section
        section = sections.calendarSearch = new ox.gui.PanelMenuSection(_("Search"), "calendarSearchSection").addTo(tab);
        // add items
        section.add(ox.widgets.searchField.calendar);
    }


    /*
     * Add handler
     */

    // event hash
    var events = {
        "calendar-edit-compose": "OX_Before_Create_Calendar",
        "calendar-edit-compose-clone": "OX_Before_Create_Calendar",
        "calendar-edit-delete": "OX_Calendar_Delete",
        "calendar-edit-edit": "OX_Calendar_Edit",
        "calendar-edit-move": "OX_Calendar_Move",
        "calendar-edit-print": "OX_Print",
        "calendar-attachments-attachment-save": "OX_Attachment_Save",
        "calendar-attachments-attachment-save-infostore": "OX_Attachment_SaveInfoStore"

    };

    // add handler function (must return true when action taken)
    ox.ToolBarController.addHandler(function (id, selections) {

        // confirmation change?
        if (id === "calendar-edit-confirmation-change") {
            // get current selection
            var selection = selections["default"];
            var tmp = [];
            if ("title" in selection[0]) {
                var i = 0, $l = selection.length;
                for (; i < $l; i++) {
                    tmp.push(new AcceptDenyObject("calendar", selection[i]));
                }
                showConfirmation();
            } else if (   storageCache.current
                       && convertAppointmentArray != identity)
            {
                storageCache.current.newIterate(selection, emptyFunction,
                    function(n, item) {
                        tmp.push(new AcceptDenyObject("calendar",
                            convertAppointmentArray(item)));
                    }, showConfirmation);
            } else {
                var collection = { columns: null, objects: [] };
                for (var i in selection) {
                    var myObj = selection[i];
                    collection.objects.push(
                            OXCalendarMapping.createKeyFromObject(
                              { module: "calendar",
                                folder_id: myObj.folder || myObj.folder_id,
                                id: myObj.id,
                                recurrence_position: myObj.recurrence_position || 0 }
                          )
                    );
                }

                OXCache.newRequest(null, "calendar", collection,
                    null, function(data) {
                        for (var i = 0; i < data.objects.length; i++) {
                            tmp.push(new AcceptDenyObject("calendar",
                                     data.objects[i]));
                        }
                        showConfirmation();
                    });
            }
        }
        function showConfirmation() {
            (new AcceptDeny(true)).showConfirmation(tmp);
        }

        // regex matches
        var m;

        // defined by event?
        if (events[id] !== undefined) {
            // trigger
            triggerEvent(events[id]);
            return true;
        }

        // is flag/mark?
        m = id.match(/^\w+-flag-(\d+)$/);
        if (m && m.length) {
            // trigger event
            triggerEvent("OX_Add_Flag", m[1]);
            return true;
        }

        // action for team selection
        var teamindex;
        if (id.match(/#[0-9]/)) {
            teamindex = id.substr( id.indexOf("#") + 1, id.length);
            var grid = getCalendarGrid("teamview");
            // delay for better gui responsiveness
            setTimeout(function() {
                 grid.loadTeamByIndex(teamindex);
            }, 10);
            return true;
        }

        switch (id) {
        case "calendar-view-calendar":
            menuswitchCalendarView('calendar');
            return true;
        case "calendar-view-list":
            menuswitchCalendarView('list');
            return true;
        case "calendar-view-team":
            menuswitchCalendarView('team');
            return true;
        case "calendar-view-detail":
            menuswitchCalendarView('detail');
            return true;
        case "calendar-view-interval-day":
            menuswitchCalendarTimerange('day');
            return true;
        case "calendar-view-interval-workweek":
            menuswitchCalendarTimerange('workweek');
            return true;
        case "calendar-view-interval-month":
            menuswitchCalendarTimerange('month');
            return true;
        case "calendar-view-interval-week":
            menuswitchCalendarTimerange('week');
            return true;
        case "calendar-view-interval-custom":
            menuswitchCalendarTimerange('custom');
            return true;
        case "calendar-teamview-add":
            triggerEvent('OX_Calendar_Teammember_Add');
            return true;
        case "calendar-teamview-remove":
            triggerEvent('OX_Calendar_Teammember_Remove');
            return true;
        case "calendar-teamview-create-team":
            triggerEvent('OX_Calendar_Team_Add_Team');
            triggerEvent('OX_Switch_View','configuration/calendar/teams');
            return true;
        case "calendar-attachments-attachment-open":
            triggerEvent("OX_Attachment_Open");
            return true;

        }

    });

    // add to top-level element
    toolbar.addTo(ox.widgets.toolBar.viewControl);

    //Bind to team-member selection
    ox.api.event.register("SubSelected", function (selection) {
        ox.ToolBarController.processSelection("team-member", false, selection, true);
        if( currentpath[2] === "attachments") {
            ox.ToolBarController.processSelection("attachments", false, [selection], true);
        }

    });

    // hide and show teamview components
    // process selection for confirmation change
    ox.api.event.register("OX_View_Changed", function () {

        toggleTeamviewSection();
        toggleAttachmentSection();

        if (currentpath[0] === "calendar") {
            if (currentpath[1] !== "detail") {
                ox.ToolBarController.processSelection("default", false, [], true);
                ox.ToolBarController.processSelection("detailappointment", false, [], true);
            } else {
                ox.ToolBarController.processSelection("default", false, [selectedAppointment], true);
            }
            if (currentpath[2] !== "attachments") {
                ox.ToolBarController.processSelection("attachments", false, [], true);
            }
        }
    });

    ox.api.event.register("Confirmation_Changed", function(en) {
        ox.widgets.toolBar.menuItems["calendar-edit-confirmation-change"].
            setEnabled(Boolean(en));
    });

    // init
    ox.ToolBarController.processSelection("team-member", false, [], true);
    ox.ToolBarController.processSelection("detailappointment", false, [], true);
    ox.ToolBarController.processSelection("attachments", false, [], true);

    // "edit" tab is default
    ox.ToolBarController.setDefaultTab(toolbar, 1);
};
/**
 * 
 * All content on this website (including text, images, source
 * code and any other original works), unless otherwise noted,
 * is licensed under a Creative Commons License.
 * 
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 * 
 * Copyright (C) Open-Xchange Inc., 2006-2012
 * Mail: info@open-xchange.com 
 * 
 * @author Matthias Biggeleben <matthias.biggeleben@open-xchange.com>
 * 
 */

/*jslint bitwise: false, nomen: false, onevar: false, plusplus: false, regexp: false, white: true, browser: true, devel: true, evil: true, forin: true, undef: true, eqeqeq: true, immed: true */

/*global ox, jQuery, $, triggerEvent, menuswitchMailView */

ox.gui.initToolBarTasks = function ($p) {
    
    var upsell = ox.api.window.core.ox.upsell;
    
    /*
     * Meta data
     */
    var meta = $.extend(ox.widgets.toolBar.meta, {
        /*
         * Tasks
         */
        tasks: {
            "compose": {
                title: _("New task"),
                icons: ["mod_tasks.png"],
                big: true
            },
            "edit": {
                title: _("Edit"),
                icons: ["edit.png"],
                big: true,
                requires: ["one", "modify"]
            },
            "delete": {
                title: _("Delete"),
                icons: ["delete.png"],
                big: true,
                requires: ["some", "delete"]
            },
            "print": {
                title: _("Print"),
                icons: ["print.png"],
                big: true,
                requires: ["some"]
            },
            "change": {
                title: _("Confirmation"),
                icons: ["confirmation_change.png"],
                big: true,
                requires: function (ability, selection, cont) {
                    if (!ability.one || AcceptDeny.visible) return false;
                    OXCache.newRequest(null, "tasks",
                        { columns: ["users"], objects: selection }, null,
                        processTask);

                    // checks if the user is a participant, calls cont if yes.
                    function processTask(data) {
                        var task = data.objects[0];
                        if (!task) cont(false);
                        else if (ox.util.isArray(task.users)) {
                            // get current user
                            var myself = ox.api.config.get("identifier");
                            // loop over users and check if current user is listed
                            var i = 0, users = task.users, $l = users.length;
                            for (; i < $l; i++) {
                                if (users[i].id === myself) {
                                    cont(true);
                                    return;
                                }
                            }
                        }
                    }
                }
            },
            "move": {
                title: _("Move"),
                icons: ["task_move.png"],
                requires: ["some", "delete"],
                visible: upsell.visible("tasks-copy-move")
            },
            "mark-as-done": {
                title: _("Mark as Done"),
                icons: ["task_mark_as_done.png"],
                requires: function(ability, selection, cont) {
                    if (!ability.some || !ability.modify) return false;
                    OXCache.newRequest(null, "tasks",
                        { columns: ["status"], objects: selection }, null,
                        function(data) {
                            for (var i in data.objects) {
                                if (data.objects[i].status != 3) {
                                    cont(true);
                                    return;
                                }
                            }
                            cont(false);
                        });
                }
            },
            "use-as-template": {
                title: _("Use as template"),
                icons: ["task_copy.png"],
                requires: ["one"],
                visible: upsell.visible("tasks-copy-move")
            },
            /*Attachments*/
            "attachment-open": {
                title: _("Open attachment"),
                icons: ["attachment_open.png"],
                selection: "attachments",
                requires: function(ability) {
                    return ability.one;
                }
            },
            "attachment-save": {
                title: _("Save as"),
                icons: ["send_as_attachment.png"],
                selection: "attachments",
                requires: function(ability) {
                    return ability.one;
                }
            },
            "attachment-save-infostore": {
                title: _("Save in InfoStore"),
                icons: ["save.png"],
                selection: "attachments",
                requires: function(ability) {
                    return ability.one;
                },
                mousedown: function (options, e) {
                    if (ox.api.config.get("modules.infostore.module") == false) {
                        corewindow.triggerEvent("Feature_Not_Available", "modules/tasks/save_to_infostore", window);
                    }
                },
                visible: ox.upsell.visible("infostore")
            },
            // VIEWS
            "hsplit": {
                title: _("H-Split"),
                icons: ["h_split_view.png"],
                big: true,
                behavior: "radio",
                group: "split-views",
                checked: function () {
                    return currentpath[1] === "split";
                }
            },
            "list": {
                title: _("List"),
                icons: ["list_view.png"],
                big: true,
                behavior: "radio",
                group: "split-views",
                checked: function () {
                    return currentpath[1] === "list";
                }
            }
        }
    });
    
    // toggles attachment section
    var toggleAttachmentSection = function () {
        if (currentpath[0] === "tasks") {
            // get parent tab
            var tab = ox.widgets.toolBar.tabs.tasksEdit;
            var section = tab.children[2];
            var visible = section.isVisible();
            // in attachment tab?
            if (currentpath[2] === "attachment") {
                if (!visible) {
                    section.setVisible(true);
                    tab.validate();
                }
            } else if (visible) {
                section.setVisible(false);
                tab.validate();
            }
        }
    };
    
    /*
     * Add widgets
     */
    
    // vars
    var tabs = ox.widgets.toolBar.tabs;
    var sections = ox.widgets.toolBar.sections;
    var items = ox.widgets.toolBar.menuItems;
    var toolbar, tab, section;
    
    // fn
    var add = $p.add;
    
    // create toolbar instance
    toolbar = ox.widgets.toolBar.views.tasks = new ox.gui.TabMenu(
        $p.getOptions({
            id: "toolbar-tasks",
            showSectionNames: false
        })
    );
    
    // new
    tab = tabs.tasksCreate = new ox.gui.PanelMenu("toolbar-tasks-create",
        //#. '&' before a letter indicates a shortcut, i.e. Alt+N for '&New'
        _("&New")).addTo(toolbar);
    // section
    section = sections.tasksCreate = new ox.gui.PanelMenuSection(_("New"), "tasksCreateSection").addTo(tab);
    // add items
    add(section, meta.create, "tasks-create",
        ["mail", "calendar", "contact", "distributionlist", "task", "file"]
    );
    
    // edit
    tab = tabs.tasksEdit = new ox.gui.PanelMenu("toolbar-tasks-edit",
        //#. '&' before a letter indicates a shortcut, i.e. Alt+E for '&Edit'
        _("&Edit")).addTo(toolbar);
    // section
    section = sections.tasksCompose = new ox.gui.PanelMenuSection(_("Compose"), "tasksComposeSection").addTo(tab);
    // add items
    add(section, meta.tasks, "tasks-edit",
        ["compose"]
    );
    // section
    section = sections.tasksEdit = new ox.gui.PanelMenuSection(_("Edit"), "tasksEditSection").addTo(tab);
    // add items
    add(section, meta.tasks, "tasks-edit",
        ["edit", "delete", "print", "change", "move", "mark-as-done", "use-as-template"]
    );
    
    // section attachments
    if (upsell.isVisible("infostore") && upsell.isVisible("tasks-attachments")) {
        section = section.tasksAttachment = new ox.gui.PanelMenuSection(_("Attachments"), "tasksAttachmentSection").addTo(tab);
        // add items
        add(section, meta.tasks, "tasks-edit",
            ["attachment-open", "attachment-save", "attachment-save-infostore"]
        );
    }
    
    // view
    tab = tabs.tasksView = new ox.gui.PanelMenu("toolbar-tasks-view",
        //#. '&' before a letter indicates a shortcut, i.e. Alt+V for '&View'
        _("&View")).addTo(toolbar);
    // section
    section = sections.tasksView = new ox.gui.PanelMenuSection(_("View"), "tasksViewSection").addTo(tab);
    // add items
    add(section, meta.tasks, "tasks-view",
        ["hsplit", "list"]
    );
    
    // Tagging (Flags & Categories)
    if (upsell.isVisible("tasks-flags")) {
        tab = tabs.tasksTagging = new ox.gui.PanelMenu("toolbar-tasks-tag",
            //#. '&' before a letter indicates a shortcut, i.e. Alt+F for '&Flags'
            _("&Flags")).addTo(toolbar);
        // section
        section = sections.tasksFlags = new ox.gui.PanelMenuSection(_("Flags"), "tasksFlagsSection").addTo(tab);
        // add items
        add(section, meta.flags, "tasks-flag",
           ["0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "10"]
        );
    }
    
    // Add section search when embbedded mode is on
    if (corewindow.embedded) {
        ox.widgets.searchField.tasks = new ox.gui.SearchField("ox-topbar-search-tasks").
        addListener("widget:search", function(e) {
            triggerEvent("OX_New_Search", ox.api.ui.getActiveModule(), e.data.value);
        });
        
        tab = tabs.tasksSearch = new ox.gui.PanelMenu("toolbar-tasks-search",
                _("&Search")).addTo(toolbar);
        // section
        section = sections.tasksSearch = new ox.gui.PanelMenuSection(_("Search"), "tasksSearchSection").addTo(tab);
        // add items
        section.add(ox.widgets.searchField.tasks);
    }
    
    // attachments section visibile only in attachments tab
    ox.api.event.register("OX_View_Changed", toggleAttachmentSection);
    
    /*
     * Add handler
     */
    
    // event hash
    var events = {
        "tasks-edit-compose": "OX_Before_Create_Task",
        "tasks-edit-compose-clone": "OX_Before_Create_Task",
        "tasks-edit-edit": "OX_Task_Edit",
        "tasks-edit-delete": "OX_Task_Delete",
        "tasks-edit-print": "OX_Print",
        "tasks-edit-change": "OX_Confirmation_Change",
        "tasks-edit-move": "OX_Task_Move",
        "tasks-edit-mark-as-done": "OX_Task_MarkAsDone",
        "tasks-edit-use-as-template": "OX_Task_Duplicate",
        "tasks-edit-attachment-save" : "OX_Attachment_Save",
        "tasks-edit-attachment-save-infostore" : "OX_Attachment_SaveInfoStore"
    };
    
    // add handler function (must return true when action taken)
    ox.ToolBarController.addHandler(function (id) {
        
        // defined by event?
        if (events[id] !== undefined) {
            // trigger
            triggerEvent(events[id]);
            return true;
        }
        
        // individual handlers
        switch (id) {
        case "tasks-view-hsplit":
        	if (currentpath[1] !== "split") {
        		switchtoView('tasks/split');
        	}
        	return true;
        case "tasks-view-list":
        	if (currentpath[1] !== "list") {
	            switchtoView('tasks/list');
        	}
        	return true;
        case "tasks-edit-attachment-open":
            triggerEvent("OX_Attachment_Open");
            return true;
        }
    });
    
    
    // add to top-level element
    toolbar.addTo(ox.widgets.toolBar.viewControl);

    // initial init
    ox.ToolBarController.processSelection("attachments",false, [], true);
    
    // "edit" tab is default
    ox.ToolBarController.setDefaultTab(toolbar, 1);
};
/**
 * 
 * All content on this website (including text, images, source
 * code and any other original works), unless otherwise noted,
 * is licensed under a Creative Commons License.
 * 
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 * 
 * Copyright (C) Open-Xchange Inc., 2006-2012
 * Mail: info@open-xchange.com 
 * 
 * @author Matthias Biggeleben <matthias.biggeleben@open-xchange.com>
 * 
 */

/*jslint bitwise: false, nomen: false, onevar: false, plusplus: false, regexp: false, white: true, browser: true, devel: true, evil: true, forin: true, undef: true, eqeqeq: true, immed: true */

/*global ox, jQuery, $, triggerEvent, menuswitchMailView */

ox.gui.initToolBarInfoStore = function ($p) {
    
    var NAME = "infostore";
    
    var upsell = ox.api.window.core.ox.upsell;
    
    /*
     * Meta data
     */
    var meta = $.extend(ox.widgets.toolBar.meta, {
        /*
         * InfoStore
         */
        infostore: {
            compose: {
                title: _("New Infoitem"),
                icons: ["mod_infostore.png"],
                big: true
            },
            "edit": {
                title: _("Edit"),
                icons: ["edit.png"],
                big: true,
                requires: function(ability, selection) {
                    return ability.one && ability.modify &&
                        selection.length == menucountlockable +
                                            menucounteditable;
                }
            },
            "delete": {
                title: _("Delete"),
                icons: ["delete.png"],
                big: true,
                requires: function(ability, selection) {
                    return ability.some && ability["delete"] &&
                        selection.length == menucountlockable +
                                            menucounteditable;
                }
            },
            "print": {
                title: _("Print"),
                icons: ["print.png"],
                big: true,
                requires: function(ability, selection) {
                    return ability.some && 
                        isPrintableView(currentpath2.join('/'));
                }
            },
            "move": {
                title: _("Move"),
                icons: ["infostore_move.png"],
                requires: function(ability, selection) {
                    return ability.some && ability["delete"] &&
                        selection.length == menucountlockable +
                                            menucounteditable;
                },
                visible: upsell.visible("infostore-copy-move")
            },
            "lock": {
                title: _("Lock"),
                icons: ["lock.png"],
                selection: "lockable",
                requires: function(ability, selection) {
                    return ability.some && ability.modify &&
                        menucountlockable == selection.length;
                }
            },
            "unlock": {
                title: _("Unlock"),
                icons: ["unlock.png"],
                selection: "lockable",
                requires: function(ability, selection) {
                    return ability.some && ability.modify &&
                        menucountunlockable == selection.length;
                }
            },
            "as-link": {
                title: _("Send as link"),
                icons: ["send_as_attachment.png"],
                requires: function(ability) {
                    return ox.api.config.get("modules.mail.module") && ability.some;
                },
                mousedown: function (options, e) {
                    if (ox.api.config.get("modules.mail.module") == false) {
                        corewindow.triggerEvent("Feature_Not_Available", "modules/infostore/send_as_link", window);
                    }
                }
            },
            "as-attachment": {
                title: _("Send as attachment"),
                icons: ["send_as_attachment.png"],
                requires: function (ability, selection, cont) {
                    // module mail disabled or na?
                    if (ox.api.config.get("modules.mail.module") === false) {
                        return false;
                    }
                    if (!selection.length) return false;
                    OXCache.newRequest(null, "infostore",
                        { objects: selection, columns: ["filename"] },
                        null, function(data) {
                            // check if selection only contain real files
                            for (var i in data.objects) {
                                if (data.objects[i].filename == undefined) {
                                    cont(false);
                                    return;
                                }
                            }
                            cont(true);
                        });
                },
                mousedown: function (options, e) {
                    if (ox.api.config.get("modules.mail.module") === false) {
                        corewindow.triggerEvent("Feature_Not_Available", "modules/infostore/send_as_attachment", window);
                    }
                }
            },
            /*Version*/
            "version-current": {
                title: _("Current"),
                icons: ["current_version.png"],
                selection: "version",
                requires: function(ability) {
                    return ability.one && ability.modify &&
                        menucountselected == menucountlockable +
                                             menucounteditable;
                }
            },
            "version-delete": {
                title: _("Delete"),
                icons: ["delete.png"],
                selection: "version",
                requires: function(ability) {
                    return ability.some && ability["delete"] &&
                        menucountselected == menucountlockable +
                                             menucounteditable;
                }
            },
            // Views
            "hsplit": {
                title: _("H-Split"),
                icons: ["h_split_view.png"],
                big: true,
                behavior: "radio",
                group: "split-views",
                checked: function () {
                    return currentpath[1] === "split";
                }
            },
            "list": {
                title: _("List"),
                icons: ["list_view.png"],
                big: true,
                behavior: "radio",
                group: "split-views",
                checked: function () {
                    return currentpath[1] === "list";
                }
            }
        }
    });
    
    // toggles attachment section
    var toggleVersionSection = function () {
        if (currentpath[0] === "infostore") {
            // get parent tab
            var tab = ox.widgets.toolBar.tabs.infostoreEdit;
            var section = tab.children[3];
            var visible = section.isVisible();
            // in version tab?
            if (currentpath[2] === "version") {
                if (!visible) {
                    section.setVisible(true);
                    tab.validate();
                }
            } else if (visible) {
                section.setVisible(false);
                tab.validate();
            }
        }
    };
    
    /*
     * Add widgets
     */
    
    // vars
    var tabs = ox.widgets.toolBar.tabs;
    var sections = ox.widgets.toolBar.sections;
    var items = ox.widgets.toolBar.menuItems;
    var toolbar, tab, section;
    
    // fn
    var add = $p.add;
    
    // create toolbar instance
    toolbar = ox.widgets.toolBar.views[NAME] = new ox.gui.TabMenu(
        $p.getOptions({
            id: "toolbar-" + NAME,
            showSectionNames: false
        })
    );
    
    // new
    tab = tabs[NAME + "Create"] = new ox.gui.PanelMenu("toolbar-" + NAME + "-create",
        //#. '&' before a letter indicates a shortcut, i.e. Alt+N for '&New'
        _("&New")).addTo(toolbar);
    // section
    section = sections.infostoreCreate = new ox.gui.PanelMenuSection(_("New"), "infostoreCreateSection").addTo(tab);
    // add items
    add(section, meta.create, NAME + "-create",
        ["mail", "calendar", "contact", "distributionlist", "task", "file"]
    );
    
    // edit
    tab = tabs[NAME + "Edit"] = new ox.gui.PanelMenu("toolbar-" + NAME + "-edit",
        //#. '&' before a letter indicates a shortcut, i.e. Alt+E for '&Edit'
        _("&Edit")).addTo(toolbar);
    // section
    section = sections.infostoreCompose = new ox.gui.PanelMenuSection(_("Compose"), "infostoreComposeSection").addTo(tab);
    // add items
    add(section, meta[NAME], NAME + "-edit",
        ["compose"]
    );
    // section
    section = sections.infostoreEdit = new ox.gui.PanelMenuSection(_("Edit"), "infostoreEditSection").addTo(tab);
    // add items
    add(section, meta[NAME], NAME + "-edit",
        ["edit", "delete", "print", "lock", "unlock", "move"]
    );
    
    section = sections.infostoreSend = new ox.gui.PanelMenuSection(_("Send"), "infostoreSendSection").addTo(tab);
    // add items
    add(section, meta[NAME], NAME + "-send", ["as-attachment", "as-link"]);
    
    // section version
    section = sections.infostoreVersion = new ox.gui.PanelMenuSection(_("Version"), "infostoreVersionSection").addTo(tab);
    // add items
    add(section, meta[NAME], [NAME] + "-version",
            ["version-current", "version-delete"]
    );
    
    // version section visibile only in versions tab
    ox.api.event.register("OX_View_Changed", function(){
        toggleVersionSection();
        if (currentpath[0] === "infostore" && currentpath[2] !== "version") {
            ox.ToolBarController.processSelection("version",
                    false, [], true);
        }
    });
    
    // Bind to version selection
    ox.api.event.register("SubSelected", function (selection) {
        var sel = new Array(selection.length);
        for (var i = 0; i < sel.length; i++) {
            sel[i] = $.extend({}, selection[i], { folder_id: activefolder });
        }
        ox.ToolBarController.processSelection("version", false, sel, true);
    });
    
    // view
    tab = tabs[NAME + "View"] = new ox.gui.PanelMenu("toolbar-" + NAME + "-view",
        //#. '&' before a letter indicates a shortcut, i.e. Alt+V for '&View'
        _("&View")).addTo(toolbar);
    // section
    section = sections.infostoreView = new ox.gui.PanelMenuSection(_("View"), "infostoreViewSection").addTo(tab);
    // add items
    add(section, meta[NAME], NAME + "-view",
        ["hsplit", "list"]
    );

    // Tagging (Flags & Categories)
    if (upsell.isVisible("infostore-flags")) {
        tab = tabs[NAME + "Tagging"] = new ox.gui.PanelMenu("toolbar-" + NAME + "-tag",
            //#. '&' before a letter indicates a shortcut, i.e. Alt+F for '&Flags'
            _("&Flags")).addTo(toolbar);
        // section
        section = sections.infostoreFlags = new ox.gui.PanelMenuSection(_("Flags"), "infostoreFlagsSection").addTo(tab);
        // add items
        add(section, meta.flags, NAME + "-flag",
           ["0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "10"]
        );
    }
    
    // Social OX
    if (ox.api.config.get(["modules","com.openexchange.subscribe"]) === true ||
            ox.api.config.get(["modules","com.openexchange.publish"]) === true ||
            ox.api.config.get("ui.upsell.features.social") === true) {
        tab = tabs.infostoreSocial = new ox.gui.PanelMenu("toolbar-infostore-socialOX",
            _("&Social")).addTo(toolbar);
        section = sections.infostoreSocialOX = new ox.gui.PanelMenuSection(_("Social"), "infostoreSocialOXSections").addTo(tab);
        add(section, meta.socialOX, "infostore-socialOX", [ "refresh", "publish", "subscribe" ]);
        
    }
    
    // Add section search when embbedded mode is on
    if (corewindow.embedded) {
        ox.widgets.searchField.infostore = new ox.gui.SearchField("ox-topbar-search-infostore").
        addListener("widget:search", function(e) {
            triggerEvent("OX_New_Search", ox.api.ui.getActiveModule(), e.data.value);
        });
        
        tab = tabs.infostoreSearch = new ox.gui.PanelMenu("toolbar-infostore-search",
                _("&Search")).addTo(toolbar);
        // section
        section = sections.infostoreSearch = new ox.gui.PanelMenuSection(_("Search"), "infostoreSearchSection").addTo(tab);
        // add items
        section.add(ox.widgets.searchField.infostore);
    }
    
    /*
     * Add handler
     */
    
    // event hash
    var events = {
        "infostore-edit-compose": "OX_Before_Create_InfoStore",
        "infostore-edit-compose-clone": "OX_Before_Create_InfoStore",
        "infostore-edit-edit": "OX_InfoStore_Edit",
        "infostore-edit-delete": "OX_InfoStore_Delete",
        "infostore-edit-print": "OX_Print",
        "infostore-edit-move": "OX_InfoStore_Move",
        "infostore-edit-lock": "OX_InfoStore_Lock",
        "infostore-edit-unlock": "OX_InfoStore_Unlock",
        "infostore-send-as-attachment": "OX_InfoStore_Send_Attachment",
        "infostore-send-as-link": "OX_InfoStore_Send_Link",
        "infostore-version-version-current": "OX_InfoStore_Version_Current",
        "infostore-version-version-delete": "OX_InfoStore_Version_Delete"
    };
    
    // initial init
    ox.ToolBarController.processSelection("version", false, [], true);
    
    // add handler function (must return true when action taken)
    ox.ToolBarController.addHandler(function (id) {        
        // defined by event?
        if (events[id] !== undefined) {
            // trigger
            triggerEvent(events[id]);
            return true;
        }
        
        // individual handlers
        switch (id) {
        case "infostore-view-hsplit":
            if (currentpath[1] !== "split") {
                switchtoView('infostore/split');
            }
            return true;
        case "infostore-view-list":
            if (currentpath[1] !== "list") {
                switchtoView('infostore/list');
            }
            return true;
        }
    });
    
    // add to top-level element
    toolbar.addTo(ox.widgets.toolBar.viewControl);
    
    // "edit" tab is default
    ox.ToolBarController.setDefaultTab(toolbar, 1);
};
/**
 *
 * All content on this website (including text, images, source
 * code and any other original works), unless otherwise noted,
 * is licensed under a Creative Commons License.
 *
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 *
 * Copyright (C) Open-Xchange Inc., 2006-2012
 * Mail: info@open-xchange.com
 *
 * @author Viktor Pracht <viktor.pracht@open-xchange.com>
 */

ox.gui.initToolBarContacts = function ($p) {

    var upsell = ox.api.window.core.ox.upsell;

    /*
     * Meta data
     */
    var meta = $.extend(ox.widgets.toolBar.meta, {
        contacts: {
            compose: {
                title: _("New Contact"),
                icons: ["mod_contacts.png"],
                big: true
            },
            composeDistributionList: {
                title: _("New Distribution List"),
                icons: ["distributionlist.png"],
                big: true
            },
            edit: {
                title: _("Edit"),
                icons: ["edit.png"],
                big: true,
                requires: ["one", "modify"]
            },
            "delete": {
                title: _("Delete"),
                icons: ["delete.png"],
                big: true,
                requires: ["some", "delete"]
            },
            print: {
                title: _("Print"),
                icons: ["print.png"],
                big: true,
                requires: function (ability) {
                    return currentpath[1] !== "cards" && ability.some;
                }
            },
            move: {
                title: _("Move"),
                icons: ["contact_move.png"],
                requires: ["some", "delete"],
                visible: upsell.visible("contacts-copy-move")
            },
            copy: {
                title: _("Copy"),
                icons: ["contact_copy.png"],
                requires: ["some"],
                visible: upsell.visible("contacts-copy-move")
            },
            sendmail: {
                title: _("Send E-Mail"),
                icons: ["mod_mail.png"],
                requires: ["some"]
            },
            vcard: {
                title: _("Send as vCard"),
                icons: ["forward.png"],
                requires: ["some"]
            },

            /*Attachments*/
            "attachment-open": {
                title: _("Open attachment"),
                icons: ["attachment_open.png"],
                selection: "attachments",
                requires: function(ability) {
                    return ability.one;
                }
            },
            "attachment-save": {
                title: _("Save as"),
                icons: ["send_as_attachment.png"],
                selection: "attachments",
                requires: function(ability) {
                    return ability.one;
                }
            },
            "attachment-save-infostore": {
                title: _("Save in InfoStore"),
                icons: ["save.png"],
                selection: "attachments",
                requires: function(ability) {
                    return ability.one;
                },
                mousedown: function (options, e) {
                    if (ox.api.config.get("modules.infostore.module") == false) {
                        corewindow.triggerEvent("Feature_Not_Available", "modules/contacts/save_to_infostore", window);
                    }
                }
            }
        },
        contactsView: {
            cards: {
                title: _("Cards"),
                icons: ["card_view.png"],
                behavior: "radio",
                group: "contacts-view",
                big: true,
                checked: function () {
                    return currentpath[1] === "cards";
                }
            },
            phonelist: {
                title: _("Phone list"),
                icons: ["list_view.png"],
                behavior: "radio",
                group: "contacts-view",
                big: true,
                checked: function () {
                    return currentpath[1] === "phonelist";
                }
            },
            detail: {
                title: _("Detail"),
                icons: ["detail_view.png"],
                requires: ["one"],
                behavior: "radio",
                group: "contacts-view",
                big: true,
                checked: function () {
                    return currentpath[1] === "detail";
                }
            }
        }
    });


    // toggles attachment section
    var toggleAttachmentSection = function () {
        if (currentpath[0] === "contacts") {
            // get parent tab
            var tab = ox.widgets.toolBar.tabs.contactsEdit;
            var section = tab.children[2];
            var visible = section.isVisible();
            // in team view?
            if (currentpath[2] === "attachment") {
                if (!visible) {
                    section.setVisible(true);
                    tab.validate();
                }
            } else if (visible) {
                section.setVisible(false);
                tab.validate();
            }
        }
    };

    /*
     * Add widgets
     */

    // vars
    var tabs = ox.widgets.toolBar.tabs;
    var sections = ox.widgets.toolBar.sections;
    var items = ox.widgets.toolBar.menuItems;
    var toolbar, tab, section;

    // fn
    var add = $p.add;

    // create toolbar instance
    toolbar = ox.widgets.toolBar.views.contacts = new ox.gui.TabMenu(
        $p.getOptions({
            id: "toolbar-contacts",
            showSectionNames: false
        })
    );

    // new
    tab = tabs.contactsCreate = new ox.gui.PanelMenu("toolbar-contacts-create",
        _("&New")).addTo(toolbar);
    section = sections.contactsCreate = new ox.gui.PanelMenuSection(_("New"), "contactsCreateSection").addTo(tab);
    add(section, meta.create, "contacts-create",
        ["mail", "calendar", "contact", "distributionlist", "task", "file"]
    );
    // edit
    tab = tabs.contactsEdit = new ox.gui.PanelMenu("toolbar-contacts-edit",
        //#. '&' before a letter indicates a shortcut, i.e. Alt+E for '&Edit'
        _("&Edit")).addTo(toolbar);
    // section
    section = sections.contactsCompose = new ox.gui.PanelMenuSection(_("Compose"), "contactsComposeSection").addTo(tab);
    // add items
    add(section, meta.contacts, "contacts-edit",
        ["compose", "composeDistributionList"]
    );
    // section
    section = sections.contactsEdit = new ox.gui.PanelMenuSection(_("Edit"), "contactsEditSection").addTo(tab);
    // add items
    add(section, meta.contacts, "contacts-edit",
        ["edit", "delete", "print", "sendmail", "vcard", "move", "copy"]
    );

    // section attachments, will only be shown in attachment tab
    if (upsell.isVisible("infostore") && upsell.isVisible("contacts-attachments")) {
        section = sections.contactsAttachment = new ox.gui.PanelMenuSection(_("Attachments"), "contactsAttachmentSection").addTo(tab);
        // add items
        add(section, meta.contacts, "contacts-edit",
            ["attachment-open", "attachment-save", "attachment-save-infostore"]
        );
    }

    // view
    tab = tabs.contactsView = new ox.gui.PanelMenu("toolbar-contacts-view",
        //#. '&' before a letter indicates a shortcut, i.e. Alt+V for '&View'
        _("&View")).addTo(toolbar);
    section = sections.contactsView = new ox.gui.PanelMenuSection(_("View"), "contactsViewSection").addTo(tab);
    add(section, meta.contactsView, "contacts-view",
        ["cards", "phonelist", "detail"]);

    // Tagging (Flags & Categories)
    if (upsell.isVisible("contacts-flags")) {
        tab = tabs.contactsTagging = new ox.gui.PanelMenu("toolbar-contacts-tag",
            //#. '&' before a letter indicates a shortcut, i.e. Alt+F for '&Flags'
            _("&Flags")).addTo(toolbar);
        section = sections.contactsFlags = new ox.gui.PanelMenuSection(_("Flags"), "contactsFlagsSections").addTo(tab);
        add(section, meta.flags, "contacts-flag",
           ["0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "10"]);
    }

    // Social OX
    if (ox.api.config.get(["modules","com.openexchange.subscribe"]) === true ||
            ox.api.config.get(["modules","com.openexchange.publish"]) === true ||
            ox.api.config.get("ui.upsell.features.social") === true) {
        tab = tabs.contactsSocial = new ox.gui.PanelMenu("toolbar-contacts-socialOX",
            _("&Social")).addTo(toolbar);
        section = sections.contactsSocialOX = new ox.gui.PanelMenuSection(_("Social"), "contactsSocialOXSections").addTo(tab);
        add(section, meta.socialOX, "contacts-socialOX", [ "refresh", "publish", "subscribe" ]);
    }

    // Add section search when embbedded mode is on
    if (corewindow.embedded) {
        ox.widgets.searchField.contacts = new ox.gui.SearchField("ox-topbar-search-contacts").
        addListener("widget:search", function(e) {
            triggerEvent("OX_New_Search", ox.api.ui.getActiveModule(), e.data.value);
        });

        tab = tabs.contactsSearch = new ox.gui.PanelMenu("toolbar-contacts-search",
                _("&Search")).addTo(toolbar);
        // section
        section = sections.contactsSearch = new ox.gui.PanelMenuSection(_("Search"), "contactsSearchSection").addTo(tab);
        // add items
        section.add(ox.widgets.searchField.contacts);
    }

    /*
     * Add handler
     */

    // event hash
    var events = {
        "contacts-edit-compose": "OX_Before_Create_Contact",
        "contacts-edit-compose-clone": "OX_Before_Create_Contact",
        "contacts-edit-composeDistributionList": "OX_Before_Create_Distributionlist",
        "contacts-edit-edit": "OX_Contact_Edit",
        "contacts-edit-print": "OX_Print",
        "contacts-edit-duplicate": "OX_Contact_Duplicate",
        "contacts-edit-move": "OX_Contact_Move",
        "contacts-edit-copy": "OX_Contact_Copy",
        "contacts-edit-delete": "OX_Contact_Delete",
        "contacts-edit-sendmail": "OX_Contact_SendMail",
        "contacts-edit-vcard": "OX_Contact_SendVCard",
        "contacts-edit-attachment-save" : "OX_Attachment_Save",
        "contacts-edit-attachment-save-infostore" : "OX_Attachment_SaveInfoStore"
    };

    var views = {
        "contacts-view-cards": "cards",
        "contacts-view-phonelist": "phonelist",
        "contacts-view-detail": "detail"
    };

    // add handler function (must return true when action taken)
    ox.ToolBarController.addHandler(function (id) {
        if (id in events) {
            triggerEvent(events[id]);
            return true;
        }
        if (id in views) {
            menuswitchContactView(views[id]);
            return true;
        }
        switch(id){
            case "contacts-edit-attachment-open":
                triggerEvent("OX_Attachment_Open");
                return true;
        }
        return false;
    });

    // attachments section visible only in attachments tab
    ox.api.event.register("OX_View_Changed", toggleAttachmentSection);
    // add to top-level element
    toolbar.addTo(ox.widgets.toolBar.viewControl);

    // initial processing of selection
    ox.ToolBarController.processSelection("attachments", false, [], true);

    // "edit" tab is default
    ox.ToolBarController.setDefaultTab(toolbar, 1);
};
/**
 * 
 * All content on this website (including text, images, source
 * code and any other original works), unless otherwise noted,
 * is licensed under a Creative Commons License.
 * 
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 * 
 * Copyright (C) Open-Xchange Inc., 2006-2012
 * Mail: info@open-xchange.com 
 * 
 * @author Viktor Pracht <viktor.pracht@open-xchange.com>
 */

ox.gui.initToolBarConfig = function ($p) {

    
    temporary.configuration = {};
    
    var auto_id = 0;
    
    /**
     * Creates a new toolbar for a configuration page.
     * @param {String} title The title of the toolbar tab.
     * @param {Array} sections An array of section definitions. Each section
     * definition is an object with the folowing members:
     * - {String} title
     * - {Array} buttons An array with button definition. Each button definition
     *                   is an options object for ox.gui.MenuItem.
     * - {String} id An optional ID for the section. If not specified, a unique
     *               ID is generated automatically. In both cases, the ID is
     *               prefixed with "
     * @param {String} id An optional ID for the toolbar. If not specified,
     * a unique ID is generated automatically. In both cases, the ID is prefixed
     * with "config-" for the ox.gui.TabMenu and with "config-tab-" for
     * the ox.gui.PanelMenu.
     * @type ox.gui.PanelMenu
     * @return The created PanelMenu instance, which can be passed to
     * showToolbar() to display it.
     */
    temporary.configuration.newToolbar = function(title, sections, id) {
        var ID = (id || auto_id++);
        var toolbar = new ox.gui.TabMenu(
            $p.getOptions({
                id: "toolbar-" + ID,
                showSectionNames: false
            })
        );
        var tab = new ox.gui.PanelMenu("toolbar-" + ID + "-" + ID,
            title).addTo(toolbar);
        for (var i = 0; i < sections.length; i++) {
            var meta = sections[i];
            var section = sections[i] = new ox.gui.PanelMenuSection(meta.title);
            section.addTo(tab);
            for (var j = 0; j < meta.buttons.length; j++) {
                var button = meta.buttons[j];
                if (button.event) {
                    button.action = (function(event) {
                        return function() { 
                            triggerEvent(event); 
                            corewindow.track({
                                type: "click",
                                dest: "menu_item",
                                data: { 
                                    id: button.id, 
                                    title: String(button.title)
                                },
                                text: "Click on menu item \"" + button.title + "\" (" + button.id + ")",
                                module: ox.UIController.getModule(),
                                view: ox.UIController.getView()
                            });
                        };
                    })(button.event);
                    delete button.event;
                }
                button.id = ID + "-" + (meta.id || i) + "-" + (button.id || j);
                var item = new ox.gui.MenuItem(button);
                ox.widgets.toolBar.menuItems[item.id] = item;
                section.add(item);
            }
        }
        toolbar.addTo(ox.widgets.toolBar.viewControl);
        // add resize handler
        toolbar.addListener("widget:togglehover", $p.resize);
        return toolbar;
    };
    
    /**
     * Switches to the specified toolbar.
     * This function is used by the configuration views instead of the default
     * handling because each configuration page has its own toolbar instance.
     * @param {ox.gui.PanelMenu} toolbar The toolbar to show.
     */
    temporary.configuration.showToolbar = function(toolbar) {
        ox.widgets.toolBar.viewControl.showView(toolbar.id);
        toolbar.setPosition(0);
        // TODO: fix temporary resize hack
        jQuery(window).trigger("resize");
    };
    
    /**
     * A section definition for the default configuration save button.
     * Normally used as the first array member of the second parameter to
     * newToolbar().
     */
    temporary.configuration.saveButton = {
        title: _("Save"),
        buttons: [{
            title: _("Save"),
            icons: ["img/menu/btn_save.gif"],
            big: true,
            event: "OX_SAVE_OBJECT"
        }]
    };
    
    temporary.configuration.defaultToolbar = temporary.configuration.newToolbar(
        _("Preferences"), [temporary.configuration.saveButton], "preferences");
};
/**
 * 
 * All content on this website (including text, images, source
 * code and any other original works), unless otherwise noted,
 * is licensed under a Creative Commons License.
 * 
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 * 
 * Copyright (C) Open-Xchange Inc., 2006-2012
 * Mail: info@open-xchange.com 
 * 
 * @author Matthias Biggeleben <matthias.biggeleben@open-xchange.com>
 * 
 */

/*jslint bitwise: false, nomen: false, onevar: false, plusplus: false, regexp: false, white: true, browser: true, devel: true, evil: true, forin: true, undef: true, eqeqeq: true, immed: true */

/*global ox, jQuery, $, triggerEvent, menuswitchMailView */

ox.gui.initToolBarPermissions = function ($p) {
    
    /*
     * Meta data
     */
    var meta = $.extend(ox.widgets.toolBar.meta, {
        /*
         * Folder/object permissions
         */
        permissions: {
            // save button
            save: {
                title: _("Save"),
                icons: ["save.png"],
                big: true,
                action: function() {
                    triggerEvent("OX_SAVE_OBJECT");
                    return true;
                }
            },
            // add permission
            add: {
                title: _("Add user"),
                icons: ["userrights.png"],
                action: function () {
                    // call magic global function with really self-explaining name TODO: fix!
                    ox.api.folder.get({
                        folder: configparentfolder != -1 ? configparentfolder : configfolder,
                        success: function (oFolder) {
                            triggerEvent('OX_Configuration_Add_User', oFolder);
                        }
                    });
                    return true;
                }
            },
            remove: {
                title: _("Remove user"),
                icons: ["userrights_remove.png"],
                action: function () {
                    // trigger event
                    triggerEvent("OX_Configuration_Remove_User");
                    return true;
                }
            },
            // folder admin
            adminYes: {
                title: _("Administrator"),
                icons: ["mod_configuration.png"],
                action: function() {
                    changeRights("ADMIN", 1);
                },
                behavior: "radio",
                group: "permissions-admin"
            },
            adminNo: {
                title: _("User"),
                icons: ["user.png"],
                action: function() {
                    changeRights("ADMIN", 0);
                },
                behavior: "radio",
                group: "permissions-admin"
            },
            // folder permissions
            folderNone: {
                title: _("None"),
                icons: ["delete_folder.png"],
                action: function() {
                    changeRights("FOLDER", 0);
                },
                behavior: "radio",
                group: "permissions-folder"
            },
            folderVisible: {
                title: _("Visible folder"),
                icons: ["folder_closed.png"],
                action: function() {
                    changeRights("FOLDER", 1);
                },
                behavior: "radio",
                group: "permissions-folder"
            },
            folderCreateObjects: {
                title: _("Create objects"),
                icons: ["folder_closed.png"],
                action: function() {
                    changeRights("FOLDER", 2);
                },
                behavior: "radio",
                group: "permissions-folder"
            },
            folderCreateFolders: {
                title: _("Create subfolders"),
                icons: ["folder_opened.png"],
                action: function() {
                    changeRights("FOLDER", 4);
                },
                behavior: "radio",
                group: "permissions-folder"
            },
            folderMax: {
                title: _("Maximum"),
                icons: ["folder_opened.png"],
                action: function() {
                    changeRights("FOLDER", 64);
                },
                behavior: "radio",
                group: "permissions-folder"
            },
            // object permissions
            // read
            readNone: {
                title: _("None"),
                icons: ["userrights_read.png"],
                action: function() {
                    changeRights("READ", 0);
                },
                behavior: "radio",
                group: "permissions-read"
            },
            readOwn: {
                title: _("Read own"),
                icons: ["userrights_read.png"],
                action: function() {
                    changeRights("READ", 1);
                },
                behavior: "radio",
                group: "permissions-read"
            },
            readAll: {
                title: _("Read all"),
                icons: ["userrights_read.png"],
                action: function() {
                    changeRights("READ", 2);
                },
                behavior: "radio",
                group: "permissions-read"
            },
            readMax: {
                title: _("Maximum"),
                icons: ["userrights_read.png"],
                action: function() {
                    changeRights("READ", 64);
                },
                behavior: "radio",
                group: "permissions-read"
            },
            // write/modify
            modifyNone: {
                title: _("None"),
                icons: ["userrights_write.png"],
                action: function() {
                    changeRights("WRITE", 0);
                },
                behavior: "radio",
                group: "permissions-write"
            },
            modifyOwn: {
                title: _("Modify own"),
                icons: ["userrights_write.png"],
                action: function() {
                    changeRights("WRITE", 1);
                },
                behavior: "radio",
                group: "permissions-write"
            },
            modifyAll: {
                title: _("Modify all"),
                icons: ["userrights_write.png"],
                action: function() {
                    changeRights("WRITE", 2);
                },
                behavior: "radio",
                group: "permissions-write"
            },
            modifyMax: {
                title: _("Maximum"),
                icons: ["userrights_write.png"],
                action: function() {
                    changeRights("WRITE", 64);
                },
                behavior: "radio",
                group: "permissions-write"
            },
            // delete
            deleteNone: {
                title: _("None"),
                icons: ["userrights_delete.png"],
                action: function() {
                    changeRights("DELETE", 0);
                },
                behavior: "radio",
                group: "permissions-delete"
            },
            deleteOwn: {
                title: _("Delete own"),
                icons: ["userrights_delete.png"],
                action: function() {
                    changeRights("DELETE", 1);
                },
                behavior: "radio",
                group: "permissions-delete"
            },
            deleteAll: {
                title: _("Delete all"),
                icons: ["userrights_delete.png"],
                action: function() {
                    changeRights("DELETE", 2);
                },
                behavior: "radio",
                group: "permissions-delete"
            },
            deleteMax: {
                title: _("Maximum"),
                icons: ["userrights_delete.png"],
                action: function() {
                    changeRights("DELETE", 64);
                },
                behavior: "radio",
                group: "permissions-delete"
            }
        }
    });
    
    /*
     * Add widgets
     */
    
    // vars
    var tabs = ox.widgets.toolBar.tabs;
    var items = ox.widgets.toolBar.menuItems;
    var toolbar, tab, section;
    
    // fn
    var add = $p.add;
    
    // create toolbar instance
    toolbar = new ox.gui.TabMenu(
        $p.getOptions({
            id: "toolbar-permissions",
            showSectionNames: false
        })
    );
    
    // shortcuts
    configToolbars.folderPermissions = ox.widgets.toolBar.views.permissions = toolbar;
    
    /*
     * Tab: Save/Add/Remove
     */
    tab = tabs.permissionsMain = new ox.gui.PanelMenu("toolbar-permissions-save",
        //#. '&' before a letter indicates a shortcut, i.e. Alt+N for '&New'
        _("&Save")).addTo(toolbar);
    
    // section "save"
    section = new ox.gui.PanelMenuSection(_("Save")).addTo(tab);
    // add items
    add(section, meta.permissions, "permissions-save",
        ["save", "add", "remove"]
    );
    
    /*
     * Tab: Folder permissions
     */
    tab = tabs.permissionsForFolders = new ox.gui.PanelMenu("toolbar-permissions-folder",
        //#. '&' before a letter indicates a shortcut, i.e. Alt+N for '&New'
        _("&Folder permissions")).addTo(toolbar);
    
    // section "folder admin"
    section = new ox.gui.PanelMenuSection(_("Folder admin")).addTo(tab);
    // add items
    add(section, meta.permissions, "permissions-folder",
        ["adminYes", "adminNo"]
    );
    
    // section "folder permissions"
    section = new ox.gui.PanelMenuSection(_("Folder permissions")).addTo(tab);
    // add items
    add(section, meta.permissions, "permissions-folder",
        ["folderNone", "folderVisible", "folderCreateObjects", "folderCreateFolders", "folderMax"]
    );
    
    /*
     * Tab: Object permissions
     */
    tab = tabs.permissionsForObjects = new ox.gui.PanelMenu("toolbar-permissions-object",
        //#. '&' before a letter indicates a shortcut, i.e. Alt+N for '&New'
        _("&Object permissions")).addTo(toolbar);
    
    // section "object permissions: READ" 
    section = new ox.gui.PanelMenuSection(pgettext("to", "Read")).addTo(tab);
    // add items
    add(section, meta.permissions, "permissions-object",
        ["readNone", "readOwn", "readAll", "readMax"]
    );
    
    // section "object permissions: WRITE"
    section = new ox.gui.PanelMenuSection(_("Modify")).addTo(tab);
    // add items
    add(section, meta.permissions, "permissions-object",
        ["modifyNone", "modifyOwn", "modifyAll", "modifyMax"]
    );
    
    // section "object permissions: DELETE"
    section = new ox.gui.PanelMenuSection(_("Delete")).addTo(tab);
    // add items
    add(section, meta.permissions, "permissions-object",
        ["deleteNone", "deleteOwn", "deleteAll", "deleteMax"]
    );
    
    // add to top-level element
    toolbar.addTo(ox.widgets.toolBar.viewControl);
};
/**
 * 
 * All content on this website (including text, images, source
 * code and any other original works), unless otherwise noted,
 * is licensed under a Creative Commons License.
 * 
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 * 
 * Copyright (C) Open-Xchange Inc., 2006-2012
 * Mail: info@open-xchange.com 
 * 
 * @author Viktor Pracht <viktor.pracht@open-xchange.com>
 */

ox.gui.initToolBarMessaging = function ($p) {

    /*
     * Meta data
     */
    var meta = $.extend(ox.widgets.toolBar.meta, {
        messaging: {
            "new": {
                title: _("New"),
                icons: ["img/mail/btn_sendmail.gif"],
                big: true
            }
        }
    });
    
    /*
     * Add widgets
     */
    
    // vars
    var tabs = ox.widgets.toolBar.tabs;
    var items = ox.widgets.toolBar.menuItems;
    var toolbar, tab, section;
    
    // fn
    var add = $p.add;
    
    // create toolbar instance
    toolbar = ox.widgets.toolBar.views.messaging = new ox.gui.TabMenu(
        $p.getOptions({
            id: "toolbar-messaging",
            showSectionNames: false
        })
    );
    
    // new
    tab = tabs.messaging = new ox.gui.PanelMenu(
        "toolbar-messaging-new", _("&Messaging")).addTo(toolbar);
    section = new ox.gui.PanelMenuSection(_("Message")).addTo(tab);
    add(section, meta.messaging, "messaging", ["new"]);
    
    // add handler function (must return true when action taken)
    ox.ToolBarController.addHandler(function (id) {
        if (id == "messaging-new") {
            triggerEvent("NewMessage");
            return true;
        }
        return false;
    });
    
    // add to top-level element
    toolbar.addTo(ox.widgets.toolBar.viewControl);
};
/**
 * 
 * All content on this website (including text, images, source
 * code and any other original works), unless otherwise noted,
 * is licensed under a Creative Commons License.
 * 
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 * 
 * Copyright (C) Open-Xchange Inc., 2006-2012
 * Mail: info@open-xchange.com 
 * 
 * @author Alexander Quast <alexander.quast@open-xchange.com>
 * 
 */

/*jslint bitwise: false, nomen: false, onevar: false, plusplus: false, regexp: false, white: true, browser: true, devel: true, evil: true, forin: true, undef: true, eqeqeq: true, immed: true */

/*global ox, jQuery, $, equals, register, triggerEvent, menuswitchMailView */
ox.gui.initToolBarNewWindow = function () {
    
    ox.widgets = {}; 
    
    var $m = ox.gui.initToolBar();
    
    // root node for menu
    var root = $("#window_toolbar_container")[0];
    
    /*
     * Add top-level view controller (containing all toolbars)
     */
    var control = ox.widgets.toolBar.viewControl = new ox.gui.ViewControl("toolbar-view-control").
        removeCSSClass("oxStretch").
        setParentDOMNode(root).
        addTo(ox.desktop);
    
    // i18n
    corewindow.register("LanguageChanged", function () {
        // event forward
        ox.desktop.triggerGlobalEvent("LanguageChanged");
    });
    
    return $m;
};

/**
 * 
 * All content on this website (including text, images, source
 * code and any other original works), unless otherwise noted,
 * is licensed under a Creative Commons License.
 * 
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 * 
 * Copyright (C) Open-Xchange Inc., 2006-2012
 * Mail: info@open-xchange.com 
 * 
 * @author  Alexander Quast <alexander.quast@open-xchange.com>
 */

ox.gui.initToolBarNewWindowMail = function () {

    var $p = ox.gui.initToolBarNewWindow(),
        upsell = ox.api.window.core.ox.upsell;
    /*
     * Meta data
     */
    var meta = $.extend(ox.widgets.toolBar.meta, {
        /*
         * New Mail Window
         */
        "window_mail": {
            "send": {
                title: _("Send"),
                icons: ["mail_send.png"],
                big: true,
                selection: "default"
            },
            "save-draft": {
                title: _("Save draft"),
                icons: ["mail_write.png"],
                big: true,
                requires: function() {
                    // check if we can save
                    return true; 
                }
            },
            "view": {
                title: _("View"),
                big: true,
                icons: ["options.png"]
            },
            "from": {
                title: pgettext("who", "From"),
                icons: ["dummy.gif"],
                behavior: "checkbox",
                checked: ox.api.config.get("gui.mail.newmail_options.from")
            },
            "replyto": {
                title: _("Reply-to"),
                icons: ["dummy.gif"],
                behavior: "checkbox",
                visible: function () {
                    return ox.api.config.get("ui.mail.replyTo.configurable", false) === true;
                },
                checked: function () {
                    return ox.api.config.get("ui.mail.replyTo.configurable", false) === true 
                    	&& ox.api.config.get("gui.mail.newmail_options.replyto", false);
                }
            },
            "cc": {
                title: _("CC"),
                icons: ["dummy.gif"],
                behavior: "checkbox",
                checked: ox.api.config.get("gui.mail.newmail_options.cc")
            },
            "bcc": {
                title: _("BCC"),
                icons: ["dummy.gif"],
                behavior: "checkbox",
                checked: ox.api.config.get("gui.mail.newmail_options.bcc")
            },
            "options": {
                title: _("Options"),
                icons: ["dummy.gif"],
                behavior: "checkbox",
                checked: ox.api.config.get("gui.mail.newmail_options.options")
            },
            "editorToolbar": {
                title: _("Editor toolbar"),
                icons: ["dummy.gif"],
                behavior: "checkbox",
                visible: function () {
                    return ox.api.config.get("gui.mail.formatmessage") !== "TEXT/PLAIN";
                },
                checked: function () {
                    return ox.api.config.get("gui.mail.newmail_options.toolbar", true);
                }
            },
            "attachment-add": {
                title: _("Add attachment"),
                icons: ["attachment_add.png"],
                big: true
            },
            "attachment-add-file": {
                title: _("Attach local file"),
                icons: ["attachment_add.png"]
            },
            "attachment-add-infoitem": {
                title: _("Attach InfoItem"),
                icons: ["attachment_add.png"],
                requires: function(ability) {
                    return ox.api.config.get("modules.infostore.module");
                },
                mousedown: function (options, e) {
                    if (ox.api.config.get("modules.infostore.module") == false) {
                        corewindow.triggerEvent("Feature_Not_Available", "modules/mail/new/add_infostore_attachment", window);
                    }
                },
                visible: upsell.visible("infostore")
            },
            "attachment-remove": {
                title: _("Remove attachment"),
                icons: ["attachment_remove.png"],
                selection: "attachments",
                requires: ["some"],
                big: true
            },
            "format-html": {
                title: _("HTML"),
                behavior: "radio",
                group: "format",
                checked: function() {
                    return configGetKey("gui.mail.formatmessage").toUpperCase()
                        === "TEXT/HTML";
                }
            },
            "format-plaintext": {
                title: _("Plain text"),
                behavior: "radio",
                group: "format",
                checked: function() {
                    return configGetKey("gui.mail.formatmessage").toUpperCase()
                        === "TEXT/PLAIN";
                }
            },
            "format-both": {
                title: _("HTML and plain text"),
                behavior: "radio",
                group: "format",
                checked: function() {
                    return configGetKey("gui.mail.formatmessage").toUpperCase()
                        === "ALTERNATIVE";
                }
            },
            "cancel": {
                 title: _("Cancel"),
                 big: true,
                 icons: ["cancel.png"]
            }
        }
    });
    
    
    /*
     * Add widgets
     */
    
    // vars
    var tabs = ox.widgets.toolBar.tabs;
    var items = ox.widgets.toolBar.menuItems;
    var toolbar, tab, section;
    
    // fn
    var add = $p.add;
    
    // create toolbar instance
    toolbar = ox.widgets.toolBar.views.detailMail = new ox.gui.TabMenu(
        $p.getOptions({
            id: "new-mail-menu",
            showSectionNames: false
        })
    );
    
    // main tab
    tab = tabs.newMailWindow = new ox.gui.PanelMenu(
        "window-mail-main", _("&E-Mail")).addTo(toolbar);
    
    section = new ox.gui.PanelMenuSection(_("Mail")).addTo(tab);
    add(section, ox.widgets.toolBar.meta.window_mail, "mail_new",
        ["send", "save-draft"]);
    
    section = new ox.gui.PanelMenuSection(_("Mail")).addTo(tab);
    add(section, ox.widgets.toolBar.meta.window_mail, "mail_new",
        ["attachment-add", "attachment-remove"]);
    
    // add sub items
    add(items["mail_new-attachment-add"], ox.widgets.toolBar.meta.window_mail, "mail_new", 
        ["attachment-add-file", "attachment-add-infoitem"]
    );
    
    section = new ox.gui.PanelMenuSection(_("View")).addTo(tab);
    // add items
    add(section, ox.widgets.toolBar.meta.window_mail, "mail_new",
        ["view"]
    );
    
    add(items["mail_new-view"], ox.widgets.toolBar.meta.window_mail, "mail_new", 
        ["from", "replyto", "cc", "bcc", "options", "editorToolbar"]
    );
    
    section = new ox.gui.PanelMenuSection(_("Format")).addTo(tab);
    add(section, ox.widgets.toolBar.meta.window_mail, "mail_new",
        ["format-html", "format-plaintext", "format-both"]);
    
    section = new ox.gui.PanelMenuSection(_("Cancel")).addTo(tab);
    add(section, ox.widgets.toolBar.meta.window_mail, "mail_new",
        ["cancel"]);
    
    // add to top-level element
    toolbar.addTo(ox.widgets.toolBar.viewControl);
    
    $p.postInit();
    
    // bind to subselected
    register("SubSelected", function(array) {
        ox.ToolBarController.processSelection("attachments", false, array ,true);
    });
    
    // event mapping
    
    var events = {
            "mail_new-send" : "OX_SAVE_OBJECT",
            "mail_new-save-draft" : "OX_Save_Template_Object",
            "mail_new-attachment-add-file": "OX_Object_Add_Attachment",
            "mail_new-attachment-add-infoitem": "OX_Object_Add_Attachment_Infostore",
            "mail_new-attachment-remove" : "OX_Object_Attachment_Delete",
            "mail_new-cancel" : "OX_Cancel_Object"
    };
    
    // add handler function (must return true when action taken)
    ox.ToolBarController.addHandler(function (id) {
        
        // regex matches
        var m;
        
        // defined by event?
        if (events[id] !== undefined) {
            // trigger
            triggerEvent(events[id]);
            return true;
        }
        
        // individual handlers
        switch (id) {
        case "mail_new-from":
            triggerEvent("OX_New_Mail_Toggle_Additional_Header", "from");
            return true;
        case "mail_new-replyto":
            triggerEvent("OX_New_Mail_Toggle_Additional_Header", "replyto");
            return true;
        case "mail_new-cc":
            triggerEvent("OX_New_Mail_Toggle_Additional_Header", "cc");
            return true;
        case "mail_new-bcc":
            triggerEvent("OX_New_Mail_Toggle_Additional_Header", "bcc");
            return true;
        case "mail_new-options":
            triggerEvent("OX_New_Mail_Toggle_Additional_Header", "options");
            return true;
        case "mail_new-editorToolbar":
            triggerEvent("OX_New_Mail_Toggle_Toolbar");
            return true;
        case "mail_new-format-html":
            triggerEvent("OX_New_Mail_Set_Format", "TEXT/HTML");
            return true;
        case "mail_new-format-plaintext":
            triggerEvent("OX_New_Mail_Set_Format", "TEXT/PLAIN");
            return true;
        case "mail_new-format-both":
            triggerEvent("OX_New_Mail_Set_Format", "ALTERNATIVE");
            return true;
        }
    });
    
    ox.desktop.validate();
    ox.api.window.core.triggerEvent("OX_Window_Toolbar_Ready", {
            handle : window.url.guid,
            type : "mail"
    });
    // initial redraw to update icon state
    ox.ToolBarController.processSelection("default", false, [], true);
    ox.ToolBarController.processSelection("attachments", false, [], true);
};
/**
 * 
 * All content on this website (including text, images, source
 * code and any other original works), unless otherwise noted,
 * is licensed under a Creative Commons License.
 * 
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 * 
 * Copyright (C) Open-Xchange Inc., 2006-2012
 * Mail: info@open-xchange.com 
 * 
 * @author  Alexander Quast <alexander.quast@open-xchange.com>
 */

ox.gui.initToolBarNewWindowMailDetail = function () {

    var $p = ox.gui.initToolBarNewWindow();
    
    /**
     * Returns true if the displayed message is nested inside another message.
     */
    function isNestedMessage() {
        if ("value" in isNestedMessage) return isNestedMessage.value;
        var msg = ox.api.window.getData(url.guid).oNestedMessage
        return isNestedMessage.value = Boolean(msg && msg.length);
    }

    /*
     * Meta data
     */
    var meta = $.extend(ox.widgets.toolBar.meta, {
        /*
         * Mail
         */
        mail: {
            "edit": {
                title: _("Edit draft"),
                icons: ["draft.png"],
                big: true,
                requires: function (ability) {
                    // current folder is drafts folder?
                    return isStdFolder("drafts");
                },
                visible: function () {
                    // current folder is drafts folder?
                    return isStdFolder("drafts");
                }
            },
            "reply": {
                title: _("Reply"),
                icons: ["reply.png"],
                big: true,
                requires: function() { return !isNestedMessage(); }
            },
            "reply-all": {
                title: _("Reply all"),
                icons: ["reply_all.png"],
                big: true,
                requires: function() { return !isNestedMessage(); }
            },
            "forward": {
                title: _("Forward"),
                icons: ["forward.png"],
                big: true,
                requires: function() { return !isNestedMessage(); }
            },
            "delete": {
                title: _("Delete"),
                icons: ["delete.png"],
                big: true,
                requires: function(ability) {
                    return ability["delete"] && !isNestedMessage();
                }
            },
            "save": {
                title: _("Save"),
                icons: ["send_as_link.png"]
            },
            "view-source": {
                title: _("Show source"),
                icons: ["mail_source.png"]
            },
            "print": {
                title: _("Print"),
                icons: ["print.png"]
            },
            "mark-as": {
                title: _("Mark as"),
                icons: ["mark_as.png"],
                requires: function() { return !isNestedMessage(); }
            },
            "mark-as-read": {
                title: _("Read")
            },
            "mark-as-unread": {
                title: _("Unread")
            },
            "mark-as-answered": {
                title: _("Answered"),
                requires: ["modify"]
            },
            "mark-as-unanswered": {
                title: _("Unanswered"),
                requires: ["modify"]
            },
            "mark-as-deleted": {
                title: _("Deleted"),
                requires: ["delete"]
            },
            "mark-as-undeleted": {
                title: _("Undeleted"),
                requires: ["delete"]
            },
            "copy": {
                title: _("Copy"),
                icons: ["mail_copy.png"],
                requires: ["some"]
            },
            /*
             * Cancel/Close
             */
            "cancel": {
                title: _("Close"),
                big: true,
                icons: ["cancel.png"]
           }
        }
    });
    
    
    // SPAM buttons
    (function () {
        
        // get some config parameter
        var spamButton = ox.api.config.get("mail.spambutton", false);
        var spamFolder = ox.api.config.get("mail.folder.spam");
        
        var folder = url.folder;
        var accountId = ox.api.folder.derive("accountId", folder);
        
        if (spamButton === true && accountId === 0 && folder !== spamFolder) {
            meta.mail.spam = {
                title: _("Spam"),
                icons: ["spam.png"]
            };
        }
        else if (spamButton === true && folder === spamFolder) {
            meta.mail.ham = {
                title: _("Not Spam"),
                icons: ["ham.png"]
            };
        }
        
    }());
    
    /*
     * Add widgets
     */
    
    // vars
    var tabs = ox.widgets.toolBar.tabs;
    var items = ox.widgets.toolBar.menuItems;
    var toolbar, tab, section;
    
    // fn
    var add = $p.add;
    
    // create toolbar instance
    toolbar = ox.widgets.toolBar.views.mail = new ox.gui.TabMenu(
        $p.getOptions({
            id: "toolbar-mail",
            showSectionNames: false
        })
    );
    
    // edit
    tab = tabs.mailEdit = new ox.gui.PanelMenu("toolbar-mail_detail-edit",
        //#. '&' before a letter indicates a shortcut, i.e. Alt+E for '&Edit'
        _("&Edit")).addTo(toolbar);
    // section
    section = new ox.gui.PanelMenuSection(_("Edit")).addTo(tab);
    // add items
    add(section, meta.mail, "mail_detail-edit",
        ["edit", "reply", "reply-all", "forward", "delete",
         "print", "mark-as", "save"
        ]
    );
    
    // add spam/ham
    if (meta.mail.spam) {
        add(section, meta.mail, "mail_detail-edit", ["spam"]);
    } else if (meta.mail.ham) {
        add(section, meta.mail, "mail_detail-edit", ["ham"]);
    }
    
    // add other items
    add(section, meta.mail, "mail_detail-edit", ["view-source"]);
    
    // section "cancel"
    section = new ox.gui.PanelMenuSection(_("Cancel")).addTo(tab);
    add(section, ox.widgets.toolBar.meta.mail, "mail_detail", ["cancel"]);
    
    // add to "mark as"
    add(items["mail_detail-edit-mark-as"], meta.mail, "mail_detail-edit",
        ["mark-as-read", "mark-as-unread",
         "mark-as-answered", "mark-as-unanswered",
         "mark-as-deleted", "mark-as-undeleted"
        ]
    );
    
    // Tagging (Flags & Categories)
    tab = tabs.mailTagging = new ox.gui.PanelMenu("toolbar-mail_detail-tag",
        //#. '&' before a letter indicates a shortcut, i.e. Alt+F for '&Flags'
        _("&Flags")).addTo(toolbar);
    // section
    section = new ox.gui.PanelMenuSection(_("Flags"), "mailFlagsSection").addTo(tab);
    // add items
    add(section, meta.flags, "mail_detail-flag",
       ["0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "10"]
    );
    
    /*
     * Add handler
     */
    
    // event hash
    var events = {
        // edit mail
        "mail_detail-compose-compose": "OX_NEW_MAIL",
        "mail_detail-edit-reply": "OX_Mail_Reply",
        "mail_detail-edit-reply-all": "OX_Mail_ReplyAll",
        "mail_detail-edit-forward": "OX_Mail_Forward",
        "mail_detail-edit-delete": "OX_Mail_Delete",
        "mail_detail-edit-edit": "OX_Mail_Edit_Draft",
        "mail_detail-edit-copy": "OX_Mail_Copy",
        "mail_detail-edit-move": "OX_Mail_Move",
        "mail_detail-edit-save": "OX_Mail_SaveMail",
        "mail_detail-edit-view-source": "OX_Mail_Show_Source",
        "mail_detail-edit-print": "OX_Print",
        "mail_detail-cancel" : "OX_Cancel_Object",
        "mail_detail-edit-spam": "OX_Mail_Spam",
        "mail_detail-edit-ham": "OX_Mail_Ham"
    };
    
    // mapping of mail flags
    var mailFlags = {
        "read": "seen", 
        "unread": "unseen",
        "answered": "answer", 
        "unanswered": "answered",
        "deleted": "delete", 
        "undeleted": "undelete"
    };
    
    // add handler function (must return true when action taken)
    ox.ToolBarController.addHandler(function (id) {
        // regex matches
        var m;
        
        // defined by event?
        if (events[id] !== undefined) {
            // trigger
            triggerEvent(events[id]);
            return true;
        }
        
        // is mail mark?
        m = id.match(/^mail_detail-edit-mark-as-(\w+)$/);
        if (m && m.length) {
            // trigger event
            triggerEvent("OX_Mail_Flag", mailFlags[m[1]]);
            return true;
        }
        
        // is flag/mark?
        m = id.match(/^\w+-flag-(\d+)$/);
        if (m && m.length) {
            // trigger event
            triggerEvent("OX_Add_Flag", m[1]);
            return true;
        }
    });
    
    // add to top-level element
    toolbar.addTo(ox.widgets.toolBar.viewControl);
    
    ox.desktop.validate();
    ox.api.window.core.triggerEvent("OX_Window_Toolbar_Ready", {
        handle : window.url.guid,
        type : "maildetail"
    });
    
    ox.ToolBarController.processSelection("default", false, [ ], true);
};
/**
 *
 * All content on this website (including text, images, source
 * code and any other original works), unless otherwise noted,
 * is licensed under a Creative Commons License.
 *
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 *
 * Copyright (C) Open-Xchange Inc., 2006-2012
 * Mail: info@open-xchange.com
 *
 * @author Matthias Biggeleben <matthias.biggeleben@open-xchange.com>
 */

ox.gui.initToolBarNewWindowCalendar = function () {

    var $p = ox.gui.initToolBarNewWindow(),
        upsell = ox.api.window.core.ox.upsell;
    /*
     * Meta data
     */
    var meta = $.extend(ox.widgets.toolBar.meta, {
        /*
         * New/Edit Appointment Window
         */
        "window_calendar": {
            "save": {
                title: _("Save"),
                icons: ["save.png"],
                big: true
            },
            "add-participant": {
                title: _("Add participant"),
                icons: ["member_add.png"],
                big: true,
                requires : function(){
                    return ox.api.config.get("participants.showDialog");
                },
                mousedown: function(options, e) {
                    if (ox.api.config.get("participants.showDialog") == false) {
                        corewindow.triggerEvent("Feature_Not_Available", "modules/calendar/new/add_participants", window);
                    }
                },
                visible: upsell.visible("participants")
            },
            "add-external-participant": {
                title: _("Add external participant"),
                icons: ["distributionlist_extcontact.png"],
                big: true,
                requires : function(){
                    return ox.api.config.get("participants.showDialog");
                },
                mousedown: function(options, e) {
                    if (ox.api.config.get("participants.showDialog") == false) {
                        corewindow.triggerEvent("Feature_Not_Available", "modules/calendar/new/add_participants", window);
                    }
                },
                visible: upsell.visible("participants")
            },
            "remove-participant": {
                title: _("Remove participant"),
                icons: ["member_remove.png"],
                big: true,
                selection: "participants",
                requires : function(ability){
                    return currentpath[1] === "participant" &&
                    ox.api.config.get("participants.showDialog") && ability.some;
                },
                mousedown: function(options, e) {
                    if (ox.api.config.get("participants.showDialog") == false) {
                        corewindow.triggerEvent("Feature_Not_Available", "modules/calendar/new/remove_participants", window);
                    }
                },
                visible: upsell.visible("participants")
            },
            "attachment-add": {
                title: _("Add attachment"),
                icons: ["attachment_add.png"],
                selection: "attachments",
                requires: function() {
                    return ox.api.config.get('modules.infostore.module');
                },
                mousedown: function (options, e) {
                    if (ox.api.config.get("modules.infostore.module") == false) {
                        corewindow.triggerEvent("Feature_Not_Available", "modules/calendar/new/add_attachment", window);
                    }
                },
                visible: upsell.visible("infostore")
            },
            "attachment-remove": {
                title: _("Remove attachment"),
                icons: ["attachment_remove.png"],
                selection: "attachments",
                requires: function(ability, selection) {
                    return currentpath[1] === "attachment" &&
                        ox.api.config.get('modules.infostore.module') &&
                        ability.some;
                },
                mousedown: function (options, e) {
                    if (ox.api.config.get("modules.infostore.module") == false) {
                        corewindow.triggerEvent("Feature_Not_Available", "modules/calendar/new/delete_attachment", window);
                    }
                },
                visible: upsell.visible("infostore")
            },
            "cancel": {
                 title: _("Cancel"),
                 icons: ["cancel.png"],
                 big: true
            }
        }
    });


    /*
     * Add widgets
     */

    // vars
    var tabs = ox.widgets.toolBar.tabs;
    var items = ox.widgets.toolBar.menuItems;
    var toolbar, tab, section;

    // fn
    var add = $p.add;

    // create toolbar instance
    toolbar = ox.widgets.toolBar.views.calendar = new ox.gui.TabMenu(
        $p.getOptions({
            id: "new-calendar-menu",
            showSectionNames: false
        })
    );

    // new
    tab = tabs.newCalendarWindow = new ox.gui.PanelMenu(
        "window-calendar-main", _("Appointment")).addTo(toolbar);
    // "save" section
    section = new ox.gui.PanelMenuSection(_("New")).addTo(tab);
    add(section, ox.widgets.toolBar.meta.window_calendar, "calendar_new",
        ["save"]);
    // "participant" section
    if (upsell.isVisible("participants")) {
        section = new ox.gui.PanelMenuSection(_("Participants")).addTo(tab);
        add(section, ox.widgets.toolBar.meta.window_calendar, "calendar_new",
            ["add-participant", "add-external-participant", "remove-participant"]
        );
    }
    // "attachment" section
    if (upsell.isVisible("infostore") && upsell.isVisible("calendar-attachments")) {
        section = new ox.gui.PanelMenuSection(_("Attachments")).addTo(tab);
        add(section, ox.widgets.toolBar.meta.window_calendar, "calendar_new",
            ["attachment-add", "attachment-remove"]);
    }
    // "cancel" section
    section = new ox.gui.PanelMenuSection(_("Cancel")).addTo(tab);
    add(section, ox.widgets.toolBar.meta.window_calendar, "calendar_new",
        ["cancel"]);

    register("OX_View_Changed", function() {
        ox.ToolBarController.processSelection("default", false, [], true);
        ox.ToolBarController.processSelection("participants", false,
            newAppointment.participants.participantsGridBig.selection.
                getSelected(),
            true);
        ox.ToolBarController.processSelection("attachments", false,
            newAppointment.attachmentGrid ?
                newAppointment.attachmentGrid.liveGrid.selection.getSelected() :
                [],
            true);
    });

    // event mapping
    var events = {
        "calendar_new-save" : "OX_SAVE_OBJECT",
        "calendar_new-add-participant" : "OX_Object_Add_Participant",
        "calendar_new-add-external-participant" : "OX_Object_Open_External",
        "calendar_new-remove-participant" : "OX_Object_Participant_Delete",
        "calendar_new-cancel" : "OX_Cancel_Object",
        "calendar_new-attachment-add" : "OX_Object_Add_Attachment",
        "calendar_new-attachment-remove" : "OX_Object_Attachment_Delete"
    };

    // add handler function (must return true when action taken)
    ox.ToolBarController.addHandler(function (id) {
        // defined by event?
        if (events[id] !== undefined) {
            // add (external) participant?
            if (id === "calendar_new-add-participant" || id === "calendar_new-add-external-participant") {
                // change tab
                changeView("calendar_new/participant");
            } else if (id === "calendar_new-attachment-add") {
                // change tab
                changeView("calendar_new/attachment");
            }
            // trigger
            triggerEvent(events[id]);
            return true;
        }
    });


    // add to top-level element
    toolbar.addTo(ox.widgets.toolBar.viewControl);

    $p.postInit();

    ox.desktop.validate();
    triggerEvent("OX_View_Changed");

    ox.api.window.core.triggerEvent("OX_Window_Toolbar_Ready", {
            handle : window.url.guid,
            type : "calendar"
    });

    ox.ToolBarController.processSelection("participants", false, [], true);
    ox.ToolBarController.processSelection("attachments", false, [], true);
};
/**
 * 
 * All content on this website (including text, images, source
 * code and any other original works), unless otherwise noted,
 * is licensed under a Creative Commons License.
 * 
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 * 
 * Copyright (C) Open-Xchange Inc., 2006-2012
 * Mail: info@open-xchange.com 
 * 
 * @author  Alexander Quast <alexander.quast@open-xchange.com>
 */

ox.gui.initToolBarNewWindowContact = function () {

    var $p = ox.gui.initToolBarNewWindow(),
        upsell = corewindow.ox.upsell;
    
    /*
     * Meta data
     */
    var meta = $.extend(ox.widgets.toolBar.meta, {
        /*
         * New Mail Window
         */
        "window_contact": {
            "save": {
                title: _("Save"),
                icons: ["save.png"],
                big: true
               
            },
            "add-picture": {
                title: _("Add picture"),
                icons: ["picture_add.png"],
                big: true
            },
            "remove-picture": {
                title: _("Remove picture"),
                icons: ["picture_remove.png"],
                big: true
            },
            "attachment-add": {
                title: _("Add attachment"),
                icons: ["attachment_add.png"],
                big: false, 
                requires: function() {
                    return ox.api.config.get('modules.infostore.module');
                },
                mousedown: function (options, e) {
                    if (ox.api.config.get("modules.infostore.module") == false) {
                        corewindow.triggerEvent("Feature_Not_Available", "modules/contacts/new/add_attachment", window);
                    }
                },
                visible: upsell.visible("infostore")
            },
            "attachment-remove": {
                title: _("Remove attachment"),
                icons: ["attachment_remove.png"],
                selection: "attachments",
                requires: function(ability) {
                    return currentpath[1] === "attachment" &&
                        ox.api.config.get('modules.infostore.module') &&
                        ability.some;
                },
                mousedown: function (options, e) {
                    if (ox.api.config.get("modules.infostore.module") == false) {
                        corewindow.triggerEvent("Feature_Not_Available", "modules/contacts/new/delete_attachment", window);
                    }
                },
                big: false,
                visible: upsell.visible("infostore")
            },
            "cancel": {
                 title: _("Cancel"),
                 big: true,
                 icons: ["cancel.png"]
            }
        }
    });
    
    
    /*
     * Add widgets
     */
    
    // vars
    var tabs = ox.widgets.toolBar.tabs;
    var items = ox.widgets.toolBar.menuItems;
    var toolbar, tab, section;
    
    // fn
    var add = $p.add;
    
    // create toolbar instance
    toolbar = ox.widgets.toolBar.views.contact = new ox.gui.TabMenu(
        $p.getOptions({
            id: "new-contact-menu",
            showSectionNames: false
        })
    );
    
    // main tab
    tab = tabs.newMailWindow = new ox.gui.PanelMenu(
        "window-contact-main", _("Contact")).addTo(toolbar);
    
    // Section 1, save contact
    section = new ox.gui.PanelMenuSection(_("Contact")).addTo(tab);
    add(section, ox.widgets.toolBar.meta.window_contact, "contact_new",
        ["save"]);

    // Section 2, add/remove contact picture
    section = new ox.gui.PanelMenuSection(_("Picture")).addTo(tab);
    add(section, ox.widgets.toolBar.meta.window_contact, "contact_new",
        ["add-picture","remove-picture"]);
    
    // Section 3, add/remove attachment
    if (upsell.isVisible("infostore") && upsell.isVisible("contacts-attachments")) {
        section = new ox.gui.PanelMenuSection(_("Attachment")).addTo(tab);
        add(section, ox.widgets.toolBar.meta.window_contact, "contact_new",
            ["attachment-add","attachment-remove"]
        );
    }
    // Section 4, cancel
    section = new ox.gui.PanelMenuSection(_("Cancel")).addTo(tab);
    add(section, ox.widgets.toolBar.meta.window_contact, "contact_new",
        ["cancel"]);
    
    // add to top-level element
    toolbar.addTo(ox.widgets.toolBar.viewControl);
    
    $p.postInit();
    
    register("OX_View_Changed", function() {
        ox.ToolBarController.processSelection("default", false, [], true);
        ox.ToolBarController.processSelection("attachments", false,
            tmp_attachment ? tmp_attachment.liveGrid.selection.getSelected()
                           : [],
            true);
    });
    
    // event mapping
    var events = {
            "contact_new-save" : "OX_SAVE_OBJECT",
            "contact_new-attachment-add" : "OX_Object_Add_Attachment",
            "contact_new-attachment-remove" : "OX_Object_Attachment_Delete",
            "contact_new-add-picture" : "OX_Picture_Add",
            "contact_new-remove-picture" : "OX_Picture_Remove",
            "contact_new-cancel" : "OX_Cancel_Object"
    }
  
    // add handler function (must return true when action taken)
    ox.ToolBarController.addHandler(function (id) {
        // defined by event?
        if (events[id] !== undefined) {
            // change tab?
            if (id === "contact_new-add-picture" || id === "contact_new-remove-picture") {
                // change tab
                changeView("contact_new/personal");
            } else if (id === "contact_new-attachment-add") {
                // change tab
                changeView("contact_new/attachment");
            }
            // trigger
            triggerEvent(events[id]);
            return true;
        }
    });
    
    ox.desktop.validate();
    
    // initial redraw to update icon state
    triggerEvent("OX_View_Changed");
    
    ox.api.window.core.triggerEvent("OX_Window_Toolbar_Ready", {
            handle : window.url.guid,
            type : "contact"
    });
    ox.ToolBarController.processSelection("attachments", false, [], true);
};
/**
 * 
 * All content on this website (including text, images, source
 * code and any other original works), unless otherwise noted,
 * is licensed under a Creative Commons License.
 * 
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 * 
 * Copyright (C) Open-Xchange Inc., 2006-2012
 * Mail: info@open-xchange.com 
 * 
 * @author Viktor Pracht <viktor.pracht@open-xchange.com>
 */

ox.gui.initToolBarNewWindowTask = function() {

    var $p = ox.gui.initToolBarNewWindow(),
        upsell = corewindow.ox.upsell;
    
    /*
     * Meta data
     */
    var meta = $.extend(ox.widgets.toolBar.meta, {
        /*
         * New Mail Window
         */
        "window_task": {
            "save": {
                title: _("Save"),
                icons: ["save.png"],
                big: true
               
            },
            "add-participant": {
                title: _("Add participant"),
                icons: ["member_add.png"],
                requires : function() {
                    return ox.api.config.get("participants.showDialog");
                },
                big: true,
                mousedown: function(options, e) {
                    if (ox.api.config.get("participants.showDialog") == false) {
                        corewindow.triggerEvent("Feature_Not_Available", "modules/tasks/new/add_participants", window);
                    }
                }
            },
            "add-external-participant": {
                title: _("Add external participant"),
                icons: ["distributionlist_extcontact.png"],
                requires : function() {
                    return ox.api.config.get("participants.showDialog");
                },
                big: true,
                mousedown: function(options, e) {
                    if (ox.api.config.get("participants.showDialog") == false) {
                        corewindow.triggerEvent("Feature_Not_Available", "modules/tasks/new/add_participants", window);
                    }
                }
            },
            "remove-participant": {
                title: _("Remove participant"),
                icons: ["member_remove.png"],
                selection: "participants",
                requires : function(ability) {
                    return currentpath[1] === "participant" && 
                    ox.api.config.get("participants.showDialog") && ability.some;
                },
                big: true,
                mousedown: function(options, e) {
                    if (ox.api.config.get("participants.showDialog") == false) {
                        corewindow.triggerEvent("Feature_Not_Available", "modules/tasks/new/remove_participants", window);
                    }
                }
            },
            "attachment-add": {
                title: _("Add attachment"),
                icons: ["attachment_add.png"],
                requires: function() {
                    return ox.api.config.get('modules.infostore.module');
                },
                mousedown: function (options, e) {
                    if (ox.api.config.get("modules.infostore.module") == false) {
                        corewindow.triggerEvent("Feature_Not_Available", "modules/tasks/new/add_attachment", window);
                    }
                },
                visible: upsell.visible("infostore")
            },
            "attachment-remove": {
                title: _("Remove attachment"),
                icons: ["attachment_remove.png"],
                selection: "attachments",
                requires: function(ability) {
                    return currentpath[1] === "attachment" &&
                        ox.api.config.get('modules.infostore.module') &&
                        ability.some;
                },
                mousedown: function (options, e) {
                    if (ox.api.config.get("modules.infostore.module") == false) {
                        corewindow.triggerEvent("Feature_Not_Available", "modules/tasks/new/delete_attachment", window);
                    }
                },
                visible: upsell.visible("infostore")
            },
            "cancel": {
                 title: _("Cancel"),
                 big: true,
                 icons: ["cancel.png"]
            }
        }
    });
    
    
    /*
     * Add widgets
     */
    
    // vars
    var tabs = ox.widgets.toolBar.tabs;
    var items = ox.widgets.toolBar.menuItems;
    var toolbar, tab, section;
    
    // fn
    var add = $p.add;
    
    // create toolbar instance
    toolbar = ox.widgets.toolBar.views.task = new ox.gui.TabMenu(
        $p.getOptions({
            id: "new-task-menu",
            showSectionNames: false
        })
    );
    
    // main tab
    tab = tabs.newMailWindow = new ox.gui.PanelMenu(
        "window-task-main", _("&New Task")).addTo(toolbar);
    
    // Section 1, save task
    section = new ox.gui.PanelMenuSection(_("Task")).addTo(tab);
    add(section, ox.widgets.toolBar.meta.window_task, "task_new", ["save"]);

    // Section 2, add/remove participant
    if (upsell.isVisible("participants")) {
        section = new ox.gui.PanelMenuSection(_("Participants")).addTo(tab);
        add(section, ox.widgets.toolBar.meta.window_task, "task_new",
            ["add-participant", "add-external-participant", "remove-participant"]
        );
    }
    
    // Section 3, add/remove attachment
    if (upsell.isVisible("infostore") && upsell.isVisible("tasks-attachments")) {
        section = new ox.gui.PanelMenuSection(_("Attachment"), "task_newAttachmentSection").addTo(tab);
        add(section, ox.widgets.toolBar.meta.window_task, "task_new",
            ["attachment-add", "attachment-remove"]
        );
    }
    // Section 4, cancel
    section = new ox.gui.PanelMenuSection(_("Cancel")).addTo(tab);
    add(section, ox.widgets.toolBar.meta.window_task, "task_new", ["cancel"]);
    
    // add to top-level element
    toolbar.addTo(ox.widgets.toolBar.viewControl);
    
    $p.postInit();
    
    register("OX_View_Changed", function() {
        ox.ToolBarController.processSelection("default", false, [], true);
        ox.ToolBarController.processSelection("participants", false,
            tmp_participant.participantsGridBig.selection.getSelected(), true);
        ox.ToolBarController.processSelection("attachments", false,
            tmp_attachment ? tmp_attachment.liveGrid.selection.getSelected()
                           : [],
            true);
    });
    
    // event mapping
    var events = {
        "task_new-save" : "OX_SAVE_OBJECT",
        "task_new-add-participant" : "OX_Object_Add_Participant",
        "task_new-add-external-participant" : "OX_Object_Open_External",
        "task_new-remove-participant" : "OX_Object_Participant_Delete",
        "task_new-attachment-add" : "OX_Object_Add_Attachment",
        "task_new-attachment-remove" : "OX_Object_Attachment_Delete",
        "task_new-cancel" : "OX_Cancel_Object"
    };
    
    // add handler function (must return true when action taken)
    ox.ToolBarController.addHandler(function (id) {        
        // defined by event?
        if (events[id] !== undefined) {
            // auto-change tab?
            if (id === "task_new-add-participant" || id === "task_new-add-external-participant") {
                // change tab
                changeView("task_new/participant");
            } else if (id === "task_new-attachment-add") {
                // change tab
                changeView("task_new/attachment");
            }
            // trigger
            triggerEvent(events[id]);
            return true;
        }
    });
    
    ox.desktop.validate();
    
    // initial redraw to update icon state
    triggerEvent("OX_View_Changed");
    ox.api.window.core.triggerEvent("OX_Window_Toolbar_Ready", {
            handle : window.url.guid,
            type : "task"
    });
    ox.ToolBarController.processSelection("participants", false, [], true);
    ox.ToolBarController.processSelection("attachments", false, [], true);
};
/**
 * 
 * All content on this website (including text, images, source
 * code and any other original works), unless otherwise noted,
 * is licensed under a Creative Commons License.
 * 
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 * 
 * Copyright (C) Open-Xchange Inc., 2006-2012
 * Mail: info@open-xchange.com 
 * 
 * @author Alexander Quast <alexander.quast@open-xchange.com>
 */

ox.gui.initToolBarNewWindowDistributionlist = function() {

    var $p = ox.gui.initToolBarNewWindow();
    
    /*
     * Meta data
     */
    var meta = $.extend(ox.widgets.toolBar.meta, {
        /*
         * New Mail Window
         */
        "window_distributionlist": {
            "save": {
                title: _("Save"),
                icons: ["save.png"],
                big: true
            },
            "add": {
                title: _("Add from addressbook"),
                icons: ["member_add.png"],
                big: true
            },
            "custom": {
                title: _("Custom form"),
                icons: ["distributionlist_extcontact.png"],
                big: true
            },
            "delete": {
                title: _("Delete"),
                icons: ["member_remove.png"],
                selection: "contact",
                requires : ["some"],
                big: true
            },
            "cancel": {
                 title: _("Cancel"),
                 big: true,
                 icons: ["cancel.png"]
            }
        }
    });
    /*
     * Add widgets
     */
    
    // vars
    var tabs = ox.widgets.toolBar.tabs;
    var items = ox.widgets.toolBar.menuItems;
    var toolbar, tab, section;
    
    // fn
    var add = $p.add;
    
    // create toolbar instance
    toolbar = ox.widgets.toolBar.views.distributionlist = new ox.gui.TabMenu(
        $p.getOptions({
            id: "new-distribution-menu",
            showSectionNames: false
        })
    );
    
    // main tab
    tab = tabs.newMailWindow = new ox.gui.PanelMenu(
        "window-distributionlist-main", _("&Distribution List")).addTo(toolbar);
    
    // Section 1, save distributionlist
    section = new ox.gui.PanelMenuSection(_("Distribution List")).addTo(tab);
    add(section, ox.widgets.toolBar.meta.window_distributionlist, "contact_new", ["save"]);

    // Section 2, add/remove/custom contact
    section = new ox.gui.PanelMenuSection(_("Contact")).addTo(tab);
    add(section, ox.widgets.toolBar.meta.window_distributionlist, "contact_new",
        ["add", "custom", "delete"]);
    
    // Section 3, cancel
    section = new ox.gui.PanelMenuSection(_("Cancel")).addTo(tab);
    add(section, ox.widgets.toolBar.meta.window_distributionlist, "contact_new", ["cancel"]);
    
    // add to top-level element
    toolbar.addTo(ox.widgets.toolBar.viewControl);
    
    $p.postInit();
    
    register("SubSelected", function() {
        ox.ToolBarController.processSelection("contact", false,
                distruBigGrid.selection.getSelected(), true);
    });
    
    // event mapping
    var events = {
        "contact_new-save" : "OX_SAVE_OBJECT",
        "contact_new-add" : "OX_Object_Distributionlist_Internal_Contact",
        "contact_new-custom" : "OX_Object_Distributionlist_External_Mail",
        "contact_new-delete" : "OX_Object_Distributionlist_Delete",
        "contact_new-cancel" : "OX_Cancel_Object"
    }
  
    // add handler function (must return true when action taken)
    ox.ToolBarController.addHandler(function(id) {
        
        // regex matches
        var m;
        
        // defined by event?
        if (events[id] !== undefined) {
            // trigger
            triggerEvent(events[id]);
            return true;
        }
    });
    
    ox.desktop.validate();
    
    ox.ToolBarController.processSelection("contact", false, [], true);
    ox.ToolBarController.processSelection("default", false, [], true);
    setTimeout(function(){triggerEvent("SubSelected");}, 10);
};
/**
 * 
 * All content on this website (including text, images, source
 * code and any other original works), unless otherwise noted,
 * is licensed under a Creative Commons License.
 * 
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 * 
 * Copyright (C) Open-Xchange Inc., 2006-2012
 * Mail: info@open-xchange.com 
 * 
 * @author Matthias Biggeleben <matthias.biggeleben@open-xchange.com>
 */

ox.gui.initToolBarNewWindowInfoStore = function() {

    var $p = ox.gui.initToolBarNewWindow();
    
    /*
     * Meta data
     */
    var meta = $.extend(ox.widgets.toolBar.meta, {
        /*
         * New Window
         */
        "window_infostore": {
            "save": {
                title: _("Save"),
                icons: ["save.png"],
                big: true
               
            },
            "cancel": {
                 title: _("Cancel"),
                 big: true,
                 icons: ["cancel.png"]
            }
        }
    });
    
    
    /*
     * Add widgets
     */
    
    // vars
    var tabs = ox.widgets.toolBar.tabs;
    var items = ox.widgets.toolBar.menuItems;
    var toolbar, tab, section;
    
    // fn
    var add = $p.add;
    
    // create toolbar instance
    toolbar = ox.widgets.toolBar.views.infostore = new ox.gui.TabMenu(
        $p.getOptions({
            id: "new-infostore-menu",
            showSectionNames: false
        })
    );
    
    // main tab
    tab = tabs.newMailWindow = new ox.gui.PanelMenu(
        "window-infostore-main", _("File")).addTo(toolbar);
    
    // Section "save"
    section = new ox.gui.PanelMenuSection(_("Save")).addTo(tab);
    add(section, ox.widgets.toolBar.meta.window_infostore, "infostore_new", ["save"]);
    
    // Section "cancel"
    section = new ox.gui.PanelMenuSection(_("Cancel")).addTo(tab);
    add(section, ox.widgets.toolBar.meta.window_infostore, "infostore_new", ["cancel"]);
    
    // add to top-level element
    toolbar.addTo(ox.widgets.toolBar.viewControl);
    
    $p.postInit();
    
    register("OX_View_Changed", function() {
        ox.ToolBarController.processSelection("default", false, [], true);
    });
    
    // event mapping
    var events = {
        "infostore_new-save" : "OX_SAVE_OBJECT",
        "infostore_new-cancel" : "OX_Cancel_Object"
    }
  
    // add handler function (must return true when action taken)
    ox.ToolBarController.addHandler(function(id) {
        
        // regex matches
        var m;
        
        // defined by event?
        if (events[id] !== undefined) {
            // trigger
            triggerEvent(events[id]);
            return true;
        }
    });
    
    ox.desktop.validate();
    
    // initial redraw to update icon state
    triggerEvent("OX_View_Changed");
    ox.api.window.core.triggerEvent("OX_Window_Toolbar_Ready", {
        handle : window.url.guid,
        type : "infostore"
    });
    ox.ToolBarController.processSelection("participants", false, [], true);
    ox.ToolBarController.processSelection("attachments", false, [], true);
};
}());
